/*************************************************************************\
*
*  DIALOG.C
*
*  This module is responsible for presenting error and confirmation 
*  dialog boxes.
*
*  (c)2002 by Robert Hildinger
*
\*************************************************************************/

#include <PalmOS.h>
#include "Power48Rsc.h"
#include "Power48BMPRsc.h"
#include <SonyCLIE.h>
#include "emu48.h"




#define alertConfirmDialogOriginX	23
#define alertConfirmDialogOriginY	67
#define alertConfirmDialogExtentX	274
#define alertConfirmDialogExtentY	188


const objectButton ACLayout[] =  // coordinates relative to upper left of Alert/Confirm dialog
{
	{  22, 157,  85, 182, 2},		// AlertConfirmNO, scope field def: 1 = active in Alert dialog
	{ 188, 157, 251, 182, 3},		// AlertConfirmYESorOK,             2 = active in Confirm dialog
};                                  //                                  3 = active in both


static UInt16 TranslatePenToACButton(Int16 x, Int16 y, UInt8 alertOrConfim)
{
	UInt16 i;
	
	for (i=0;i<maxAlertConfirmButtons;i++)
	{
		if ((x >= ((ACLayout[i].upperLeftX+alertConfirmDialogOriginX)>>1)) &&
		    (x <= ((ACLayout[i].lowerRightX+alertConfirmDialogOriginX)>>1)) &&
		    (y >= ((ACLayout[i].upperLeftY+alertConfirmDialogOriginY)>>1)) &&
		    (y <= ((ACLayout[i].lowerRightY+alertConfirmDialogOriginY)>>1)) &&
		    (alertOrConfim&ACLayout[i].scope))
		{
			return i;
		}
	}
	return noAlertConfirmButton;
}


static void DepressAlertConfirmButton(UInt16 acCode)
{
	RectangleType 	myRect;
	RGBColorType  	prevRGB;
	
	WinSetForeColorRGB(&redRGB,&prevRGB);
	WinSetBackColorRGB(&greyRGB,NULL);
	WinSetDrawMode(winSwap);
	
	myRect.topLeft.x = ACLayout[acCode].upperLeftX+alertConfirmDialogOriginX;
	myRect.topLeft.y = ACLayout[acCode].upperLeftY+alertConfirmDialogOriginY;
	myRect.extent.x = ACLayout[acCode].lowerRightX-ACLayout[acCode].upperLeftX+1;
	myRect.extent.y = ACLayout[acCode].lowerRightY-ACLayout[acCode].upperLeftY+1;

	HRWinPaintRectangle(hrLibRefNum,&myRect,10);
	WinSetDrawMode(winPaint);
	WinSetBackColorRGB(&backRGB,NULL);	
	WinSetForeColorRGB(&prevRGB,NULL);
}


static void ReleaseAlertConfirmButton(UInt16 acCode)
{
	RectangleType myRect;
	RGBColorType  prevRGB;
	
	WinSetForeColorRGB(&redRGB,&prevRGB);
	WinSetBackColorRGB(&greyRGB,NULL);
	WinSetDrawMode(winSwap);
			
	myRect.topLeft.x = ACLayout[acCode].upperLeftX+alertConfirmDialogOriginX;
	myRect.topLeft.y = ACLayout[acCode].upperLeftY+alertConfirmDialogOriginY;
	myRect.extent.x = ACLayout[acCode].lowerRightX-ACLayout[acCode].upperLeftX+1;
	myRect.extent.y = ACLayout[acCode].lowerRightY-ACLayout[acCode].upperLeftY+1;
			
	HRWinPaintRectangle(hrLibRefNum,&myRect,10);
	WinSetDrawMode(winPaint);
	WinSetForeColorRGB(&prevRGB,NULL);
	WinSetBackColorRGB(&backRGB,NULL);
}




/*************************************************************************\
*
*  FUNCTION:  Int8 OpenAlertConfirmDialog(UInt8 alertOrConfirm,DmResID stringID,Boolean skipSweep)
*
\*************************************************************************/
Int8 OpenAlertConfirmDialog(dialogType alertOrConfirm, DmResID stringID, 
							Boolean skipSweep, Boolean skipSave)
{
	RectangleType 	oldRect,newRect,opRect;
    EventType 		event;
	Int16 			penX, penY;
	Boolean 		penDown;
	RGBColorType 	prevRGB,prevRGB2;
	FieldType 		*textField;
	FormPtr 		frmP;
	MemHandle 		textHandle,oldHandle;
	char			*textPtr;
	UInt16 			textFieldIndex;
	UInt16			acCode;
	UInt16			acReleasePending=noAlertConfirmButton;
	int				applyChangesOrCancel=0;
	int				fileIndex=0;
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	WinHandle		dialogBackStore;
	WinHandle		tempWinStore;
	Err				error;
	Coord 			picWidth,picHeight;
	Boolean			revertScreen=false;

	if ((option320x320)&&(statusDisplayMode2x2))
	{
		toggle_lcd(); // put calc screen in 1x1 mode
		revertScreen=true;
	}

	if (!skipSave)
	{
		if (skipSweep)
		{
			// save area directly behind dialog window in new offscreen window
			dialogBackStore = HRWinCreateOffscreenWindow(hrLibRefNum,
				alertConfirmDialogExtentX,alertConfirmDialogExtentY,screenFormat,&error);
			if (error) return -1;
			opRect.topLeft.x=alertConfirmDialogOriginX;
			opRect.topLeft.y=alertConfirmDialogOriginY;
			opRect.extent.x=alertConfirmDialogExtentX;
			opRect.extent.y=alertConfirmDialogExtentY;
			HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,dialogBackStore,&opRect,0,0,winPaint);
		}
		else
		{	
			// save snapshot of emulator and do sweep in effect...
			opRect.topLeft.x=0;
			opRect.topLeft.y=0;
			opRect.extent.x=320;
			opRect.extent.y=alertConfirmDialogOriginY+alertConfirmDialogExtentY;
			HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,backStoreWindow,&opRect,opRect.topLeft.x,opRect.topLeft.y,winPaint);

			oldRect.topLeft.x = 0;
			oldRect.topLeft.y = 0;
			oldRect.extent.x  = 320;
			oldRect.extent.y  = ((option320x320)?320:480);
			newRect.topLeft.x = alertConfirmDialogOriginX;    // dialog box dimensions
			newRect.topLeft.y = alertConfirmDialogOriginY;
			newRect.extent.x  = alertConfirmDialogExtentX;
			newRect.extent.y  = alertConfirmDialogExtentY;
			rect_sweep(&oldRect,&newRect,8);
		}
	}
	
redrawdialog:
	UIColorSetTableEntry(UIFieldBackground,&greyRGB);
	UIColorSetTableEntry(UIFieldText,&blackRGB);
	UIColorSetTableEntry(UIFieldTextHighlightBackground,&greyRGB);
	UIColorSetTableEntry(UIFieldTextHighlightForeground,&greyRGB);
	WinSetForeColorRGB(&greyRGB,&prevRGB);
	WinSetBackColorRGB(&greyRGB,&prevRGB2);

	frmP = FrmGetActiveForm();
	textFieldIndex = FrmGetObjectIndex(frmP,MainDialogDescField);
	textField = FrmGetObjectPtr(frmP, textFieldIndex);

	// Draw dialog box...
	picHandle = DmGetResource(bitmapRsc, ((alertOrConfirm==1)?AlertDialogBitmapFamily:ConfirmDialogBitmapFamily));
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	tempWinStore = HRWinCreateOffscreenWindow(hrLibRefNum, 320,picHeight,screenFormat,&error);
	if (error) 
	{
		WinSetDrawWindow(mainDisplayWindow);
		FastDrawBitmap(picBitmap,alertConfirmDialogOriginX,alertConfirmDialogOriginY);
	}
	else
	{
		WinSetDrawWindow(tempWinStore);
		FastDrawBitmap(picBitmap,alertConfirmDialogOriginX,0);
		WinSetDrawWindow(mainDisplayWindow);
		PixelDissolve(tempWinStore,mainDisplayWindow,320L*(UInt32)picHeight,0,alertConfirmDialogOriginY,true);
		WinDeleteWindow(tempWinStore,false);
	}
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	oldHandle = FldGetTextHandle(textField);
	textHandle = DmGetResource(strRsc, stringID);
	textPtr = MemHandleLock(textHandle);
	if (StrLen(textPtr)>126)
		FldSetFont(textField, stdFont);
	else
		FldSetFont(textField, boldFont);
	MemHandleUnlock(textHandle);
	FldSetTextHandle( textField, textHandle );
	DmReleaseResource (textHandle);
	if (oldHandle != NULL) MemHandleFree(oldHandle);
    FrmShowObject(frmP,textFieldIndex);
	FldDrawField(textField);
	FldSetUsable(textField,false);
	WinSetForeColorRGB(&prevRGB,NULL);
	WinSetBackColorRGB(&prevRGB2,NULL);
	
	EvtFlushPenQueue();
	EvtFlushKeyQueue();

    do 
    {
        EvtGetEvent(&event, evtWaitForever);
        
        if (event.eType==penDownEvent)
        {
        	PenGetPoint(&penX, &penY, &penDown);
			acCode = TranslatePenToACButton(penX,penY,alertOrConfirm);
			if (acCode != noAlertConfirmButton)
			{
				DepressAlertConfirmButton(acCode);
				acReleasePending = acCode;
				continue;
			}
			else
			{
				acReleasePending = noAlertConfirmButton;
			}
		}
        if (event.eType==penUpEvent)
		{
			if (acReleasePending != noAlertConfirmButton)
			{
				PenGetPoint(&penX, &penY, &penDown);
				acCode = TranslatePenToACButton(penX,penY,alertOrConfirm);
				ReleaseAlertConfirmButton(acReleasePending);
					
				if (acReleasePending == acCode)
				{
					switch(acReleasePending)
					{
						case AlertConfirmNO:
							applyChangesOrCancel=1;
							break;
						case AlertConfirmYESorOK:
							applyChangesOrCancel=2;
							break;
						default: break;
					}
					continue;
				}
			}
		}
		
		if (event.eType==appStopEvent)  // respond to app-switching events.
		{
			if (alertOrConfirm==alert)
				DepressAlertConfirmButton(AlertConfirmYESorOK);
			else
				DepressAlertConfirmButton(AlertConfirmNO);
			SysTaskDelay(10);
			if (alertOrConfirm==alert)
				ReleaseAlertConfirmButton(AlertConfirmYESorOK);
			else
				ReleaseAlertConfirmButton(AlertConfirmNO);
			statusQuit=true;
			EvtAddEventToQueue(&event);  // put app stop back on queue
			applyChangesOrCancel=1;
			continue;
		}

		if (event.eType==keyDownEvent)
		{
			if (event.data.keyDown.chr == chrLineFeed)
			{
				DepressAlertConfirmButton(AlertConfirmYESorOK);
				SysTaskDelay(10);
				ReleaseAlertConfirmButton(AlertConfirmYESorOK);
				applyChangesOrCancel=2;
				continue;
			}
		}

		if ((event.eType==volumeUnmountedEvent) && (statusRUN))
		{
			if (!statusInUnmountDialog)
			{
				if (event.data.generic.datum[0]==msVolRefNum)
				{
					// Restart dialog with volume unmounted string
					EvtFlushPenQueue();
					FldSetTextHandle( textField, NULL );
					FldSetUsable(textField,false);  // I don't like how FrmHideObject erases...
					
					statusInUnmountDialog = true;
					
					if (OpenAlertConfirmDialog(alert,pleaseReinsertCardString,true,true))
						statusVolumeMounted=false;
					else
						statusVolumeMounted=true;
					statusInUnmountDialog = false;	
					
					goto redrawdialog;
				}			
			}
			continue;
		}

		if (event.eType==volumeMountedEvent)
		{
			CheckForRemount();
			if ((statusVolumeMounted) && (statusInUnmountDialog))
			{
				DepressAlertConfirmButton(AlertConfirmYESorOK);
				SysTaskDelay(10);
				ReleaseAlertConfirmButton(AlertConfirmYESorOK);
				applyChangesOrCancel=1;
			}
			continue;
		}
		
		if (event.eType==displayChangeEvent)
		{
			continue;
		}
		
        if (! SysHandleEvent(&event))
        {
			FrmDispatchEvent(&event);
        }
    } 
    while (!applyChangesOrCancel);

	
	EvtFlushPenQueue();

	FldSetTextHandle( textField, NULL );
	DmReleaseResource (textHandle);
		
	FldSetUsable(textField,false);  // I don't like how FrmHideObject erases...
	
	if (!skipSave)
	{
		if (skipSweep)
		{
			// just restore the backing store image and delete the window
			opRect.topLeft.x=0;
			opRect.topLeft.y=0;
			opRect.extent.x=alertConfirmDialogExtentX;
			opRect.extent.y=alertConfirmDialogExtentY;
			HRWinCopyRectangle(hrLibRefNum,dialogBackStore,mainDisplayWindow,&opRect,alertConfirmDialogOriginX,alertConfirmDialogOriginY,winPaint);
			WinDeleteWindow(dialogBackStore,false);
		}
		else
		{
			// do the sweep thing
			oldRect=newRect;
			newRect.topLeft.x = 0;
			newRect.topLeft.y = 0;
			newRect.extent.x  = 320;
			newRect.extent.y  = ((option320x320)?320:480);
			rect_sweep(&oldRect,&newRect,8);

			opRect.topLeft.x=0;
			opRect.topLeft.y=0;
			opRect.extent.x=320;
			opRect.extent.y=alertConfirmDialogOriginY+alertConfirmDialogExtentY;
			HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&opRect,opRect.topLeft.x,opRect.topLeft.y,winPaint);
		}
	}
	
	if (revertScreen)
		toggle_lcd();

	if (applyChangesOrCancel==1)
		return 0;
		
	return 1;
}
