/*************************************************************************\
*
*  DISPLAY.C
*
*  This module is responsible for all emulator display and GUI elements of
*  the emulator
*
*  (c)2002 by Robert Hildinger
*
\*************************************************************************/


#include "emu48.h"
#include <PalmOS.h>
#include <SonyCLIE.h>
#include "IO.H"

BitmapPtr drawBitmap;
BitmapPtr basePixel2Bitmap;
BitmapPtr basePixel1Bitmap;
BitmapPtr annunciatorBitmap;
UInt32 *drawMemPtrBase;
UInt32 *lcd2x2MemPtr1Base;
UInt16 *lcd1x1MemPtrBase;
UInt16 *softKeyMemPtrBase;
UInt16 *labelMemPtrBase;
UInt32 *lcd2x2AnnAreaPtrBase;
UInt32 *lcd1x1AnnAreaPtrBase;
UInt32 *contrastPixel2Map[16];
UInt16 *contrastPixel1Map[16];
UInt16 *annunciatorMemPtrBase;
UInt32 *nibbleMap2x2Base;
UInt16 *nibbleMap1x1Base;

Boolean lcdFullyInitialized=false;




#define 	softKeyAIntX 	12
#define 	softKeyBIntX 	59
#define 	softKeyCIntX 	106
#define 	softKeyDIntX 	12
#define 	softKeyEIntX 	59
#define 	softKeyFIntX 	106

#define 	softKeyRow1Y	25
#define 	softKeyRow2Y	52

#define		softLabelStart	57

#define 	skipToKeyB		52
#define 	skipToKeyC		52
#define 	skipToKeyD		17050
#define 	skipToKeyE		52
#define 	skipToKeyF		52







inline char* GetRealAddressNoIO2(long d)
{
  	if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
	{
		return d+(char*)mcRAMDelta;
	}
	else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
	{
		return d+(char*)mcCE2Delta;
	}
	else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
	{
		return d+(char*)mcCE1Delta;
	}
 	else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
	{
		return d+(char*)mcNCE3Delta;
	}
  	else   //ROM
  	{
  		if (status49GActive)
  		{
  			d&=0x7ffff;   // 49 ROM is mirrored
  			if (d<0x40000)  // lo bank
  				return flashRomLo+d;
  			else			// hi bank
  				return flashRomHi+d;
  		}
  		else
			return rom+d;
	}
}





/*************************************************************************\
*
*  FUNCTION:  DrawDisplayLine16x2()
*
*  This function draws one line of the display at 16 bit color depth and
*  2x2 pixel size
*
\*************************************************************************/
/*void DrawDisplayLine16x2(void) 
{
	UInt32 	*lcd1MemPtr;			// (UInt32) = copy 32 bits at a time
	UInt32 	*lcd2MemPtr;			// (UInt32) = copy 32 bits at a time
	UInt32 	*contrastPixelMemPtr;	// (UInt32) = copy 32 bits at a time
	Int16  	offset;
	UInt16  switchValue;
	UInt8	*valPtr;
	UInt32 	**contrastP2Map;

	if (!lcdOn) return;  
	
    offset = (320*lcdCurrentLine);
    lcd1MemPtr = lcd2x2MemPtr1Base + offset;
	lcd2MemPtr = lcd1MemPtr + 160;
	valPtr = (UInt8*)lcdLineAddress[lcdCurrentLine];
	switchValue = lcdPixelOffset&3;
	contrastP2Map=contrastPixel2Map;
	
	switch (switchValue) 
	{
		case 0:	contrastPixelMemPtr = contrastP2Map[*valPtr++];  // nibble 0
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				break;
		case 1:	contrastPixelMemPtr = contrastP2Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr++;  //skip forward one pixel first row
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=73; //skip forward one pixel second row
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr;
				break;
		case 2:	contrastPixelMemPtr = contrastP2Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr+=2;  //skip forward two pixels first row
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=74;  //skip forward two pixels second row
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				break;
		case 3:	contrastPixelMemPtr = contrastP2Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr+=3;  //skip forward three pixels first row
				*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=75;  //skip forward three pixels second row
				*lcd2MemPtr++ = *contrastPixelMemPtr;
				break;
	}
	
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 1
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
		contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 2
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
		contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 3
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
		contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 4
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 5
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 6
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 7
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 8
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 9
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 10
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 11
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 12
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 13
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 14
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 15
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 16
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 17
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 18
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 19
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 20
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 21
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 22
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 23
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 24
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 25
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 26
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 27
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 28
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 29
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 30
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 31
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;

	switch (switchValue) 
	{
		case 0: contrastPixelMemPtr = contrastPixel2Map[*valPtr];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=73;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr = *contrastPixelMemPtr;
				break;
		case 1:	contrastPixelMemPtr = contrastPixel2Map[*valPtr];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr = *contrastPixelMemPtr;
				break;
		case 2:	contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr];  // nibble 33
				*lcd1MemPtr = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=75;
				*lcd2MemPtr = *contrastPixelMemPtr;
				break;
		case 3:	contrastPixelMemPtr = contrastPixel2Map[*valPtr++];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=72;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr++;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr++ = *contrastPixelMemPtr;
				contrastPixelMemPtr = contrastPixel2Map[*valPtr];  // nibble 33
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr = *contrastPixelMemPtr++;
				contrastPixelMemPtr+=74;
				*lcd2MemPtr++ = *contrastPixelMemPtr++;*lcd2MemPtr = *contrastPixelMemPtr;
				break;
	}
	
	lcdRedrawBuffer[lcdCurrentLine] = 0;
}*/

/*************************************************************************\
*
*  FUNCTION:  DrawDisplayLine16x2()
*
*  This function draws one line of the display at 16 bit color depth and
*  2x2 pixel size
*
\*************************************************************************/
asm void DrawDisplayLine16x2(void)
{
			movem.l		a1-a4,-(a7)  // a1 safe

			tst.w		lcdOn
			beq.w		ddexit2
			
    //offset = (320*lcdCurrentLine);
    		moveq		#0,d1
			move.w		lcdCurrentLine,d1
			move.l		d1,d0
			muls.w		#1280,d1
			
    //lcd1MemPtr = lcd2x2MemPtr1Base + offset;
			movea.l		lcd2x2MemPtr1Base,a3
			lea			(a3,d1.l),a3
			
	//lcd2MemPtr = lcd1MemPtr + 160;
			lea			640(a3),a4
			
	//lcdRedrawBuffer[lcdCurrentLine] = 0;
			lea 		lcdRedrawBuffer,a0
			clr.b 		(a0,d0.l)
			
	//valPtr = (UInt8*)lcdLineAddress[lcdCurrentLine];
			lsl.l		#2,d0
			lea 		lcdLineAddress,a0
			movea.l		(a0,d0.l),a2
			
	//switchValue = lcdPixelOffset&3;
	
			move.w		lcdPixelOffset,d2
			andi.w		#0x3,d2
			
			lea 		contrastPixel2Map,a0
			
	//switch (switchValue) 
			move.w		d2,d0
			beq.s		scase0;
			subq.w		#1,d0
			beq.s		scase1;
			subq.w		#1,d0
			beq.s		scase2;
			subq.w		#1,d0
			beq.s		scase3;
			
scase0:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			bra.s		midline

scase1:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			addq.l		#4,a1
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		292(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			bra.s		midline

scase2:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			addq.l		#8,a1
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		296(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			bra.s		midline

scase3:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			addq.l		#4,a1
			addq.l		#8,a1
			move.l		(a1)+,(a3)+
			lea	 		300(a1),a1
			move.l		(a1)+,(a4)+

// draw middle of line

midline:	moveq		#0,d0         // nibble 1
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 2
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 3
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 4
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 5
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 6
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 7
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 8
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 9
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 10
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 11
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 12
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 13
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 14
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 15
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 16
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 17
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 18
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 19
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 20
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 21
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 22
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 23
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 24
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 25
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 26
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 27
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 28
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 29
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 30
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

			moveq		#0,d0         // nibble 31
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+


	//switch (switchValue) 
			move.w		d2,d0
			beq.s		sfcase0;
			subq.w		#1,d0
			beq.s		sfcase1;
			subq.w		#1,d0
			beq.s		sfcase2;
			subq.w		#1,d0
			beq.s		sfcase3;
			
sfcase0:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		292(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			bra			ddexit2

sfcase1:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			bra.s		ddexit2

sfcase2:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			
			moveq		#0,d0         // nibble 33
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			lea	 		300(a1),a1
			move.l		(a1)+,(a4)+
			bra.s		ddexit2

sfcase3:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		288(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+
			
			moveq		#0,d0         // nibble 33
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a1

			move.l		(a1)+,(a3)+
			move.l		(a1)+,(a3)+
			lea	 		296(a1),a1
			move.l		(a1)+,(a4)+
			move.l		(a1)+,(a4)+

ddexit2:	movem.l 	(a7)+,a1-a4  // a1 safe
			rts
}






/*************************************************************************\
*
*  FUNCTION:  DrawDisplayLine16x1()
*
*  This function draws one line of the display at 16 bit color depth and
*  1x1 pixel size
*
\*************************************************************************/
/*void DrawDisplayLine16x1(void) 
{
	UInt16 	*lcd1MemPtr;
	UInt16 	*contrastPixelMemPtr;
	UInt16	*labelMemPtr;
	Int16  	offset,offset2;
	UInt16  switchValue;
	char    *valPtr;
	UInt16	white=0xffff;
	
	if (!lcdOn) return;  
	
    offset = (320*lcdCurrentLine);
    lcd1MemPtr = lcd1x1MemPtrBase + offset;
	valPtr = lcdLineAddress[lcdCurrentLine];
	switchValue = lcdPixelOffset&3;
		
	switch (switchValue) 
	{
		case 0:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 0
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				break;
		case 1:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr++;  //skip forward one pixel
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;
				break;
		case 2:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr+=2;  //skip forward two pixels
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				break;
		case 3:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 0
				contrastPixelMemPtr+=3;  //skip forward three pixels
				*lcd1MemPtr++ = *contrastPixelMemPtr++;
				break;
	}
	
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 1
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 2
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 3
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 4
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 5
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 6
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 7
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 8
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 9
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 10
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 11
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 12
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 13
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 14
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 15
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 16
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 17
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 18
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 19
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 20
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 21
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 22
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 23
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 24
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 25
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 26
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 27
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 28
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 29
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 30
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 31
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;

	switch (switchValue) 
	{
		case 0: contrastPixelMemPtr = contrastPixel1Map[*valPtr];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr = *contrastPixelMemPtr;
				break;
		case 1:	contrastPixelMemPtr = contrastPixel1Map[*valPtr];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr = *contrastPixelMemPtr;
				break;
		case 2:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr];  // nibble 33
				*lcd1MemPtr = *contrastPixelMemPtr;
				break;
		case 3:	contrastPixelMemPtr = contrastPixel1Map[*valPtr++];  // nibble 32
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr++ = *contrastPixelMemPtr++;
				contrastPixelMemPtr = contrastPixel1Map[*valPtr];  // nibble 33
				*lcd1MemPtr++ = *contrastPixelMemPtr++;*lcd1MemPtr = *contrastPixelMemPtr;
				break;
	}
	if (lcdCurrentLine>=softLabelStart)   //  are we in the menu area
	{
		if (lcdLineCount==55)  // Is menu active?
		{
			offset2 = (320*(lcdCurrentLine-softLabelStart));
			lcd1MemPtr = softKeyMemPtrBase + offset2;
			labelMemPtr = labelMemPtrBase + offset2;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			
			lcd1MemPtr+= skipToKeyB;
			labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			
			lcd1MemPtr+= skipToKeyC;
			labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			
			lcd1MemPtr+= skipToKeyD;
			labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			
			lcd1MemPtr+= skipToKeyE;
			labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			
			lcd1MemPtr+= skipToKeyF;
			labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
			*lcd1MemPtr++ = *labelMemPtr++;
		}
		else
		{
			lcd1MemPtr = softKeyMemPtrBase + (320*(lcdCurrentLine-softLabelStart));
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			lcd1MemPtr+= skipToKeyB;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			lcd1MemPtr+= skipToKeyC;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			lcd1MemPtr+= skipToKeyD;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			lcd1MemPtr+= skipToKeyE;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			lcd1MemPtr+= skipToKeyF;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			*lcd1MemPtr++ = white;
			
			
//			EraseLabelLine(lcdCurrentLine); // erase menu labels on top of softkey
		}
	}
	
	lcdRedrawBuffer[lcdCurrentLine] = 0;
}
*/




/*************************************************************************\
*
*  FUNCTION:  DrawDisplayLine16x1()
*
*  This function draws one line of the display at 16 bit color depth and
*  1x1 pixel size
*
\*************************************************************************/
asm void DrawDisplayLine16x1(void) 
{
			movem.l		a2-a4,-(a7)  // a1 safe

			tst.w		lcdOn
			beq.w		ddexit2
			
    //offset = (320*lcdCurrentLine);
    		moveq		#0,d1
			move.w		lcdCurrentLine,d1
			move.l		d1,d0
			muls.w		#640,d1
			
    //lcd1MemPtr = lcd1x1MemPtrBase + offset;
			movea.l		lcd1x1MemPtrBase,a3
			lea			(a3,d1.l),a3
						
	//lcdRedrawBuffer[lcdCurrentLine] = 0;
			lea 		lcdRedrawBuffer,a0
			clr.b 		(a0,d0.l)
			
	//valPtr = (UInt8*)lcdLineAddress[lcdCurrentLine];
			lsl.l		#2,d0
			lea 		lcdLineAddress,a0
			movea.l		(a0,d0.l),a2
			
	//switchValue = lcdPixelOffset&3;
	
			move.w		lcdPixelOffset,d2
			andi.w		#0x3,d2
			
			lea 		contrastPixel1Map,a0
			
	//switch (switchValue) 
			move.w		d2,d0
			beq.s		scase0;
			subq.w		#1,d0
			beq.s		scase1;
			subq.w		#1,d0
			beq.s		scase2;
			subq.w		#1,d0
			beq.s		scase3;
			
scase0:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+
			bra.s		midline

scase1:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			addq.l		#2,a4
			move.l		(a4)+,(a3)+
			move.w		(a4)+,(a3)+
			bra.s		midline

scase2:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			addq.l		#4,a4
			move.l		(a4)+,(a3)+
			bra.s		midline

scase3:		moveq		#0,d0
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			addq.l		#6,a4
			move.w		(a4)+,(a3)+

// draw middle of line

midline:	moveq		#0,d0         // nibble 1
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 2
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 3
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 4
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 5
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 6
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 7
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 8
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 9
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 10
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 11
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 12
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 13
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 14
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 15
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 16
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 17
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 18
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 19
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 20
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 21
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 22
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 23
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 24
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 25
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 26
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 27
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 28
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 29
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 30
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+

			moveq		#0,d0         // nibble 31
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+


	//switch (switchValue) 
			move.w		d2,d0
			beq.s		sfcase0;
			subq.w		#1,d0
			beq.s		sfcase1;
			subq.w		#1,d0
			beq.s		sfcase2;
			subq.w		#1,d0
			beq.s		sfcase3;
			
sfcase0:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.w		(a4)+,(a3)+
			bra.s		ddcont2

sfcase1:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+
			bra.s		ddcont2

sfcase2:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+
			
			moveq		#0,d0         // nibble 33
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.w		(a4)+,(a3)+
			bra.s		ddcont2

sfcase3:	moveq		#0,d0         // nibble 32
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			move.l		(a4)+,(a3)+
			
			moveq		#0,d0         // nibble 33
			move.b		(a2)+,d0
			lsl.w		#2,d0
			movea.l		(a0,d0.w),a4

			move.l		(a4)+,(a3)+
			
ddcont2:	
	// if (lcdCurrentLine>=softLabelStart)   //  are we in the menu area
			cmpi.w		#softLabelStart,lcdCurrentLine
			blt.w		ddexit2
			
	// if (lcdLineCount==55)  // Is menu active?
			cmpi.w		#55,lcdLineCount
			bne.w		eraseline
			
	// offset2 = (320*(lcdCurrentLine-softLabelStart));
			moveq 		#-softLabelStart,d1
			add.w		lcdCurrentLine,d1
			muls.w		#320,d1
			
	// lcd1MemPtr = softKeyMemPtrBase + offset2;
			add.w		d1,d1
			movea.l 	softKeyMemPtrBase,a2
			lea 		(a2,d1.w),a2
			
	// if (status49GActive) lcd1MemPtr -= 960;  //  back up three line for 49G skin
			tst.b		status49GActive
			beq.s		ddcont3
			lea			-1920(a2),a2
			
	// labelMemPtr = labelMemPtrBase + offset2;
ddcont3:	movea.l		labelMemPtrBase,a3
			lea			(a3,d1.w),a3
			
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			
			lea			skipToKeyB(a2),a2
			addq.l		#2,a3
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			
			lea			skipToKeyC(a2),a2
			addq.l		#2,a3
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			
			lea			skipToKeyD(a2),a2
			addq.l		#2,a3
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			
			lea			skipToKeyE(a2),a2
			addq.l		#2,a3
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			
			lea			skipToKeyF(a2),a2
			addq.l		#2,a3
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.l		(a3)+,(a2)+
			move.w		(a3)+,(a2)+
			bra.w		ddexit2
			
			
eraseline:	
	//lcd1MemPtr = softKeyMemPtrBase + (320*(lcdCurrentLine-softLabelStart));
			moveq		#-softLabelStart,d0
			add.w		lcdCurrentLine,d0
			muls.w		#320,d0
			add.l		d0,d0
			movea.l		softKeyMemPtrBase,a2
			lea			(a2,d0.w),a2
			
	// if (status49GActive) lcd1MemPtr -= 960;  //  back up three line for 49G skin
			tst.b		status49GActive
			beq.s		ddcont4
			lea			-1920(a2),a2
			
ddcont4:	move.l		#-1,d1

			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+
			
			lea			skipToKeyB(a2),a2
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+

			lea			skipToKeyC(a2),a2
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+

			lea			skipToKeyD(a2),a2
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+

			lea			skipToKeyE(a2),a2
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+

			lea			skipToKeyF(a2),a2
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.l		d1,(a2)+
			move.w		d1,(a2)+

			
ddexit2:	movem.l 	(a7)+,a2-a4  // a1 safe
			rts
}






/*************************************************************************\
*
*  FUNCTION:  RecalculateDisplayParameters()
*
\*************************************************************************/
void RecalculateDisplayParameters(void) 
{
	int nibblesPerLine;
	int skipOneNibble;
	int i,j;
	char* realAddress;
	int numLines;
	
#ifdef INCLUDE_TRACER
	char tmp[80];
	if (collecting)
	{
		StrPrintF(tmp,"Recalc disp - da:%lx   ma:%lx   lc:%d   po:%d   bo:%d\n",lcdDisplayAddressStart,lcdMenuAddressStart,lcdLineCount,lcdPixelOffset,lcdByteOffset);
		FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
	}
#endif

	lcdTouched = 0;
	
	// Calculate grob end addresses
	if (lcdPixelOffset&4)
	{
		nibblesPerLine = 36;
		skipOneNibble = 1;
	}
	else
	{
		nibblesPerLine = 34;
		skipOneNibble = 0;
	}
	
	if (lcdLineCount==0)
		numLines=64;
	else
		numLines=lcdLineCount+1;
		
	lcdDisplayAreaWidth = (lcdByteOffset+nibblesPerLine)&0xfffe;
	lcdDisplayAddressEnd = lcdDisplayAddressStart + (numLines*lcdDisplayAreaWidth);
	if (lcdDisplayAddressEnd < lcdDisplayAddressStart)
	{
		lcdDisplayAddressStart2 = lcdDisplayAddressEnd - lcdDisplayAreaWidth;
		lcdDisplayAddressEnd = lcdDisplayAddressStart - lcdDisplayAreaWidth;
	}
	else
	{
		lcdDisplayAddressStart2 = lcdDisplayAddressStart;
	}
	lcdMenuAddressEnd = lcdMenuAddressStart + (64-numLines)*34;
	
	// Precalculate line start addresses
	realAddress = GetRealAddressNoIO2(lcdDisplayAddressStart);
	for (i=0;i<numLines;i++)
	{
		lcdLineAddress[i] = realAddress + (lcdDisplayAreaWidth*i) + skipOneNibble;
		lcdRedrawBuffer[i] = 1;  // mark line for redraw
	}
	if (i<64)
	{
		realAddress = GetRealAddressNoIO2(lcdMenuAddressStart);
		for (j=i;j<64;j++)
		{
			lcdLineAddress[j] = realAddress + (34*(j-i));
			lcdRedrawBuffer[j] = 1;  // mark line for redraw
		}
	}
}







/*************************************************************************\
*
*  FUNCTION:  build_contrast_map()
*
\*************************************************************************/
void build_contrast_map(void)
{
	UInt16 			r,c;
	RGBColorType 	backgroundRGB;
	RGBColorType 	pixelRGB;
	RGBColorType 	newBackgroundRGB,newPixelRGB;
	Coord 			picWidth,picHeight;
	Int16			level;
	Int16			diffR,diffG,diffB;
	UInt16			*baseWinPtr,*baseWinPtrBase,*contWinPtr,*contWinPtrBase; 
	UInt16			pxColor,bgColor;
	Int16			i,j;
	UInt16			shift;
	UInt16			offset;
	
	
	if (!lcdFullyInitialized) return;  // This function might get called before
	                                   // init_lcd() -> BIG NO NO!
	                                   
	MemMove(&backgroundRGB,statusBackColor,sizeof(RGBColorType));
	MemMove(&pixelRGB,statusPixelColor,sizeof(RGBColorType));

	for (j=0;j<2;j++)
	{
		if (j==0)
		{
			BmpGetDimensions(basePixel2Bitmap,&picWidth,&picHeight,NULL);
			baseWinPtrBase = BmpGetBits(basePixel2Bitmap);
			contWinPtrBase = BmpGetBits(WinGetBitmap(nibbleMap2x2));
			shift=8;
			offset = 144;
		}
		else
		{
			BmpGetDimensions(basePixel1Bitmap,&picWidth,&picHeight,NULL);
			baseWinPtrBase = BmpGetBits(basePixel1Bitmap);
			contWinPtrBase = BmpGetBits(WinGetBitmap(nibbleMap1x1));
			shift=4;
			offset = 72;
		}
					
		for (i=0;i<19;i++)
		{
			if (optionTarget==HP48SX)
			{
				if (i==9)   // set to default
				{
					newBackgroundRGB=backgroundRGB; 
					newPixelRGB=pixelRGB;
				}
				else if (i<9)   // fade pixel to background (0-8)
				{
					newBackgroundRGB=backgroundRGB;
					level = i;  // adjust to 0-8;
					diffR = ((pixelRGB.r - backgroundRGB.r)/9);
					diffG = ((pixelRGB.g - backgroundRGB.g)/9);
					diffB = ((pixelRGB.b - backgroundRGB.b)/9);
					newPixelRGB.r = (backgroundRGB.r + (level*diffR));
					newPixelRGB.g = (backgroundRGB.g + (level*diffG));
					newPixelRGB.b = (backgroundRGB.b + (level*diffB));
				}
				else   // fade background to pixel (10-18)
				{
					newPixelRGB=pixelRGB;
					level = i-9;  // adjust to 1-9;
					if (level>9) level=9;
					diffR = ((pixelRGB.r - backgroundRGB.r)/9);
					diffG = ((pixelRGB.g - backgroundRGB.g)/9);
					diffB = ((pixelRGB.b - backgroundRGB.b)/9);
					newBackgroundRGB.r = (backgroundRGB.r + (level*diffR));
					newBackgroundRGB.g = (backgroundRGB.g + (level*diffG));
					newBackgroundRGB.b = (backgroundRGB.b + (level*diffB));
				}
			}
			else //48GX or 49G
			{
				if (i==6)   // set to default
				{
					newBackgroundRGB=backgroundRGB;
					newPixelRGB=pixelRGB;
				}
				else if (i<6)   // fade pixel to background (0-5)
				{
					newBackgroundRGB=backgroundRGB;
					level = i;  // adjust to 0-5;
					diffR = ((pixelRGB.r - backgroundRGB.r)/6);
					diffG = ((pixelRGB.g - backgroundRGB.g)/6);
					diffB = ((pixelRGB.b - backgroundRGB.b)/6);
					newPixelRGB.r = (backgroundRGB.r + (level*diffR));
					newPixelRGB.g = (backgroundRGB.g + (level*diffG));
					newPixelRGB.b = (backgroundRGB.b + (level*diffB));
				}
				else   // fade background to pixel (7-17)
				{
					newPixelRGB=pixelRGB;
					level = i-6;  // adjust to 1-11;
					if (level>11) level=11;
					diffR = ((pixelRGB.r - backgroundRGB.r)/11);
					diffG = ((pixelRGB.g - backgroundRGB.g)/11);
					diffB = ((pixelRGB.b - backgroundRGB.b)/11);
					newBackgroundRGB.r = (backgroundRGB.r + (level*diffR));
					newBackgroundRGB.g = (backgroundRGB.g + (level*diffG));
					newBackgroundRGB.b = (backgroundRGB.b + (level*diffB));
				}
			}
			
			pxColor=(((UInt16)newPixelRGB.r<<8)&0xf800)|(((UInt16)newPixelRGB.g<<3)&0x07e0)|(newPixelRGB.b>>3);
			bgColor=(((UInt16)newBackgroundRGB.r<<8)&0xf800)|(((UInt16)newBackgroundRGB.g<<3)&0x07e0)|(newBackgroundRGB.b>>3);
			
			contWinPtr = contWinPtrBase + (shift*i);
			baseWinPtr = baseWinPtrBase;
			
			r=picHeight;
			do
			{
				c=picWidth;
				do
				{
					if (*baseWinPtr++ == 0xffff)
						*contWinPtr++ = bgColor;
					else
						*contWinPtr++ = pxColor;
				} while (--c);
				contWinPtr += offset;
			} while (--r);
		}
	}
}



/*************************************************************************\
*
*  FUNCTION:  build_quick_color_contrast_map()
*
\*************************************************************************/
void build_quick_color_contrast_map(RGBColorType *pixRGB, RGBColorType *bgRGB)
{
	UInt16 			r,c;
	Coord 			picWidth,picHeight;
	UInt16			*baseWinPtr,*baseWinPtrBase,*contWinPtr,*contWinPtrBase; 
	UInt16			pxColor,bgColor;
	Int16			j;
	UInt16			shift;
	UInt16			offset;
	
	if (!lcdFullyInitialized) return;  // This function might get called before
	                                   // init_lcd() -> BIG NO NO!
	                                   
	for (j=0;j<2;j++)
	{
		if (j==0)
		{
			BmpGetDimensions(basePixel2Bitmap,&picWidth,&picHeight,NULL);
			baseWinPtrBase = BmpGetBits(basePixel2Bitmap);
			contWinPtrBase = BmpGetBits(WinGetBitmap(nibbleMap2x2));
			shift=8;
			offset = 144;
		}
		else
		{
			BmpGetDimensions(basePixel1Bitmap,&picWidth,&picHeight,NULL);
			baseWinPtrBase = BmpGetBits(basePixel1Bitmap);
			contWinPtrBase = BmpGetBits(WinGetBitmap(nibbleMap1x1));
			shift=4;
			offset = 72;
		}
			
		pxColor=(((UInt16)pixRGB->r<<8)&0xf800)|(((UInt16)pixRGB->g<<3)&0x07e0)|(pixRGB->b>>3);
		bgColor=(((UInt16)bgRGB->r<<8)&0xf800)|(((UInt16)bgRGB->g<<3)&0x07e0)|(bgRGB->b>>3);
			
		contWinPtr = contWinPtrBase;
		baseWinPtr = baseWinPtrBase;
			
		r=picHeight;
		do
		{
			c=picWidth;
			do
			{
				if (*baseWinPtr++ == 0xffff)
					*contWinPtr++ = bgColor;
				else
					*contWinPtr++ = pxColor;
			} while (--c);
			contWinPtr += offset;
		} while (--r);
	}
}



/*************************************************************************\
*
*  FUNCTION:  adjust_contrast()
*
\*************************************************************************/
void adjust_contrast(void)
{
	Int16 contrast,i;
	
	contrast = lcdContrastLevel;

	if (optionTarget==HP48SX)
		contrast-=2;
	else
		contrast-=8;
	if (contrast<0) contrast=0;
	if (contrast>18) contrast=18;
	
	for (i=0;i<16;i++)
	{
		contrastPixel2Map[i]=nibbleMap2x2Base+(contrast*4)+(152*i);
		contrastPixel1Map[i]=nibbleMap1x1Base+(contrast*4)+(76*i);
	}
}


/*************************************************************************\
*
*  FUNCTION:  rect_sweep()
*
\*************************************************************************/
void rect_sweep(RectangleType *oldRect,RectangleType *newRect, int numSteps)
{
	int i;
	int stepX,stepY,stepExtX,stepExtY;
	RectangleType saveRect,copyRect;
	RGBColorType prevRGB;
	
	saveRect=*oldRect;
	
	stepX=(newRect->topLeft.x-oldRect->topLeft.x)/numSteps;
	stepY=(newRect->topLeft.y-oldRect->topLeft.y)/numSteps;
	stepExtX=(newRect->extent.x-oldRect->extent.x)/numSteps;
	stepExtY=(newRect->extent.y-oldRect->extent.y)/numSteps;
	
	WinSetForeColorRGB(&whiteRGB,&prevRGB);
	for (i=0;i<numSteps;i++)
	{
		oldRect->topLeft.x+=stepX;
		oldRect->topLeft.y+=stepY;
		oldRect->extent.x+=stepExtX;
		oldRect->extent.y+=stepExtY;
		HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame,oldRect);
		SysTaskDelay(1);
	}
	*oldRect=saveRect;
	for (i=0;i<numSteps;i++)
	{
		oldRect->topLeft.x+=stepX;
		oldRect->topLeft.y+=stepY;
		oldRect->extent.x+=stepExtX;
		oldRect->extent.y+=stepExtY;
		
		copyRect.topLeft.x=oldRect->topLeft.x-1;
		copyRect.topLeft.y=oldRect->topLeft.y-2;
		copyRect.extent.x =oldRect->extent.x+3;
		copyRect.extent.y =3;
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);
		copyRect.topLeft.y=oldRect->topLeft.y+oldRect->extent.y-1;
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);
		copyRect.topLeft.y=oldRect->topLeft.y-1;
		copyRect.topLeft.x-=1;
		copyRect.extent.x =3;
		copyRect.extent.y =oldRect->extent.y+3;
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);
		copyRect.topLeft.x=oldRect->topLeft.x+oldRect->extent.x-1;
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);

		SysTaskDelay(1);
	}
	WinSetForeColorRGB(&prevRGB,NULL);
}






/*************************************************************************\
*
*  FUNCTION:  toggle_lcd()
*
\*************************************************************************/
void toggle_lcd(void)
{
	int i;
	RGBColorType frameColorRGB={0,115,113,99},prevRGB;
	RectangleType opRect,oldRect,newRect;
	
	// toggle back and forth between 2x2 and 1x1 LCDs
	if (statusDisplayMode2x2)
	{
		// switch to 1x1 display mode
		statusDisplayMode2x2=false;
		virtLayout = (squareDef*)virtLayout1x1;
				
		// setup LCD for delayed complete refresh 
		for (i=0;i<64;i++) lcdRedrawBuffer[i] = 0;
		delayRefresh=true;
		
		// do shrink graphic / clear out remnants of 2x2 mode via back store copy
		oldRect.topLeft.x=fullScreenOriginX-3;
		oldRect.topLeft.y=fullScreenOriginY-3;
		oldRect.extent.x=fullScreenExtentX+6;
		oldRect.extent.y=fullScreenExtentY+6;
		newRect.topLeft.x=lcd1x1AnnOriginX;
		newRect.topLeft.y=lcd1x1AnnOriginY;
		newRect.extent.x=lcd1x1FSExtentX+lcd1x1AnnExtentX+4;
		newRect.extent.y=lcd1x1FSExtentY;
		rect_sweep(&oldRect,&newRect,8);
		
		opRect.topLeft.x=fullScreenOriginX-4;
		opRect.topLeft.y=fullScreenOriginY-4;
		opRect.extent.x=fullScreenExtentX+8;
		opRect.extent.y=fullScreenExtentY+8;
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&opRect,opRect.topLeft.x,opRect.topLeft.y,winPaint);
		
		display_clear();
	}
	else
	{
		// switch to 2x2 display mode
		statusDisplayMode2x2=true;
		virtLayout = (squareDef*)virtLayout2x2;
		
		// setup LCD for delayed complete refresh 
		for (i=0;i<64;i++) lcdRedrawBuffer[i] = 0;
		delayRefresh=true;
		
		// save copy of background
		opRect.topLeft.x=fullScreenOriginX-4;
		opRect.topLeft.y=fullScreenOriginY-4;
		opRect.extent.x=fullScreenExtentX+8;
		opRect.extent.y=fullScreenExtentY+8;
		HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,backStoreWindow,&opRect,opRect.topLeft.x,opRect.topLeft.y,winPaint);

		// do expansion graphic
		newRect.topLeft.x=fullScreenOriginX-3;
		newRect.topLeft.y=fullScreenOriginY-3;
		newRect.extent.x=fullScreenExtentX+6;
		newRect.extent.y=fullScreenExtentY+6;
		oldRect.topLeft.x=lcd1x1AnnOriginX;
		oldRect.topLeft.y=lcd1x1AnnOriginY;
		oldRect.extent.x=lcd1x1FSExtentX+lcd1x1AnnExtentX+4;
		oldRect.extent.y=lcd1x1FSExtentY;
		rect_sweep(&oldRect,&newRect,8);
		
		// draw frame?
		WinSetForeColorRGB(&frameColorRGB,&prevRGB);
		opRect.topLeft.x=fullScreenOriginX-2;
		opRect.topLeft.y=fullScreenOriginY-2;
		opRect.extent.x=fullScreenExtentX+4;
		opRect.extent.y=fullScreenExtentY+4;
		HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame,&opRect);
		opRect.topLeft.x++;
		opRect.topLeft.y++;
		opRect.extent.x-=2;
		opRect.extent.y-=2;
		HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame,&opRect);
		opRect.topLeft.x++;
		opRect.topLeft.y++;
		opRect.extent.x-=2;
		opRect.extent.y-=2;
		HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame,&opRect);
		WinSetForeColorRGB(&prevRGB,NULL);
		
		display_clear();
	}
}




/*************************************************************************\
*
*  FUNCTION:  init_lcd()
*
\*************************************************************************/
void init_lcd() 
{
	drawBitmap = WinGetBitmap(WinGetDrawWindow());
	drawMemPtrBase = BmpGetBits(drawBitmap);
	basePixel2Bitmap = WinGetBitmap(hp48NibbleToScreenWindow2x2);
	basePixel1Bitmap = WinGetBitmap(hp48NibbleToScreenWindow1x1);
	annunciatorBitmap = WinGetBitmap(annunciatorBitmapWindow);
	annunciatorMemPtrBase = BmpGetBits(annunciatorBitmap);
	nibbleMap1x1Base=BmpGetBits(WinGetBitmap(nibbleMap1x1));
	nibbleMap2x2Base=BmpGetBits(WinGetBitmap(nibbleMap2x2));

	lcd2x2MemPtr1Base = drawMemPtrBase + (((lcdOriginY*320) + lcdOriginX)/2);   // Move to start of hp48 display area (2x2 mode)
	lcd1x1MemPtrBase = (UInt16*)drawMemPtrBase + (lcd1x1OriginY*320) + lcd1x1OriginX;   // Move to start of hp48 display area (1x1 mode)
	lcd2x2AnnAreaPtrBase = drawMemPtrBase + (((annunciator2x2OriginY*320) + annunciator2x2OriginX)/2); // Move to start of annunciator area.
	lcd1x1AnnAreaPtrBase = drawMemPtrBase + (((annunciator1x1OriginY*320) + annunciator1x1OriginX)/2); // Move to start of annunciator area.
	softKeyMemPtrBase = (UInt16*)drawMemPtrBase + (softKeyRow1Y*320) + softKeyAIntX;   // Move to start of softKey label area (1x1 mode)
	labelMemPtrBase = (UInt16*)drawMemPtrBase + ((lcd1x1OriginY+softLabelStart)*320) + lcd1x1OriginX;   // Move to start of lcd label area (1x1 mode)
	lcdFullyInitialized=true;  // Now we're free to call lcd functions.
}



/*************************************************************************\
*
*  FUNCTION:  exit_lcd()
*
\*************************************************************************/
void exit_lcd() {

  return;
}



/*************************************************************************\
*
*  FUNCTION:  display_lear()
*
\*************************************************************************/
void display_clear(void) 
{
	RectangleType screenRect;
	RGBColorType prevRGB;
	RGBColorType divColor={0,112,119,96};
	
	if (statusDisplayMode2x2)
	{
		WinSetForeColorRGB(statusBackColor,&prevRGB);
		screenRect.topLeft.x = fullScreenOriginX;
		screenRect.topLeft.y = fullScreenOriginY;
		screenRect.extent.x = fullScreenExtentX;
		screenRect.extent.y = fullScreenExtentY;
		HRWinDrawRectangle(hrLibRefNum, &screenRect, 0);
		
		WinSetForeColorRGB(&prevRGB,NULL);
	}
	else
	{
		WinSetForeColorRGB(statusBackColor,&prevRGB);
		screenRect.topLeft.x = lcd1x1FSOriginX;
		screenRect.topLeft.y = lcd1x1FSOriginY;
		screenRect.extent.x = lcd1x1FSExtentX;
		screenRect.extent.y = lcd1x1FSExtentY;
		HRWinDrawRectangle(hrLibRefNum, &screenRect, 0);
		screenRect.topLeft.x = lcd1x1AnnOriginX;
		screenRect.topLeft.y = lcd1x1AnnOriginY;
		screenRect.extent.x = lcd1x1AnnExtentX;
		screenRect.extent.y = lcd1x1AnnExtentY;
		HRWinDrawRectangle(hrLibRefNum, &screenRect, 0);
		screenRect.topLeft.x = lcd1x1DividerOriginX;
		screenRect.topLeft.y = lcd1x1DividerOriginY;
		screenRect.extent.x = lcd1x1DividerExtentX;
		screenRect.extent.y = lcd1x1DividerExtentY;
		WinSetForeColorRGB(&divColor,NULL);
		HRWinDrawRectangle(hrLibRefNum,&screenRect, 0);
		
		WinSetForeColorRGB(&prevRGB,NULL);
	}
}




UInt16 annunciatorPixelColor;
UInt16 annunciatorBackColor;

static void draw_annunciator(UInt16* lcdMemPtr, UInt16 *annunciatorMemPtr, UInt32 draw)
{
	UInt32 i,count;

	for (i=0;i<annHeight;i++)
	{
		count=10;
		do
		{
			if ((*annunciatorMemPtr++ == 0xFFFF) || (draw==0))
			{
				*lcdMemPtr++ = annunciatorBackColor;
			} 
			else 
			{
				*lcdMemPtr++ = annunciatorPixelColor;
			}
		} while (--count);
		lcdMemPtr+=310;   // skip to the next line: 310=320 - number of ptr copy ops 
		annunciatorMemPtr+=annunciatorBitmapWidth-10;   // skip to next line in offscreen bitmap;
	}
}


#define annSLBGoffset2x2		ann2x2ShiftLeftPositionX+262
#define annSRBGoffset2x2		ann2x2ShiftRightPositionX+262
#define annAlphaBGoffset2x2		ann2x2AlphaPositionX+262
#define annAlarmBGoffset2x2		ann2x2AlarmPositionX+262
#define annBusyBGoffset2x2		ann2x2BusyPositionX+262
#define annIOBGoffset2x2		ann2x2IOPositionX+262

#define annSLBGoffset1x1		ann1x1ShiftLeftPositionY*148+1
#define annSRBGoffset1x1		ann1x1ShiftRightPositionY*148+1
#define annAlphaBGoffset1x1		ann1x1AlphaPositionY*148+1
#define annAlarmBGoffset1x1		ann1x1AlarmPositionY*148+1
#define annBusyBGoffset1x1		ann1x1BusyPositionY*148+1
#define annIOBGoffset1x1		ann1x1IOPositionY*148+1


/*************************************************************************\
*
*  FUNCTION:  change_ann()
*
\*************************************************************************/
void change_ann(RGBColorType *annPixelRGBColor, RGBColorType *annBackRGBColor) 
{
	int c;
	UInt16 *lcdMemPtr;
	UInt16 *annunciatorMemPtr;
	


#ifdef INCLUDE_TRACER
	char tmp[60];
	if (collecting)
	{
		StrPrintF(tmp,"Change annunciator\n");
		FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
	}
#endif
	
	c = lcdAnnunciatorState;
	if ((!(c&AON)) || (!IO_TIMER_RUN)) c=0;
	
	if (annPixelRGBColor==NULL)
		annunciatorPixelColor=(((UInt16)statusPixelColor->r<<8)&0xf800)|(((UInt16)statusPixelColor->g<<3)&0x07e0)|(statusPixelColor->b>>3);
	else
		annunciatorPixelColor=(((UInt16)annPixelRGBColor->r<<8)&0xf800)|(((UInt16)annPixelRGBColor->g<<3)&0x07e0)|(annPixelRGBColor->b>>3);
		
	if (annBackRGBColor==NULL)
		annunciatorBackColor=(((UInt16)statusBackColor->r<<8)&0xf800)|(((UInt16)statusBackColor->g<<3)&0x07e0)|(statusBackColor->b>>3);
	else
		annunciatorBackColor=(((UInt16)annBackRGBColor->r<<8)&0xf800)|(((UInt16)annBackRGBColor->g<<3)&0x07e0)|(annBackRGBColor->b>>3);

	if ((c&LA1) != (lcdPreviousAnnunciatorState&LA1))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2ShiftLeftPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1ShiftLeftPositionY*320);
		}
			
		annunciatorMemPtr = annunciatorMemPtrBase + annShiftLeftOffset;

		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA1);
	}
	if ((c&LA2) != (lcdPreviousAnnunciatorState&LA2))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2ShiftRightPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1ShiftRightPositionY*320);
		}
			
		annunciatorMemPtr = annunciatorMemPtrBase + annShiftRightOffset;
		
		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA2);
	}
	if ((c&LA3) != (lcdPreviousAnnunciatorState&LA3))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2AlphaPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1AlphaPositionY*320);
		}

		annunciatorMemPtr = annunciatorMemPtrBase + annAlphaOffset;
		
		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA3);
	}
	if ((c&LA4) != (lcdPreviousAnnunciatorState&LA4))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2AlarmPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1AlarmPositionY*320);
		}
			
		annunciatorMemPtr = annunciatorMemPtrBase + annAlarmOffset;
		
		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA4);
	}
	if ((c&LA5) != (lcdPreviousAnnunciatorState&LA5))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2BusyPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1BusyPositionY*320);
		}
			
		annunciatorMemPtr = annunciatorMemPtrBase + annBusyOffset;
		
		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA5);
	}
	if ((c&LA6) != (lcdPreviousAnnunciatorState&LA6))
	{
		if (statusDisplayMode2x2)
		{
			lcdMemPtr = (UInt16*)lcd2x2AnnAreaPtrBase + ann2x2IOPositionX;
		}
		else
		{
			lcdMemPtr = (UInt16*)lcd1x1AnnAreaPtrBase + (ann1x1IOPositionY*320);
		}
			
		annunciatorMemPtr = annunciatorMemPtrBase + annIOOffset;
		
		draw_annunciator(lcdMemPtr,annunciatorMemPtr,c&LA6);
	}
	lcdPreviousAnnunciatorState = c;
}






#ifdef PRINT_IPS 

void update_ips(void)
{
	char tmp[80];
	UInt32 currentTicks;
	float instDelta;
	float timeDelta;
	UInt32 ips;
	
	currentTicks = TimGetTicks();
	instDelta = instCount - previousCount;
	timeDelta = (float)(currentTicks - lastTicks) / 100.0;
	ips = (UInt32)(instDelta / timeDelta);
	
	WinSetBackColorRGB(&backRGB,NULL);
	WinSetTextColorRGB(&textRGB,NULL);
	HRFntSetFont(hrLibRefNum, hrTinyFont);
	
	StrPrintF(tmp,"IPS=%lu   ", ips);
	HRWinDrawChars(hrLibRefNum, tmp, StrLen(tmp),1,170);
	
	lastTicks = TimGetTicks();
	previousCount = instCount;
}

#endif






