/*************************************************************************\
*
*  MEMORY.C
*
*  This module is responsible for trapping writes to memory mapped IO
*
*  (c)1995 Sebastien Carlier
*
*  Portions (c)2002 by Christoff Giesselink
*
*  Portions (c)2002 by Robert Hildinger
*
\*************************************************************************/

#include "emu48.h"
#include "IO.H"

inline void IOBit(long d, char b, Boolean s)	// set/clear bit in I/O section
{
	if (s)
		ioram[d] |= b;			// set bit
	else
		ioram[d] &= ~b;			// clear bit
}



// This function has been rendered a1 safe by the SAVEPC/RESTOREPC combination
Boolean write_io_multiple(long d, char *a, int s) 
{
	char c;
	int i;
	int tempInt;
	long tempLong;
	Boolean contrastTouched    = false;
	Boolean annunciatorTouched = false;
	Boolean dispAddressTouched = false;
	Boolean lineOffsetTouched  = false;
	Boolean lineCountTouched   = false;
	Boolean menuAddressTouched = false;
	Boolean GENIOINTERRUPT = false;	
	//char tmp[80];
	
	do
	{
		c=*a++;
		switch (d) 
		{
// 00100 =  NS:DISPIO
// 00100 @  Display bit offset and DON [DON OFF2 OFF1 OFF0]
// 00100 @  3 nibs for display offset (scrolling), DON=Display ON
			case 0x00:
				if (lcdOn!=(c>>3)) 
				{ 
					lcdOn = c>>3;
					if (!lcdOn) 
					{
						SAVEPC
						display_clear();
						RESTOREPC
					}
					lcdTouched = 1;  
				}
				if (lcdPixelOffset!=(c&7)) // OFF2, OFF1, or OFF0 changed
				{ 
					lcdTouched = 1; 
					lcdPixelOffset = c&7; 
				}
				break;
				
// 00101 =  NS:CONTRLSB
// 00101 @  Contrast Control [CON3 CON2 CON1 CON0]
// 00101 @  Higher value = darker screen

// 00102 =  NS:DISPTEST
// 00102 @  Display test [VDIG LID TRIM CON4] [LRT LRTD LRTC BIN]
// 00102 @  Normally zeros
			case 0x01:
			case 0x02: contrastTouched=true; break;
			case 0x03: break;
			
// 00104 =  HP:CRC
// 00104 @  16 bit hardware CRC (104-107) (X^16+X^12+X^5+1)
// 00104 @  crc = ( crc >> 4 ) ^ ( ( ( crc ^ nib ) & 0x000F ) * 0x1081 );
	 		case 0x04: crc=crc&0xfff0;crc|=(c); break;		// CRC
	  		case 0x05: crc=crc&0xff0f;crc|=(c<<4); break;	// CRC
	 		case 0x06: crc=crc&0xf0ff;crc|=(c<<8); break;	// CRC
	  		case 0x07: crc=crc&0x0fff;crc|=(c<<12); break;	// CRC

// 00108 =  NS:POWERSTATUS
// 00108 @  Low power registers (108-109)
// 00108 @  [LB2 LB1 LB0 VLBI] (read only)
// 00108 @  LowBat(2) LowBat(1) LowBat(S) VeryLowBat
			case 0x08: break;  // read-only
			
// 00109 =  NS:POWERCTRL
// 00109 @  [ELBI EVLBI GRST RST] (read/write)
			case 0x09: break; 

// 0010A =  NS:MODE
// 0010A @  Mode Register (read-only)
			case 0x0A: break;
			
// 0010B =  HP:ANNCTRL
// 0010B @  Annunciator control [LA4 LA3 LA2 LA1] = [ alarm alpha -> <- ]
			case 0x0B:
			case 0x0C: 
				if (c!=ioram[d])
				{
					annunciatorTouched=true; 
				}
				break;
			
// 0010D =  NS:BAUD
// 0010D @  Serial baud rate [UCK BD2 BD1 BD0] (bit 3 is read-only)
// 0010D @  3 bits = {1200 1920 2400 3840 4800 7680 9600 15360}
			case 0x0D: c = (ioram[d]&8)|(c&7);  // bit 3 is read-only
			// Call procedure to set baud rate
			break;

// 0010E =  NS:CARDCTL
// 0010E @  [ECDT RCDT SMP SWINT] (read/write)
// 0010E @  Enable Card Det., Run Card Det., Set Module Pulled, Software interrupt
	  		case 0x0E:  									// NS:CARDCTRL
	    		if (c&1) GENIOINTERRUPT = true;
	    		if (c&2) { ioram[0x19] = 2; HST|=MP; GENIOINTERRUPT = true; }
	    		if (c&8)
	      			if (c&4) ioram[0x0F] = 0;
	    		break;
	    		
// 0010F =  NS:CARDSTATUS
// 0010F @  [P2W P1W P2C P1C] (read-only) Port 2 writable .. Port 1 inserted
	  		case 0x0F: break;  // read-only

// 00110 =  HP:IOC
// 00110 @  Serial I/O Control [SON ETBE ERBF ERBZ]
// 00110 @  Serial On, Interrupt On Recv.Buf.Empty, Full, Buzy
	 		case 0x10: break;  	
	 									
// 00111 =  HP:RCS
// 00111  Serial Receive Control/Status [RX RER RBZ RBF] (bit 3 is read-only)
	  		case 0x11: c = (ioram[d]&8)|(c&7);break;
	  		
// 00112 =  HP:TCS
// 00112 @  Serial Transmit Control/Status [BRK LPB TBZ TBF]
	 		case 0x12: break;
	 		
// 00113 =  HP:CRER
// 00113 @  Serial Clear RER (writing anything clears RER bit)
	  		case 0x13: IOBit(RCS,RER,false); break;
	  		
// 00114 =  HP:RBR
// 00114 @  Serial Receive Buffer Register (Reading clears RBF bit)
	  		case 0x14:
	  		case 0x15: break;

// 00116 =  HP:TBR
// 00116 @  Serial Transmit Buffer Register (Writing sets TBF bit)
	  		case 0x16: break;
	  		case 0x17: ioram[TCS] |= 1; break;

// 00118 =  NS:SRR
// 00118 @  Service Request Register (read-only)
// 00118 @  [ISRQ TSRQ USRQ VSRQ] [KDN NINT2 NINT LSRQ]
	  		case 0x18: 
	  		case 0x19: break;

// 0011A =  HP:IRC
// 0011A @  IR Control Register [IRI EIRU EIRI IRE] (bit 3 is read-only)
// 0011A @  IR Input, Enable IR UART mode, Enable IR Interrupt, IR Event
	  		case 0x1A: c = (ioram[d]&8)|(c&7);break;

// 0011B =  NS:BASENIBOFF
// 0011B @  Used as addressto get BASENIB from 11F to the 5th nibble
	  		case 0x1B: break;

// 0011C =  NS:LCR
// 0011C @  Led Control Register [LED ELBE LBZ LBF] (Setting LED is draining)
	  		case 0x1C: 
			// HP49G new mapping on LED bit change
			if ((optionTarget==HP49G) && ((c^ioram[d])&LED))
			{
				ioram[d]=c;			// save new value for mapping
				// Adjust ROM mapping???
				//SAVEPC
				//StrPrintF(tmp,"LED change, val = %d",((c&0x8)>>3));
				//ErrorPrint(tmp,true);
				//SysTaskDelay(100);
				//RESTOREPC
			}
	  		break;

// 0011D =  NS:LBR
// 0011D @  Led Buffer Register [0 0 0 LBO] (bits 1-3 read zero)
	  		case 0x1D: c &= 1; break;

// 0011E =  NS:SCRATCHPAD
// 0011E @  Scratch pad
	  		case 0x1E: break;

// 0011F =  NS:BASENIB
// 0011F @  7 or 8 for base memory
	  		case 0x1F: break;

// 00120 =  NS:DISPADDR
// 00120 @  Display Start Address (write only)
// 00120 @  bit 0 is ignored (display must start on byte boundary)
	  		case 0x20:
	  		case 0x21:
	  		case 0x22:
	  		case 0x23:
	  		case 0x24: dispAddressTouched=true; break;

// 00125 =  NS:LINEOFFS
// 00125 @  Display Line offset (write only) (no of bytes skipped after each line)
// 00125 @  MSG sign extended
			case 0x25: 
			case 0x26:
			case 0x27: lineOffsetTouched=true; break;

// 00128 =  NS:LINECOUNT
// 00128 @  Display Line Counter and miscellaneous (28-29)
// 00128 @  [LC3 LC2 LC1 LC0] [DA19 M32 LC5 LC4]
// 00128 @  Line counter 6 bits -> max = 2^6-1 = 63 = disp height
// 00128 @  Normally has 55 -> Menu starts at display row 56
			case 0x28: 
			case 0x29: 
				lineCountTouched=true; 
				if ((c^ioram[d])&DA19)	// DA19 changed
				{
					ioram[d]^=DA19;		// save new DA19
					// Adjust ROM mapping???
				}
				break;

			case 0x2A:
			case 0x2B:
			case 0x2C:
			case 0x2D: break;
			

// 0012E =  NS:TIMER1CTRL
// 0012E @  TIMER1 Control [SRQ WKE INT XTRA]
			case 0x2E: c &= 0xE; break;

// 0012F =  NS:TIMER2CTRL
// 0012F @  TIMER2 Control [SRQ WKE INT RUN]
			case 0x2F: 
			IO_TIMER_RUN=(c&RUN); 
			annunciatorTouched=true;
			break;

// 00130 =  NS:MENUADDR
// 00130 @  Display Secondary Start Address (write only) (30-34)
// 00130 @  Menu Display Address, no line offsets
	  		case 0x30:
			case 0x31:
			case 0x32:
			case 0x33:
			case 0x34: menuAddressTouched=true; break;
			
			case 0x35:
			case 0x36: break;
			

// 00137 =  HP:TIMER1
// 00137 @  Decremented 16 times/s
			case 0x37: 
				t1 = c;
				SAVEPC
				prevTickStamp = TimGetTicks();
				RESTOREPC
				break;
					   
// 00138 =  HP:TIMER2
// 00138 @  hardware timer (38-3F), decremented 8192 times/s
			case 0x38: t2=(t2&0xFFFFFFF0)|(long)c; break;
			case 0x39: t2=(t2&0xFFFFFF0F)|((long)c<<4); break;
			case 0x3A: t2=(t2&0xFFFFF0FF)|((long)c<<8); break;
			case 0x3B: t2=(t2&0xFFFF0FFF)|((long)c<<12); break;
			case 0x3C: t2=(t2&0xFFF0FFFF)|((long)c<<16); break;
			case 0x3D: t2=(t2&0xFF0FFFFF)|((long)c<<20); break;
			case 0x3E: t2=(t2&0xF0FFFFFF)|((long)c<<24); break;
			case 0x3F: t2=(t2&0x0FFFFFFF)|((long)c<<28); break;
			default: break;
		}
		ioram[d] = c;
		d++;
	} while (--s);

	if (contrastTouched)
	{
		tempInt = ((ioram[0x02]&0x1)<<4)|ioram[0x01];
		if (tempInt != lcdContrastLevel)
		{
			lcdContrastLevel = tempInt;
			SAVEPC
			adjust_contrast();
			RESTOREPC
			
			for (i=0;i<64;i++) lcdRedrawBuffer[i] = 1;  // signal complete redraw
		}
	}
	
	if (annunciatorTouched)
	{
		lcdAnnunciatorState = (ioram[0x0c]<<4)|ioram[0x0b];
		SAVEPC
		change_ann(NULL,NULL);
		RESTOREPC
	}
	
	if (dispAddressTouched)
	{
		tempLong = ((long)ioram[0x24]<<16)|((long)ioram[0x23]<<12)|(ioram[0x22]<<8)|(ioram[0x21]<<4)|(ioram[0x20]&0xe);
		if (tempLong != lcdDisplayAddressStart)
		{
			lcdDisplayAddressStart = tempLong;
			lcdTouched = 1;
		}
	}
	
	if (lineOffsetTouched)
	{
		tempInt = (ioram[0x27]<<8)|(ioram[0x26]<<4)|ioram[0x25];
		if (tempInt&0x800) tempInt-=0x1000;
		if (tempInt != lcdByteOffset)
		{
			lcdByteOffset = tempInt;
			lcdTouched = 1;
		}
	}
	
	if (lineCountTouched)
	{
		tempInt =  ((ioram[0x29]&0x3)<<4)|ioram[0x28];
		if (tempInt != lcdLineCount)
		{
			lcdLineCount = tempInt;
			lcdTouched = 1;
		}
	}
	
	if (menuAddressTouched)
	{
		tempLong = ((long)ioram[0x34]<<16)|((long)ioram[0x33]<<12)|(ioram[0x32]<<8)|(ioram[0x31]<<4)|(ioram[0x30]&0xe);
		if (tempLong != lcdMenuAddressStart)
		{
			lcdMenuAddressStart = tempLong;
			lcdTouched = 1;
		}
	}
	
	if (lcdTouched)
	{
		SAVEPC
		RecalculateDisplayParameters();
		RESTOREPC
	}
	return GENIOINTERRUPT;
}
