/*************************************************************************\
*
*  PREFS.C
*
*  This module is responsible for implementing the preferences screens
*
*  (c)2002 by Robert Hildinger
*
\*************************************************************************/

#include <PalmOS.h>
#include "Power48Rsc.h"
#include "Power48BMPRsc.h"
#include <SonyCLIE.h>
#include "emu48.h"



Boolean tempTarget;
Boolean tempPowerSave;
Boolean tempVerbose;
Boolean tempScrollFast;
Boolean temp320x320;
UInt16  tempVolume;

Boolean	temp48GXBackdrop;
Boolean	temp48SXBackdrop;
Boolean	temp49GBackdrop;
Boolean	temp48GXColorCycle;
Boolean	temp48SXColorCycle;
Boolean	temp49GColorCycle;
Boolean	temp48GX128KExpansion;
Boolean	temp48SX128KExpansion;



const prefButton prefLayout[] =   // coordinates relative to upper left of pref dialog
{
	{   3,  35,  84,  60, 0xE },		// prefGeneral
	{  89,  35, 150,  60, 0xD },		// pref48GX
	{ 155,  35, 216,  60, 0xB },		// pref48SX
	{ 221,  35, 278,  60, 0x7 },		// pref49G
	{  19, 251,  88, 285, 0xF },		// prefCancel
	{ 193, 251, 262, 285, 0xF },		// prefOK
	{   6,  88,  22, 104, 0x1 },		// prefGenCPUDoze
	{   6, 117,  22, 133, 0x1 },		// prefGenHiResPlus
	{ 999, 999, 999, 999, 0x1 },		// prefGenTurbo - UNUSED
	{ 145,  88, 161, 104, 0x1 },		// prefGenVerbose
	{ 168, 126, 184, 142, 0x1 },		// prefGenVerboseSlow
	{ 219, 126, 235, 142, 0x1 },		// prefGenVerboseFast
	{ 205, 203, 224, 220, 0x1 },		// prefGenVolUp
	{ 205, 224, 224, 241, 0x1 },		// prefGenVolDown
	{ 147, 160, 208, 191, 0x1 },		// prefGenReset
	{   8, 160,  69, 191, 0x1 },		// prefGenColorSelect
	{ 999, 999, 999, 999, 0x2 },		// pref48GXBackdrop - UNUSED
	{ 999, 999, 999, 999, 0x2 },		// pref48GXColorCycle - UNUSED
	{  88, 138, 104, 154, 0x2 },		// pref48GX128KExpansion
	{  32, 180,  48, 196, 0x2 },		// pref48GXActive
	{ 999, 999, 999, 999, 0x4 },		// pref48SXBackdrop - UNUSED
	{ 999, 999, 999, 999, 0x4 },		// pref48SXColorCycle - UNUSED
	{  88, 138, 104, 154, 0x4 },		// pref48SX128KExpansion
	{  32, 180,  48, 196, 0x4 },		// pref48SXActive
	{ 999, 999, 999, 999, 0x8 },		// pref49GBackdrop - UNUSED
	{ 999, 999, 999, 999, 0x8 },		// pref49GColorCycle - UNUSED
	{  32, 180,  48, 196, 0x8 },		// pref49GActive
}; 



/***********************************************************************
 *
 * FUNCTION:    TranslatePenToPref(Int16 x, Int16 y)
 *
 * DESCRIPTION: This routine translates a penDown/penUp event into
 *              the press release of a preference button
 *
 * PARAMETERS:  
 *
 * RETURNED:    
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static UInt16 TranslatePenToPref(Int16 x, Int16 y)
{
	UInt16 i;
	
	for (i=0;i<maxPrefButtons;i++)
	{
		if ((x >= (((prefLayout[i].upperLeftX+prefDialogOriginX+1)>>1)-1)) &&
		    (x <= (((prefLayout[i].lowerRightX+prefDialogOriginX+1)>>1)+1)) &&
		    (y >= (((prefLayout[i].upperLeftY+prefDialogOriginY+1)>>1)-1)) &&
		    (y <= (((prefLayout[i].lowerRightY+prefDialogOriginY+1)>>1)+1)) &&
		    (statusPrefTabActive&prefLayout[i].scope))
		{
			return i;
		}
	}
	return noPref;
}


static Boolean InsertOrRemove128KCard(void)
{
	void    *mem;
	UInt32  memSize;
	UInt16 	ftrNum;
	UInt32	fileSize;
	Err		error;
	UInt32	count;
	UInt8	*buf;
	UInt8	*a,*b;
	FileRef	myRef;
	UInt32	numRead;
	
	CARDSTATUS=0;
	
	if (optionTarget==HP48SX)
	{
		ftrNum = ramMemSXFtrNum;
		memSize = RAM_SIZE_SX+16; // The extra 16 bytes is just a precautionary cushion
	}
	else if (optionTarget==HP48GX)
	{
		ftrNum = ramMemGXFtrNum;
		memSize = RAM_SIZE_GX+16; // The extra 16 bytes is just a precautionary cushion
	}

	if (status128KExpansion)
	{
		// Insert 128K card in port 1
		memSize += 0x40000;
		if (FtrPtrResize(appFileCreator, ftrNum, memSize, &mem))
			return false;
		ram=(char *)mem;
		if (optionTarget==HP48SX)
			port1=ram+0x10000;
		else if (optionTarget==HP48GX)
			port1=ram+0x40000;
			
		// load port1 image file if available
		fileSize=0x20000;  // half the size in nibbles
		
		if (optionTarget==HP48SX)
			error = VFSFileOpen(msVolRefNum,sxVFSPORT1FileName,vfsModeRead,&myRef);
		else if (optionTarget==HP48GX)
			error = VFSFileOpen(msVolRefNum,gxVFSPORT1FileName,vfsModeRead,&myRef);
				
		if (error) 
		{
			// no image file available, clear mem
			OpenAlertConfirmDialog(alert,unableToFindPort1FileString,true,false);
			MemSemaphoreReserve(true);
			MemSet(port1,0x40000,0);
			MemSemaphoreRelease(true);
		}
		else
		{
			// image file available, load it
			buf=(UInt8*)port1;
			
			MemSemaphoreReserve(true);
			VFSFileRead(myRef,fileSize,&buf[fileSize],&numRead);
			VFSFileClose(myRef);
			
			a=&buf[0];
			b=&buf[fileSize];
			count=fileSize;
			
			do
			{
				*a++=*b&0xf;
				*a++=*b++>>4;
			} while(--count);

			MemSemaphoreRelease(true);
		}
		if (optionTarget==HP48SX)
			CARDSTATUS=0x5;
		else if (optionTarget==HP48GX)
			CARDSTATUS=0xa;
	}
	else
	{
		// Save port1 image
		SaveRAM(true);
		
		// Remove 128K card from port 1
		if (FtrPtrResize(appFileCreator, ftrNum, memSize, &mem))
			return false;
		ram=(char *)mem;
		port1=NULL;
	}
	
	// adjust ram and port1 pointer references
	if (optionTarget==HP48SX)
		ce1addr=port1;
	else if (optionTarget==HP48GX)
		ce2addr=port1;
	
	if (mcRAMAddrConfigured) mcRAMDelta = (long)ram-mcRAMBase;
	if (mcCE1AddrConfigured) mcCE1Delta = (long)ce1addr-mcCE1Base;
	if (mcCE2AddrConfigured) mcCE2Delta = (long)ce2addr-mcCE2Base;
	
	RecalculateDisplayParameters();
		
	return true;
}


static void UpdatePrefTab(void)
{
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	Err				error;
	DmResID			rID;
	WinHandle		ptBackStore;
	Coord 			picWidth,picHeight;
	RectangleType	myRect;
	
	switch (statusPrefTabActive)
	{
		case 1: rID = PrefGeneralDialogBitmapFamily;break;
		case 2: rID = Pref48GXDialogBitmapFamily;break;
		case 4: rID = Pref48SXDialogBitmapFamily;break;
		case 8: rID = Pref49GDialogBitmapFamily;break;
		default:
			statusPrefTabActive=1;	rID = PrefGeneralDialogBitmapFamily;break;
	}
	picHandle = DmGetResource(bitmapRsc, rID);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	ptBackStore = HRWinCreateOffscreenWindow(hrLibRefNum, 320,picHeight,screenFormat,&error);
	if (!error) 
	{
		WinSetDrawWindow(ptBackStore);
		FastDrawBitmap(picBitmap,prefDialogOriginX,0);
	}
	else
	{
		WinSetDrawWindow(mainDisplayWindow);
		FastDrawBitmap(picBitmap,prefDialogOriginX,prefDialogOriginY);
	}
	if (((statusPrefTabActive==2) && (!status48GXROMAvailable)) ||
		((statusPrefTabActive==4) && (!status48SXROMAvailable)) ||
		((statusPrefTabActive==8) && (!status49GROMAvailable)))
	{
		myRect.topLeft.x = 11+prefDialogOriginX;
		myRect.topLeft.y = 75;
		myRect.extent.x = 60;
		myRect.extent.y = 138;
		GreyOutRectangle(myRect,ptBackStore);
	}
	if (statusPrefTabActive&0xe)  // For OS4.1 or lower, grey out OS5 features
	{
		myRect.topLeft.x = 88+prefDialogOriginX;
		myRect.topLeft.y = 74;
		myRect.extent.x = 147;
		myRect.extent.y = 47;
		GreyOutRectangle(myRect,ptBackStore);		
	}
	if (statusPrefTabActive==1)
	{
		if (!silkInUse)
		{
			myRect.topLeft.x = 4+prefDialogOriginX;
			myRect.topLeft.y = 114;
			myRect.extent.x = 108;
			myRect.extent.y = 22;
			GreyOutRectangle(myRect,ptBackStore);
		}
		if (!statusRUN)  // grey out Color and Reset buttons if not running active target
		{
			myRect.topLeft.x = 8+prefDialogOriginX;
			myRect.topLeft.y = 160;
			myRect.extent.x = 125;
			myRect.extent.y = 32;
			GreyOutRectangle(myRect,ptBackStore);		
			myRect.topLeft.x = 147+prefDialogOriginX;
			GreyOutRectangle(myRect,ptBackStore);		
		}
	}
	WinSetDrawWindow(mainDisplayWindow);
	if (!error) PixelDissolve(ptBackStore,mainDisplayWindow,320L*(UInt32)picHeight,0,prefDialogOriginY,true);
		
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	if (!error) WinDeleteWindow(ptBackStore,false);	
}



static void UpdateGreyArea(void)
{
	RectangleType	myRect;
	Err	error;
	
	if (statusPrefTabActive==1) 
	{
		if (!tempVerbose)
		{
			myRect.topLeft.x = 165+prefDialogOriginX;
			myRect.topLeft.y = 112+prefDialogOriginY;
			myRect.extent.x = 96;
			myRect.extent.y = 33;
			if (greyBackStore==NULL)
				greyBackStore = HRWinCreateOffscreenWindow(hrLibRefNum,
								myRect.extent.x,myRect.extent.y,screenFormat,&error);
			if (greyBackStore!=NULL)
				HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,greyBackStore,&myRect,0,0,winPaint);
			GreyOutRectangle(myRect,mainDisplayWindow);
		}
		else
		{
			if (greyBackStore!=NULL)
			{
				myRect.topLeft.x = 0;
				myRect.topLeft.y = 0;
				myRect.extent.x = 96;
				myRect.extent.y = 33;
				HRWinCopyRectangle(hrLibRefNum,greyBackStore,mainDisplayWindow,&myRect,165+prefDialogOriginX,112+prefDialogOriginY,winPaint);
			}
		}
	}		
}



static void UpdateHeapSpace(void)
{
	UInt16 numHeaps0;
	UInt32 heapFree,heapMax;
	Int16  length,position;
	char   tmp[20];
	int    i;
	
	if (statusPrefTabActive!=1) return;
	
	numHeaps0 = MemNumHeaps(0);
	for (i=0;i<numHeaps0;i++)
	{
		if (MemHeapDynamic(i))
		{
			MemHeapFreeBytes(i, &heapFree, &heapMax);
			HRFntSetFont(hrLibRefNum, hrTinyFont);
			StrPrintF(tmp,"%ld",heapFree);
			length = FntCharsWidth(tmp, StrLen(tmp));
			position=((79-length)/2)+1;
			if ((position<0)||(position>79))
				position=0;
			WinSetTextColorRGB(&blackRGB,NULL);
			WinSetBackColorRGB(&greyRGB,NULL);
			HRWinDrawTruncChars(hrLibRefNum, tmp, StrLen(tmp), position+15+prefDialogOriginX, prefDialogOriginY+229, 77);
		}
	}
}



static void UpdateCheckBoxes(void)
{
	RectangleType trueRect;
	RectangleType falseRect;

	trueRect.topLeft.x = 0;
	trueRect.topLeft.y = 0;
	trueRect.extent.x = 17;
	trueRect.extent.y = 17;
	falseRect.topLeft.x = 17;
	falseRect.topLeft.y = 0;
	falseRect.extent.x = 17;
	falseRect.extent.y = 17;
	
	switch (statusPrefTabActive)
	{
		case 1: 
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempPowerSave)?&trueRect:&falseRect)
								,prefLayout[prefGenCPUDoze].upperLeftX+prefDialogOriginX,prefLayout[prefGenCPUDoze].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempVerbose)?&trueRect:&falseRect)
								,prefLayout[prefGenVerbose].upperLeftX+prefDialogOriginX,prefLayout[prefGenVerbose].upperLeftY+prefDialogOriginY,winPaint);
			if (silkInUse)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((!temp320x320)?&trueRect:&falseRect)
									,prefLayout[prefGenHiResPlus].upperLeftX+prefDialogOriginX,prefLayout[prefGenHiResPlus].upperLeftY+prefDialogOriginY,winPaint);
			if (tempVerbose)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((!tempScrollFast)?&trueRect:&falseRect)
									,prefLayout[prefGenVerboseSlow].upperLeftX+prefDialogOriginX,prefLayout[prefGenVerboseSlow].upperLeftY+prefDialogOriginY,winPaint);
			if (tempVerbose)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempScrollFast)?&trueRect:&falseRect)
									,prefLayout[prefGenVerboseFast].upperLeftX+prefDialogOriginX,prefLayout[prefGenVerboseFast].upperLeftY+prefDialogOriginY,winPaint);
			break;
		case 2: 
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48GXBackdrop)?&trueRect:&falseRect)
								,prefLayout[pref48GXBackdrop].upperLeftX+prefDialogOriginX,prefLayout[pref48GXBackdrop].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48GXColorCycle)?&trueRect:&falseRect)
								,prefLayout[pref48GXColorCycle].upperLeftX+prefDialogOriginX,prefLayout[pref48GXColorCycle].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48GX128KExpansion)?&trueRect:&falseRect)
								,prefLayout[pref48GX128KExpansion].upperLeftX+prefDialogOriginX,prefLayout[pref48GX128KExpansion].upperLeftY+prefDialogOriginY,winPaint);
			if (status48GXROMAvailable)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempTarget==HP48GX)?&trueRect:&falseRect)
									,prefLayout[pref48GXActive].upperLeftX+prefDialogOriginX,prefLayout[pref48GXActive].upperLeftY+prefDialogOriginY,winPaint);
			break;
		case 4: 
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48SXBackdrop)?&trueRect:&falseRect)
								,prefLayout[pref48SXBackdrop].upperLeftX+prefDialogOriginX,prefLayout[pref48SXBackdrop].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48SXColorCycle)?&trueRect:&falseRect)
								,prefLayout[pref48SXColorCycle].upperLeftX+prefDialogOriginX,prefLayout[pref48SXColorCycle].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp48SX128KExpansion)?&trueRect:&falseRect)
								,prefLayout[pref48SX128KExpansion].upperLeftX+prefDialogOriginX,prefLayout[pref48SX128KExpansion].upperLeftY+prefDialogOriginY,winPaint);
			if (status48SXROMAvailable)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempTarget==HP48SX)?&trueRect:&falseRect)
									,prefLayout[pref48SXActive].upperLeftX+prefDialogOriginX,prefLayout[pref48SXActive].upperLeftY+prefDialogOriginY,winPaint);
			break;
		case 8: // reseved for 49G
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp49GBackdrop)?&trueRect:&falseRect)
								,prefLayout[pref49GBackdrop].upperLeftX+prefDialogOriginX,prefLayout[pref49GBackdrop].upperLeftY+prefDialogOriginY,winPaint);
			HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((temp49GColorCycle)?&trueRect:&falseRect)
								,prefLayout[pref49GColorCycle].upperLeftX+prefDialogOriginX,prefLayout[pref49GColorCycle].upperLeftY+prefDialogOriginY,winPaint);
			if (status49GROMAvailable)
				HRWinCopyRectangle(hrLibRefNum,checkBoxWindow,mainDisplayWindow,((tempTarget==HP49G)?&trueRect:&falseRect)
									,prefLayout[pref49GActive].upperLeftX+prefDialogOriginX,prefLayout[pref49GActive].upperLeftY+prefDialogOriginY,winPaint);
			break;
			break;
		default:
			break;
	}
}



static void UpdateVolumeDisplay(void)
{
	char tmp[10];
	RectangleType 	myRect;
	RGBColorType	prevRGB;
	
	if (statusPrefTabActive!=1) return;

	myRect.topLeft.x = prefDialogOriginX+232;
	myRect.topLeft.y = prefDialogOriginY+209;
	myRect.extent.x = 39;
	myRect.extent.y = 27;
	WinSetForeColorRGB(&greyRGB,&prevRGB);
	HRWinDrawRectangle(hrLibRefNum,&myRect,0);
	WinSetForeColorRGB(&prevRGB,NULL);

	StrPrintF(tmp,"%d",tempVolume);
	HRFntSetFont(hrLibRefNum, hrLargeFont);
	WinSetTextColorRGB(&blackRGB,NULL);
	WinSetBackColorRGB(&greyRGB,NULL);
	HRWinDrawChars(hrLibRefNum, tmp, StrLen(tmp), prefDialogOriginX+236, prefDialogOriginY+208);
}






static void DepressPreferenceButton(UInt16 prefCode)
{
	RectangleType 	myRect;
	RGBColorType  	prevRGB;
	Int16			penX,penY;
	Boolean			penDown=false;
	Int16			count=0;
	
	WinSetForeColorRGB(&redRGB,&prevRGB);

	switch (prefCode)
	{
		case prefGenVolUp:
		case prefGenVolDown:
			WinSetBackColorRGB(&blackRGB,NULL);
			WinSetDrawMode(winSwap);
			myRect.topLeft.x = prefLayout[prefCode].upperLeftX+prefDialogOriginX;
			myRect.topLeft.y = prefLayout[prefCode].upperLeftY+prefDialogOriginY;
			myRect.extent.x = prefLayout[prefCode].lowerRightX-prefLayout[prefCode].upperLeftX+1;
			myRect.extent.y = prefLayout[prefCode].lowerRightY-prefLayout[prefCode].upperLeftY+1;
			HRWinPaintRectangle(hrLibRefNum,&myRect,0);
			do 
			{
				EvtGetPen(&penX,&penY,&penDown);
				if (!count)
				{
					count = 200;
					if (prefCode==prefGenVolUp) 
					{
						if (tempVolume<64) tempVolume++;
					}
					if (prefCode==prefGenVolDown) 
					{
						if (tempVolume>0) tempVolume--;
					}
					UpdateVolumeDisplay();
				}
				count--;
			} while (penDown);
			WinSetBackColorRGB(&blackRGB,NULL);
			HRWinPaintRectangle(hrLibRefNum,&myRect,0);
			WinSetDrawMode(winPaint);
			WinSetBackColorRGB(&backRGB,NULL);
			break;
		case prefGenCPUDoze:
		case prefGenVerbose:
		case prefGenHiResPlus:
		case prefGenVerboseSlow:
		case prefGenVerboseFast:
		case pref48GXBackdrop:
		case pref48GXColorCycle:
		case pref48GX128KExpansion:
		case pref48GXActive:
		case pref48SXBackdrop:
		case pref48SXColorCycle:
		case pref48SX128KExpansion:
		case pref48SXActive:
		case pref49GBackdrop:
		case pref49GColorCycle:
		case pref49GActive:
			WinSetBackColorRGB(&blackRGB,NULL);
			WinSetDrawMode(winSwap);
			myRect.topLeft.x = prefLayout[prefCode].upperLeftX+prefDialogOriginX;
			myRect.topLeft.y = prefLayout[prefCode].upperLeftY+prefDialogOriginY;
			myRect.extent.x = prefLayout[prefCode].lowerRightX-prefLayout[prefCode].upperLeftX+1;
			myRect.extent.y = prefLayout[prefCode].lowerRightY-prefLayout[prefCode].upperLeftY+1;
			HRWinPaintRectangle(hrLibRefNum,&myRect,0);
			WinSetDrawMode(winPaint);
			WinSetBackColorRGB(&backRGB,NULL);
			break;
		case prefGeneral:
		case pref48GX:
		case pref48SX:
		case pref49G:
		case prefGenReset:
		case prefGenColorSelect:
		case prefCancel:
		case prefOK:
			WinSetBackColorRGB(&greyRGB,NULL);
			WinSetDrawMode(winSwap);
			myRect.topLeft.x = prefLayout[prefCode].upperLeftX+prefDialogOriginX;
			myRect.topLeft.y = prefLayout[prefCode].upperLeftY+prefDialogOriginY;
			myRect.extent.x = prefLayout[prefCode].lowerRightX-prefLayout[prefCode].upperLeftX+1;
			myRect.extent.y = prefLayout[prefCode].lowerRightY-prefLayout[prefCode].upperLeftY+1;
			HRWinPaintRectangle(hrLibRefNum,&myRect,10);
			WinSetDrawMode(winPaint);
			WinSetBackColorRGB(&backRGB,NULL);
			break;
		default:break;
	}
	
	WinSetForeColorRGB(&prevRGB,NULL);
}


static void ReleasePreferenceButton(UInt16 prefCode)
{
	RectangleType myRect;
	RGBColorType  prevRGB;
	
	WinSetForeColorRGB(&redRGB,&prevRGB);

	switch (prefCode)
	{
		case prefGenVolUp:
		case prefGenVolDown:
			//do either/or switch
			break;
		case prefGenCPUDoze:
		case prefGenVerbose:
		case prefGenHiResPlus:
		case prefGenVerboseSlow:
		case prefGenVerboseFast:
		case pref48GXBackdrop:
		case pref48GXColorCycle:
		case pref48GX128KExpansion:
		case pref48GXActive:
		case pref48SXBackdrop:
		case pref48SXColorCycle:
		case pref48SX128KExpansion:
		case pref48SXActive:
		case pref49GBackdrop:
		case pref49GColorCycle:
		case pref49GActive:
			WinSetBackColorRGB(&blackRGB,NULL);
			WinSetDrawMode(winSwap);
			myRect.topLeft.x = prefLayout[prefCode].upperLeftX+prefDialogOriginX;
			myRect.topLeft.y = prefLayout[prefCode].upperLeftY+prefDialogOriginY;
			myRect.extent.x = prefLayout[prefCode].lowerRightX-prefLayout[prefCode].upperLeftX+1;
			myRect.extent.y = prefLayout[prefCode].lowerRightY-prefLayout[prefCode].upperLeftY+1;
			HRWinPaintRectangle(hrLibRefNum,&myRect,0);
			WinSetDrawMode(winPaint);
			break;
		case prefGeneral:
		case pref48GX:
		case pref48SX:
		case pref49G:
		case prefGenReset:
		case prefGenColorSelect:
		case prefCancel:
		case prefOK:
			WinSetBackColorRGB(&greyRGB,NULL);
			WinSetDrawMode(winSwap);
			myRect.topLeft.x = prefLayout[prefCode].upperLeftX+prefDialogOriginX;
			myRect.topLeft.y = prefLayout[prefCode].upperLeftY+prefDialogOriginY;
			myRect.extent.x = prefLayout[prefCode].lowerRightX-prefLayout[prefCode].upperLeftX+1;
			myRect.extent.y = prefLayout[prefCode].lowerRightY-prefLayout[prefCode].upperLeftY+1;
			HRWinPaintRectangle(hrLibRefNum,&myRect,10);
			WinSetDrawMode(winPaint);
			break;
		default:break;
	}
	
	WinSetForeColorRGB(&prevRGB,NULL);
	WinSetBackColorRGB(&backRGB,NULL);
}



/***********************************************************************
 *
 * FUNCTION:    DisplayPreferenceScreen
 *
 * DESCRIPTION: 
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void DisplayPreferenceScreen(void)
{
	RectangleType 	copyRect;
	//int				i;
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	Coord 			picWidth,picHeight;
	Int16			penX,penY;
	Boolean			penDown=false;
	int				applyChangesOrCancel=0;
	EventType		event,newEvent;
	UInt16			prefCode;
	UInt16			prefReleasePending=noPref;
	Err				error;
	Boolean			revertScreen=false;
	Boolean			delayClear=false;
	
	tempTarget					= optionTarget;
	tempPowerSave				= optionPowerSave;
	tempVerbose					= optionVerbose;
	tempScrollFast				= optionScrollFast;
	temp320x320 				= option320x320;
	tempVolume					= optionVolume;
	temp48GXBackdrop			= option48GXBackdrop;
	temp48SXBackdrop			= option48SXBackdrop;
	temp49GBackdrop				= option49GBackdrop;
	temp48GXColorCycle			= option48GXColorCycle;
	temp48SXColorCycle			= option48SXColorCycle;
	temp49GColorCycle			= option49GColorCycle;
	temp48GX128KExpansion		= option48GX128KExpansion;
	temp48SX128KExpansion		= option48SX128KExpansion;
	
	if ((option320x320)&&(statusDisplayMode2x2))
	{
		toggle_lcd(); // put calc screen in 1x1 mode
		revertScreen=true;
	}
	
	greyBackStore=NULL;
	
	copyRect.topLeft.x = 0;
	copyRect.topLeft.y = 0;
	copyRect.extent.x = 320;
	copyRect.extent.y = ((option320x320)?320:480);
	HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,backStoreWindow,&copyRect,0,0,winPaint);

	// Get dialog check box graphic
	picHandle = DmGetResource(bitmapRsc, PrefCheckboxGraphicBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	checkBoxWindow = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (!error) 
	{
		WinSetDrawWindow(checkBoxWindow);
		HRWinDrawBitmap(hrLibRefNum,picBitmap,0,0);
		WinSetDrawWindow(mainDisplayWindow);
	}
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	if (error)
	{
		OpenAlertConfirmDialog(alert,errorDisplayingPrefScreenString,true,false);
		goto dpsexit;
	}
	
	UpdatePrefTab();
	UpdateVolumeDisplay();
	UpdateCheckBoxes();
	UpdateHeapSpace();
	UpdateGreyArea();
	
	EvtFlushPenQueue();
	do
	{
		EvtGetEvent(&event,evtWaitForever);
		if (event.eType!=nilEvent)
		{
			//process event
			if (event.eType==penDownEvent)
			{
				PenGetPoint(&penX, &penY, &penDown);
				prefCode = TranslatePenToPref(penX,penY);
				if (prefCode != noPref)
				{
					DepressPreferenceButton(prefCode);
					prefReleasePending = prefCode;
					continue;
				}
				else
				{
					prefReleasePending = noPref;
				}
			}
			if (event.eType==penUpEvent)
			{
				if (prefReleasePending != noPref)
				{
					PenGetPoint(&penX, &penY, &penDown);
					prefCode = TranslatePenToPref(penX,penY);
					ReleasePreferenceButton(prefReleasePending);
					
					if (prefReleasePending == prefCode)
					{
						switch(prefReleasePending)
						{
							case prefGeneral:
								statusPrefTabActive=1;
								UpdatePrefTab();
								UpdateVolumeDisplay();
								UpdateCheckBoxes();
								UpdateHeapSpace();
								UpdateGreyArea();
								break;
							case pref48GX:
								statusPrefTabActive=2;
								UpdatePrefTab();
								UpdateCheckBoxes();
								break;
							case pref48SX:
								statusPrefTabActive=4;
								UpdatePrefTab();
								UpdateCheckBoxes();
								break;
							case pref49G:
								statusPrefTabActive=8;
								UpdatePrefTab();
								UpdateCheckBoxes();
								break;
							case prefGenVolUp:
								break;
							case prefGenVolDown:
								break;
							case prefGenCPUDoze:
								if (tempPowerSave)
									tempPowerSave=false;
								else
									tempPowerSave=true;
								UpdateCheckBoxes();
								break;
							case prefGenVerbose:
								if (tempVerbose)
									tempVerbose=false;
								else
									tempVerbose=true;
								UpdateGreyArea();
								UpdateCheckBoxes();
								break;
							case prefGenHiResPlus:
								if (silkInUse)
								{
									if (temp320x320)
										temp320x320=false;
									else
										temp320x320=true;
									UpdateCheckBoxes();
								}
								break;
							case prefGenVerboseSlow:
								if (tempVerbose)
								{
									if (tempScrollFast)
										tempScrollFast=false;
									UpdateCheckBoxes();
								}
								break;								
							case prefGenVerboseFast:
								if (tempVerbose)
								{
									if (!tempScrollFast)
										tempScrollFast=true;
									UpdateCheckBoxes();
								}
								break;								
							case prefGenReset:
								if (statusRUN)
								{
									if (OpenAlertConfirmDialog(confirm,checkResetString,true,false))
									{
										PinResetEmulator();
										applyChangesOrCancel=1;  // Discard any changes
									}
								}
								break;
							case prefGenColorSelect:
								if (statusRUN)
								{
									// bring up the color selector after dialog closes
									newEvent.eType = invokeColorSelectorEvent;
									applyChangesOrCancel=1;  // Discard any changes
									EvtAddEventToQueue(&newEvent);
								}
								break;
																
							case pref48GXBackdrop:
								break;
							case pref48GXColorCycle:
								break;
							case pref48GX128KExpansion:
								if (temp48GX128KExpansion)
									temp48GX128KExpansion=false;
								else
									temp48GX128KExpansion=true;
								UpdateCheckBoxes();
								break;
							case pref48GXActive:
								if (status48GXROMAvailable)
								{
									tempTarget=HP48GX;
									UpdateCheckBoxes();
								}
								break;
								
							case pref48SXBackdrop:
								break;
							case pref48SXColorCycle:
								break;
							case pref48SX128KExpansion:
								if (temp48SX128KExpansion)
									temp48SX128KExpansion=false;
								else
									temp48SX128KExpansion=true;
								UpdateCheckBoxes();
								break;
							case pref48SXActive:
								if (status48SXROMAvailable)
								{
									tempTarget=HP48SX;
									UpdateCheckBoxes();
								}
								break;
								
							case pref49GBackdrop:
								break;
							case pref49GColorCycle:
								break;
							case pref49GActive:
								if (status49GROMAvailable)
								{
									tempTarget=HP49G;
									UpdateCheckBoxes();
								}
								break;
								
							case prefCancel:
								applyChangesOrCancel=1;
								break;
							case prefOK:
								applyChangesOrCancel=2;
								break;
							default: break;
						}
						continue;
					}
				}
			}
			
			if (event.eType==appStopEvent)  // respond to app-switching events.
			{
				DepressPreferenceButton(prefCancel);
				SysTaskDelay(10);
				ReleasePreferenceButton(prefCancel);
				statusQuit=true;
				applyChangesOrCancel=1;
				EvtAddEventToQueue(&event);  // put app stop back on queue
				continue;
			}

			if (event.eType==keyDownEvent)
			{
				if (event.data.keyDown.chr == chrLineFeed)
				{
					DepressPreferenceButton(prefOK);
					SysTaskDelay(10);
					ReleasePreferenceButton(prefOK);
					applyChangesOrCancel=2;
					continue;
				}
			}
			
			if (event.eType==volumeUnmountedEvent)
			{
				if (event.data.generic.datum[0]==msVolRefNum)
				{
					statusInUnmountDialog = true;
					if (OpenAlertConfirmDialog(alert,pleaseReinsertCardString,true,false))
						statusVolumeMounted=false;
					else
						statusVolumeMounted=true;
					statusInUnmountDialog = false;
				}
				continue;
			}
			
			if (event.eType==volumeMountedEvent)
			{
				CheckForRemount();
				continue;
			}
			
			if (! SysHandleEvent(&event))  // respond to all system events, like power-down, etc...
			{
				FrmDispatchEvent(&event);
			}
		}
	} while (!applyChangesOrCancel);
	
	
	if (applyChangesOrCancel==2)
	{
		//apply all the changes
		optionVerbose             = tempVerbose;
		optionScrollFast		  = tempScrollFast;
		optionPowerSave           = tempPowerSave;
		optionVolume              = tempVolume;
		
		if (option320x320 != temp320x320)
		{
			option320x320 = temp320x320;
			if (option320x320)
			{
				SilkLibResizeDispWin(silkLibRefNum, silkResizeNormal);  // switch to 320x320
				SilkLibDisableResize(silkLibRefNum);
			}
			else
			{
				SilkLibEnableResize(silkLibRefNum);
				SilkLibResizeDispWin(silkLibRefNum, silkResizeMax);  // switch to 320x480
			}
			statusPendingGUISwitch = true;
		}
		
		if (tempTarget==HP48SX)
		{
			if (option48SX128KExpansion != temp48SX128KExpansion)
			{
				if (lcdOn) 
				{
					OpenAlertConfirmDialog(alert,turnOffCalcBeforeCardString,true,false);
					temp48SX128KExpansion = option48SX128KExpansion;
				}
				else
				{
					status128KExpansion = temp48SX128KExpansion;
					InsertOrRemove128KCard();
				}
			}
		}
		else if (tempTarget==HP48GX)
		{
			if (option48GX128KExpansion != temp48GX128KExpansion)
			{
				if (lcdOn)
				{
					OpenAlertConfirmDialog(alert,turnOffCalcBeforeCardString,true,false);
					temp48GX128KExpansion = option48GX128KExpansion;
				}
				else
				{
					status128KExpansion = temp48GX128KExpansion;
					InsertOrRemove128KCard();
				}
			}
		}
		else
		{
			// handle 49G options
		}
		
		option48SXBackdrop = temp48SXBackdrop;
		option48SXColorCycle = temp48SXColorCycle;
		option48SX128KExpansion = temp48SX128KExpansion;
		option48GXBackdrop = temp48GXBackdrop;
		option48GXColorCycle = temp48GXColorCycle;
		option48GX128KExpansion = temp48GX128KExpansion;
		option49GBackdrop = temp49GBackdrop;
		option49GColorCycle = temp49GColorCycle;
		

		if (optionTarget != tempTarget)
		{
			statusSwitchEmulation = 1;
			statusQuit = true;           // stop the emulator so it can be restarted.
			statusNextTarget=tempTarget;
		}
	}

	
	WinDeleteWindow(checkBoxWindow,false);

dpsexit:

	if (greyBackStore!=NULL) WinDeleteWindow(greyBackStore,false);
	
	EvtFlushPenQueue();
	
	PixelDissolve(backStoreWindow,mainDisplayWindow,320L*prefDialogExtentY,prefDialogOriginY,prefDialogOriginY,false);

	if (delayClear) display_clear();
	
	if (revertScreen)
		toggle_lcd();
}






