/*************************************************************************\
*
*  Power48.c
*
*  This is the module that provides all palm-related startup and shutdown
*  features, as well as global option and preference processing. Some
*  GUI elements are defined here as well.
*
*  (c)2002 Robert Hildinger
*
\*************************************************************************/

#include <PalmOS.h>
#include "Power48Rsc.h"
#include "Power48BMPRsc.h"
#include <SonyCLIE.h>
#include "emu48.h"




/*********************************************************************** 
 *
 *	Global variables
 *
 ***********************************************************************/

UInt16 hrLibRefNum;
UInt16 silkLibRefNum;
Boolean silkInUse;

static Coord displayExtentX=0, displayExtentY=0;
static Coord windowExtentX=0, windowExtentY=0;

WinHandle mainDisplayWindow;
WinHandle hp48NibbleToScreenWindow1x1;
WinHandle hp48NibbleToScreenWindow2x2;
WinHandle annunciatorBitmapWindow;	
WinHandle backStoreWindow;	
WinHandle miniBitmapWindow;	
WinHandle miniSaveWindow;	
WinHandle checkBoxWindow;	
WinHandle greyBackStore;

WinHandle nibbleMap1x1;
WinHandle nibbleMap2x2;

RGBColorType dialogBackRGB;

RGBColorType backRGB = {0,255,255,255};  // default white
RGBColorType textRGB = {0,0,0,0};        // default black

static UInt16 screenY=lcdOriginY-13;
static UInt16 skipOnce=1;
static UInt16 errorScreenY=errorPrintOriginY+1;
static UInt16 errorSkipOnce=1;
UInt16 msVolRefNum=0;

keyDef* keyLayout;
squareDef* virtLayout;
squareDef keyToSquare[maxKeys];


const RGBColorType redRGB={0,255,0,0};
const RGBColorType greenRGB={0,0,255,0};
const RGBColorType blueRGB={0,0,0,255};
const RGBColorType whiteRGB={0,255,255,255};
const RGBColorType blackRGB={0,0,0,0};
const RGBColorType greyRGB={0,199,199,199};

const point circPoint[109] =
{
	{ 350,525 },
	{ 330,495 },
	{ 310,465 },
	{ 290,435 },
	{ 270,405 },
	{ 250,375 },
	{ 230,345 },
	{ 210,315 },
	{ 190,285 },
	{ 170,255 },	
	{ 150,225 },
	{ 130,195 },
	{ 110,165 },
	{  90,135 },
	{  70,105 },	
	{  58, 75 },
	{  48, 45 },
	{  40, 15 },
	{  34, 0 },

	{ 28, -27 },  // circle motion starts here
	{ 26, -29 },
	{ 24, -31 },
	{ 22, -33 },
	{ 20, -34 },
	{ 17, -35 },
	{ 14, -37 },
	{ 12, -38 },
	{ 9, -38 },
	{ 6, -39 },
	{ 4, -39 },
	{ 1, -39 },
	{ -1, -39 },
	{ -4, -39 },
	{ -6, -39 },
	{ -9, -38 },
	{ -12, -38 },
	{ -14, -37 },
	{ -17, -35 },
	{ -19, -34 },
	{ -22, -33 },
	{ -24, -31 },
	{ -26, -29 },
	{ -28, -27 },
	{ -30, -25 },
	{ -32, -23 },
	{ -33, -21 },
	{ -35, -18 },
	{ -36, -16 },
	{ -37, -13 },
	{ -38, -11 },
	{ -39, -8 },
	{ -39, -5 },
	{ -39, -2 },
	{ -40, 0 },
	{ -39, 2 },
	{ -39, 5 },
	{ -39, 8 },
	{ -38, 11 },
	{ -37, 13 },
	{ -36, 16 },
	{ -35, 18 },
	{ -33, 21 },
	{ -32, 23 },
	{ -30, 25 },
	{ -28, 27 },
	{ -26, 29 },
	{ -24, 31 },
	{ -22, 33 },
	{ -20, 34 },
	{ -17, 35 },
	{ -14, 37 },
	{ -12, 38 },
	{ -9, 38 },
	{ -6, 39 },
	{ -4, 39 },
	{ -1, 39 },
	{ 1, 39 },
	{ 4, 39 },
	{ 6, 39 },
	{ 9, 38 },
	{ 12, 38 },
	{ 14, 37 },
	{ 17, 35 },
	{ 19, 34 },
	{ 22, 33 },
	{ 24, 31 },
	{ 26, 29 },
	{ 28, 27 },
	{ 30, 25 },
	{ 32, 23 },
	{ 33, 21 },
	{ 35, 18 },
	{ 36, 16 },
	{ 37, 13 },
	{ 38, 11 },
	{ 39, 8 },
	{ 39, 5 },
	{ 39, 2 },
	{ 40, 0 },
	{ 39, -2 },  // circle motion starts here
	{ 39, -5 },
	{ 39, -8 },
	{ 38, -11 },
	{ 37, -13 },
	{ 36, -16 },
	{ 35, -18 },
	{ 33, -21 },
	{ 32, -23 },
	{ 30, -25 }
};



const char *aboutText = 
	"\0\1"
	"(c) 2002/2003 by Robert Hildinger\0\2"
	"\0\1"
	"\0\1"
	"Version: " appVersionNum "\0\2"
	"\0\1"
	"Build Date: " __DATE__ "\0\2"
	"\0\1"
	"\0\1"
	"\0\1"
	"HP48/49 ROM images are the property of\0\1"
	"and copyrighted by Hewlett-Packard.\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"Power48 is based primarily on Emu48 written\0\1"
	"by Sebastien Carlier, though it also borrows\0\1"
	"a few tricks from the later incarnations of\0\1"
	"Emu48 as maintained and updated by Christoph\0\1"
	"Gieelink. The code has been almost completely\0\1"
	"rewritten and optimized for the Palm platform\0\1"
	"in an attempt to squeeze in every ounce of\0\1"
	"performance possible.\0\1"
	"\0\1"
	"\0\1"
	"48GX graphic representation based on work of\0\1"
	"Jeffery L. McMahan\0\1"
	"\0\1"
	"48SX graphic representation based on work of\0\1"
	"Jeff Breadner\0\1"
	"\0\1"
	"49G graphic representation based on work of\0\1"
	"Eric Rechlin\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"For all Power48 support issues...\0\2"
	"email:\0\1"
	"support@power48.mobilevoodoo.com\0\1"
	"website:\0\1"
	"http://power48.mobilevoodoo.com/\0\3"
	"\0\1"
	"\0\1"
	"\0\1"
	"Notes:\0\1"
	"\0\1"
	"I would like to thank everyone who has\0\1"
	"donated to the continued development of\0\1"
	"Power48. Your generosity is very much ap-\0\1"
	"preciated!\0\1"
	"\0\1"
	"Power48 is a free program distributed under\0\1"
	"the GPL, but unfortunately testing hardware\0\1"
	"and development environments are not. Working\0\1"
	"on this program also requires large amounts\0\1"
	"of my free time. If you're feeling generous\0\1"
	"and find this program useful and would like\0\1"
	"to support it's continued development, perhaps\0\1"
	"you'd consider making a donation to help offset\0\1"
	"the costs of developing it? If you feel so\0\1"
	"inclined, you can send your donation through\0\1"
	"PayPal to the following email address:\0\1"
	"support@power48.mobilevoodoo.com.\0\1"
	"\0\1"
	"Thank you very much,\0\4"
	"-Robert Hildinger\0\4"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"GPL statement\0\1"
	"\0\1"
	"This program is free software; you can redis-\0\1"
	"tribute it and/or modify it under the terms of\0\1"
	"the GNU General Public License as published by\0\1"
	"the Free Software Foundation; either version 2\0\1"
	"of the license, or (at your option) any later\0\1"
	"version.\0\1"
	"\0\1"
	"This program is distributed in the hope that it\0\1"
	"will be useful, but WITHOUT ANY WARRANTY;\0\1"
	"without even the implied warranty of \0\1"
	"MERCHANTABILITY or FITNESS FOR A PAR-\0\1"
	"TICULAR PURPOSE. See the GNU General Public\0\1"
	"License for more details.\0\1"
	"\0\1"
	"You should have received a copy of the GNU\0\1"
	"General Public License along with this program;\0\1"
	"if not, write to:\0\1"
	"The Free Software Foundation, Inc.\0\1"
	"59 Temple Place, Suite 330\0\1"
	"Boston, MA  02111-1307\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"\0\1"
	"QQ";



/***********************************************************************
 *
 *	Global Option variables
 *
 ***********************************************************************/


calcTarget 	optionTarget				= HP48GX;
Boolean 	optionPowerSave             = false;
Boolean 	optionVerbose				= true;
Boolean 	optionScrollFast			= false;
Boolean 	option320x320				= false;
UInt16  	optionVolume				= 32;     	// Max = 64
Boolean	 	option48GXBackdrop			= false;
Boolean	 	option48SXBackdrop			= false;
Boolean	 	option49GBackdrop			= false;
Boolean	 	option48GXColorCycle		= false;
Boolean	 	option48SXColorCycle		= false;
Boolean	 	option49GColorCycle			= false;
Boolean	 	option48GX128KExpansion		= false;
Boolean	 	option48SX128KExpansion		= false;
RGBColorType	option48GXPixelColor	= {0,0,0,80};
RGBColorType	option48GXBackColor		= {0,120,132,96};
RGBColorType	option48SXPixelColor	= {0,0,0,0};
RGBColorType	option48SXBackColor		= {0,144,152,120};
RGBColorType	option49GPixelColor		= {0,0,0,0};
RGBColorType	option49GBackColor		= {0,144,152,120};

Boolean 	statusQuit					= false;
Boolean 	statusCompleteReset			= false;
Boolean 	statusSwitchEmulation 		= false;
Boolean 	status128KExpansion         = false;
Boolean 	statusDisplayMode2x2		= false;
Boolean		statusBackdrop				= false;
Boolean 	statusColorCycle			= false;
Boolean		status48SXROMAvailable		= false;
Boolean		status48GXROMAvailable      = false;
Boolean		status49GROMAvailable		= false;
Boolean		statusVolumeMounted			= false;
Boolean		statusInUnmountDialog		= false;
Boolean		statusRUN					= false;
Boolean		statusPendingGUISwitch		= false;
Boolean		statusSkipScreenDraw		= false;
UInt8		statusPrefTabActive		    = 1;    	// 1 =general,2=48GX,4=48SX,8=49G
UInt8		statusObjectLSActive		= 1;		// 1 = load, 2 = save
Boolean		status49GActive				= false;
calcTarget 	statusNextTarget			= HP49G;
Boolean		statusPerformReset			= false;
RGBColorType	*statusPixelColor		= &option48GXPixelColor;
RGBColorType	*statusBackColor		= &option48GXBackColor;


/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/



/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}



/***********************************************************************
 *
 * FUNCTION:    FastDrawBitmap
 *
 * DESCRIPTION: This routine uncompresses an 8-bit, Packbits-compressed,
 *              version 3 bitmap
 *              with color table and displays it in the desired window. 
 *              It does not do any bounds checking.
 *              It does not support transparency. 
 *              The bitmap must fit completely
 *              inside the enclosing window.
 *
 *
 ***********************************************************************/
void FastDrawBitmap(BitmapPtr sourceBitmap, Coord x, Coord y)
{
	UInt8 *data,*endData;
	UInt16 *dest;
	UInt32 dataCount;
	UInt16 runCount;
	BitmapPtr destBitmap;
	Coord destWidth,destHeight;
	Coord srcWidth,srcHeight;
	UInt16 rowCount=0;
	UInt32 rowSkip;
	UInt16 c1;
	UInt16 colorTable[256];
	UInt16 numCTEntries;
	UInt16 i;
	UInt8 index,cr,cg,cb;
	UInt16 *ctPtr;
		
	destBitmap = WinGetBitmap(WinGetDrawWindow());
	dest = BmpGetBits(destBitmap);
	BmpGetDimensions(destBitmap,&destWidth,&destHeight,NULL);
	dest += ((Int32)destWidth*(Int32)y)+(Int32)x;
	
	BmpGetDimensions(sourceBitmap,&srcWidth,&srcHeight,NULL);
	rowSkip = destWidth-srcWidth;
	
	data = (UInt8*)BmpGetColortable(sourceBitmap);
	// data now points to color table
	numCTEntries=(data[0]<<8)|data[1];
	data+=2;

	for (i=0;i<numCTEntries;i++)
	{
		index=*data++;
		cr=(*data++)>>3;
		cg=(*data++)>>2;
		cb=(*data++)>>3;
		colorTable[index]=((UInt16)cr<<11)|((UInt16)cg<<5)|cb;
	}
	ctPtr=colorTable;
	
	data=BmpGetBits(sourceBitmap);
	
	// data now points to compressed data size indicator (limit size to < 0xFFFF)
	dataCount=(data[0]<<8)|data[1];
	dataCount--;
	
	endData=data+dataCount;
	data+=2;

	do
	{
		if (*data&0x80)
		{ 	// repeat data
			runCount=-(*(Int8*)data++)+1;
			c1=ctPtr[*data++];
			
			rowCount+=runCount;
			do { *dest++ = c1; } while (--runCount);
			if (rowCount>=srcWidth)
			{
				rowCount=0;
				dest += rowSkip;
			}
		}
		else
		{	// run data
			runCount=(*data++)+1;

			rowCount+=runCount;
			do { *dest++ = ctPtr[*data++]; } while (--runCount);
			if (rowCount>=srcWidth)
			{
				rowCount=0;
				dest += rowSkip;
			}
		}
	} while (data<=endData);
}




/***********************************************************************
 *
 * FUNCTION:    GreyOutRectangle
 *
 * DESCRIPTION: This routine "Greys Out" the portion of the main display
 *              window specified by OpRect
 *
 *
 ***********************************************************************/
void GreyOutRectangle(RectangleType opRect, WinHandle win)
{
	UInt16 *winBase;
	UInt16 valRed,valGreen,valBlue;
	UInt16 deltaRed,deltaGreen,deltaBlue;
	UInt16 baseRed,baseGreen,baseBlue;
	BitmapPtr tempBitmap;
	int i,j;

	tempBitmap = WinGetBitmap(win);
	winBase = BmpGetBits(tempBitmap);

	baseRed = 24;  // 16 bit representation of gray dialog background
	baseGreen=48;
	baseBlue= 24;

	winBase += ((Int32)opRect.topLeft.y*320L)+opRect.topLeft.x;
	
	for (i=0;i<opRect.extent.y;i++)
	{
		for (j=0;j<opRect.extent.x;j++)
		{
			valRed = *winBase;
			valBlue=valRed&0x1f;
			valRed>>=5;
			valGreen=valRed&0x3f;
			valRed>>=6;
				
			deltaRed   = ((baseRed-valRed)/2);
			deltaBlue  = ((baseBlue-valBlue)/2);
			deltaGreen = ((baseGreen-valGreen)/2);
				 
			valRed=(valRed + deltaRed)&0x1f;
			valBlue=(valBlue + deltaBlue)&0x1f;
			valGreen=(valGreen + deltaGreen)&0x3f;
				
			valRed<<=6;
			valRed|=valGreen;
			valRed<<=5;
			valRed|=valBlue;
			*winBase++ = valRed;
		}
	winBase += 320-opRect.extent.x;
	}
}




/***********************************************************************
 *
 * FUNCTION:    DrawProgressBar(UInt8 percent)
 *
 * DESCRIPTION: This function is responsible for drawing and updating
 *              a 0 to 100% progress bar.
 * 
 *              Bar type: STARTUP: 0 to 100
 *                        SWITCH:  0 to 200
 *                        SHUTDOWN:0 to 100
 *
 ***********************************************************************/
UInt8 previousProgress=0;
UInt8 progressMax=100;
UInt8 divisor = 1;

static void DrawProgressBar(UInt8 progress, ProgBarType progBar)
{
	RectangleType opRect;
	RGBColorType prevRGB;
	UInt8 currentProgress;
	
	if (optionVerbose) return;
		
	WinSetForeColorRGB(statusPixelColor,NULL);
	WinSetTextColorRGB(statusPixelColor,&prevRGB);
	WinSetBackColorRGB(statusBackColor, NULL);
		
	if (progress==0)
	{
		if (progBar == SWITCH)
		{
			divisor = 2;
			progressMax=200;
		}
		else
		{
			divisor = 1;
			progressMax=100;
		}
		
		display_clear();
		
		// draw initial bar
		if (statusDisplayMode2x2)
		{
			// double size display bar
			HRFntSetFont(hrLibRefNum, hrBoldFont);
			if (progBar==STARTUP)
				HRWinDrawChars(hrLibRefNum, "Starting Up...", 14, fullScreenOriginX+30, fullScreenOriginY+44);
			else if (progBar==SWITCH)
				HRWinDrawChars(hrLibRefNum, "Switching Target...", 19, fullScreenOriginX+30, fullScreenOriginY+44);
			else if (progBar==SHUTDOWN)
				HRWinDrawChars(hrLibRefNum, "Shutting Down...", 16, fullScreenOriginX+30, fullScreenOriginY+44);
			opRect.topLeft.x=fullScreenOriginX+30;
			opRect.topLeft.y=fullScreenOriginY+70;
			opRect.extent.x =204;
			opRect.extent.y =16;
			HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame, &opRect);
		}
		else
		{
			// regular size display bar
			HRFntSetFont(hrLibRefNum, hrTinyFont);
			WinSetDrawMode(winOverlay);
			if (progBar==STARTUP)
				HRWinDrawChars(hrLibRefNum, "Starting Up...", 14, lcd1x1FSOriginX+15, lcd1x1FSOriginY+16);
			else if (progBar==SWITCH)
				HRWinDrawChars(hrLibRefNum, "Switching Target...", 19, lcd1x1FSOriginX+15, lcd1x1FSOriginY+16);
			else if (progBar==SHUTDOWN)
				HRWinDrawChars(hrLibRefNum, "Shutting Down...", 16, lcd1x1FSOriginX+15, lcd1x1FSOriginY+16);
			opRect.topLeft.x=lcd1x1FSOriginX+15;
			opRect.topLeft.y=lcd1x1FSOriginY+30;
			opRect.extent.x =102;
			opRect.extent.y =10;
			HRWinDrawRectangleFrame(hrLibRefNum, simpleFrame, &opRect);
		}
		previousProgress=0;
	}
	else
	{
		currentProgress = previousProgress+progress;
		
		if (currentProgress>progressMax) currentProgress=progressMax;
		
		// increase bar length by progress
		if (statusDisplayMode2x2)
		{
			opRect.topLeft.x=fullScreenOriginX+32+((previousProgress*2)/divisor);
			opRect.topLeft.y=fullScreenOriginY+72;
			opRect.extent.x =(progress*2)/divisor;
			opRect.extent.y =12;
			HRWinDrawRectangle(hrLibRefNum, &opRect, 0);
		}
		else
		{
			opRect.topLeft.x=lcd1x1FSOriginX+16+(previousProgress/divisor)-(previousProgress!=0);
			opRect.extent.x =(progress/divisor)+(previousProgress!=0);
			opRect.topLeft.y=lcd1x1FSOriginY+31;
			opRect.extent.y =8;
			HRWinDrawRectangle(hrLibRefNum, &opRect, 0);
		}
		previousProgress=currentProgress;
	}
	WinSetForeColorRGB(&prevRGB,NULL);
}



/***********************************************************************
 *
 * FUNCTION:    PixelDissolve
 *
 * DESCRIPTION: This function performs a pixel dissolve from one window 
 *              to another.
 *
 ***********************************************************************/
void PixelDissolve(WinHandle fromWin, WinHandle toWin, UInt32 limit, UInt32 fromOffset, UInt32 toOffset, Boolean transparent)
{
	UInt32			val,mask;
	UInt16			*fromBasePtr,*toBasePtr,*fromPtr,*toPtr;
	BitmapPtr		fromBM,toBM;
	UInt16			tc=0xffff;
	
	val = 1;
	fromBM = WinGetBitmap(fromWin);
	fromBasePtr = BmpGetBits(fromBM);
	toBM = WinGetBitmap(toWin);
	toBasePtr = BmpGetBits(toBM);
	fromBasePtr += (fromOffset*320L);
	toBasePtr += (toOffset*320L);
		
	mask=0x00006000;
	if (limit>16383)   // 2^14-1
		mask=0x00006000;
	if (limit>32767)   // 2^15-1
		mask=0x0000B400;
	if (limit>65535)   // 2^16-1
		mask=0x00012000;
	if (limit>131071)  // 2^17-1
		mask=0x00020400;
	

	if (transparent)
	{
		do 
		{
			if (val<=limit)
			{
				fromPtr = fromBasePtr + val - 1;
				toPtr = toBasePtr + val - 1;
				if (*fromPtr!=tc)
					*toPtr = *fromPtr;
			}
			if (val&1)
				val = (val>>1)^mask;
			else
				val>>=1;
		}
		while (val != 1);
	}
	else
	{
		do 
		{
			if (val<=limit)
			{
				fromPtr = fromBasePtr + val - 1;
				toPtr = toBasePtr + val - 1;
				*toPtr = *fromPtr;
			}
			if (val&1)
				val = (val>>1)^mask;
			else
				val>>=1;
		}
		while (val != 1);
	}
}



/***********************************************************************
 *
 * FUNCTION:    GetNextTarget
 *
 *
 ***********************************************************************/
static calcTarget GetNextTarget(calcTarget currentTarget)
{
	calcTarget nextTarget;
	
	if (currentTarget==HP48SX)
	{
		if (status48GXROMAvailable)
			nextTarget=HP48GX;
		else
			nextTarget=HP49G;
	}
	else if (currentTarget==HP48GX)
	{
		if (status49GROMAvailable)
			nextTarget=HP49G;
		else
			nextTarget=HP48SX;
	}
	else
	{
		if (status48SXROMAvailable)
			nextTarget=HP48SX;
		else
			nextTarget=HP48GX;
	}
	
	return nextTarget;
}






/***********************************************************************
 *
 * FUNCTION:    DisplayHP48Screen
 *
 *
 ***********************************************************************/
Boolean firstTime=true;

void DisplayHP48Screen(calcTarget loadTarget)
{	
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	DmResID			part1BF,part2BF,part3BF,part4BF,miniBF;
	RectangleType	copyRect;
	calcTarget		nextTarget;
	Int16			i;
	
	
#ifdef ENABLE_CLEAR_BUTTON
	RGBColorType	prevRGB;
#endif
	
	if (option320x320)
	{
		statusDisplayMode2x2=false;
		virtLayout = (squareDef*)virtLayout1x1;
		if (loadTarget==HP49G)
			keyLayout = (keyDef*)keyLayout320_hp49;
		else
			keyLayout = (keyDef*)keyLayout320;
	}
	else
	{
		statusDisplayMode2x2=true;
		virtLayout = (squareDef*)virtLayout2x2;
		if (loadTarget==HP49G)
			keyLayout = (keyDef*)keyLayout480_hp49;
		else
			keyLayout = (keyDef*)keyLayout480;
	}		

	for (i=0;i<maxKeys;i++)  // precalculate so we don't have to do it later.
	{
		keyToSquare[i].upperLeftX = (keyLayout[i].upperLeftX>>1)-EDGE_EXPANSION;
		keyToSquare[i].lowerRightX = (keyLayout[i].lowerRightX>>1)+EDGE_EXPANSION;
		keyToSquare[i].upperLeftY = (keyLayout[i].upperLeftY>>1)-EDGE_EXPANSION;
		keyToSquare[i].lowerRightY = (keyLayout[i].lowerRightY>>1)+EDGE_EXPANSION;
	}
	
	if (loadTarget==HP48SX)
	{
		if (option320x320)
		{
			part1BF = Hp48sx320p1BitmapFamily;
			part2BF = Hp48sx320p2BitmapFamily;
			part3BF = Hp48sx320p3BitmapFamily;
		}
		else
		{
			part1BF = Hp48sxp1BitmapFamily;
			part2BF = Hp48sxp2BitmapFamily;
			part3BF = Hp48sxp3BitmapFamily;
			part4BF = Hp48sxp4BitmapFamily;
		}
		miniBF  = Hp48gxMiniBitmapFamily;	// only applicable in DRY-RUN scenario
		statusBackdrop = option48SXBackdrop;
		statusPixelColor = &option48SXPixelColor;
		statusBackColor	= &option48SXBackColor;
	}
	else if (loadTarget==HP48GX)
	{
		if (option320x320)
		{
			part1BF = Hp48gx320p1BitmapFamily; 
			part2BF = Hp48gx320p2BitmapFamily;
			part3BF = Hp48gx320p3BitmapFamily;
		}
		else
		{
			part1BF = Hp48gxp1BitmapFamily;
			part2BF = Hp48gxp2BitmapFamily;
			part3BF = Hp48gxp3BitmapFamily;
			part4BF = Hp48gxp4BitmapFamily;
		}
		miniBF  = Hp49gMiniBitmapFamily;	// only applicable in DRY-RUN scenario
		statusBackdrop = option48GXBackdrop;
		statusPixelColor = &option48GXPixelColor;
		statusBackColor	= &option48GXBackColor;
	}
	else
	{
		if (option320x320)
		{
			part1BF = Hp49g320p1BitmapFamily; 
			part2BF = Hp49g320p2BitmapFamily;
			part3BF = Hp49g320p3BitmapFamily;
		}
		else
		{
			part1BF = Hp49gp1BitmapFamily;
			part2BF = Hp49gp2BitmapFamily;
			part3BF = Hp49gp3BitmapFamily;
			part4BF = Hp49gp4BitmapFamily;
		}
		miniBF  = Hp48sxMiniBitmapFamily;	// only applicable in DRY-RUN scenario
		statusBackdrop = option49GBackdrop;
		statusPixelColor = &option49GPixelColor;
		statusBackColor	= &option49GBackColor;
	}
	
	if (statusRUN)	// override miniBF if emulator is running
	{
		nextTarget=GetNextTarget(loadTarget); // set miniBF based on next available ROM target
		if (nextTarget==HP48SX)
			miniBF = Hp48sxMiniBitmapFamily;
		else if (nextTarget==HP48GX)
			miniBF = Hp48gxMiniBitmapFamily;
		else
			miniBF = Hp49gMiniBitmapFamily;
	}
	
	WinSetDrawWindow(backStoreWindow);

	// Display graphic representation of HP48
	picHandle = DmGetResource(bitmapRsc, part1BF);
	picBitmap = MemHandleLock(picHandle);
	FastDrawBitmap(picBitmap,0,0);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	picHandle = DmGetResource(bitmapRsc, part2BF);
	picBitmap = MemHandleLock(picHandle);
	FastDrawBitmap(picBitmap,0,120);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	picHandle = DmGetResource(bitmapRsc, part3BF);
	picBitmap = MemHandleLock(picHandle);
	FastDrawBitmap(picBitmap,0,240);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	if (!option320x320)
	{
		picHandle = DmGetResource(bitmapRsc, part4BF);
		picBitmap = MemHandleLock(picHandle);
		FastDrawBitmap(picBitmap,0,360);
		MemHandleUnlock(picHandle);
		DmReleaseResource(picHandle);
	}
	
#ifdef ENABLE_CLEAR_BUTTON
	WinSetDrawWindow(backStoreWindow);
	// Draw clear button if profiler in use
	copyRect.topLeft.x = keyLayout[keyClear].upperLeftX;
	copyRect.topLeft.y = keyLayout[keyClear].upperLeftY;
	copyRect.extent.x = keyLayout[keyClear].lowerRightX-keyLayout[keyClear].upperLeftX;
	copyRect.extent.y = keyLayout[keyClear].lowerRightY-keyLayout[keyClear].upperLeftY;
	WinSetForeColorRGB(&whiteRGB,&prevRGB);
	HRWinDrawRectangle(hrLibRefNum, &copyRect, 3);
	WinSetForeColorRGB(&prevRGB,NULL);
#endif

	if (firstTime)
	{
		copyRect.topLeft.x = 0;
		copyRect.topLeft.y = 0;
		copyRect.extent.x = 320;
		copyRect.extent.y = ((option320x320)?320:480);
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,0,0,winPaint);
	}

	// Get mini bitmap graphic
	picHandle = DmGetResource(bitmapRsc, miniBF);
	picBitmap = MemHandleLock(picHandle);
	WinSetDrawWindow(miniBitmapWindow);
	FastDrawBitmap(picBitmap,0,0);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	// Get annunciator graphic
	picHandle = DmGetResource(bitmapRsc, AnnunciatorMapBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	WinSetDrawWindow(annunciatorBitmapWindow);
	HRWinDrawBitmap(hrLibRefNum,picBitmap,0,0);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	WinSetDrawWindow(mainDisplayWindow);
	
	if (!firstTime)
	{
		PixelDissolve(backStoreWindow,mainDisplayWindow,((option320x320)?102400:153600),0,0,false);				
	}
	firstTime=false;
}




/***********************************************************************
 *
 * FUNCTION:    RestoreBottomThird()
 *
 * DESCRIPTION: Restore the bottom third of screen when in 320x480 mode
 *              and the graffiti area is brought up
 *
 ***********************************************************************/
void RestoreBottomThird(void)
{
	RectangleType copyRect;
	
	copyRect.topLeft.x = 0;
	copyRect.topLeft.y = 320;
	copyRect.extent.x = 320;
	copyRect.extent.y = 160;
	HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);
}



/***********************************************************************
 *
 * FUNCTION:    DisplayAboutScreen
 *
 * DESCRIPTION: 
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void DisplayAboutScreen(void)
{
	WinHandle 		textScrollWindow;
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	Err				error;
	int				i,j;
	Int16			penX,penY;
	Boolean			penDown;
	Boolean			revertScreen=false;
	char 			*line,*test;
	int				numLines;
	UInt8			val;
	HRFontID 		font;
	Boolean 		center;
	RectangleType	drawRect;
	RectangleType 	vacatedRect,scrollRect,copyRect;
	Int16			length,position,height;
	Int16			offset=0;
	UInt32			nextTick;
	Boolean 		logoDisplay;
			
	if ((option320x320)&&(!statusDisplayMode2x2))
	{
		toggle_lcd(); // put calc screen in 2x2 mode
		revertScreen=true;
	}
	else
		display_clear();
		
	textScrollWindow = HRWinCreateOffscreenWindow(hrLibRefNum,fullScreenExtentX,fullScreenExtentY+30,screenFormat,&error);
	if (error)
	{
		goto dasexit;
	}

	drawRect.topLeft.x = 0;
	drawRect.topLeft.y = 0;
	drawRect.extent.x = fullScreenExtentX;
	drawRect.extent.y = fullScreenExtentY+30;
	WinSetForeColorRGB(&blackRGB,NULL);
	WinSetBackColorRGB(&blackRGB,NULL);
	WinSetDrawWindow(textScrollWindow);
	HRWinDrawRectangle(hrLibRefNum,&drawRect,0);
	
	// get Title logo for display
	picHandle = DmGetResource(bitmapRsc, TitleLogoBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	HRWinPaintBitmap(hrLibRefNum,picBitmap,14,18);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	line=(char*)aboutText;
	test=line;
	val=0;
	numLines=0;
	
	while(1)
	{
		val=*test++;
		if (val==0) numLines++;
		if (val=='Q')
			if (*test=='Q') break;
	}
	
	copyRect.topLeft.x = 0;
	copyRect.topLeft.y = 0;
	copyRect.extent.x = fullScreenExtentX;
	copyRect.extent.y = fullScreenExtentY;
		
	HRWinCopyRectangle(hrLibRefNum,textScrollWindow,mainDisplayWindow,&copyRect,fullScreenOriginX,fullScreenOriginY,winPaint);
	logoDisplay=true;
	
	nextTick=TimGetTicks();
	test=line;
	for (i=0;i<numLines;i++)
	{
		if (!logoDisplay)
		{
			while (*test++!=0) {}
			val=*test++;
			switch (val)
			{
				case 1:
					WinSetTextColorRGB(&whiteRGB,NULL);
					font=hrTinyFont;
					center=true;
					break;
				case 2:
					WinSetTextColorRGB(&whiteRGB,NULL);
					font=hrSmallBoldFont;
					center=true;
					break;
				case 3:
					WinSetTextColorRGB(&redRGB,NULL);
					font=hrSmallBoldFont;
					center=true;
					break;
				// case 4:
				default:
					WinSetTextColorRGB(&whiteRGB,NULL);
					font=hrTinyBoldFont;
					center=false;
					break;
			}
			HRFntSetFont(hrLibRefNum,font);
			height = FntLineHeight();
			
			if (line != NULL)
			{
				if (center)
				{
					length = FntCharsWidth(line, StrLen(line));
					if (length<(fullScreenExtentX-3))
						position=(((fullScreenExtentX-2)-length)/2)+1;
					else
						position=1;
					HRWinDrawTruncChars(hrLibRefNum,line, StrLen(line), position, fullScreenExtentY, fullScreenExtentX-2);
				}
				else
				{
					HRWinDrawTruncChars(hrLibRefNum,line, StrLen(line), 1, fullScreenExtentY, fullScreenExtentX-2);
				}
			}
			
			scrollRect.topLeft.x = 0;
			scrollRect.topLeft.y = 0;
			scrollRect.extent.x = fullScreenExtentX;
			scrollRect.extent.y = fullScreenExtentY+height+2;
		}
		else
		{
			height=70;  // draw Title logo for the equivalent of 70 anim frames.
		}
		for (j=0;j<height;j++)
		{
			do 
			{
				EvtGetPen(&penX, &penY, &penDown);
				if (penDown)
				{
					do 
					{
						EvtGetPen(&penX, &penY, &penDown);
						if ((penX<(fullScreenOriginX>>1)) || (penX>((fullScreenOriginX+fullScreenExtentX)>>1)) ||
						    (penY<(fullScreenOriginY>>1)) || (penY>((fullScreenOriginY+fullScreenExtentY)>>1)))
						{
							EvtFlushPenQueue( );
							i=numLines;
							j=height;
						}
					} while (penDown);
				}
			} while (TimGetTicks()<nextTick);
			
			nextTick=TimGetTicks()+3;
			
			EvtFlushPenQueue( );
			
			HRWinCopyRectangle(hrLibRefNum,textScrollWindow,mainDisplayWindow,&copyRect,fullScreenOriginX,fullScreenOriginY,winPaint);
			
			if (!logoDisplay)
			{
				HRWinScrollRectangle(hrLibRefNum,&scrollRect, winUp, 1, &vacatedRect);
			}
		}
		line=test;
		if (logoDisplay) logoDisplay=false;
	} 
	
	WinSetDrawWindow(mainDisplayWindow);
	WinSetTextColorRGB(&blackRGB,NULL);
		
	WinDeleteWindow(textScrollWindow,false);

dasexit:	
	if (revertScreen)
		toggle_lcd();
	else
	{
		display_clear(); // clear calcscreen	
		for (i=0;i<64;i++) lcdRedrawBuffer[i] = 1;
	}
}




/***********************************************************************
 *
 * FUNCTION:    DisplaySwitchGraphic
 *
 * DESCRIPTION: 
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void DisplaySwitchGraphic(void)
{
	MemHandle 		picHandle;
	BitmapPtr 		picBitmap;
	Coord 			picWidth,picHeight;
	Coord			newX,newY,oldX,oldY;
	RectangleType	srcRect,picRect,copyRect;
	Int16			penX,penY;
	UInt16			i=0;
	Boolean			penDown=false;
	Boolean			quitLoop = false;
	
	if (optionTarget==HP48SX)
	{
		if ((!status48GXROMAvailable) && (!status49GROMAvailable) && (statusRUN))
			return;		// 48GX/49G ROM not available, don't switch unless in dry run mode
	}
	else if (optionTarget==HP48GX)
	{
		if ((!status48SXROMAvailable) && (!status49GROMAvailable)  && (statusRUN))
			return;		// 48SX/49G ROM not available, don't switch unless in dry run mode
	}
	else
	{
		if ((!status48SXROMAvailable) && (!status48GXROMAvailable)  && (statusRUN))
			return;		// 48SX/48GX ROM not available, don't switch unless in dry run mode
	}
	
	
	WinSetDrawWindow(mainDisplayWindow);
	
	picHandle = DmGetResource(bitmapRsc, Hp48gxMiniBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	copyRect.topLeft.x = 0;
	copyRect.topLeft.y = 0;
	copyRect.extent.x = 320;
	copyRect.extent.y = 150;  // only resave the LCD screen, the rest hasn't changed
	HRWinCopyRectangle(hrLibRefNum,mainDisplayWindow,backStoreWindow,&copyRect,0,0,winPaint);
		
	picRect.topLeft.x = 0;
	picRect.topLeft.y = 0;
	picRect.extent.x = picWidth;
	picRect.extent.y = picHeight;
	srcRect.topLeft.x = 0;
	srcRect.topLeft.y = 0;
	srcRect.extent.x = picWidth;
	srcRect.extent.y = picHeight;
	
	EvtGetPen(&penX, &penY, &penDown);
	if (option320x320)
	{
		oldX = (penX<<1)+40-circPoint[i].deltaX-(picWidth>>1);
		i=4;
	}
	else
	{
		oldX = (penX<<1)+40+circPoint[i].deltaX-(picWidth>>1);
	}
	oldY = (penY<<1)+40+circPoint[i].deltaY-(picHeight>>1);
	goto loopSt;
	
	do
	{
		EvtGetPen(&penX, &penY, &penDown);
		// Copy saved window store to its prior location
		HRWinCopyRectangle(hrLibRefNum,miniSaveWindow,backStoreWindow,&picRect,oldX,oldY,winPaint);
		// Make new backing store copy of new location
loopSt: if (i<((option320x320)?13:10)) penDown=true;  // must perform at least a few iterations before allowing exit
		if (option320x320)
			newX = (penX<<1)-circPoint[i].deltaX-(picWidth>>1);
		else
			newX = (penX<<1)+circPoint[i].deltaX-(picWidth>>1);
		newY = (penY<<1)+circPoint[i].deltaY-(picHeight>>1);
		srcRect.topLeft.x = newX;
		srcRect.topLeft.y = newY;		
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,miniSaveWindow,&srcRect,0,0,winPaint);
		// Draw mini pic at new location
		HRWinCopyRectangle(hrLibRefNum,miniBitmapWindow,backStoreWindow,&picRect,newX,newY,winPaint);
		if (newX>oldX)
		{
			copyRect.topLeft.x = oldX;
			copyRect.extent.x = newX+picWidth-oldX;
		}
		else
		{
			copyRect.topLeft.x = newX;
			copyRect.extent.x = oldX+picWidth-newX;
		}
		if (newY>oldY)
		{
			copyRect.topLeft.y = oldY;
			copyRect.extent.y = newY+picHeight-oldY;
		}
		else
		{
			copyRect.topLeft.y = newY;
			copyRect.extent.y = oldY+picHeight-newY;
		}
		HRWinCopyRectangle(hrLibRefNum,backStoreWindow,mainDisplayWindow,&copyRect,copyRect.topLeft.x,copyRect.topLeft.y,winPaint);
		oldX = newX;
		oldY = newY;
		i++;
		if (i>=109) i=19;
	} while (penDown);
	
	EvtFlushPenQueue( );

	
	srcRect.topLeft.x = newX;
	srcRect.topLeft.y = newY;
	srcRect.extent.x = picWidth;
	srcRect.extent.y = picHeight;
	copyRect.topLeft.x = 0;
	copyRect.topLeft.y = 0;
	copyRect.extent.x = 320;
	copyRect.extent.y = ((option320x320)?320:480);
	rect_sweep(&srcRect,&copyRect,10);
	
	statusSwitchEmulation = 1;
	statusQuit = true;              // stop the emulator so it can be restarted.
	statusNextTarget = GetNextTarget(optionTarget);
}





/***********************************************************************
 *
 * FUNCTION:    ErrorPrint
 *
 * DESCRIPTION: Print Error messages to HP48 simulation
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void ErrorPrint( char* screenText, Boolean newLine )
{
	RectangleType scrollRect,vacatedRect;
	Boolean doNotIncrease=false;


	WinSetDrawWindow(mainDisplayWindow);
	// Set background color and text color to match HP48 screen
	WinSetBackColorRGB(&greyRGB,NULL);
	WinSetTextColorRGB(&blackRGB,NULL);
	WinSetForeColorRGB(&greyRGB,NULL);
	// Set the appropriate Font
	HRFntSetFont(hrLibRefNum, hrTinyFont);
	
	if (newLine)
	{
		if ((errorScreenY+lineDiff)>((((option320x320)?errorPrint320ExtentY:errorPrintExtentY)+errorPrintOriginY)-lineDiff))
		{
			if (errorSkipOnce)
			{
				errorSkipOnce = 0;
			}
			else
			{
				// Scroll screen up 1 line
				scrollRect.topLeft.x = errorPrintOriginX;
				scrollRect.topLeft.y = errorPrintOriginY;
				scrollRect.extent.x = errorPrintExtentX;
				scrollRect.extent.y = errorPrintExtentY;
				HRWinScrollRectangle(hrLibRefNum, &scrollRect, winUp, lineDiff, &vacatedRect);
				HRWinDrawRectangle(hrLibRefNum, &vacatedRect, 0);
			}
			doNotIncrease=true;
		}
	}
	
	if (screenText != NULL)	
		HRWinDrawTruncChars(hrLibRefNum, screenText, StrLen(screenText), 
							errorPrintOriginX, errorScreenY, errorPrintExtentX);
	
	if (newLine)
		if (!doNotIncrease)
			errorScreenY=errorScreenY+lineDiff;
	
}



/***********************************************************************
 *
 * FUNCTION:    ScreenPrint
 *
 * DESCRIPTION: Print info messages to HP48 screen simulation
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void ScreenPrint( char* screenText, Boolean newLine )
{
	RectangleType scrollRect,vacatedRect;
	Boolean doNotIncrease=false;
	
	if (!optionVerbose) return;
	
	if ((option320x320)&&(!statusDisplayMode2x2))
	{
		toggle_lcd(); // put calc screen in 2x2 mode
		delayEvent=60;  // wait 60 keyboard ticks before triggering a return to 1x1.
		delayKey=virtKeyVideoSwap;  
	}


	// Set background color and text color to match HP48 screen
	WinSetBackColorRGB(statusBackColor,NULL);
	WinSetForeColorRGB(statusBackColor,NULL);
	WinSetTextColorRGB(&textRGB,NULL);

	// Set the appropriate Font
	HRFntSetFont(hrLibRefNum, hrTinyFont);
	
	if (newLine)
	{
		if ((screenY+lineDiff)>(screenPrintExtentY-lineDiff))
		{
			if (skipOnce)
			{
				skipOnce = 0;
			}
			else
			{
				// Scroll screen up 1 line
				scrollRect.topLeft.x = screenPrintOriginX;
				scrollRect.topLeft.y = screenPrintOriginY;
				scrollRect.extent.x = screenPrintExtentX;
				scrollRect.extent.y = screenPrintExtentY;
				HRWinScrollRectangle(hrLibRefNum, &scrollRect, winUp, lineDiff, &vacatedRect);
				HRWinDrawRectangle(hrLibRefNum, &vacatedRect, 0);
			}
			doNotIncrease=true;
		} 
	}
	
	if (screenText != NULL)	
		HRWinDrawTruncChars(hrLibRefNum, screenText, StrLen(screenText), 
							screenPrintOriginX, screenY, screenPrintExtentX);
	
	if (newLine)
		if (!doNotIncrease)
			screenY=screenY+lineDiff;
				
	if (newLine) 
	{
		if (optionScrollFast)
			SysTaskDelay(10);
		else
			SysTaskDelay(50);
	}
}


/***********************************************************************
 *
 * FUNCTION:    ScreenPrintReset
 *
 * DESCRIPTION: Print info messages to HP48 screen simulation
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void ScreenPrintReset( )
{
	RectangleType screenRect;
	RGBColorType prevRGB;
	
	WinSetForeColorRGB(&backRGB,&prevRGB);
		
	screenRect.topLeft.x = screenPrintOriginX;
	screenRect.topLeft.y = screenPrintOriginY;
	screenRect.extent.x = screenPrintExtentX;
	screenRect.extent.y = screenPrintExtentY;
	
	HRWinDrawRectangle(hrLibRefNum, &screenRect, 0);
	
	WinSetForeColorRGB(&prevRGB,NULL);
	
	screenY = lcdOriginY-14;
	skipOnce = 1;
}






/***********************************************************************
 *
 * FUNCTION:    AllocateRAM()
 *
 * DESCRIPTION: This function is responsible for allocating a 256 meg
 *              chunk of memory from the storage heap
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static Boolean AllocateRAM(void)
{	
	void    *mem;
	UInt32  memSize;
	UInt16 	ftrNum;
	
	if (optionTarget==HP48SX)
	{
		ftrNum = ramMemSXFtrNum;
		memSize = RAM_SIZE_SX+16; // The extra 16 bytes is just a precautionary cushion
	}
	else if (optionTarget==HP48GX)
	{
		ftrNum = ramMemGXFtrNum;
		memSize = RAM_SIZE_GX+16; // The extra 16 bytes is just a precautionary cushion
	}
	else
	{
		ftrNum = ramMem49FtrNum;
		memSize = IRAM_SIZE_49+ERAM_SIZE_49+16; // The extra 16 bytes is just a precautionary cushion
	}
	
	if ((status128KExpansion) && (optionTarget!=HP49G))
		memSize += 0x40000;

	if (FtrPtrNew(appFileCreator, ftrNum, memSize, &mem))
	{
		ScreenPrint("...Unable to allocate memory for RAM",true);
		OpenAlertConfirmDialog(alert,((status128KExpansion)?unableToAllocateRAMPORT1String:unableToAllocateRAMString),false,false);
		return false;
	}
	ram=(char *)mem;
	
	if (status128KExpansion)
	{
		if (optionTarget==HP48SX)
		{
			port1=ram+RAM_SIZE_SX;
		}
		else if (optionTarget==HP48GX)
		{
			port1=ram+RAM_SIZE_GX;
		}
	}
	if (optionTarget==HP49G)
	{
		port1=ram+IRAM_SIZE_49;
		port2=port1+(ERAM_SIZE_49/2);
	}
		
	return true;
}





/***********************************************************************
 *
 * FUNCTION:    CheckForRemount()
 *
 * DESCRIPTION: This function is responsible for checking if a newly 
 *              mounted volume is our storage volume for RAM / ROM
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
void CheckForRemount(void)
{
	UInt32		volIterator = vfsIteratorStart;
	FileRef		dirRef;
	Boolean		dirFound=false;
	Err			error;
	
	while (volIterator != vfsIteratorStop)
	{
		if (VFSVolumeEnumerate(&msVolRefNum, &volIterator)) return;
		error = VFSFileOpen(msVolRefNum,VFSROMRAMDirectory,vfsModeRead,&dirRef);
		if (!error) 
		{
			dirFound=true;
			break;
		}
	}
	VFSFileClose(dirRef);
	if (!dirFound) 
	{
		return;
	}
	statusVolumeMounted = true;	
}





/***********************************************************************
 *
 * FUNCTION:    CheckROMAvailability()
 *
 * DESCRIPTION: This function is responsible for checking for all ROMs 
 *              that this emulator supports.
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void CheckROMAvailability(void)
{
	UInt32		volIterator = vfsIteratorStart;
	FileRef		myRef,dirRef;
	Boolean		dirFound=false;
	Err			error;
	
	while (volIterator != vfsIteratorStop)
	{
		if (VFSVolumeEnumerate(&msVolRefNum, &volIterator)) return;
		error = VFSFileOpen(msVolRefNum,VFSROMRAMDirectory,vfsModeRead,&dirRef);
		if (!error) 
		{
			dirFound=true;
			break;
		}
	}
	VFSFileClose(dirRef);
	if (!dirFound) 
	{
		return;
	}
	statusVolumeMounted = true;
	error = VFSFileOpen(msVolRefNum,gxVFSROMFileName,vfsModeRead,&myRef);
	if (!error)
	{
		VFSFileClose(myRef);
		status48GXROMAvailable=true;
	}
	error = VFSFileOpen(msVolRefNum,sxVFSROMFileName,vfsModeRead,&myRef);
	if (!error)
	{
		VFSFileClose(myRef);
		status48SXROMAvailable=true;
	}
	error = VFSFileOpen(msVolRefNum,g49VFSROMFileName,vfsModeRead,&myRef);
	if (!error)
	{
		VFSFileClose(myRef);
		status49GROMAvailable=true;
	}
	
}







/***********************************************************************
 *
 * FUNCTION:    LoadRAMFile()
 *
 * DESCRIPTION: This function is responsible for loading in the saved RAM
 *              file that contains a sanpshot of the calc RAM last time
 *              the application stopped.
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean LoadRAMFile(void)
{
	Err				error;
	UInt32			ramSize;
	Boolean			romSwap=false;
	UInt32			numRead;
	FileRef			myRef;
	UInt32			fileSize;
	UInt32			count;
	UInt8			*buf;
	UInt8			*a,*b;
	
	if (optionTarget==HP48SX)
		ramSize = RAM_SIZE_SX;
	else if (optionTarget==HP48GX)
		ramSize = RAM_SIZE_GX;
	else
		ramSize = IRAM_SIZE_49;
	
	// load main RAM image
	fileSize=(ramSize>>1);  // half the size in nibbles
	
	if (optionTarget==HP48SX)
		error = VFSFileOpen(msVolRefNum,sxVFSRAMFileName,vfsModeRead,&myRef);
	else if (optionTarget==HP48GX)
		error = VFSFileOpen(msVolRefNum,gxVFSRAMFileName,vfsModeRead,&myRef);
	else
		error = VFSFileOpen(msVolRefNum,g49VFSIRAMFileName,vfsModeRead,&myRef);
		
	if (!error)
	{
		// attempt to reload database
		ScreenPrint("...VFS RAM image found", true);
	}
	else
	{
		ScreenPrint("...Unable to find VFS RAM image", true);
		OpenAlertConfirmDialog(alert,unableToFindRAMFileString,false,false);
		return false;
	}
	
	buf=(UInt8*)ram;
	
	ScreenPrint("...Loading RAM image from VFS", true);
	
	error=VFSFileReadData(myRef,fileSize,buf,fileSize,&numRead);
	VFSFileClose(myRef);
	if (error)
	{
		ScreenPrint("...Error reading VFS RAM image", true);
		OpenAlertConfirmDialog(alert,errorReadingRAMString,false,false);
		return false;
	}		
	
	DrawProgressBar(5,ANY);
	
    ScreenPrint("...Expanding RAM image...",true);
    
	MemSemaphoreReserve(true);	

	a=&buf[0];
	b=&buf[fileSize];
	count=fileSize;
	
	do
	{
		*a++=*b&0xf;
		*a++=*b++>>4;
	} while(--count);

	MemSemaphoreRelease(true);

	ScreenPrint("...Expansion finished",true);

	DrawProgressBar(5,ANY);
		
	// load port1 image if needed
	if ((status128KExpansion) || (optionTarget==HP49G))
	{
		if (optionTarget==HP49G)
			ramSize = ERAM_SIZE_49;
		else
			ramSize = PORT1_SIZE;
			
		fileSize=(ramSize>>1);  // half the size in nibbles
		
		if (optionTarget==HP48SX)
			error = VFSFileOpen(msVolRefNum,sxVFSPORT1FileName,vfsModeRead,&myRef);
		else if (optionTarget==HP48GX)
			error = VFSFileOpen(msVolRefNum,gxVFSPORT1FileName,vfsModeRead,&myRef);
		else
			error = VFSFileOpen(msVolRefNum,g49VFSERAMFileName,vfsModeRead,&myRef);
				
		if (!error)
		{
			// attempt to reload database
			ScreenPrint("...VFS Port1 image found", true);
		}
		else
		{
			ScreenPrint("...Unable to find VFS Port1 image", true);
			OpenAlertConfirmDialog(alert,unableToFindPort1FileString,false,false);
			return false;
		}
		
		buf=(UInt8*)port1;
		
		ScreenPrint("...Loading Port1 image from VFS", true);
		
		error=VFSFileReadData(myRef,fileSize,ram,ramSize+fileSize,&numRead);
		VFSFileClose(myRef);
		if (error)
		{
			ScreenPrint("...Error reading VFS Port1 image", true);
			OpenAlertConfirmDialog(alert,errorReadingPort1String,false,false);
			return false;
		}		
		
	    ScreenPrint("...Expanding Port1 image...",true);
		
		MemSemaphoreReserve(true);

		a=&buf[0];
		b=&buf[fileSize];
		count=fileSize;
		
		do
		{
			*a++=*b&0xf;
			*a++=*b++>>4;
		} while(--count);

		MemSemaphoreRelease(true);

		ScreenPrint("...Expansion finished",true);
	}
	DrawProgressBar(10,ANY);

	return true;
}







/***********************************************************************
 *
 * FUNCTION:    LoadROMFile()
 *
 * DESCRIPTION: This function is responsible for loading in a ROM database 
 *              file
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean LoadROMFile(void)
{
	Err				error;
	UInt32			romSize;
	UInt16 			ftrNum;
	void			*mem;
	Boolean			romSwap=false;
	Boolean			badHeader=false;
	unsigned char 	four[4];
	UInt32			numRead;
	FileRef			myRef;
	UInt32			fileSize;
	UInt32			count;
	UInt8			*buf;
	UInt8			*a,*b;
	
	if (optionTarget==HP48SX)
	{
		romSize = ROM_SIZE_SX;
		ftrNum  = romMemSXFtrNum;
	}
	else if (optionTarget==HP48GX)
	{
		romSize = ROM_SIZE_GX;
		ftrNum  = romMemGXFtrNum;
	}
	else
	{
		romSize = ROM_SIZE_49;
		ftrNum  = romMem49FtrNum;
	}
	
	fileSize=(romSize>>1);
	
	if (optionTarget==HP48SX)
		error = VFSFileOpen(msVolRefNum,sxVFSROMFileName,vfsModeRead,&myRef);
	else if (optionTarget==HP48GX)
		error = VFSFileOpen(msVolRefNum,gxVFSROMFileName,vfsModeRead,&myRef);
	else
		error = VFSFileOpen(msVolRefNum,g49VFSROMFileName,vfsModeRead,&myRef);
	
	
	if (!error)
	{
		// attempt to reload database
		ScreenPrint("...VFS ROM image found", true);
	}
	else
	{
		ScreenPrint("...Unable to find VFS ROM image", true);
		OpenAlertConfirmDialog(alert,unableToFindROMFileString,false,false);
		return false;
	}
	
	ScreenPrint("...Allocating memory space for ROM", true);
	if (FtrPtrNew(appFileCreator, ftrNum, romSize, &mem))
	{
		VFSFileClose(myRef);
		ScreenPrint("...Unable to allocate memory for ROM",true);
		OpenAlertConfirmDialog(alert,unableToAllocateROMString,false,false);
		return false;
	}
	rom=(char *)mem;
	buf=(UInt8*)mem;
	
	DrawProgressBar(10,ANY);
	
	ScreenPrint("...Loading ROM from VFS", true);
	
	
	error=VFSFileReadData(myRef,fileSize,mem,fileSize,&numRead);
	VFSFileClose(myRef);
	if (error)
	{
		ScreenPrint("...Error reading VFS ROM image", true);
		OpenAlertConfirmDialog(alert,errorReadingROMString,false,false);
		return false;
	}		
	
	DrawProgressBar(20,ANY);
	
	MemSemaphoreReserve(true);
	if (((buf[fileSize] == 0x23) && (optionTarget!=HP49G)) ||
	    ((buf[fileSize] == 0x62) && (optionTarget==HP49G)))
	{
		ScreenPrint("...ROM file is BIG_ENDIAN",true);
		romSwap=false;
	}
	
	if (((buf[fileSize] == 0x32) && (optionTarget!=HP49G)) ||
	    ((buf[fileSize] == 0x26) && (optionTarget==HP49G)))
	{
		ScreenPrint("...ROM file is LITTLE_ENDIAN",true);
		romSwap=true;
	} 
	
	if (optionTarget==HP49G)
	{
		if (romSwap)
		{
			if ((buf[fileSize] != 0x26)   || (buf[fileSize+1] != 0x49) ||
				(buf[fileSize+2] != 0x80) || (buf[fileSize+3] != 0x10))
				badHeader=true;
		}
		else
		{
			if ((buf[fileSize] != 0x62) ||   (buf[fileSize+1] != 0x94) ||
				(buf[fileSize+2] != 0x08) || (buf[fileSize+3] != 0x01))
				badHeader=true;
		}
	}
	else
	{
		if (romSwap)
		{
			if ((buf[fileSize] != 0x32)   || (buf[fileSize+1] != 0x96) ||
				(buf[fileSize+2] != 0x1b) || (buf[fileSize+3] != 0x80))
				badHeader=true;
		}
		else
		{
			if ((buf[fileSize] != 0x23) ||   (buf[fileSize+1] != 0x69) ||
				(buf[fileSize+2] != 0xb1) || (buf[fileSize+3] != 0x08))
				badHeader=true;
		}
	}
	
	if (badHeader)
	{
		MemSemaphoreRelease(true);
		ScreenPrint("...Invalid ROM file header",true);
		OpenAlertConfirmDialog(alert,invalidROMHeaderString,false,false);
		return false;
	}
	
	
    ScreenPrint("...ROM file header valid, expanding...",true);
	
	
	
	a=&buf[0];
	b=&buf[fileSize];
	count=fileSize;
	
	if (romSwap)
	{
		do
		{
			*a++=*b&0xf;
			*a++=*b++>>4;
		} while(--count);
	}
	else
	{
		do
		{
			*a++=*b>>4;
			*a++=*b++&0xf;
		} while(--count);
	}
		
	MemSemaphoreRelease(true);

	ScreenPrint("...Expansion finished",true);
	
	DrawProgressBar(15,ANY);
		
	ScreenPrint("Patching ROM for BEEP extension",true);

	four[0]=0x8;
	four[1]=0x1;
	four[2]=0xB;
	four[3]=0x1;
	if (optionTarget==HP49G)
		DmWrite(rom, 0x4157A, four, 4);
	else
		DmWrite(rom, 0x017A6, four, 4);
  
	ScreenPrint("ROM patched.",true);

	if (optionTarget==HP48GX)
	{
		ScreenPrint("Patching ROM for CRC removal",true);

		if ((rom[0x0A08A]!=0x1)||(rom[0x0A08B]!=0x5)||(rom[0x0A08C]!=0xA)||(rom[0x0A08D]!=0x3)||
			(rom[0x0A093]!=0x1)||(rom[0x0A094]!=0x5)||(rom[0x0A095]!=0xE)||(rom[0x0A096]!=0x3)||
			(rom[0x0A07B]!=0x5)||(rom[0x0A07C]!=0x6)||(rom[0x0A07D]!=0xf))
		{
			ScreenPrint("ROM patch check failed!",true);
			OpenAlertConfirmDialog(alert,romPatchFailedString,false,false);
			return false;
		}
		four[0]=0x6;
		four[1]=0x3;
		four[2]=0x0;
		four[3]=0x0;
		DmWrite(rom, 0x0A08A, four, 4);
		DmWrite(rom, 0x0A093, four, 4);
		four[0]=0x4;
		four[1]=0x2;
		four[2]=0x0;
		DmWrite(rom, 0x0A07B, four, 3);
	}
	
	DrawProgressBar(5,ANY);
	return true;
}






/***********************************************************************
 *
 * FUNCTION:    LoadState()
 *
 * DESCRIPTION: This function is responsible for loading the previous 
 *              saved state of the calculator.
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static Boolean LoadState(FileHand fileHandle)
{
	Boolean stateError=false;
	
	if (FileRead(fileHandle,A,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,B,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,C,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,D,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,R0,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,R1,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,R2,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,R3,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,R4,1,16,NULL) != 16 ) stateError = true;
	if (FileRead(fileHandle,ST,1,4,NULL) != 4 ) stateError = true; 
	
	if (FileRead(fileHandle,&HST,1,1,NULL) != 1 ) stateError = true; 
	if (FileRead(fileHandle,&P,1,1,NULL) != 1 ) stateError = true; 
	if (FileRead(fileHandle,&OUT,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&IN,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&SHUTDN,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&WAKEUP,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&INTP,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&INTE,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&INTD,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&rstkp,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&MODE,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&MODE_MINUS_ONE,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&CARRY,1,1,NULL) != 1 ) stateError = true; 
	if (FileRead(fileHandle,rstk,1,32,NULL) != 32 ) stateError = true; 
    if (FileRead(fileHandle,&pc_reg,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&d0reg,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&d1reg,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&CARDSTATUS,1,1,NULL) != 1 ) stateError = true; 

	if (FileRead(fileHandle,ioram,1,64,NULL) != 64 ) stateError = true;
	if (FileRead(fileHandle,&mcIOAddrConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcIOBase,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcIOCeil,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcRAMSizeConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcRAMAddrConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcRAMBase,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcRAMCeil,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcRAMSize,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE1SizeConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcCE1AddrConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcCE1Base,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE1Ceil,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE1Size,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE2SizeConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcCE2AddrConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcCE2Base,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE2Ceil,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcCE2Size,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcNCE3SizeConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcNCE3AddrConfigured,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&mcNCE3Base,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcNCE3Ceil,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&mcNCE3Size,1,4,NULL) != 4 ) stateError = true;
	if (FileRead(fileHandle,&flashBankLo,1,2,NULL) != 2 ) stateError = true;
	if (FileRead(fileHandle,&flashBankHi,1,2,NULL) != 2 ) stateError = true;
	if (FileRead(fileHandle,&crc,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&t1,1,1,NULL) != 1 ) stateError = true;
	if (FileRead(fileHandle,&t2,1,4,NULL) != 4 ) stateError = true; 
	
	if (FileRead(fileHandle,&lcdDisplayAddressStart,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&lcdDisplayAddressEnd,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&lcdMenuAddressStart,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&lcdMenuAddressEnd,1,4,NULL) != 4 ) stateError = true; 
	if (FileRead(fileHandle,&lcdCurrentLine,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdDisplayAreaWidth,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdLineCount,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdByteOffset,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdContrastLevel,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdPixelOffset,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdAnnunciatorState,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdPreviousAnnunciatorState,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdOn,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,&lcdTouched,1,2,NULL) != 2 ) stateError = true; 
	if (FileRead(fileHandle,lcdRedrawBuffer,1,64,NULL) != 64 ) stateError = true; 
	if (FileRead(fileHandle,kbd_row,1,9,NULL) != 9 ) stateError = true; 
	F_s[0]=P;
	F_l[1]=P+1;
	if (stateError) 
	{
		ScreenPrint("...unable to read state file correctly", true);
		OpenAlertConfirmDialog(alert,unableToLoadStateFileString,false,false);
		return false;
	}
	return true;
}



/***********************************************************************
 *
 * FUNCTION:    SaveState()
 *
 * DESCRIPTION: This function is responsible for storing a snapshot of 
 *              the calculator state for quick return next time. 
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static void SaveState(void)
{
	Boolean 		stateError=false;
	Err				error;
	FileHand		fileHandle;

	if (optionTarget==HP48SX)
		fileHandle = FileOpen(0, sxStateFileName, sxStateFileType, appFileCreator, fileModeReadWrite, &error);
	else if (optionTarget==HP48GX)
		fileHandle = FileOpen(0, gxStateFileName, gxStateFileType, appFileCreator, fileModeReadWrite, &error);
	else
		fileHandle = FileOpen(0, g49StateFileName, g49StateFileType, appFileCreator, fileModeReadWrite, &error);
    
	if (!error)
	{
		if (FileWrite(fileHandle,A,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,B,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,C,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,D,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,R0,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,R1,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,R2,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,R3,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,R4,1,16,NULL) != 16 ) stateError = true;
		if (FileWrite(fileHandle,ST,1,4,NULL) != 4 ) stateError = true; 
		
		if (FileWrite(fileHandle,&HST,1,1,NULL) != 1 ) stateError = true; 
		if (FileWrite(fileHandle,&P,1,1,NULL) != 1 ) stateError = true; 
		if (FileWrite(fileHandle,&OUT,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&IN,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&SHUTDN,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&WAKEUP,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&INTP,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&INTE,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&INTD,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&rstkp,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&MODE,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&MODE_MINUS_ONE,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&CARRY,1,1,NULL) != 1 ) stateError = true; 
		if (FileWrite(fileHandle,rstk,1,32,NULL) != 32 ) stateError = true; 
	    if (FileWrite(fileHandle,&pc_reg,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&d0reg,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&d1reg,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&CARDSTATUS,1,1,NULL) != 1 ) stateError = true; 

		if (FileWrite(fileHandle,ioram,1,64,NULL) != 64 ) stateError = true;
		if (FileWrite(fileHandle,&mcIOAddrConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcIOBase,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcIOCeil,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcRAMSizeConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcRAMAddrConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcRAMBase,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcRAMCeil,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcRAMSize,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE1SizeConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE1AddrConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE1Base,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE1Ceil,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE1Size,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE2SizeConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE2AddrConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE2Base,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE2Ceil,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcCE2Size,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcNCE3SizeConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcNCE3AddrConfigured,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&mcNCE3Base,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcNCE3Ceil,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&mcNCE3Size,1,4,NULL) != 4 ) stateError = true;
		if (FileWrite(fileHandle,&flashBankLo,1,2,NULL) != 2 ) stateError = true;
		if (FileWrite(fileHandle,&flashBankHi,1,2,NULL) != 2 ) stateError = true;
		if (FileWrite(fileHandle,&crc,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&t1,1,1,NULL) != 1 ) stateError = true;
		if (FileWrite(fileHandle,&t2,1,4,NULL) != 4 ) stateError = true; 
		
		if (FileWrite(fileHandle,&lcdDisplayAddressStart,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdDisplayAddressEnd,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdMenuAddressStart,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdMenuAddressEnd,1,4,NULL) != 4 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdCurrentLine,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdDisplayAreaWidth,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdLineCount,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdByteOffset,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdContrastLevel,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdPixelOffset,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdAnnunciatorState,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdPreviousAnnunciatorState,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdOn,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,&lcdTouched,1,2,NULL) != 2 ) stateError = true; 
		if (FileWrite(fileHandle,lcdRedrawBuffer,1,64,NULL) != 64 ) stateError = true; 
		if (FileWrite(fileHandle,kbd_row,1,9,NULL) != 9 ) stateError = true; 
	}
	
	if (stateError) 
	{
		OpenAlertConfirmDialog(alert,unableToSaveStateFileString,false,false);
	}
	
	DrawProgressBar(10,ANY);
	FileClose(fileHandle);
}



/***********************************************************************
 *
 * FUNCTION:    SaveRAM()
 *
 * DESCRIPTION: This function is responsible for storing a snapshot of 
 *              calculator RAM for quick return next time.
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
Boolean SaveRAM(Boolean skipToPort1)
{
	Err				error;
	UInt32			ramSize;
	UInt32			volIterator = vfsIteratorStart;
	Boolean			dirFound=false;
	UInt32			fileSize;
	UInt32			count;
	UInt8			*buf;
	UInt8			*a,*b;
	FileRef			myRef;
	UInt32			numWritten;
	
	if (skipToPort1) goto saveport1;

	// save main RAM image
	if (optionTarget==HP48SX)
		ramSize = RAM_SIZE_SX;
	else if (optionTarget==HP48GX)
		ramSize = RAM_SIZE_GX;
	else
		ramSize = IRAM_SIZE_49;

	fileSize=(ramSize>>1);

	buf=(UInt8*)ram;

	a=&buf[ramSize];
	b=a;
	count=fileSize;
	
	MemSemaphoreReserve(true);
	
	do
	{
		*--a = *--b<<4;
		*a |= *--b&0xf;
	} while(--count);
	
	MemSemaphoreRelease(true);
	
	DrawProgressBar(20,ANY);
	
	while (!statusVolumeMounted)
	{
		if (!OpenAlertConfirmDialog(confirm,retryAfterInsertString,false,false))
			return false;
	}
	
retrysaveram:

	if (optionTarget==HP48SX)
	{
		VFSFileDelete(msVolRefNum,sxVFSRAMFileName);
		VFSFileCreate(msVolRefNum,sxVFSRAMFileName);
	}
	else if (optionTarget==HP48GX)
	{
		VFSFileDelete(msVolRefNum,gxVFSRAMFileName);
		VFSFileCreate(msVolRefNum,gxVFSRAMFileName);
	}
	else
	{
		VFSFileDelete(msVolRefNum,g49VFSIRAMFileName);
		VFSFileCreate(msVolRefNum,g49VFSIRAMFileName);
	}
	
	 
	if (optionTarget==HP48SX)
		error = VFSFileOpen(msVolRefNum,sxVFSRAMFileName,vfsModeWrite,&myRef);
	else if (optionTarget==HP48GX)
		error = VFSFileOpen(msVolRefNum,gxVFSRAMFileName,vfsModeWrite,&myRef);
	else
		error = VFSFileOpen(msVolRefNum,g49VFSIRAMFileName,vfsModeWrite,&myRef);
    
    if (error)
    {
    	OpenAlertConfirmDialog(alert,unableToSaveRAMImageString,false,false);
    }
    else
    {
		VFSFileWrite(myRef,fileSize,&buf[fileSize],&numWritten);
		VFSFileClose(myRef);
		
		if (numWritten!=fileSize)
		{
			if (OpenAlertConfirmDialog(confirm,savedRAMFileInvalidString,false,false))
				goto retrysaveram;
		}
	}
	DrawProgressBar(20,ANY);


	// save Port1 image if necessary
	if ((status128KExpansion) || (optionTarget==HP49G))
	{
saveport1:	
		
		if (optionTarget==HP49G)
			ramSize = ERAM_SIZE_49;
		else
			ramSize = PORT1_SIZE;
	
		fileSize=(ramSize>>1);	

		buf=(UInt8*)port1;

		a=&buf[ramSize];
		b=a;
		count=fileSize;
		
		MemSemaphoreReserve(true);
		
		do
		{
			*--a = *--b<<4;
			*a |= *--b&0xf;
		} while(--count);
		
		MemSemaphoreRelease(true);
		
retrysaveport1:

		if (optionTarget==HP48SX)
		{
			VFSFileDelete(msVolRefNum,sxVFSPORT1FileName);
			VFSFileCreate(msVolRefNum,sxVFSPORT1FileName);
		}
		else if (optionTarget==HP48GX)
		{
			VFSFileDelete(msVolRefNum,gxVFSPORT1FileName);
			VFSFileCreate(msVolRefNum,gxVFSPORT1FileName);
		}
		else
		{
			VFSFileDelete(msVolRefNum,g49VFSERAMFileName);
			VFSFileCreate(msVolRefNum,g49VFSERAMFileName);
		}
		
		if (optionTarget==HP48SX)
			error = VFSFileOpen(msVolRefNum,sxVFSPORT1FileName,vfsModeWrite,&myRef);
		else if (optionTarget==HP48GX)
			error = VFSFileOpen(msVolRefNum,gxVFSPORT1FileName,vfsModeWrite,&myRef);
		else
			error = VFSFileOpen(msVolRefNum,g49VFSERAMFileName,vfsModeWrite,&myRef);
	    
	    if (error)
	    {
	    	OpenAlertConfirmDialog(alert,unableToSavePORT1ImageString,false,false);
	    }
	    else
	    {
			VFSFileWrite(myRef,fileSize,&buf[fileSize],&numWritten);
			VFSFileClose(myRef);
			if (numWritten!=fileSize)
			{
				if (OpenAlertConfirmDialog(confirm,savedPORT1FileInvalidString,false,false))
					goto retrysaveport1;
			}
		}
	}
	if (!skipToPort1) DrawProgressBar(40,ANY);
	return true;
}



/***********************************************************************
 *
 * FUNCTION:    CleanUpRAM()
 *
 * DESCRIPTION: This function is responsible for clearing all allocated pointers
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void CleanUpRAM(void)
{
	FtrPtrFree(appFileCreator, romMemSXFtrNum);
	FtrPtrFree(appFileCreator, ramMemSXFtrNum);
	FtrPtrFree(appFileCreator, romMemGXFtrNum);
	FtrPtrFree(appFileCreator, ramMemGXFtrNum);
	FtrPtrFree(appFileCreator, romMem49FtrNum);
	FtrPtrFree(appFileCreator, ramMem49FtrNum);

	DrawProgressBar(10,ANY);
}




/***********************************************************************
 *
 * FUNCTION:    PinResetEmulator()
 *
 * DESCRIPTION: This function is responsible for clearing the calculator 
 *              state variables to simulate a press of the reset button.
 *
 ***********************************************************************/
 
void PinResetEmulator(void)
{
	int i;
	
	update_t1=4;

	// set rom bank to correct bank
	if (optionTarget==HP49G)
	{
		flashBankHi=0;
		flashBankLo=0;
		flashRomHi=flashAddr[flashBankHi]-0x40000;
		flashRomLo=flashAddr[flashBankLo];
	}
	
	// set pc to 0
	statusPerformReset=true;
		
	rstkp=0;
	for (i=0;i<8;i++) rstk[i]=0;
	HST=0;
	SHUTDN=1;
	INTE=1;
	INTP=0;
	INTD=0;
	crc=0;
	
	// reset IO
	for (i=0;i<64;i++)  ioram[i]=0;
	
	// reset MMU
	mcIOAddrConfigured=false;
	mcIOBase=0x100000;
	mcIOCeil=0;

	mcRAMSizeConfigured=false;
	mcRAMAddrConfigured=false;
	mcRAMBase=0x100000;
	mcRAMCeil=0;
	mcRAMSize=0;
	mcRAMDelta=0;

	mcCE1SizeConfigured=false;
	mcCE1AddrConfigured=false;
	mcCE1Base=0x100000;
	mcCE1Ceil=0;
	mcCE1Size=0;
	mcCE1Delta=0;

	mcCE2SizeConfigured=false;
	mcCE2AddrConfigured=false;
	mcCE2Base=0x100000;
	mcCE2Ceil=0;
	mcCE2Size=0;
	mcCE2Delta=0;

	mcNCE3SizeConfigured=false;
	mcNCE3AddrConfigured=false;
	mcNCE3Base=0x100000;
	mcNCE3Ceil=0;
	mcNCE3Size=0;
	mcNCE3Delta=0;
	
	t1=0;
	t2=0;
	lcdLineCount=0;
	lcdByteOffset=0;
	lcdContrastLevel=0;
	lcdPixelOffset=0;
	IO_TIMER_RUN=(ioram[0x2f]&1);
	
	prevTimeStamp = *timeReg;
	prevTickStamp = TimGetTicks();
}




/***********************************************************************
 *
 * FUNCTION:    ClearCalculator()
 *
 * DESCRIPTION: This function is responsible for clearing the calculator 
 *              RAM and state variables to simulate a factory default
 *              condition.
 *
 ***********************************************************************/
 
static void ClearCalculator(void)
{
	Int16			i;
	Int32           j;
	UInt32 			ramSize;
	void            *dummy=NULL;

	if (optionTarget==HP48SX)
		ramSize = RAM_SIZE_SX;
	else if (optionTarget==HP48GX)
		ramSize = RAM_SIZE_GX;
	else
		ramSize = IRAM_SIZE_49+ERAM_SIZE_49;
		
	if (status128KExpansion)
		ramSize += PORT1_SIZE;
		
	ScreenPrint("...clearing RAM",true);

	dummy=MemPtrNew(0x8000);
	if (dummy!=0)
	{
		MemSet(dummy,0x8000,0);
		for (j=0;j<ramSize;j+=0x8000)
			DmWrite(ram,j,dummy,0x8000);
		MemPtrFree(dummy);
	}
	
	ScreenPrint("...clearing state information",true);
	for (i=0;i<16;i++)
	{
		A[i]=0;
		B[i]=0;
		C[i]=0;
		D[i]=0;
		R0[i]=0;
		R1[i]=0;
		R2[i]=0;
		R3[i]=0;
		R4[i]=0;
	}
	for (i=0;i<4;i++) ST[i]=0;
	HST=0;
	P=0;
	OUT=0;
	IN=0;
	SHUTDN=0;
	WAKEUP=0;
	INTP=0;
	INTE=0;
	INTD=0;
	rstkp=0;
	MODE=16;
	MODE_MINUS_ONE=15;
	CARRY=0;
	IR15X=0x0000;
	for (i=0;i<8;i++) rstk[i]=0;
	pc_reg=0;
	d0reg=0;
	d1reg=0;
	F_s[0]=0;
	F_l[1]=0;

	for (i=0;i<9;i++) kbd_row[i]=0;
	
	lcdDisplayAddressStart=0;
	lcdDisplayAddressEnd=0;
	lcdMenuAddressStart=0;
	lcdMenuAddressEnd=0;
	lcdCurrentLine=0;
	lcdDisplayAreaWidth=0;
	lcdLineCount=0;
	lcdByteOffset=0;
	lcdContrastLevel=0;
	lcdPixelOffset=0;
	lcdAnnunciatorState=0;
	lcdPreviousAnnunciatorState=0;
	lcdOn=0;
	lcdTouched=0;
	for (i=0;i<64;i++) 
	{
		lcdRedrawBuffer[i]=0;
		ioram[i]=0;
	}
	
	mcIOAddrConfigured=false;
	mcIOBase=0x100000;
	mcIOCeil=0;

	mcRAMSizeConfigured=false;
	mcRAMAddrConfigured=false;
	mcRAMBase=0x100000;
	mcRAMCeil=0;
	mcRAMSize=0;
	mcRAMDelta=0;

	mcCE1SizeConfigured=false;
	mcCE1AddrConfigured=false;
	mcCE1Base=0x100000;
	mcCE1Ceil=0;
	mcCE1Size=0;
	mcCE1Delta=0;

	mcCE2SizeConfigured=false;
	mcCE2AddrConfigured=false;
	mcCE2Base=0x100000;
	mcCE2Ceil=0;
	mcCE2Size=0;
	mcCE2Delta=0;

	mcNCE3SizeConfigured=false;
	mcNCE3AddrConfigured=false;
	mcNCE3Base=0x100000;
	mcNCE3Ceil=0;
	mcNCE3Size=0;
	mcNCE3Delta=0;

	flashBankLo=0;
	flashBankHi=0;
	
	crc=0;
	t1=0;
	t2=0;
	saturn_speed=0;
	
}




/***********************************************************************
 *
 * FUNCTION:    DumpProfile()
 *
 * DESCRIPTION: This function is responsible for dumping the profile
 *              information to a text file. Profiling inclusion is
 *              controlled in emu48.h with the INCLUDE_PROFILER define
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

#ifdef INCLUDE_PROFILER
#define dirName 	"/PALM/PROGRAMS/Power48/"
#define profName 	"/PALM/PROGRAMS/Power48/profile.txt"

static void DumpProfile(void)
{
 	UInt16		volRefNum;
	UInt32		volIterator = vfsIteratorStart;
	FileRef		myRef;
	char 		tmp2[80];
	int			i,j;
	int			opcode[500];
	Int32		microsecondAvg[500];
	UInt32		tempCount,tempTime;
	int			tempOpcode;
	Int32		tempAvg;
	float 		timerPeriod;
	float		avgFloat;
	
	for (i=0;i<500;i++) opcode[i]=i;
	
	ErrorPrint("Computing avgs...",true);
	
	timerPeriod = (1000000.0/5526869.0);
	
	for (i=0;i<500;i++) 
	{
		if (count[i]>0)
		{
			avgFloat = ((float)time[i]*timerPeriod)/(float)count[i];
			microsecondAvg[i]= (Int32)avgFloat;
		}
		else
		{
			microsecondAvg[i]=0;
		}
	}
	
	ErrorPrint("Sorting profiler info...",true);
	
	for (i=0;i<500-1;i++)
	{
		for (j=0;j<500-1-i;j++)
		{
			if (time[j+1]>time[j])
			{
				tempCount = count[j];
				tempTime = time[j];
				tempOpcode = opcode[j];
				tempAvg = microsecondAvg[j];
				count[j] = count[j+1];
				time[j] = time[j+1];
				opcode[j] = opcode[j+1];
				microsecondAvg[j] = microsecondAvg[j+1];
				count[j+1] = tempCount;
				time[j+1] = tempTime;
				opcode[j+1] = tempOpcode;
				microsecondAvg[j+1] = tempAvg;
			}
		}
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries sorted.",i);
			ErrorPrint(tmp2,true);
		}		
	}

	ErrorPrint("Dumping profiler info...",true);

	
	while (volIterator != vfsIteratorStop)
	{
		if (VFSVolumeEnumerate(&volRefNum, &volIterator)) return;

		// check for presence of directory
		if (!VFSFileOpen(volRefNum,dirName,vfsModeRead,&myRef)) break;
	}

	if (VFSFileClose(myRef)) return;
	
	if (VFSFileCreate(volRefNum,profName)) return;
	
	if (VFSFileOpen(volRefNum,profName,vfsModeWrite,&myRef)) return;
	
	
	for (i=0;i<500;i++)
	{
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries written.",i);
			ErrorPrint(tmp2,true);
		}
		StrPrintF(tmp2,"%d.  Opcode %d:      count=%ld,      time=%ld,     avg=%ld\n",i,opcode[i],count[i],time[i],microsecondAvg[i]);
		VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	}
	
	StrPrintF(tmp2," \n");
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	
	ErrorPrint("Sorting profiler info by avg...",true);
	
	for (i=0;i<500-1;i++)
	{
		for (j=0;j<500-1-i;j++)
		{
			if (microsecondAvg[j+1]>microsecondAvg[j])
			{
				tempCount = count[j];
				tempTime = time[j];
				tempOpcode = opcode[j];
				tempAvg = microsecondAvg[j];
				count[j] = count[j+1];
				time[j] = time[j+1];
				opcode[j] = opcode[j+1];
				microsecondAvg[j] = microsecondAvg[j+1];
				count[j+1] = tempCount;
				time[j+1] = tempTime;
				opcode[j+1] = tempOpcode;
				microsecondAvg[j+1] = tempAvg;
			}
		}
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries sorted.",i);
			ErrorPrint(tmp2,true);
		}		
	}

	ErrorPrint("Dumping profiler avg info...",true);

    for (i=0;i<500;i++)
	{
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries written.",i);
			ErrorPrint(tmp2,true);
		}
		StrPrintF(tmp2,"%d.  Opcode %d:      count=%ld,      time=%ld,     avg=%ld\n",i,opcode[i],count[i],time[i],microsecondAvg[i]);
		VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	}

	StrPrintF(tmp2," \n");
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	
	ErrorPrint("Sorting profiler info by opcode...",true);
	
	for (i=0;i<500-1;i++)
	{
		for (j=0;j<500-1-i;j++)
		{
			if (opcode[j+1]<opcode[j])
			{
				tempCount = count[j];
				tempTime = time[j];
				tempOpcode = opcode[j];
				tempAvg = microsecondAvg[j];
				count[j] = count[j+1];
				time[j] = time[j+1];
				opcode[j] = opcode[j+1];
				microsecondAvg[j] = microsecondAvg[j+1];
				count[j+1] = tempCount;
				time[j+1] = tempTime;
				opcode[j+1] = tempOpcode;
				microsecondAvg[j+1] = tempAvg;
			}
		}
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries sorted.",i);
			ErrorPrint(tmp2,true);
		}		
	}

	ErrorPrint("Dumping profiler opcode info...",true);

    for (i=0;i<500;i++)
	{
		if ((i&0x3f)==0)
		{
			StrPrintF(tmp2,"%d entries written.",i);
			ErrorPrint(tmp2,true);
		}
		StrPrintF(tmp2,"%d.  Opcode %d:      count=%ld,      time=%ld,     avg=%ld\n",i,opcode[i],count[i],time[i],microsecondAvg[i]);
		VFSFileWrite(myRef,StrLen(tmp2),tmp2,NULL);
	}

    VFSFileClose(myRef);
}

#endif





/***********************************************************************
 *
 * FUNCTION:    LoadEmulator()
 *
 * DESCRIPTION: This function is responsible for loading up the RAM and
 *              ROM files and starting the main emulator loop.
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static Err LoadEmulator(void) 
{
	Int16 		i;
	FileHand 	fileHandle;
	Err 		error;


	CheckROMAvailability();
	
	if ((!status48SXROMAvailable)&&(!status48GXROMAvailable)&&(!status49GROMAvailable))
	{
		if (!statusSkipScreenDraw) DisplayHP48Screen(optionTarget);
		statusSkipScreenDraw=false;
		OpenAlertConfirmDialog(alert,noROMFilesString,true,false);
		return errUnableToFindROMFiles;
	}
	else
	{
		if ((optionTarget==HP48SX)&&(!status48SXROMAvailable))
		{
			if (status48GXROMAvailable)
				optionTarget=HP48GX;       // 48SX ROM not available, start 48GX instead
			else
				optionTarget=HP49G;        // 48GX ROM not available, start 49G instead
		}
		else if ((optionTarget==HP48GX)&&(!status48GXROMAvailable))
		{
			if (status49GROMAvailable)
				optionTarget=HP49G;       // 48GX ROM not available, start 49G instead
			else
				optionTarget=HP48SX;       // 49G ROM not available, start 48SX instead
		}
		else if (!status49GROMAvailable)
		{
			if (status48SXROMAvailable)
				optionTarget=HP48SX;       // 49G ROM not available, start 48SX instead
			else
				optionTarget=HP48GX;       // 48SX ROM not available, start 48GX instead
		}
	}
	
	
	if (optionTarget==HP48SX)
	{
		statusColorCycle 	 	= option48SXColorCycle;
		status128KExpansion 	= option48SX128KExpansion;
	}
	else if (optionTarget==HP48GX)
	{
		statusColorCycle 		= option48GXColorCycle;
		status128KExpansion 	= option48GX128KExpansion;
	}
	else
	{
		statusColorCycle 		= option49GColorCycle;
		status128KExpansion 	= false;
	}
			
	if (!statusSkipScreenDraw) 
	{
		DisplayHP48Screen(optionTarget);
		DrawProgressBar(0,STARTUP);
	}
		
	DrawProgressBar(5,ANY);

	ScreenPrint("Emulator startup commencing...",true);

	ScreenPrint("Allocating mem for RAM/port1 from feature mem...",true);
	if (AllocateRAM())
	{
		ScreenPrint("     SUCCEEDED",true);
	}
	else
	{
		ScreenPrint("     FAILED",true);
		CleanUpRAM();
		statusSkipScreenDraw=false;
		return errUnableToAllocateRAM;
	}
	ScreenPrint(NULL,true);
	
	DrawProgressBar(5,ANY);
		
		
	if (!statusCompleteReset)
	{			
		ScreenPrint("Loading RAM and Port1 images from saved files...",true);
		if (LoadRAMFile())
		{
			ScreenPrint("     SUCCEEDED",true);
		}
		else
		{
			ScreenPrint("     FAILED",true);
			statusCompleteReset=true;
		}
		ScreenPrint(NULL,true);
	}
	
	// DrawProgressBar(20,ANY);  LoadRAMFile does this for us

	CARDSTATUS=0;
	
	if (status128KExpansion)
	{
		if (optionTarget==HP48SX)
			CARDSTATUS=0x5;
		else if (optionTarget==HP48GX)
			CARDSTATUS=0xa;
	}		
	if (optionTarget==HP49G)
		CARDSTATUS=0xF;
	
	if (!statusCompleteReset)
	{
		if (optionTarget==HP48SX)
			fileHandle = FileOpen(0, sxStateFileName, sxStateFileType, appFileCreator, fileModeReadOnly, &error);
		else if (optionTarget==HP48GX)
			fileHandle = FileOpen(0, gxStateFileName, gxStateFileType, appFileCreator, fileModeReadOnly, &error);
		else
			fileHandle = FileOpen(0, g49StateFileName, g49StateFileType, appFileCreator, fileModeReadOnly, &error);
			
		if (!error)
		{
			ScreenPrint("Loading calculator state file...",true);
			if (LoadState(fileHandle))
			{
				ScreenPrint("     SUCCEEDED",true);
			}
			else
			{
				ScreenPrint("     FAILED",true);
				statusCompleteReset=true;
			}
			ScreenPrint(NULL,true);
		}
		else
		{
			statusCompleteReset=true;
		}
		FileClose(fileHandle);
	}
	
	DrawProgressBar(10,ANY);

	if (statusCompleteReset)
	{
		ScreenPrint("Resetting all calculator data...",true);
		ClearCalculator();
		ScreenPrint("     FINISHED",true);
		ScreenPrint(NULL,true);
		statusCompleteReset=false;
		delayRefresh = false;
	}
	else
	{
		delayRefresh = true;
	}
		
	RecalculateDisplayParameters();
	
	if (delayRefresh)
		for (i=0;i<64;i++)
			lcdRedrawBuffer[i] = 0;
	
	DrawProgressBar(10,ANY);

	ScreenPrint("Loading ROM from saved ROM File...",true);
	if (LoadROMFile())
	{
		ScreenPrint("     SUCCEEDED",true);
	}
	else
	{
		ScreenPrint("     FAILED",true);
		CleanUpRAM();
		statusSkipScreenDraw=false;
		return errUnableToLoadROMFile;
	}
	ScreenPrint(NULL,true);
	
	// DrawProgressBar(50,ANY);  LoadROMFile() does this for us

	if (optionTarget==HP48SX)
	{
		ce1addr=port1;
		ce2addr=port2;
		nce3addr=NULL;
	}
	else
	{
		ce1addr=NULL;
		ce2addr=port1;
		nce3addr=port2;
	}
	
	if (mcRAMAddrConfigured) mcRAMDelta = (long)ram-mcRAMBase;
	if (mcCE1AddrConfigured) mcCE1Delta = (long)ce1addr-mcCE1Base;
	if (mcCE2AddrConfigured) mcCE2Delta = (long)ce2addr-mcCE2Base;
	if (mcNCE3AddrConfigured) mcNCE3Delta = (long)nce3addr-mcNCE3Base;
	
	if (optionTarget==HP49G)
	{
		for (i=0;i<8;i++)
			flashAddr[i]=rom+((long)i*0x40000);
		
		for (i=8;i<16;i++)
			flashAddr[i]=rom+0x1ffd00+((long)(i-8)*0x20);
			
		flashRomHi=flashAddr[flashBankHi]-0x40000;  // Subtract 0x40000 here so we don't have to later
		flashRomLo=flashAddr[flashBankLo];
		
		status49GActive=true;
	}
	else
		status49GActive=false;
	
	ScreenPrint("Initializing LCD...",true);
	init_lcd();
	ScreenPrint("     FINISHED",true);
	ScreenPrint(NULL,true);
	
	display_clear();
    
#ifdef INCLUDE_PROFILER
	for (i=0;i<500;i++) {time[i]=0;count[i]=0;}
#endif

	saturn_speed=SATURN_INSTRUCTIONS_PER_TICK;
	statusPerformReset=false;
	statusRUN=true;
	
	build_contrast_map();
	adjust_contrast();
	
	// Start the Emulator
	MemSemaphoreReserve(true);
	
	emulate();
	
	MemSemaphoreRelease(true);

	// Emulator is quitting, save state information
	exit_lcd();
	
	if (statusSwitchEmulation)
	{
		DisplayHP48Screen(statusNextTarget);
		statusSkipScreenDraw=true;
		DrawProgressBar(0,SWITCH);
	}				
	else
	{
		statusSkipScreenDraw=false;
		DrawProgressBar(0,SHUTDOWN);
	}
	
#ifdef INCLUDE_PROFILER
	DumpProfile();
#endif

	if (SaveRAM(false))     // ensure that state is saved only if matching RAM is saved
		SaveState();   // Prevents corruption if user elects to not save RAM to VFS
		
	CleanUpRAM();
	
	EvtFlushPenQueue( );
	
  	return errNone;
}






/***********************************************************************
 *
 * FUNCTION:    DryEmulate()
 *
 * DESCRIPTION: This function is bringing up the shell of the emulator
 *              without actually running it. It provides an event loop
 *              so the user can alter preferences.
 *
 ***********************************************************************/

static void DryEmulate(void) 
{
	EventType event;
	
restartDEloop:
	statusRUN=false;
	statusQuit=false;
	statusSwitchEmulation=false;
	
	status49GActive=(optionTarget==HP49G);
	
	display_clear();
	while (!statusQuit)
	{
		WinSetForeColorRGB(statusPixelColor, NULL);
		WinSetTextColorRGB(statusPixelColor, NULL);
		WinSetBackColorRGB(statusBackColor, NULL);
		HRFntSetFont(hrLibRefNum, hrBoldFont);
		if (statusDisplayMode2x2)
		{
			HRWinDrawChars(hrLibRefNum, "Dry run mode", 12, fullScreenOriginX+66, fullScreenOriginY+56);
		}
		else
		{
			HRWinDrawChars(hrLibRefNum, "Dry run mode", 12, lcd1x1FSOriginX+1, lcd1x1FSOriginY+19);
		}
		if (optionPowerSave)
			EvtGetEvent(&event,evtWaitForever);  // Allow Palm CPU to doze
		else
			EvtGetEvent(&event,0);

		if (event.eType!=nilEvent)
	    	ProcessPalmEvent(&event);
	}
	if (statusSwitchEmulation)
	{
		statusQuit=false;
		if (optionTarget==HP48SX)
			optionTarget=HP48GX;
		else if (optionTarget==HP48GX)
			optionTarget=HP49G;
		else
			optionTarget=HP48SX;
		DisplayHP48Screen(optionTarget);		
		goto restartDEloop;
	}
	EvtFlushPenQueue( );
}
	    

	
/***********************************************************************
 *
 * FUNCTION:    AdjustWindowBounds()
 *
 * DESCRIPTION: 
 *
 ***********************************************************************/
void AdjustWindowBounds(void)
{
	Coord extX, extY;
	RectangleType winRect;
	
	HRWinGetDisplayExtent(hrLibRefNum, &extX, &extY);
	if ((extX != displayExtentX) || (extY != displayExtentY))
	{
		HRWinGetWindowBounds(hrLibRefNum, &winRect);
		winRect.extent.x = extX;
		winRect.extent.y = extY;
		HRWinSetWindowBounds(hrLibRefNum, WinGetDrawWindow(), &winRect);
	}
	displayExtentX = extX;
	displayExtentY = extY;
	HRWinGetWindowExtent(hrLibRefNum, &windowExtentX, &windowExtentY);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}




/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;
	EventType newEvent;
	Err error;
	
	switch (eventP->eType) 
		{
		case menuEvent:
			break; // no menu code here!

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			
			FrmHideObject(frmP,FrmGetObjectIndex(frmP,MainSaveFileField));
			FrmHideObject(frmP,FrmGetObjectIndex(frmP,MainDialogDescField));
			
			if (!option320x320)
			{
				// The following line triggers the start of the emulator
				// by sending the sysNotifyDisplayChangeEvent event to the
				// application launch handler. (For the case of an NR70)
				SilkLibResizeDispWin(silkLibRefNum, silkResizeMax);
			}
			else
			{
				// Just start the emulator without resizing the screen
				newEvent.eType = displayChangeEvent;
				EvtAddEventToQueue(&newEvent);
			}
			handled = true;
			break;
			
		case frmUpdateEvent:
			// To do any custom drawing here, first call FrmDrawForm(), then do your
			// drawing, and then set handled to true.
			break;
			
		case displayChangeEvent:
			AdjustWindowBounds();
						
			do
			{
				statusSwitchEmulation=false;  // this is now taken card of in DisplayHP48screen
				
				error=LoadEmulator();  // Load up the ROM and RAM contents and start the emulator
				
				if (error) DryEmulate();   // Just bring up the emulator shell
				
				if (statusSwitchEmulation)
				{
					statusQuit=false;
					optionTarget=statusNextTarget;
				}
			} while (statusSwitchEmulation);
					

			// Send Appstop event message to form loop
			newEvent.eType = appStopEvent;
			EvtAddEventToQueue(&newEvent);
			break;
		
		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the temporary initial event loop for the
 *              Power48 application. It is only active until the screen
 *              size is changed and event handling switches to the main calc
 *              loop in Saturn.c
 *
 * PARAMETERS:  nothing 
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     Open Silk Library
 *
 ***********************************************************************/
static Boolean OpenSilk(SonySysFtrSysInfoP infoP)
{
	Err error;
	
	// is virtual silkscreen library available?
	if ((infoP->extn & sonySysFtrSysInfoExtnSilk) &&
		(infoP->libr & sonySysFtrSysInfoLibrSilk))
	{
		// library available, now get refNum;
		error = SysLibFind(sonySysLibNameSilk, &silkLibRefNum);
		if (error) return false;
			
		// library refNum found, now load library
		error = SysLibLoad( 'libr', sonySysFileCSilkLib, &silkLibRefNum );
		if (error) return false;
			
		// library loaded, now open library
		error = SilkLibOpen(silkLibRefNum);
		if (error) return false;
	}
	else
	{
		return false;
	}
	return true;
}

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	UInt16 prefsSize;
	UInt32 width, height, depth;
	Boolean enableColor;
	UInt16 cardNo;
	LocalID dbID;
	Err error;
	Coord picWidth,picHeight;
	MemHandle picHandle;
	BitmapPtr picBitmap;
	SonySysFtrSysInfoP infoP;
	pref_t options;
	pref_t_090b *options_090b;
	Boolean offscreenError=false;
	Boolean regError=false;
	Int16 myPrefVersion;
	
	// Read the preference information for the calculator app
	prefsSize = sizeof(pref_t);
	myPrefVersion = PrefGetAppPreferences(appFileCreator, appPrefID, &options, &prefsSize, true);
	if (myPrefVersion == noPreferenceFound)
	{
		statusCompleteReset=true;
	}
	else if (myPrefVersion == appPrefVersionNum_090b)
	{
		options_090b = (pref_t_090b*)&options;
		optionVolume               = options_090b->volume;
		optionTarget               = ((options_090b->loadSX)?HP48SX:HP48GX);
		optionPowerSave            = options_090b->powerSave;
		optionVerbose              = options_090b->verbose;
		optionScrollFast           = options_090b->scrollFast;
		option320x320              = options_090b->o320x320;
		option48GX128KExpansion    = options_090b->o48GX128KExpansion;
		option48SX128KExpansion    = options_090b->o48SX128KExpansion;
	}
	else if (myPrefVersion == appPrefVersionNum)
	{
		optionVolume               = options.volume;
		optionTarget               = options.target;
		optionPowerSave            = options.powerSave;
		optionVerbose              = options.verbose;
		optionScrollFast           = options.scrollFast;
		option320x320              = options.o320x320;
		option48GXBackdrop         = options.o48GXBackdrop;
		option48SXBackdrop         = options.o48SXBackdrop;
		option49GBackdrop          = options.o49GBackdrop;
		option48GXColorCycle       = options.o48GXColorCycle;
		option48SXColorCycle       = options.o48SXColorCycle;
		option49GColorCycle        = options.o49GColorCycle;
		option48GX128KExpansion    = options.o48GX128KExpansion;
		option48SX128KExpansion    = options.o48SX128KExpansion;
		MemMove(&option48GXPixelColor,&options.o48GXPixelColor,sizeof(RGBColorType));
		MemMove(&option48GXBackColor,&options.o48GXBackColor,sizeof(RGBColorType));
		MemMove(&option48SXPixelColor,&options.o48SXPixelColor,sizeof(RGBColorType));
		MemMove(&option48SXBackColor,&options.o48SXBackColor,sizeof(RGBColorType));
		MemMove(&option49GPixelColor,&options.o49GPixelColor,sizeof(RGBColorType));
		MemMove(&option49GBackColor,&options.o49GBackColor,sizeof(RGBColorType));
	}
    if (optionVolume>64) optionVolume=64;
    
    // blank out unused pref options
	option48GXBackdrop			= false;
	option48SXBackdrop			= false;
	option49GBackdrop			= false;
	option48GXColorCycle		= false;
	option48SXColorCycle		= false;
	option49GColorCycle			= false;

	// get Sony system information
	error=FtrGet(sonySysFtrCreator, sonySysFtrNumSysInfoP, (UInt32 *)&infoP);
	if (error)
	{
		FrmAlert(NoHiResLibraryAlert);
		return error;
	}
	
	// is HR library available?
	if (infoP->libr & sonySysFtrSysInfoLibrHR)
	{
		// library available, now get refNum;
		error = SysLibFind(sonySysLibNameHR, &hrLibRefNum);
		if (error)
		{
			FrmAlert(NoHiResLibraryAlert);
			return error;
		}
				
		// library refNum found, now load library
		error = SysLibLoad( 'libr', sonySysFileCHRLib, &hrLibRefNum );
		if (error)
		{
			FrmAlert(NoHiResLibraryAlert);
			return error;
		}
		
		//library loaded, now enable HR mode
		error = HROpen(hrLibRefNum);
		if (error)
		{
			FrmAlert(NoHiResLibraryAlert);
			return error;
		}
		
		// Does the display support color
		if (!HRWinScreenMode(hrLibRefNum,winScreenModeGetSupportsColor,NULL,NULL,NULL,&enableColor))
		{
			if (enableColor==false)
			{
				FrmAlert(NoColorLowDepthAlert);
				return sysErrParamErr;
			}
		}
		
		// Does the display support 16 bit depth
		if (!HRWinScreenMode(hrLibRefNum,winScreenModeGetSupportedDepths,NULL,NULL,&depth,NULL))
		{
			if ((depth&0x8000)==0)
			{
				FrmAlert(NoColorLowDepthAlert);
				return sysErrParamErr;
			}
		}
		
		// High-resolution mode enabled, now switch window.
		width = hrWidth;
		height = hrHeight;
		depth = 16;
		enableColor = true;
		error = HRWinScreenMode(hrLibRefNum, winScreenModeSet,
								&width, &height, &depth, &enableColor);
		if (error)
		{
			FrmAlert(NoHiResLibraryAlert);
			return error;
		}
	}
	else
	{
		FrmAlert(NoHiResLibraryAlert);
		return sysErrLibNotFound;
	}
	
	silkInUse=false;
	if (OpenSilk(infoP))
	{
		silkInUse=true;
		if (!option320x320) SilkLibEnableResize(silkLibRefNum);  // resize to 320x480
	}
	else
		if (!option320x320)     // if we can't open the silk library we must use 320x320 mode
			option320x320=true; 
			
	mainDisplayWindow=WinGetDisplayWindow();
	
	// Setup offscrean windows
	picHandle = DmGetResource(bitmapRsc, ScreenNibbleMap1x1BitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	hp48NibbleToScreenWindow1x1 = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (error) 
		offscreenError=true;
	else
	{
		WinSetDrawWindow(hp48NibbleToScreenWindow1x1);
		HRWinDrawBitmap(hrLibRefNum,picBitmap,0,0);
	}
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	
	picHandle = DmGetResource(bitmapRsc, ScreenNibbleMap2x2BitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	hp48NibbleToScreenWindow2x2 = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (error) 
		offscreenError=true;
	else
	{
		WinSetDrawWindow(hp48NibbleToScreenWindow2x2);
		HRWinDrawBitmap(hrLibRefNum,picBitmap,0,0);
	}
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);


	// Setup backStore window
	backStoreWindow = HRWinCreateOffscreenWindow(hrLibRefNum,
			320,480,screenFormat,&error);
	if (error) offscreenError=true;

	// Get annunciator bitmap graphic
	picHandle = DmGetResource(bitmapRsc, AnnunciatorMapBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	annunciatorBitmapWindow = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (error) offscreenError=true;
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);

	// create mini bitmap window and save window
	picHandle = DmGetResource(bitmapRsc, Hp48gxMiniBitmapFamily);
	picBitmap = MemHandleLock(picHandle);
	BmpGetDimensions(picBitmap,&picWidth,&picHeight,NULL);
	miniBitmapWindow = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (error) offscreenError=true;
	miniSaveWindow = HRWinCreateOffscreenWindow(hrLibRefNum,
			picWidth,picHeight,screenFormat,&error);
	if (error) offscreenError=true;
	MemHandleUnlock(picHandle);
	DmReleaseResource(picHandle);
	
	// create screen nibble bitmap
	nibbleMap1x1 = HRWinCreateOffscreenWindow(hrLibRefNum,4*19,16,screenFormat,&error);
	if (error) offscreenError=true;
	nibbleMap2x2 = HRWinCreateOffscreenWindow(hrLibRefNum,8*19,32,screenFormat,&error);
	if (error) offscreenError=true;
	
	if (offscreenError)
	{
		FrmAlert(NoOffscreenWindowsAlert);
		return sysErrNoFreeResource;
	}
	
	WinSetDrawWindow(mainDisplayWindow);

	HRWinGetDisplayExtent(hrLibRefNum, &displayExtentX, &displayExtentY);
	HRWinGetWindowExtent(hrLibRefNum, &windowExtentX, &windowExtentY);
	
	// Register to receive display area change events
	error = SysCurAppDatabase(&cardNo, &dbID);
	if (error) 
	{
		FrmAlert(UnableToRegisterAlert);
		return error;
	}
	
	if (SysNotifyRegister(cardNo, dbID, sysNotifyVolumeUnmountedEvent, NULL, sysNotifyNormalPriority, NULL)) regError=true;	
	if (SysNotifyRegister(cardNo, dbID, sysNotifyVolumeMountedEvent, NULL, sysNotifyNormalPriority, NULL)) regError=true;	
	if (SysNotifyRegister(cardNo, dbID, sysNotifyTimeChangeEvent, NULL, sysNotifyNormalPriority, NULL)) regError=true;	
	if (SysNotifyRegister(cardNo, dbID, sysNotifyLateWakeupEvent, NULL, sysNotifyNormalPriority, NULL)) regError=true;	
	if (SysNotifyRegister(cardNo, dbID, sysNotifyDeviceUnlocked, NULL, sysNotifyNormalPriority, NULL)) regError=true;	
	
	if (silkInUse)
	{
		if (SysNotifyRegister(cardNo, dbID, sysNotifyDisplayChangeEvent, NULL, sysNotifyNormalPriority, NULL)) regError=true;
	}
	if (regError)
	{
		FrmAlert(UnableToRegisterAlert);
		return sysErrNoFreeResource;
	}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	UInt16 prefsSize;
	pref_t options, oldOptions;
	UInt16 cardNo;
	LocalID dbID;
	Int16 myPrefVersion;
	
	// Write the preference information for the calculator app. This data 
	// will saved during a HotSync backup.
	options.volume                   = optionVolume;
	options.target                   = optionTarget;
	options.powerSave                = optionPowerSave;
	options.verbose                  = optionVerbose;
	options.scrollFast               = optionScrollFast;
	options.o320x320                 = option320x320;
	options.o48GXBackdrop            = option48GXBackdrop;
	options.o48SXBackdrop            = option48SXBackdrop;
	options.o49GBackdrop             = option49GBackdrop;
	options.o48GXColorCycle          = option48GXColorCycle;
	options.o48SXColorCycle          = option48SXColorCycle;
	options.o49GColorCycle           = option49GColorCycle;
	options.o48GX128KExpansion       = option48GX128KExpansion;
	options.o48SX128KExpansion       = option48SX128KExpansion;
	MemMove(&options.o48GXPixelColor,&option48GXPixelColor,sizeof(RGBColorType));
	MemMove(&options.o48GXBackColor,&option48GXBackColor,sizeof(RGBColorType));
	MemMove(&options.o48SXPixelColor,&option48SXPixelColor,sizeof(RGBColorType));
	MemMove(&options.o48SXBackColor,&option48SXBackColor,sizeof(RGBColorType));
	MemMove(&options.o49GPixelColor,&option49GPixelColor,sizeof(RGBColorType));
	MemMove(&options.o49GBackColor,&option49GBackColor,sizeof(RGBColorType));
	
	prefsSize = sizeof(pref_t);
	myPrefVersion = PrefGetAppPreferences(appFileCreator, appPrefID, &oldOptions, &prefsSize, true);   // get previous stored prefs
	// only save prefs if different from saved to reduce hotsyncs.
	if ((options.volume                != oldOptions.volume                ) ||
		(options.target                != oldOptions.target                ) ||
		(options.powerSave             != oldOptions.powerSave             ) ||
		(options.verbose               != oldOptions.verbose               ) ||
		(options.scrollFast            != oldOptions.scrollFast            ) ||
		(options.o320x320              != oldOptions.o320x320              ) ||
		(options.o48GXBackdrop         != oldOptions.o48GXBackdrop ) ||
		(options.o48SXBackdrop         != oldOptions.o48SXBackdrop ) ||
		(options.o49GBackdrop          != oldOptions.o49GBackdrop  ) ||
		(options.o48GXColorCycle       != oldOptions.o48GXColorCycle  ) ||
		(options.o48SXColorCycle       != oldOptions.o48SXColorCycle  ) ||
		(options.o49GColorCycle        != oldOptions.o49GColorCycle   ) ||
		(options.o48GX128KExpansion    != oldOptions.o48GX128KExpansion    ) ||
		(options.o48SX128KExpansion    != oldOptions.o48SX128KExpansion    ) ||
		(MemCmp(&options.o48GXPixelColor,&oldOptions.o48GXPixelColor,sizeof(RGBColorType))) ||
		(MemCmp(&options.o48GXBackColor,&oldOptions.o48GXBackColor,sizeof(RGBColorType))) ||
		(MemCmp(&options.o48SXPixelColor,&oldOptions.o48SXPixelColor,sizeof(RGBColorType))) ||
		(MemCmp(&options.o48SXBackColor,&oldOptions.o48SXBackColor,sizeof(RGBColorType))) ||
		(MemCmp(&options.o49GPixelColor,&oldOptions.o49GPixelColor,sizeof(RGBColorType))) ||
		(MemCmp(&options.o49GBackColor,&oldOptions.o49GBackColor,sizeof(RGBColorType))) ||
		(myPrefVersion                 != appPrefVersionNum))
	{
		PrefSetAppPreferences(appFileCreator, appPrefID, appPrefVersionNum, &options, sizeof(pref_t), true);
	}
	
	// Delete any existing memory handles
	WinDeleteWindow(nibbleMap2x2,false);
	WinDeleteWindow(nibbleMap1x1,false);
	
	WinDeleteWindow(miniSaveWindow,false);
	WinDeleteWindow(miniBitmapWindow,false);
	WinDeleteWindow(annunciatorBitmapWindow,false);
	WinDeleteWindow(backStoreWindow,false);
	WinDeleteWindow(hp48NibbleToScreenWindow2x2,false);
	WinDeleteWindow(hp48NibbleToScreenWindow1x1,false);

	// Close all the open forms.
	FrmCloseAllForms ();

	if (silkInUse)
	{
		SilkLibResizeDispWin(silkLibRefNum, silkResizeNormal);
		SilkLibDisableResize(silkLibRefNum);
		HRClose(silkLibRefNum);
	}	
	
	HRWinScreenMode(hrLibRefNum, winScreenModeSetToDefaults,NULL,NULL,NULL,NULL);					
	HRClose(hrLibRefNum);
				
	SysCurAppDatabase(&cardNo, &dbID);
	SysNotifyUnregister(cardNo, dbID, sysNotifyVolumeUnmountedEvent,
								sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyVolumeMountedEvent,
								sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyTimeChangeEvent,
								sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyLateWakeupEvent,
								sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyDeviceUnlocked,
								sysNotifyNormalPriority);
	if (silkInUse)
	{
		SysNotifyUnregister(cardNo, dbID, sysNotifyDisplayChangeEvent,
								sysNotifyNormalPriority);
	}
}










/* all code from here to end of file should use no global variables */
#pragma warn_a5_access on

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static Err RomVersionCompatible(UInt32 minVersion, UInt32 maxVersion, UInt16 launchFlags)
{
    UInt32 romVersion;

    /* See if we're on in minimum required version of the ROM or later. */
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
    if ((romVersion < minVersion) || (romVersion >= maxVersion))
    {
        if ((launchFlags & 
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
            FrmAlert (RomIncompatibleAlert);

            /* Palm OS versions before 2.0 will continuously relaunch this
             * app unless we switch to another safe one. */
            if (romVersion < kPalmOS20Version)
            {
                AppLaunchWithCommand(
                    sysFileCDefaultApp, 
                    sysAppLaunchCmdNormalLaunch, NULL);
            }
        }

        return sysErrRomIncompatible;
    }

    return errNone;
}



/***********************************************************************
 *
 * FUNCTION:    Power48Main
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPBP - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch, errNone if all went OK
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static UInt32 Power48Main(
    UInt16 cmd, 
    MemPtr cmdPBP, 
    UInt16 launchFlags)
{
	Err error;
	EventType newEvent;
	
	error = RomVersionCompatible (ourMinVersion, ourMaxVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;
			
		case sysAppLaunchCmdNotify:
			if (((SysNotifyParamType*)cmdPBP)->notifyType == 
				sysNotifyDisplayChangeEvent)
			{
				newEvent.eType = displayChangeEvent;
				EvtAddEventToQueue(&newEvent);
			}
			if (((SysNotifyParamType*)cmdPBP)->notifyType == 
				sysNotifyVolumeUnmountedEvent)
			{
				newEvent.eType = volumeUnmountedEvent;
				newEvent.data.generic.datum[0] = (UInt16)((SysNotifyParamType*)cmdPBP)->notifyDetailsP;
				EvtAddEventToQueue(&newEvent);
			}
			if (((SysNotifyParamType*)cmdPBP)->notifyType == 
				sysNotifyVolumeMountedEvent)
			{
				((SysNotifyParamType*)cmdPBP)->handled |= vfsHandledStartPrc;
				((SysNotifyParamType*)cmdPBP)->handled |= vfsHandledUIAppSwitch;
				newEvent.eType = volumeMountedEvent;
				EvtAddEventToQueue(&newEvent);
			}
			if ((((SysNotifyParamType*)cmdPBP)->notifyType == sysNotifyTimeChangeEvent) ||
				(((SysNotifyParamType*)cmdPBP)->notifyType == sysNotifyLateWakeupEvent) ||
				(((SysNotifyParamType*)cmdPBP)->notifyType == sysNotifyDeviceUnlocked))
			{
				if (launchFlags & sysAppLaunchFlagSubCall)  // okay to use A5 globals
					AdjustTime();
			}
			break;

		default:
			break;

		}
	
	return errNone;
}

/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPBP - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch, errNone if all went OK
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return Power48Main(cmd, cmdPBP, launchFlags);
}

/* turn a5 warning off to prevent it being set off by C++
 * static initializer code generation */
#pragma warn_a5_access reset
