/*************************************************************************\
*
*  SATURN.C
*
*  This is the core of the emulator
*
*  (c)1995 Sebastien Carlier
*  (c)2002 Robert Hildinger
*
\*************************************************************************/

#include "emu48.h"
#include <SonyCLIE.h>
#include "IO.H"

#define INTERRUPT(t)  {INTP=1;rstkpush_pc0();PCINTERRUPT();}
#define PCHANGED      {F_s[0]=P;F_l[1]=P+1;}
#define OPEND 		{asm { tst.w d7    bgt.w o_fetch } goto o_done2; }
//#define OPEND goto o_done;   // this should be used with profiler

char A[16]={0,};
char B[16]={0,};
char C[16]={0,};
char D[16]={0,};
char R0[16]={0,};
char R1[16]={0,};
char R2[16]={0,};
char R3[16]={0,};
char R4[16]={0,};
char ST[4]={0,};


char HST=0, P=0;
int  OUT = 0, IN = 0;
int  SHUTDN=0, WAKEUP=0, INTP=0, INTE=0, INTD=0, rstkp = 0, MODE = 16, MODE_MINUS_ONE = 15;
UInt8 CARRY=0;
int  IR15X=0x0000;
long rstk[8] = {0,};
long pc_reg=0; 
long d0reg=0, d1reg=0;
char CARDSTATUS = 0;
char X[16];
UInt8 kbd_row[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
char ioram[64] = {0,};

long 	lcdDisplayAddressStart = 0;
long 	lcdDisplayAddressStart2 = 0;
long 	lcdDisplayAddressEnd = 0;
long 	lcdMenuAddressStart = 0;
long 	lcdMenuAddressEnd = 0;
int  	lcdCurrentLine = 0;
int  	lcdDisplayAreaWidth = 0;
int  	lcdLineCount = 0;
int  	lcdByteOffset = 0;
int  	lcdContrastLevel = 0;
int  	lcdPixelOffset = 0;
int  	lcdAnnunciatorState = 0;
int  	lcdPreviousAnnunciatorState = 0;
int  	lcdOn = 0;
int  	lcdTouched = 0;
char*	lcdLineAddress[64] = {NULL,};
UInt8	lcdRedrawBuffer[64] = {0,};


Boolean 	mcIOAddrConfigured=false;
long 		mcIOBase=0x100000;
long 		mcIOCeil=0;

Boolean 	mcRAMSizeConfigured=false;
Boolean 	mcRAMAddrConfigured=false;
long 		mcRAMBase=0x100000;
long		mcRAMCeil=0;
long		mcRAMSize=0;
long		mcRAMDelta=0;

Boolean 	mcCE1SizeConfigured=false;
Boolean 	mcCE1AddrConfigured=false;
long 		mcCE1Base=0x100000;
long		mcCE1Ceil=0;
long		mcCE1Size=0;
long		mcCE1Delta=0;

Boolean 	mcCE2SizeConfigured=false;
Boolean 	mcCE2AddrConfigured=false;
long 		mcCE2Base=0x100000;
long		mcCE2Ceil=0;
long		mcCE2Size=0;
long		mcCE2Delta=0;

Boolean 	mcNCE3SizeConfigured=false;
Boolean 	mcNCE3AddrConfigured=false;
long 		mcNCE3Base=0x100000;
long		mcNCE3Ceil=0;
long		mcNCE3Size=0;
long		mcNCE3Delta=0;

char *rom=NULL;
char *ram=NULL;
char *port1=NULL;
char *port2=NULL;
char *ce1addr=NULL;
char *ce2addr=NULL;
char *nce3addr=NULL;
char* flashAddr[16]={NULL,};
int flashBankLo=0;
int flashBankHi=0;
char *flashRomLo=NULL;
char *flashRomHi=NULL;


UInt16 	crc=0;
UInt8 	t1=0;
UInt32	t2=0;
UInt16	saturn_speed=0;

Boolean delayRefresh=false;
UInt16	delayEvent=0;
UInt16	delayKey=noKey;
UInt16	delayEvent2=0;
UInt16	delayKey2=noKey;

UInt32	pc_save;

int IO_TIMER_RUN=0;


UInt8 F_s[16] = {0/*P*/,0       ,2,0,15,3 ,0,0 ,0,0,0,0,0,0,0,0};
UInt8 F_l[16] = {1     ,0/*P+1*/,1,3,1 ,12,2,16,0,0,0,0,0,0,0,5};

Int16 update_t1;

#ifdef PRINT_IPS
UInt32 lastTicks = 0;
Int16 ipsLoop;
UInt32 instCount;
UInt32 previousCount;
#endif

#ifndef INCLUDE_PROFILER
#define RFL(val) 
#else
#define RFL(val) timeIndex=val;
#endif

#ifdef INCLUDE_PROFILER
UInt32 	time[500];
UInt32	count[500];
int		timeIndex;
UInt16	timeStart;
UInt16	timeEnd;
#endif

#define TCN1   0xfffff608;
volatile UInt16 *timeReg;
UInt16 prevTimeStamp;    // dragonball high resolution timer used for Saturn Timer 2
UInt32 prevTickStamp;    // PalmOS 10ms low resolution timer used for Saturn Timer 1


#ifdef INCLUDE_TRACER
Boolean collecting=false;
FileHand tracerStream;
UInt32 traceNum=0;
#endif

// FILE *wire;






#ifdef INCLUDE_TRACER

static void CollectTrace(long pc)
{
	char tmp[160];
	UInt32 code=0;
	
		
	traceNum++;
	asm
	{
		move.l  a1,a0
		clr.l   d0
		move.b  (a0)+,d0
	    lsl.b   #4,d0
	    or.b    (a0)+,d0
	    lsl.w   #4,d0
	    or.b    (a0)+,d0
	    lsl.w   #4,d0
	    or.b    (a0)+,d0
	    lsl.l   #4,d0
	    or.b    (a0)+,d0
	    lsl.l   #4,d0
	    or.b    (a0)+,d0
	    lsl.l   #4,d0
	    or.b    (a0)+,d0
	    lsl.l   #4,d0
	    or.b    (a0)+,d0
	    move.l	d0,code
	}
	StrPrintF(tmp,"%ld pc:%lx   code:%lx   d0:%lx   d1:%lx   OUT:%x   IN:%x   IR15X:%d   CY:%d\n",traceNum,pc,code,d0reg,d1reg,OUT,IN,IR15X,CARRY);
	FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
}

static void TraceDisplayHit(void)
{
	char tmp[40];
	
	StrPrintF(tmp,"DISPLAY MEMORY HIT\n");
	FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
	StrPrintF(tmp,"  \n");
	FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
}

static void TraceMenuHit(void)
{
	char tmp[40];
	
	StrPrintF(tmp,"MENU MEMORY HIT\n");
	FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
	StrPrintF(tmp,"  \n");
	FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
}

#endif






/***********************************************************************************
 *
 *  Nibble Operator Primitives
 *
 *
 *
 *
 ***********************************************************************************/


inline asm long Npack5AASM( ) 
{
	lea     A+4,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.l   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack5CASM( ) 
{
	lea     C+4,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.l   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack5DASM( ) 
{
	lea     D+4,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.l   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack4AASM( ) 
{
	lea     A+3,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack4CASM( ) 
{
	lea     C+3,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack3CASM( ) 
{
	clr.l   d0
   move.b  C+2,d0
    lsl.b   #4,d0
    or.b    C+1,d0
    lsl.w   #4,d0
    or.b    C,d0
}


inline asm long Npack5I2ASM( )  // Assumption: a1 -> I[3] - we need to start with I[6]
{
	move.l	a1,a3
	addq.l  #3,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.l   #4,d0
    or.b    -(a3),d0
}

inline asm long Npack5I0ASM( ) // Assumptions: address to pack is in a2
{
	move.l  a2,a3
	addq.l  #4,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.l   #4,d0
    or.b    -(a3),d0
}



inline asm long Npack4I2ASM( ) // Assumption: a1 -> I[3] - we need to start with I[5]
{
	move.l	a1,a3
	addq.l  #2,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
}


// This function is the same as Npack4I2ASM above, except it decrements a1 by 2 to avoid PCMINUSTHREE
inline asm long Npack4I2ASM_m3( ) // Assumption: a1 -> I[3] - we need to start with I[5]
{
	addq.l  #2,a1      // a1 -> I[5]
	clr.l   d0
	move.b  (a1),d0    // a1 -> I[5]
    lsl.b   #4,d0
    or.b    -(a1),d0   // a1 -> I[4]
    lsl.w   #4,d0
    or.b    -(a1),d0   // a1 -> I[3]
    lsl.w   #4,d0
    or.b    -(a1),d0   // a1 -> I[2]
    subq.l  #2,a1      // a1 -> I[0]
}


inline asm long Npack2I2ASM( )  // Assumption: a1 -> I[3] - we need to start with I[3]
{
	move.l	a1,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
}


inline asm long Npack3I1ASM( )  // Assumption: a1 -> I[3] - we need to start with I[3]
{
	move.l	a1,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
    lsl.w   #4,d0
    or.b    -(a3),d0
}

// This function is the same as Npack3I1ASM above, except it decrements a1 by 3 to avoid PCMINUSTHREE
inline asm long Npack3I1ASM_m3( )  // Assumption: a1 -> I[3] - we need to start with I[3]
{
	clr.l   d0
	move.b  (a1),d0   // a1 -> I[3]
    lsl.b   #4,d0
    or.b    -(a1),d0  // a1 -> I[2]
    lsl.w   #4,d0
    or.b    -(a1),d0  // a1 -> I[1]
    subq.l  #1,a1     // a1 -> I[0]
}


inline asm long Npack2I1ASM( )  // Assumption: a1 -> I[3] - we need to start with I[2]
{
	move.l	a1,a3
	subq.l  #1,a3
	clr.l   d0
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
}

// This function is the same as Npack2I1ASM above, except it decrements a1 by 3 to avoid PCMINUSTHREE
inline asm long Npack2I1ASM_m3( )  // Assumption: a1 -> I[3] - we need to start with I[2]
{
	subq.l  #1,a1     // a1 -> I[2]
	clr.l   d0
	move.b  (a1),d0   // a1 -> I[2]
    lsl.b   #4,d0
    or.b    -(a1),d0  // a1 -> I[1]
    subq.l  #1,a1     // a1 -> I[0]
}


inline asm signed char Npack2I0ASM( )  // Assumption: a1 -> I[0] - we need to start with I[1]
{
	move.l	a1,a3
	addq.l  #1,a3
	move.b  (a3),d0
    lsl.b   #4,d0
    or.b    -(a3),d0
}



inline asm long Npack2CPageASM( ) 
{
	clr.l	d0
	move.b	C+4,d0
	lsl.b	#4,d0
	or.b	C+3,d0
	moveq	#12,d1
	lsl.l	d1,d0
}









inline void Nunpack3(char *a, unsigned long b)  
{
	*a++=b&0xf;
	b>>=4;
	*a++=b&0xf;
	b>>=4;
	*a=b&0xf;                        
}


inline void Nunpack4(char *a, unsigned long b)  
{ 
	*a++=b&0xf;
	b>>=4;
	*a++=b&0xf;
	b>>=4;
	*a++=b&0xf;                        
	b>>=4;
	*a=b&0xf;
}


inline void Nunpack5(char *a, unsigned long b)  
{
	*a++=b&0xf;
	b>>=4;
	*a++=b&0xf;
	b>>=4;
	*a++=b&0xf;                        
	b>>=4;
	*a++=b&0xf;
	b>>=4;
	*a=b;
}







inline void Ncopy_circular_C_pc_minus_one( )  
{
	asm
	{
		// This version of ncopy_circular assumes
		// that the immediate data is at pc - 1.
	
		move.b    P,d0
		ext.w     d0
		lea       C,a2
		lea       (a2,d0.w),a2   // a2=C+P
		
		subq    #1,a1   
		move.b	P,d0
		ext.w   d0
		move.w  d0,d1
		add.w   d5,d1    
		cmpi.w  #16,d1
		bge.s   skipa  
		
		moveq	#16,d0  // no wraparound
		sub.w	d5,d0		
		
skipa:	add.w	d0,d0   // wraparound
		lea		ncpys1, a0
		add.w	d0,a0
		jmp		(a0)

ncpys1: move.b    (a1)+,(a2)+ //16
		move.b    (a1)+,(a2)+ //15
		move.b    (a1)+,(a2)+ //14
		move.b    (a1)+,(a2)+ //13
		move.b    (a1)+,(a2)+ //12
		move.b    (a1)+,(a2)+ //11
		move.b    (a1)+,(a2)+ //10
		move.b    (a1)+,(a2)+ //9
		move.b    (a1)+,(a2)+ //8
		move.b    (a1)+,(a2)+ //7
		move.b    (a1)+,(a2)+ //6
		move.b    (a1)+,(a2)+ //5
		move.b    (a1)+,(a2)+ //4
		move.b    (a1)+,(a2)+ //3
		move.b    (a1)+,(a2)+ //2
		move.b    (a1)+,(a2)+ //1

		cmpi.w  #16,d1
		bge.s   cont  
	}
	return;

	asm 
	{
cont:	lea     C,a2
		moveq	#32,d0
		sub.w	d5,d0
		sub.b   P,d0
		add.w	d0,d0
		lea		ncpys2, a0
		add.w	d0,a0
		jmp		(a0)
	
ncpys2: move.b    (a1)+,(a2)+ //16
		move.b    (a1)+,(a2)+ //15
		move.b    (a1)+,(a2)+ //14
		move.b    (a1)+,(a2)+ //13
		move.b    (a1)+,(a2)+ //12
		move.b    (a1)+,(a2)+ //11
		move.b    (a1)+,(a2)+ //10
		move.b    (a1)+,(a2)+ //9
		move.b    (a1)+,(a2)+ //8
		move.b    (a1)+,(a2)+ //7
		move.b    (a1)+,(a2)+ //6
		move.b    (a1)+,(a2)+ //5
		move.b    (a1)+,(a2)+ //4
		move.b    (a1)+,(a2)+ //3
		move.b    (a1)+,(a2)+ //2
		move.b    (a1)+,(a2)+ //1
	}
}

inline void Ncopy_circular_A_pc_plus_one( )  
{
	asm
	{
		// This version of ncopy_circular assumes
		// that the immediate data is at pc + 1.
	
		move.b    P,d0
		ext.w     d0
		lea       A,a2
		lea       (a2,d0.w),a2   // a2=A+P
		
		addq    #1,a1   
		move.b	P,d0
		ext.w   d0
		move.w  d0,d1
		add.w   d5,d1   
		cmpi.w  #16,d1
		bge.s   skipa2  
		
		moveq	#16,d0  // no wraparound
		sub.w	d5,d0		
		
skipa2:	add.w	d0,d0   // wraparound
		lea		ncpys12, a0
		add.w	d0,a0
		jmp		(a0)

ncpys12:move.b    (a1)+,(a2)+ //16
		move.b    (a1)+,(a2)+ //15
		move.b    (a1)+,(a2)+ //14
		move.b    (a1)+,(a2)+ //13
		move.b    (a1)+,(a2)+ //12
		move.b    (a1)+,(a2)+ //11
		move.b    (a1)+,(a2)+ //10
		move.b    (a1)+,(a2)+ //9
		move.b    (a1)+,(a2)+ //8
		move.b    (a1)+,(a2)+ //7
		move.b    (a1)+,(a2)+ //6
		move.b    (a1)+,(a2)+ //5
		move.b    (a1)+,(a2)+ //4
		move.b    (a1)+,(a2)+ //3
		move.b    (a1)+,(a2)+ //2
		move.b    (a1)+,(a2)+ //1

		cmpi.w  #16,d1
		bge.s   cont2  
	}
	return;

	asm 
	{
cont2:	lea     A,a2
		moveq	#32,d0
		sub.w	d5,d0
		sub.b   P,d0
		add.w	d0,d0
		lea		ncpys22, a0
		add.w	d0,a0
		jmp		(a0)
	
ncpys22:move.b    (a1)+,(a2)+ //16
		move.b    (a1)+,(a2)+ //15
		move.b    (a1)+,(a2)+ //14
		move.b    (a1)+,(a2)+ //13
		move.b    (a1)+,(a2)+ //12
		move.b    (a1)+,(a2)+ //11
		move.b    (a1)+,(a2)+ //10
		move.b    (a1)+,(a2)+ //9
		move.b    (a1)+,(a2)+ //8
		move.b    (a1)+,(a2)+ //7
		move.b    (a1)+,(a2)+ //6
		move.b    (a1)+,(a2)+ //5
		move.b    (a1)+,(a2)+ //4
		move.b    (a1)+,(a2)+ //3
		move.b    (a1)+,(a2)+ //2
		move.b    (a1)+,(a2)+ //1
	}
}




inline void Ncopy4(char *a, char *b)  
{
  *(UInt32*)a    = *(UInt32*)b;
}


inline void Ncopy5(char *a, char *b)  
{
  UInt32  *ap,*bp;
  
  ap=(UInt32*)a;
  bp=(UInt32*)b;
  
  *ap++ = *bp++;
  *(UInt8*)ap = *(UInt8*)bp;
}


inline void Ncopy3(char *a, char *b)  
{
  UInt16  *ap,*bp;
  
  ap=(UInt16*)a;
  bp=(UInt16*)b;
  
  *ap++ = *bp++;
  *(UInt8*)ap = *(UInt8*)bp;
}





inline void Nxchg3(char *a, char *b)  
{
  char spare[3];  
  
  *(UInt16*)spare      = *(UInt16*)b; 
  *(UInt8*)(spare+2)   = *(UInt8*)(b+2); 
  *(UInt16*)b          = *(UInt16*)a; 
  *(UInt8*)(b+2)       = *(UInt8*)(a+2); 
  *(UInt16*)a          = *(UInt16*)spare; 
  *(UInt8*)(a+2)       = *(UInt8*)(spare+2); 
}


inline void Nxchg5(char *a, char *b)  
{
  char spare[5];  
  
  *(UInt32*)spare      = *(UInt32*)b; 
  *(UInt8*)(spare+4)   = *(UInt8*)(b+4); 
  *(UInt32*)b          = *(UInt32*)a; 
  *(UInt8*)(b+4)       = *(UInt8*)(a+4); 
  *(UInt32*)a          = *(UInt32*)spare; 
  *(UInt8*)(a+4)       = *(UInt8*)(spare+4); 
}


inline void Nxchg4(char *a, char *b)  
{
  char spare[4];

  *(UInt32*)spare      = *(UInt32*)b; 
  *(UInt32*)b          = *(UInt32*)a; 
  *(UInt32*)a          = *(UInt32*)spare; 
}








inline void Ninc5(char *a)  
{
  char m=(char)MODE;
  if (++(*a)<m) { CARRY=0; return; }
  *a++=0;
  
  if (++(*a)<m) { CARRY=0; return; }
  *a++=0;
  
  if (++(*a)<m) { CARRY=0; return; }
  *a++=0;
  
  if (++(*a)<m) { CARRY=0; return; }
  *a++=0;
  
  if (++(*a)<m) { CARRY=0; return; }
  *a=0;
  
  CARRY = 1;
}



inline void Ndec5(char *a)  
{
  char m=(char)MODE_MINUS_ONE;
  if ((*a)--) { CARRY = 0; return; }
  *a++ = m;
  
  if ((*a)--) { CARRY = 0; return; }
  *a++ = m;
  
  if ((*a)--) { CARRY = 0; return; }
  *a++ = m;
  
  if ((*a)--) { CARRY = 0; return; }
  *a++ = m;
  
  if ((*a)--) { CARRY = 0; return; }
  *a = m;

  CARRY = 1;
}








inline void NFaddconstant(char *a) 
{ 
	char m=(char)MODE;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d6 }  // assumptions f is in d6, a1 -> I[5], we need I[3]

  	count=F_l[f];
	a+=F_s[f];

	asm {                     // assumptions f is in d6, a1 -> I[5], we need I[5], will increase to I[6] to avoid PCPLUSONE
		move.b (a1)+,d6 
		addq.b #1,d6
	}
	
	CARRY=0;
	
	*a += (char)f;
	do 
	{
		*a += CARRY;
		if (*a < m)
			{ CARRY = 0;break; }
		else
			{ *a++ -= m; CARRY = 1; }
	} while (--count);
}


inline void Nadd5(char *a, char *b)  
{
	char m=(char)MODE;
	CARRY=0;
	*(UInt32*)a += *(UInt32*)b;
	*(UInt8*)(a+4) += *(UInt8*)(b+4);
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a >= m) { *a -= m; CARRY=1; }
}








inline void NFsubconstant(char *a) 
{ 
	char m=(char)MODE;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d6 }  // assumptions f is in d6, a1 -> I[5], we need I[3]

  	count=F_l[f];
	a+=F_s[f];

	asm {                     // assumptions f is in d6, a1 -> I[5], we need I[5], will increase to I[6] to avoid PCPLUSONE
		move.b (a1)+,d6 
		addq.b #1,d6
	}
  
	CARRY=0;
	
	*a -= (char)f;
	do 
	{
		*a -= CARRY;
		if (*a<0)
			{ *a++ += m; CARRY = 1; }
		else
			{ CARRY=0;break;}
	} while (--count);
}


inline void Nsub5(char *a, char *b)  
{
	char m=(char)MODE;
	CARRY=0;
	*a++ -= *b++;
	*a++ -= *b++;
	*a++ -= *b++;
	*a++ -= *b++;
	*a -= *b;
	a-=4;
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a < 0) { *a += m; CARRY=1; }
}


inline void Nrsub5(char *a, char *b)  
{
	char m=(char)MODE;
	CARRY=0;
	*a++ = *b++ - *a;
	*a++ = *b++ - *a;
	*a++ = *b++ - *a;
	*a++ = *b++ - *a;
	*a   = *b   - *a;
	a-=4;
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a++ < 0) { *--a += m; (*++a)--; }
	if (*a < 0) { *a += m; CARRY=1; }
}







inline void NFand(char *a, char *b) { 
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d4 }  // assumptions f is in d4, a1 -> I[4], we need I[2]
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (count--) { *a++ &= *b++; };
}



inline void NFor(char *a, char *b) { 
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d4 }  // assumptions f is in d4, a1 -> I[4], we need I[2]
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (count--) { *a++ |= *b++; };
}







inline void Nzero4(char *a) 
{
  *(UInt32*)a = 0;
}


inline void Nzero5(char *a)  
{
  *(UInt32*)a = 0;
  *(UInt8*)(a+4) = 0;
}

inline void Nzero3(char *a)  
{
  *(UInt16*)a = 0;
  *(UInt8*)(a+2) = 0;
}







inline void Nnot5(char *a)   
{
  char m=(char)MODE_MINUS_ONE;
  *a++=(m-*a);
  *a++=(m-*a);
  *a++=(m-*a);
  *a++=(m-*a);
  *a=(m-*a);
  CARRY = 0;
}






inline void Nneg5(char *a) {    
  char m=(char)MODE_MINUS_ONE;
  if (*a++) {(*--a)--;goto nneg51;}
  if (*a++) {(*--a)--;goto nneg52;}
  if (*a++) {(*--a)--;goto nneg53;}
  if (*a++) {(*--a)--;goto nneg54;}
  if (*a) {(*a)--;goto nneg55;}
  CARRY=0; 
  return;
  nneg51: *a++ = m-(*a);
  nneg52: *a++ = m-(*a);
  nneg53: *a++ = m-(*a);
  nneg54: *a++ = m-(*a);
  nneg55: *a = m-(*a);
  CARRY=1;
}









inline void Nsl5(char *a)   
{
  char *b;

  a+=5;
  b=a-1;
  *--a=*--b;
  *--a=*--b;
  *--a=*--b;
  *--a=*--b;
  (*--a)=0;
}




inline void Nsr5(char *a)   
{
  char *b;
  
  b=a+1;
  if (*a) HST|=SB;
  *a++ = *b++;
  *a++ = *b++;
  *a++ = *b++;
  *a++ = *b;
  (*a)=0;
}









inline void Nbit0_4(char *a) 
{  
	char b;
	b=0;
	asm { move.b (a1)+,d4 }  // assumptions b is in d4, a1 -> I[4], we need I[4], will increment to I[5] to avoid PCPLUSONE

	a[b>>2]&=~(1<<(b&3));
}

inline void Nbit1_4(char *a) 
{  
	char b;
	b=0;
	asm { move.b (a1)+,d4 }  // assumptions b is in d4, a1 -> I[4], we need I[4], will increment to I[5] to avoid PCPLUSONE

	a[b>>2]|=1<<(b&3);
}

inline void Nbit0_2(char *a) 
{  
	char b;
	b=0;
	asm { move.b -1(a1),d4 }  // assumptions b is in d4, a1 -> I[3], we need I[2]

	a[b>>2]&=~(1<<(b&3));
}

inline void Nbit1_2(char *a) 
{  
	char b;
	b=0;
	asm { move.b -1(a1),d4 }  // assumptions b is in d4, a1 -> I[3], we need I[2]

	a[b>>2]|=1<<(b&3);
}







inline void Nsrb(char *a, int s) { 

  if ((*a)&1) HST|=SB;
  while (--s) {
    (*a)>>=1;
    if (a[1]&1) (*a)|=8;
    a++;
  }
  *a>>=1;      
}


inline void NFsrb(char *a) 
{ 
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d5 }  // assumptions f is in d5, a1 -> I[5], we need I[3]

  	count=F_l[f];
	a+=F_s[f];

	if ((*a)&1) HST|=SB;
	while (--count) {
		(*a)>>=1;
		if (a[1]&1) (*a)|=8;
		a++;
	}
	*a>>=1;      
}










inline void Ndbl5(char *a)  
{
	char m=(char)MODE;
	CARRY=0;
	*(UInt32*)a<<=1;
	*(UInt8*)(a+4)<<=1;
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a++ >= m) { *--a -= m; (*++a)++; }
	if (*a >= m) { *a -= m; CARRY=1; }
}







/***********************************************************************************
 *
 *  Nibble Test primitives
 *
 *
 *
 *
 ***********************************************************************************/



inline void Tbit0_4(char *a)
{
	char b;
	b=0;
	asm { move.b (a1)+,d4 }  // assumptions b is in d4, a1 -> I[4], we need I[4], will increment to I[5] to avoid PCPLUSONE

	if (a[b>>2]&(1<<(b&3)))
		CARRY = 0;
	else
		CARRY = 1;
}

inline void Tbit1_4(char *a)
{
	char b;
	b=0;
	asm { move.b (a1)+,d4 }  // assumptions b is in d4, a1 -> I[4], we need I[4], will increment to I[5] to avoid PCPLUSONE

	if (a[b>>2]&(1<<(b&3)))
		CARRY = 1;
	else
		CARRY = 0;
}

inline void Tbit0_2(char *a)
{
	char b;
	b=0;
	asm { move.b -1(a1),d4 }  // assumptions b is in d4, a1 -> I[3], we need I[2]

	if (a[b>>2]&(1<<(b&3)))
		CARRY = 0;
	else
		CARRY = 1;
}

inline void Tbit1_2(char *a)
{
	char b;
	b=0;
	asm { move.b -1(a1),d4 }  // assumptions b is in d4, a1 -> I[3], we need I[2]

	if (a[b>>2]&(1<<(b&3)))
		CARRY = 1;
	else
		CARRY = 0;
}





inline void TFe(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d5 }  // assumptions f is in d5, a1 -> I[3], we need I[1]
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	do { if (*a++!=*b++) { CARRY = 0; return; } } while (--count);
	CARRY = 1;
}



inline void TFne(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d5 }  // assumptions f is in d5, a1 -> I[3], we need I[1]
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	do { if (*a++!=*b++) { CARRY=1; return; } } while (--count);
	CARRY = 0;
}




inline void Te5(char *a, char *b) 
{
  if (*(UInt32*)a != *(UInt32*)b) { CARRY = 0; return; }
  CARRY = (*(UInt8*)(a+4) == *(UInt8*)(b+4));
}



inline void Tne5(char *a, char *b) 
{
  if (*(UInt32*)a != *(UInt32*)b) { CARRY = 1; return; }
  CARRY = (*(UInt8*)(a+4) != *(UInt8*)(b+4));
}








inline void TFz(char *a) 
{
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d5 }  // assumptions f is in d5, a1 -> I[3], we need I[1]

  	count=F_l[f];
	a+=F_s[f];

	do { if (*a++ != 0) { CARRY = 0; return; } } while (--count);
	CARRY = 1;
}


inline void Tz5(char *a) 
{
  if (*(UInt32*)a != 0) { CARRY = 0; return; }
  CARRY = (*(UInt8*)(a+4) == 0);
}







inline void TFnz(char *a) 
{
	register int count;
	register int f;
	
	f=0;

	asm { move.b -2(a1),d5 }  // assumptions f is in d5, a1 -> I[3], we need I[1]

  	count=F_l[f];
	a+=F_s[f];

	do { if (*a++ != 0) { CARRY=1; return; } } while (--count);
	CARRY = 0;
}


inline void Tnz5(char *a) 
{
  if (*(UInt32*)a != 0) { CARRY = 1; return; }
  CARRY = (*(UInt8*)(a+4) != 0);
}







inline void TFa(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm {                      // assumptions f is in d5, a1 -> I[3], we need I[1]
		move.b    -2(a1),d5
		andi.w    #0x7,d5 
	}
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (--count) if (a[count]!=b[count]) break;
	CARRY = (a[count]>b[count]);
}


inline void Ta5(char *a, char *b) 
{
  a+=5;
  b+=5;
  if (*--a != *--b) goto ta5exit;
  if (*--a != *--b) goto ta5exit;
  if (*--a != *--b) goto ta5exit;
  if (*--a != *--b) goto ta5exit;
  a--;b--;
  ta5exit:CARRY=(*a>*b);return;
}





inline void TFb(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm {                      // assumptions f is in d5, a1 -> I[3], we need I[1]
		move.b    -2(a1),d5
		andi.w    #0x7,d5 
	}
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (--count) if (a[count]!=b[count]) break;
	CARRY = (a[count]<b[count]);
}


inline void Tb5(char *a, char *b)
{
  a+=5;
  b+=5;
  if (*--a != *--b) goto tb5exit;
  if (*--a != *--b) goto tb5exit;
  if (*--a != *--b) goto tb5exit;
  if (*--a != *--b) goto tb5exit;
  a--;b--;
  tb5exit:CARRY=(*a<*b);return;
}






inline void TFae(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm {                      // assumptions f is in d5, a1 -> I[3], we need I[1]
		move.b    -2(a1),d5
		andi.w    #0x7,d5 
	}
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (--count) if (a[count]!=b[count]) break;
	CARRY = (a[count]>=b[count]);
}


inline void Tae5(char *a, char *b) 
{
  a+=5;
  b+=5;
  if (*--a != *--b) goto tae5exit;
  if (*--a != *--b) goto tae5exit;
  if (*--a != *--b) goto tae5exit;
  if (*--a != *--b) goto tae5exit;
  a--;b--;
  tae5exit:CARRY=(*a>=*b);return;
}






inline void TFbe(char *a, char *b) 
{
	register int offset;
	register int count;
	register int f;
	
	f=0;

	asm {                      // assumptions f is in d5, a1 -> I[3], we need I[1]
		move.b    -2(a1),d5
		andi.w    #0x7,d5 
	}
	
  	offset=F_s[f];
  	count=F_l[f];
	a+=offset;
	b+=offset;

	while (--count) if (a[count]!=b[count]) break;
	CARRY = (a[count]<=b[count]);
}


inline void Tbe5(char *a, char *b) 
{
  a+=5;
  b+=5;
  if (*--a != *--b) goto tbe5exit;
  if (*--a != *--b) goto tbe5exit;
  if (*--a != *--b) goto tbe5exit;
  if (*--a != *--b) goto tbe5exit;
  a--;b--;
  tbe5exit:CARRY=(*a<=*b);return;
}








/***********************************************************************************
 *
 *  Bus Primitives
 *
 *
 *
 *
 ***********************************************************************************/


/* Memory Controllers

        48SX:            48GX              49G
        -----            ----              ---
HDW     0 IO             0 IO              0 IO
RAM     1 RAM            1 RAM             1 RAM
CE2     2 PORT 1         2 bank switcher   2 bank switcher
CE1     3 PORT 2         3 PORT 1          3 PORT 1
NCE3    4 UNUSED         4 PORT 2          4 PORT 2
ROM     5 ROM            5 ROM             5 ROM

*/




inline void config(void) 
{
	long d;
	
	d=Npack2CPageASM();   // keep to 0x1000 nibble boundary
	
	// Controllers are configured in the following order:
	// HDW, RAM, CE1, CE2, NCE3
	//
	if (!mcIOAddrConfigured) 
	{ 
		mcIOAddrConfigured = true;
		mcIOBase = Npack5CASM()&0xFFFC0;			// keep to 64 nibble boundary
		mcIOCeil = mcIOBase + 0x40;			
		return;
	}
	
	if (!mcRAMAddrConfigured)
	{
		if (!mcRAMSizeConfigured)	
		{
			mcRAMSizeConfigured = true;
			mcRAMSize = 0x100000-d;
			return;
		}
		mcRAMAddrConfigured = true;
		mcRAMBase = d;	
		mcRAMCeil = d+mcRAMSize;
		mcRAMDelta = (long)ram-d;
		return;
	}
	
	if (!mcCE1AddrConfigured)
	{
		if (!mcCE1SizeConfigured)	
		{
			mcCE1SizeConfigured = true;
			mcCE1Size = 0x100000-d;
			return;
		}
		mcCE1AddrConfigured = true;
		mcCE1Base = d;	
		mcCE1Ceil = d+mcCE1Size;
		mcCE1Delta = (long)ce1addr-d;
		return;
	}
	
	if (!mcCE2AddrConfigured)
	{
		if (!mcCE2SizeConfigured)	
		{
			mcCE2SizeConfigured = true;
			mcCE2Size = 0x100000-d;
			return;
		}
		mcCE2AddrConfigured = true;
		mcCE2Base = d;	
		mcCE2Ceil = d+mcCE2Size;
		mcCE2Delta = (long)ce2addr-d;
		return;
	}
	
	if (!mcNCE3AddrConfigured)
	{
		if (!mcNCE3SizeConfigured)	
		{
			mcNCE3SizeConfigured = true;
			mcNCE3Size = 0x100000-d;
			return;
		}
		mcNCE3AddrConfigured = true;
		mcNCE3Base = d;	
		mcNCE3Ceil = d+mcNCE3Size;
		mcNCE3Delta = (long)nce3addr-d;
		return;
	}
}


//#define HDW_UNCONFIG_ALARM

inline void unconfig(void) 
{
	long d;

	d=Npack2CPageASM();   // keep to 0x1000 nibble boundary
	
	// Controllers are un-configured in the following order:
	// HDW, RAM, CE2, CE1, NCE3
	//
	// To speed things up, we're going to violate this rule and leave
	// out the ability to unconfigure the HDW controller as it'll probably 
	// never be unconfiged through anything but a reset. I hope this works...
	//

#ifdef HDW_UNCONFIG_ALARM
	if ((Npack5CASM()&0xFFFC0)==mcIOBase)
	{
		ErrorPrint("ALERT: IO unconfigured outside of RESET",true);
		mcIOAddrConfigured=false;
		mcIOCeil=0;
		return;
	}
#endif

	if ((mcRAMAddrConfigured)&&(d==mcRAMBase))
	{
		mcRAMAddrConfigured=false;
		mcRAMSizeConfigured=false;
		mcRAMCeil = 0;	// Setting the ceiling to zero will prevent the address decoders
		return; 		// from selecting this controller. Leave the base untouched
	}
		
	if ((mcCE2AddrConfigured)&&(d==mcCE2Base))
	{
		mcCE2AddrConfigured=false;
		mcCE2SizeConfigured=false;
		mcCE2Ceil = 0;	// Setting the ceiling to zero will prevent the address decoders
		return; 		// from selecting this controller. Leave the base untouched
	}
		
	if ((mcCE1AddrConfigured)&&(d==mcCE1Base))
	{
		mcCE1AddrConfigured=false;
		mcCE1SizeConfigured=false;
		mcCE1Ceil = 0;	// Setting the ceiling to zero will prevent the address decoders
		return; 		// from selecting this controller. Leave the base untouched
	}
		
	if ((mcNCE3AddrConfigured)&&(d==mcNCE3Base))
	{
		mcNCE3AddrConfigured=false;
		mcNCE3SizeConfigured=false;
		mcNCE3Ceil = 0;	// Setting the ceiling to zero will prevent the address decoders
		return; 		// from selecting this controller. Leave the base untouched
	}
}


inline void reset() 
{	
	mcIOAddrConfigured=false;
	mcIOBase=0x100000;
	mcIOCeil=0;

	mcRAMSizeConfigured=false;
	mcRAMAddrConfigured=false;
	mcRAMBase=0x100000;
	mcRAMCeil=0;
	mcRAMSize=0;
	mcRAMDelta=0;

	mcCE1SizeConfigured=false;
	mcCE1AddrConfigured=false;
	mcCE1Base=0x100000;
	mcCE1Ceil=0;
	mcCE1Size=0;
	mcCE1Delta=0;

	mcCE2SizeConfigured=false;
	mcCE2AddrConfigured=false;
	mcCE2Base=0x100000;
	mcCE2Ceil=0;
	mcCE2Size=0;
	mcCE2Delta=0;

	mcNCE3SizeConfigured=false;
	mcNCE3AddrConfigured=false;
	mcNCE3Base=0x100000;
	mcNCE3Ceil=0;
	mcNCE3Size=0;
	mcNCE3Delta=0;
}



inline void c_eq_id(void) 
{
	long val;
	
	// Controllers are configured in the following order:
	// HDW, RAM, CE1, CE2, NCE3
	//

	if (!mcIOAddrConfigured) 
	{
		val=mcIOBase|0x19;
		goto repack;
	}
	
	if (!mcRAMAddrConfigured)  
	{
		if (!mcRAMSizeConfigured)  
		{
			val=(0x100000-mcRAMSize)|0x03;
			goto repack;
		}
		val=mcRAMBase|0xF4;
		goto repack;
	}

	if (!mcCE1AddrConfigured)
	{
		if (!mcCE1SizeConfigured)
		{
			val=(0x100000-mcCE1Size)|0x05;
			goto repack;
		}
		val=mcCE1Base|0xF6;
		goto repack;
	}
	
	
	if (!mcCE2AddrConfigured)  
	{
		if (!mcCE2SizeConfigured)
		{
			val=(0x100000-mcCE2Size)|0x07;
			goto repack;
		}
		val=mcCE2Base|0xF8;
		goto repack;
	}
	
	if (!mcNCE3AddrConfigured) 
	{
		if (!mcNCE3SizeConfigured) 
		{
			val=(0x100000-mcNCE3Size)|0x01;
			goto repack;
		}
		val=mcNCE3Base|0xF2;
		goto repack;
	}
	
	C[0]=0;
	C[1]=0;
	C[2]=0;
	C[4]=0;
	C[5]=0;
	return;
	
repack:
	Nunpack5(C,val);
}


inline long rstkpop() {
  long r;
  int t;

  t = rstkp;
  t--;
  t&=7;
  rstkp = t;
  r = rstk[t];
  rstk[t] = 0;

  return r;
}


inline void rstkpop_to_a1() 
{
	asm
	{
		move.w    rstkp,d1   // t=rstkp
		subq.w    #1,d1      // t--
		andi.w    #0x7,d1    // t&=7
		move.w    d1,rstkp   // rstkp=t
		lsl.w     #2,d1
		lea       rstk,a0
		move.l    (a0,d1.w),d0  // r(a1)=rstk[t];
		clr.l     (a0,d1.w)     // rstk[t]=0;
		move.l    d0,d3			//temporarily restore pc
		
								// d0 now contains the stack address d
		
p_ram:	cmp.l     mcRAMBase,d0
		blt.s     p_ce2
		cmp.l     mcRAMCeil,d0
		bge.s     p_ce2
		movea.l   mcRAMDelta,a1
		lea       (a1,d0.l),a1
	}
	return;
	asm
	{
p_ce2:	cmp.l     mcCE2Base,d0
		blt.s     p_ce1
		cmp.l     mcCE2Ceil,d0
		bge.s     p_ce1
		movea.l   mcCE2Delta,a1
		lea       (a1,d0.l),a1
	}
	return;
	asm
	{
p_ce1:	cmp.l     mcCE1Base,d0
		blt.s     p_rom
		cmp.l     mcCE1Ceil,d0
		bge.s     p_rom
		movea.l   mcCE1Delta,a1
		lea       (a1,d0.l),a1
	}
	asm
	{
p_rom:	tst.b     status49GActive
		beq.s     p_rom4
		andi.l    #0x7ffff,d0
		cmpi.l    #262144,d0
		bge.s     p_rom2
  		movea.l   flashRomLo,a1
		lea	      (a1,d0.l),a1
		bra.s     p_rom5
p_rom2: movea.l   flashRomHi,a1
		lea       (a1,d0.l),a1
		bra.s     p_rom5
p_rom4: movea.l   rom,a1
		lea       (a1,d0.l),a1
	}
p_rom5:
	return;
}



inline void rstkpush(long d) {
  int t;

  t = rstkp;   
  rstk[t] = d;
  t++;
  t &= 7;
  rstkp = t;
}


inline void rstkpush_pc0( ) // This function leaves pc untouched
{
	asm
	{
		move.w    rstkp,d4  // d4 = rstkp
		move.w    d4,d0
		lsl.w     #2,d4     
		lea       rstk,a0
		move.l    d3,(a0,d4.w)  // rstk[d4] = pc
		addq.w    #1,d0         // d0++
		andi.w    #0x7,d0       // d0&=7
		move.w    d0,rstkp      // rstkp = d0
	}
}

inline void rstkpush_pc4( )  // This function leaves pc untouched
{
	asm
	{
		move.w    rstkp,d4  // d4 = rstkp
		move.w    d4,d0 
		move.l    d3,d1     // d1=pc
		addq.l    #4,d1     // d1 = d1 + 4
		lsl.w     #2,d4     
		lea       rstk,a0
		move.l    d1,(a0,d4.w)  // rstk[d4] = pc
		addq.w    #1,d0         // d0++
		andi.w    #0x7,d0       // d0&=7
		move.w    d0,rstkp      // rstkp = d0
	}
}

inline void rstkpush_pc6( )   // This function leaves pc untouched
{
	asm
	{
		move.w    rstkp,d4  // d4 = rstkp
		move.w    d4,d0
		move.l    d3,d1     // d1=pc
		addq.l    #6,d1     // d1 = d1 + 6
		lsl.w     #2,d4     
		lea       rstk,a0
		move.l    d1,(a0,d4.w)  // rstk[d4] = pc
		addq.w    #1,d0         // d0++
		andi.w    #0x7,d0       // d0&=7
		move.w    d0,rstkp      // rstkp = d0
	}
}

inline void rstkpush_pc7( )   // This function increases pc by 7
{
	asm
	{
		move.w    rstkp,d4  // d4 = rstkp
		move.w    d4,d0
		addq.l    #7,d3     // pc = pc + 7
		lsl.w     #2,d4     
		lea       rstk,a0
		move.l    d3,(a0,d4.w)  // rstk[d4] = pc
		addq.w    #1,d0         // d0++
		andi.w    #0x7,d0       // d0&=7
		move.w    d0,rstkp      // rstkp = d0
	}
}

inline void PCINTERRUPT()  // This function is sets pc=0xf, and register a1 to correct mem
{
	asm
	{
			moveq 		#15,d3
			tst.l		mcRAMBase
			beq.s		pciskip1
			tst.b		status49GActive
			beq.s		pciskip3
			move.l		flashRomLo,a1
			bra.s		pciskip2
pciskip3:	move.l 		rom,a1
			bra.s		pciskip2 
pciskip1:	move.l		ram,a1
pciskip2:	addq.l 		#8,a1 
			addq.l 		#7,a1  
			move.l 		a1,pc_save
	}
}


inline void PCRESET()  // This function sets pc=0, and register a1 to correct mem
{
	asm
	{
			moveq 		#0,d3
			move.l 		rom,a1
			move.l 		a1,pc_save
	}
}



/***********************************************************************************
 *
 *  Memory Primitives
 *
 *
 *
 *
 ***********************************************************************************/


inline char* GetRealAddressNoIO(long d)
{		
  	if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
	{
		return d+(char*)mcRAMDelta;
	}
	else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
	{
		return d+(char*)mcCE2Delta;
	}
	else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
	{
		return d+(char*)mcCE1Delta;
	}
  	else   //ROM
  	{
  		if (status49GActive)
  		{
  			d&=0x7ffff;   // 49 ROM is mirrored
  			if (d<0x40000)  // lo bank
  				return flashRomLo+d;
  			else			// hi bank
  				return flashRomHi+d;
  		}
  		else
			return rom+d;
	}
}


inline void ResetPCinRegA1(long d) // assumption: d is in d3
{
  	if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
	{
		asm
		{
			movea.l   mcRAMDelta,a1
			lea       (a1,d3.l),a1
		}
	}
	else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
	{
		asm
		{
			movea.l   mcCE2Delta,a1
			lea       (a1,d3.l),a1
		}
	}
	else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
	{
		asm
		{
			movea.l   mcCE1Delta,a1
			lea       (a1,d3.l),a1
		}
	}
  	else   //ROM
  	{
  		if (status49GActive)
  		{
  			asm
  			{
  				move.l 		d3,d2
  				andi.l		#0x7ffff,d2
  				cmpi.l		#262144,d2
  				bge.s		rpaskip1
  				movea.l		flashRomLo,a1
				lea			(a1,d2.l),a1
				bra.s		rpaskip2
	rpaskip1:	movea.l		flashRomHi,a1
				lea			(a1,d2.l),a1
			}
  		}
  		else
  		{
			asm
			{
				movea.l   rom,a1
				lea       (a1,d3.l),a1
			}
		}
	}
rpaskip2:
	return;
}


const unsigned int crc_table[16] =
{
	0x0000, 0x1081, 0x2102, 0x3183, 0x4204, 0x5285, 0x6306, 0x7387,
	0x8408, 0x9489, 0xA50A, 0xB58B, 0xC60C, 0xD68D, 0xE70E, 0xF78F
};


inline void CRC_calc(char nib)
{
	crc = (unsigned int)((crc>>4)^crc_table[(crc^nib)&0xf]);
}



inline void DisplayMainTouch(long d)
{	
  	asm
  	{
  	
#ifdef INCLUDE_TRACER
			move.l    a1,pc_save
			jsr       TraceDisplayHit
			move.l    pc_save,a1
#endif

			move.l    d4,d2                     // d2 = d
			
			move.w    lcdDisplayAreaWidth,d1    // if (width<0) width=-width;
			bge.s     dmtskip1                  //
			neg.w     d1                        //
			
dmtskip1:	sub.l     lcdDisplayAddressStart,d2 // d-=lcdDisplayAddressStart;

			move.w    d1,d0                     // d+=64*width
			lsl.w     #6,d0                     //
			ext.l     d0                        //
			add.l     d0,d2                     //
			
			divu      d1,d2                     // y=d/width - 64;
			sub.w     #64,d2                    //
			
			move.w    d2,d0                     // if (y<0) y=-y;
			bge.s     dmtskip2                  //
			neg.w     d0                        //
			
dmtskip2:	lea       lcdRedrawBuffer,a0        // lcdRedrawBuffer[y] = 1;
			move.b    #1,(a0,d0.w)              //
	}
}


inline void DisplayMenuTouch(long d)
{
	if ((lcdLineCount==0) || (lcdLineCount==63))
		return;
  	asm
  	{
  	
#ifdef INCLUDE_TRACER
			move.l    a1,pc_save
			jsr       TraceMenuHit
			move.l    pc_save,a1
#endif

			move.l    d4,d0
			sub.l     lcdMenuAddressStart,d0
			moveq     #34,d1
			divu      d1,d0
			add.w     lcdLineCount,d0
			addq.w    #1,d0
			lea       lcdRedrawBuffer,a0
			move.b    #1,(a0,d0.w)
	}
}




static void RomSwitch(long d)
{
	char tmp[80];
	return;
	SAVEPC
	StrPrintF(tmp,"BS read, addr=%lx",d);
	ErrorPrint(tmp,true);
	SysTaskDelay(100);
	RESTOREPC
}

static void RomSwitchWrite(long d,int s)
{
//	char tmp[80];
	SAVEPC
//	StrPrintF(tmp,"BS write, addr=%lx, s=%d",d,s);
//	ErrorPrint(tmp,true);
	//SysTaskDelay(100);
	d=d+s-1;
	d=(d>>1)&0x3f;
	flashBankHi=d&0xf;
	flashBankLo=d>>4;
//	StrPrintF(tmp,"lobank=%d,    hibank=%d",flashBankLo,flashBankHi);
//	ErrorPrint(tmp,true);
//	SysTaskDelay(5);
	flashRomHi=flashAddr[flashBankHi]-0x40000;
	flashRomLo=flashAddr[flashBankLo];
	RESTOREPC
}

static void FlashWrite(long d)
{
	char tmp[80];
	return;
	SAVEPC
	StrPrintF(tmp,"P2 write, addr=%lx",d);
	ErrorPrint(tmp,true);
	SysTaskDelay(100);
	RESTOREPC
}

static void FlashRead(long d,int s)
{
	char tmp[80];
	return;
	SAVEPC
	StrPrintF(tmp,"P2 read, addr=%lx, s=%d",d,s);
	ErrorPrint(tmp,true);
	SysTaskDelay(200);
	RESTOREPC
}




/***********************************************************************************
 *
 *  Hardware Primitives
 *
 *
 *
 *
 ***********************************************************************************/


inline void update_in() 
{ 
	int testOUT=OUT;
	int testIN=IR15X;

	if (testOUT & 0x001) testIN|=kbd_row[0];
	if (testOUT & 0x002) testIN|=kbd_row[1];
	if (testOUT & 0x004) testIN|=kbd_row[2];
	if (testOUT & 0x008) testIN|=kbd_row[3];
	if (testOUT & 0x010) testIN|=kbd_row[4];
	if (testOUT & 0x020) testIN|=kbd_row[5];
	if (testOUT & 0x040) testIN|=kbd_row[6];
	if (testOUT & 0x080) testIN|=kbd_row[7];
	if (testOUT & 0x100) testIN|=kbd_row[8];
	IN=testIN;
}


inline void update_out() 
{ 
	// Perform any necessary OUT register machinations..
}


inline EmulateBeep(void)
{
	long			frequency, duration;
	char			*n;
	SndCommandPtr	sndCommand=NULL;
	
	SAVEPC
	frequency = Npack5DASM();
	duration = Npack5CASM();
	if (optionTarget==HP48SX)
		n=GetRealAddressNoIO(0x706D2);
	else if (optionTarget==HP48GX)
		n=GetRealAddressNoIO(0x80850);
	else
		n=GetRealAddressNoIO(0x80F0F);
	CARRY=1;
	
	if (!(*n &0x8) && (frequency))
	{
		sndCommand->cmd = sndCmdFreqDurationAmp;
		sndCommand->param1=frequency;
		sndCommand->param2=(UInt16)(duration&0xFFFF);
		sndCommand->param3=(optionVolume);
		SndDoCmd(NULL,sndCommand,0);
		
		P=0;
		CARRY=0;
	}
	RESTOREPC
}


void ResetTime(void)
{
	unsigned long long 	ticks,timeoff;
	UInt32				seconds;
	int 				i;
	char				*n;
	
	seconds = TimGetSeconds();
	ticks = (unsigned long long)seconds*8192;
	ticks += TIME_OFFSET;
	timeoff = ticks + 0x4B0000;
	
	if (optionTarget==HP48SX)
		n=ram+0x52;
	else
		n=ram+0x58;
			
	for (i=0;i<13;i++)
	{
		*n++ = ticks&0xf;
		ticks>>=4;
	}
	for (i=0;i<4;i++)
	{
		*n++ = 0;
	}
	for (i=0;i<13;i++)
	{
		*n++ = timeoff&0xf;
		timeoff>>=4;
	}
	*n=0xf;
}


inline void read_io_m(long d, char *a8, int s)
{
	char c;
	UInt16 timeDelta;
	UInt16 currTimeStamp;
	
	if (d>=0x38)
	{
		currTimeStamp = *timeReg;
		if (currTimeStamp>prevTimeStamp)
		{
			timeDelta=(currTimeStamp-prevTimeStamp)/674;
			prevTimeStamp+=timeDelta*674;
		}
		else
		{
			SAVEPC
			timeDelta=((currTimeStamp+55268)-prevTimeStamp)/674;
			RESTOREPC
			prevTimeStamp+=timeDelta*674;
			if (prevTimeStamp>55268)
				prevTimeStamp-=55268;
		}
		if (timeDelta>t2)
		{
			if (ioram[0x2f]&4) { ioram[0x2f]|=8; WAKEUP=1; }
			if (ioram[0x2f]&2) { ioram[0x2f]|=8; if (!INTP) INTERRUPT("T2"); }
		}
		t2-=timeDelta;
	}
	
	do
	{
		switch (d) 
		{
			case 0x04: c = (char)(crc); break;
			case 0x05: c = (char)(crc>>4); break;
			case 0x06: c = (char)(crc>>8); break;
			case 0x07: c = (char)(crc>>12); break;
			case 0x0F: c = CARDSTATUS; break;
			case 0x15: ioram[0x11]&=0xe; c=ioram[d]; break;
			case 0x16: /**/
			case 0x17: /**/ c=3;break;
			case 0x18: /**/
			case 0x19: /* ScreenPrint("Nibbles #118/#119 read.",true); */ c=3;break;
			case 0x1B: 
			case 0x1C:                // LED CONTROL
			case 0x1D: c = 0; break;  // LED BUFFER
			case 0x20: /**/
			case 0x21: /**/
			case 0x22: /**/
			case 0x23: /**/
			case 0x24: /**/
			case 0x25: /**/
			case 0x26: /**/
			case 0x27: /**/ c=3;break;
			case 0x28: c = (char)lcdCurrentLine; break;
			case 0x29: c = (char)(lcdCurrentLine>>4); break;
			case 0x30: /**/
			case 0x31: /**/
			case 0x32: /**/
			case 0x33: /**/
			case 0x34: /**/ c = 3; break;      
			case 0x37: c = (char)(t1); break;
			case 0x38: c = (char)(t2); break;
			case 0x39: c = (char)(t2>>4); break;
			case 0x3A: c = (char)(t2>>8); break;
			case 0x3B: c = (char)(t2>>12); break;
			case 0x3C: c = (char)(t2>>16); break;
			case 0x3D: c = (char)(t2>>20); break;
			case 0x3E: c = (char)(t2>>24); break;
			case 0x3F: c = (char)(t2>>28); break;
			default: c=ioram[d];
		}
		*a8++ = (c&0xf);
		d++;
	} while (--s);
}


static void print_check(long d)
{
	char tmp[80];
	
	StrPrintF(tmp,"addr=%lx",d);
	ErrorPrint(tmp,true);
	SysTaskDelay(100);
}

/***********************************************************************
 *
 * FUNCTION:    emulate()
 *
 * DESCRIPTION: The main emulator loop... it all happens here, baby!
 *
 * PARAMETERS:  
 *
 * RETURNED:   
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

void emulate(void) 
{
	long d,e;
	char *a8,*b8,*c8,*n8;
	UInt32 *a32,*b32,*c32;
	char m,n,o;
	int loopCount,fieldIndex;
	signed char jmp;
	EventType event;
	register long pc=0;
	register Int16 icounter=0;
	UInt32 currTickStamp;
    
#ifdef INCLUDE_TRACER
	char tmp[60];
#endif

    
#ifdef PRINT_IPS
	instCount=0;
	previousCount=0;	
	ipsLoop=20;
	lastTicks = TimGetTicks();
#endif


	timeReg=(volatile UInt16 *)TCN1;
	prevTimeStamp = *timeReg;
	prevTickStamp = TimGetTicks();
	
	icounter = saturn_speed;
	update_t1=4;
	pc=pc_reg;
	IO_TIMER_RUN=(ioram[0x2f]&1);
	
	ResetTime();  // not a1 safe
	
	ResetPCinRegA1(pc);
	SAVEPC
	
	goto mainloopstart;
	
	
	
	
o00: 		RFL(0) HST|=XM; rstkpop_to_a1(); OPEND
o01: 		RFL(1) rstkpop_to_a1(); OPEND
o02: 		RFL(2) CARRY=1; rstkpop_to_a1(); OPEND
o03: 		RFL(3) CARRY=0; rstkpop_to_a1(); OPEND

o100: 		RFL(31) a32=(UInt32*)R0; b32=(UInt32*)A; goto ncopy16finish;
o101: 		RFL(32) a32=(UInt32*)R1; b32=(UInt32*)A; goto ncopy16finish;
o102: 		RFL(33) a32=(UInt32*)R2; b32=(UInt32*)A; goto ncopy16finish;
o103: 		RFL(34) a32=(UInt32*)R3; b32=(UInt32*)A; goto ncopy16finish;
o104: 		RFL(35) a32=(UInt32*)R4; b32=(UInt32*)A; goto ncopy16finish;
o108: 		RFL(36) a32=(UInt32*)R0; b32=(UInt32*)C; goto ncopy16finish;
o109: 		RFL(37) a32=(UInt32*)R1; b32=(UInt32*)C; goto ncopy16finish;
o10A: 		RFL(38) a32=(UInt32*)R2; b32=(UInt32*)C; goto ncopy16finish;
o10B: 		RFL(39) a32=(UInt32*)R3; b32=(UInt32*)C; goto ncopy16finish;
o10C: 		RFL(40) a32=(UInt32*)R4; b32=(UInt32*)C; 
			ncopy16finish:pc+=3;*a32++=*b32++;*a32++=*b32++;*a32++=*b32++;*a32=*b32;OPEND
o110: 		RFL(41) a32=(UInt32*)A; b32=(UInt32*)R0; goto ncopy16finish;
o111: 		RFL(42) a32=(UInt32*)A; b32=(UInt32*)R1; goto ncopy16finish;
o112: 		RFL(43) a32=(UInt32*)A; b32=(UInt32*)R2; goto ncopy16finish;
o113: 		RFL(44) a32=(UInt32*)A; b32=(UInt32*)R3; goto ncopy16finish;
o114: 		RFL(45) a32=(UInt32*)A; b32=(UInt32*)R4; goto ncopy16finish;
o118: 		RFL(46) a32=(UInt32*)C; b32=(UInt32*)R0; goto ncopy16finish;
o119: 		RFL(47) a32=(UInt32*)C; b32=(UInt32*)R1; goto ncopy16finish;
o11A: 		RFL(48) a32=(UInt32*)C; b32=(UInt32*)R2; goto ncopy16finish;
o11B: 		RFL(49) a32=(UInt32*)C; b32=(UInt32*)R3; goto ncopy16finish;
o11C: 		RFL(50) a32=(UInt32*)C; b32=(UInt32*)R4; goto ncopy16finish;
			

o120: 		RFL(51) a32=(UInt32*)A; b32=(UInt32*)R0; goto nxchg16finish;
o121: 		RFL(52) a32=(UInt32*)A; b32=(UInt32*)R1; goto nxchg16finish;
o122: 		RFL(53) a32=(UInt32*)A; b32=(UInt32*)R2; goto nxchg16finish;
o123: 		RFL(54) a32=(UInt32*)A; b32=(UInt32*)R3; goto nxchg16finish;
o124: 		RFL(55) a32=(UInt32*)A; b32=(UInt32*)R4;
			nxchg16finish:pc+=3;c32=(UInt32*)X;
				*c32 = *b32;*b32++ = *a32;*a32++ = *c32++;*c32 = *b32;*b32++ = *a32;*a32++ = *c32++;
				*c32 = *b32;*b32++ = *a32;*a32++ = *c32++;*c32 = *b32;*b32 = *a32;*a32 = *c32;OPEND
o128: 		RFL(56) a32=(UInt32*)C; b32=(UInt32*)R0; goto nxchg16finish;
o129: 		RFL(57) a32=(UInt32*)C; b32=(UInt32*)R1; goto nxchg16finish;
o12A: 		RFL(58) a32=(UInt32*)C; b32=(UInt32*)R2; goto nxchg16finish;
o12B: 		RFL(59) a32=(UInt32*)C; b32=(UInt32*)R3; goto nxchg16finish;
o12C: 		RFL(60) a32=(UInt32*)C; b32=(UInt32*)R4; goto nxchg16finish;

o130: 		RFL(61) pc+=3; d0reg=Npack5AASM(); OPEND
o131: 		RFL(62) pc+=3; d1reg=Npack5AASM(); OPEND
o132: 		RFL(63) pc+=3; d = d0reg; d0reg=Npack5AASM(); Nunpack5(A, d); OPEND
o133: 		RFL(64) pc+=3; d = d1reg; d1reg=Npack5AASM(); Nunpack5(A, d); OPEND
o134: 		RFL(65) pc+=3; d0reg=Npack5CASM(); OPEND
o135: 		RFL(66) pc+=3; d1reg=Npack5CASM(); OPEND
o136: 		RFL(67) pc+=3; d = d0reg; d0reg=Npack5CASM(); Nunpack5(C, d); OPEND
o137: 		RFL(68) pc+=3; d = d1reg; d1reg=Npack5CASM(); Nunpack5(C, d); OPEND
o138: 		RFL(69) pc+=3; *((UInt16*)(&d0reg)+1)=Npack4AASM(); OPEND
o139: 		RFL(70) pc+=3; *((UInt16*)(&d1reg)+1)=Npack4AASM(); OPEND
o13A: 		RFL(71) pc+=3; d = d0reg; *((UInt16*)(&d0reg)+1)=Npack4AASM(); Nunpack4(A, d); OPEND
o13B: 		RFL(72) pc+=3; d = d1reg; *((UInt16*)(&d1reg)+1)=Npack4AASM(); Nunpack4(A, d); OPEND
o13C: 		RFL(73) pc+=3; *((UInt16*)(&d0reg)+1)=Npack4CASM(); OPEND
o13D: 		RFL(74) pc+=3; *((UInt16*)(&d1reg)+1)=Npack4CASM(); OPEND
o13E: 		RFL(75) pc+=3; d = d0reg; *((UInt16*)(&d0reg)+1)=Npack4CASM(); Nunpack4(C, d); OPEND
o13F: 		RFL(76) pc+=3; d = d1reg; *((UInt16*)(&d1reg)+1)=Npack4CASM(); Nunpack4(C, d); OPEND



mainloopstart:
	while (1) 
	{
#ifdef INCLUDE_PROFILER
		timeStart=*timeReg;
#endif

		



o_fetch:
    	icounter--;
    	

#ifdef INCLUDE_TRACER
		SAVEPC
		if (collecting) CollectTrace(pc);
		RESTOREPC
#endif


//	if ((pc==0x0a00e) || (pc==0x09B73) || (pc==0x05944) ||
//		(pc==0x0597e) || (pc==0x05981))
//	{
//		SAVEPC
//		print_check(pc);
//		RESTOREPC
//	}






// Master three-level 16K jump table inserted here...
//
#include "JUMPTABLE.H"
//
// End Master jump table


	
	
	
decode0E:
	asm 
	{                        // a1 -> I[3]
		move.b	(a1)+,d0     // a1 -> I[4]
		ext.w   d0
		add.w	d0,d0
		add.w	d0,d0
		jmp		2(pc,d0)
jstart5:jmp		o0Ef0
		jmp		o0Ef1
		jmp		o0Ef2
		jmp		o0Ef3
		jmp		o0Ef4
		jmp		o0Ef5
		jmp		o0Ef6
		jmp		o0Ef7
		jmp		o0Ef8
		jmp		o0Ef9
		jmp		o0EfA
		jmp		o0EfB
		jmp		o0EfC
		jmp		o0EfD
		jmp		o0EfE
		jmp		o0EfF
	}


o04: 		RFL(4) pc+=2; PCMINUSONE MODE=16; MODE_MINUS_ONE=15; OPEND
o05: 		RFL(5) pc+=2; PCMINUSONE MODE=10; MODE_MINUS_ONE=9; OPEND
o06: 		RFL(6) pc+=2; PCMINUSONE rstkpush(Npack5CASM()); OPEND
o07: 		RFL(7) pc+=2; PCMINUSONE Nunpack5(C, rstkpop()); OPEND
o08: 		RFL(8) pc+=2; PCMINUSONE Nzero3(ST); OPEND
o09: 		RFL(9) pc+=2; PCMINUSONE Ncopy3(C, ST); OPEND
o0A: 		RFL(10) pc+=2; PCMINUSONE Ncopy3(ST, C); OPEND
o0B: 		RFL(11) pc+=2; PCMINUSONE Nxchg3(C, ST); OPEND
o0C: 		RFL(12) pc+=2; PCMINUSONE P++; if (P>15) { P=0; CARRY=1; } else CARRY=0; PCHANGED OPEND
o0D: 		RFL(13) pc+=2; PCMINUSONE P--; if (P<0) { P=15; CARRY=1; } else CARRY=0; PCHANGED OPEND
o0Ef0: 		RFL(14) NFand(A,B); pc+=4; OPEND
o0Ef1: 		RFL(15) NFand(B,C); pc+=4; OPEND
o0Ef2: 		RFL(16) NFand(C,A); pc+=4; OPEND
o0Ef3: 		RFL(17) NFand(D,C); pc+=4; OPEND
o0Ef4: 		RFL(18) NFand(B,A); pc+=4; OPEND
o0Ef5: 		RFL(19) NFand(C,B); pc+=4; OPEND
o0Ef6: 		RFL(20) NFand(A,C); pc+=4; OPEND
o0Ef7: 		RFL(21) NFand(C,D); pc+=4; OPEND
o0Ef8: 		RFL(22) NFor(A,B); pc+=4; OPEND
o0Ef9: 		RFL(23) NFor(B,C); pc+=4; OPEND
o0EfA: 		RFL(24) NFor(C,A); pc+=4; OPEND
o0EfB: 		RFL(25) NFor(D,C); pc+=4; OPEND
o0EfC: 		RFL(26) NFor(B,A); pc+=4; OPEND
o0EfD: 		RFL(27) NFor(C,B); pc+=4; OPEND
o0EfE: 		RFL(28) NFor(A,C); pc+=4; OPEND
o0EfF: 		RFL(29) NFor(C,D); pc+=4; OPEND
o0F: 		RFL(30) if (INTD&&INTE) {INTD=0;PCINTERRUPT(); OPEND} else {INTP=0;rstkpop_to_a1(); OPEND}





o140: 		RFL(77) a8=A;d=d0reg; goto write5;
o141: 		RFL(78) a8=A;d=d1reg; goto write5;
o144: 		RFL(79) a8=C;d=d0reg; goto write5;
o145: 		RFL(80) a8=C;d=d1reg;
			write5:pc+=3;
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
			  	if (write_io_multiple(d-mcIOBase,a8,5)) INTERRUPT("IO")
			  	OPEND
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
				if ((d<lcdDisplayAddressEnd)&&(d>=lcdDisplayAddressStart2))
					DisplayMainTouch(d);
				else if ((d<lcdMenuAddressEnd)&&(d>=lcdMenuAddressStart))
					DisplayMenuTouch(d);
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitchWrite(d,5);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashWrite(d);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
				n8=rom+d;
			}
			*n8++=*a8++;
			*n8++=*a8++;
			*n8++=*a8++;
			*n8++=*a8++;
			*n8=*a8;
			OPEND

o148: 		RFL(81) a8=A;d=d0reg; goto write2;
o149: 		RFL(82) a8=A;d=d1reg; goto write2;
o14C: 		RFL(83) a8=C;d=d0reg; goto write2;
o14D: 		RFL(84) a8=C;d=d1reg;
			write2:pc+=3;
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
			  	if (write_io_multiple(d-mcIOBase,a8,2)) INTERRUPT("IO")
		 		OPEND
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
				if ((d<lcdDisplayAddressEnd)&&(d>=lcdDisplayAddressStart2))
					DisplayMainTouch(d);
				else if ((d<lcdMenuAddressEnd)&&(d>=lcdMenuAddressStart))
					DisplayMenuTouch(d);
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitchWrite(d,2);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashWrite(d);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
				n8=rom+d;
			}
			*n8++=*a8++;
			*n8=*a8;
			OPEND

o150a: 		RFL(85) a8=A;d=d0reg; goto writef;
o151a: 		RFL(86) a8=A;d=d1reg; goto writef;
o154a: 		RFL(87) a8=C;d=d0reg; goto writef;
o155a: 		RFL(88) a8=C;d=d1reg;
			writef:fieldIndex=0;
			asm { move.b (a1)+,d6 }  // assumptions fieldIndex is in d6, a1 -> I[3], we need I[3]
			pc+=4;
			//PCPLUSONE - took care of this with the postincrement asm above
			loopCount=F_l[fieldIndex];
			a8+=F_s[fieldIndex];
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
			  	if (write_io_multiple(d-mcIOBase,a8,loopCount)) INTERRUPT("IO")
		 		OPEND
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
				if ((d<lcdDisplayAddressEnd)&&(d>=lcdDisplayAddressStart2))
					DisplayMainTouch(d);
				else if ((d<lcdMenuAddressEnd)&&(d>=lcdMenuAddressStart))
					DisplayMenuTouch(d);
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitchWrite(d,loopCount);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashWrite(d);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
				n8=rom+d;
			}
			do { *n8++=*a8++; } while (--loopCount);
			OPEND

o158x: 		RFL(89) a8=A;d=d0reg; goto writex;
o159x: 		RFL(90) a8=A;d=d1reg; goto writex;
o15Cx: 		RFL(91) a8=C;d=d0reg; goto writex;
o15Dx: 		RFL(92) a8=C;d=d1reg;
			writex:loopCount=0;
			asm { move.b (a1)+,d5 }  // assumptions loopCount is in d5, a1 -> I[3], we need I[3]
			loopCount++;
			pc+=4;
			//PCPLUSONE - took care of this with the postincrement asm above
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
			  	if (write_io_multiple(d-mcIOBase,a8,loopCount)) INTERRUPT("IO")
		 		OPEND
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
				if ((d<lcdDisplayAddressEnd)&&(d>=lcdDisplayAddressStart2))
					DisplayMainTouch(d);
				else if ((d<lcdMenuAddressEnd)&&(d>=lcdMenuAddressStart))
					DisplayMenuTouch(d);
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitchWrite(d,loopCount);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashWrite(d);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
				n8=rom+d;
			}
			do { *n8++=*a8++; } while (--loopCount);
			OPEND

o142: 		RFL(93) a8=A;d=d0reg; goto read5;
o143: 		RFL(94) a8=A;d=d1reg; goto read5;
o146: 		RFL(95) a8=C;d=d0reg; goto read5;
o147: 		RFL(96) a8=C;d=d1reg;
			read5:pc+=3;
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
				d-=mcIOBase;
				loopCount=5;
				goto read_io;
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitch(d);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashRead(d,5);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
  				if (status49GActive)
  				{
  					e=d&0x7ffff;
  					if (e<0x40000)  // lo bank
  						n8=flashRomLo+e;
  					else			// hi bank
  						n8=flashRomHi+e;
  				}
  				else
					n8=rom+d;
			}
			*a8++=*n8++;
			*a8++=*n8++;
			*a8++=*n8++;
			*a8++=*n8++;
			*a8=*n8;
			OPEND

o14A: 		RFL(97) a8=A;d=d0reg; goto read2;
o14B: 		RFL(98) a8=A;d=d1reg; goto read2;
o14E: 		RFL(99) a8=C;d=d0reg; goto read2;
o14F: 		RFL(100) a8=C;d=d1reg;
			read2:pc+=3;
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
				d-=mcIOBase;
				loopCount=2;
				goto read_io;
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitch(d);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashRead(d,2);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
  				if (status49GActive)
  				{
  					e=d&0x7ffff;
  					if (e<0x40000)  // lo bank
  						n8=flashRomLo+e;
  					else			// hi bank
  						n8=flashRomHi+e;
  				}
  				else
					n8=rom+d;
			}
			*a8++=*n8++;
			*a8=*n8;
			OPEND

o152a: 		RFL(101) a8=A;d=d0reg; goto readf;
o153a: 		RFL(102) a8=A;d=d1reg; goto readf;
o156a: 		RFL(103) a8=C;d=d0reg; goto readf;
o157a: 		RFL(104) a8=C;d=d1reg;
			readf:fieldIndex=0;
			asm { move.b (a1)+,d6 }  // assumptions fieldIndex is in d6, a1 -> I[3], we need I[3]
			pc+=4;
			//PCPLUSONE - took care of this with the postincrement asm above
			loopCount=F_l[fieldIndex];
			a8+=F_s[fieldIndex];
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
				d-=mcIOBase;
				goto read_io;
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitch(d);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashRead(d,loopCount);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
  				if (status49GActive)
  				{
  					e=d&0x7ffff;
  					if (e<0x40000)  // lo bank
  						n8=flashRomLo+e;
  					else			// hi bank
  						n8=flashRomHi+e;
  				}
  				else
					n8=rom+d;
			}
			do { *a8++=*n8++; } while (--loopCount);
			OPEND

o15Ax: 		RFL(105) a8=A;d=d0reg; goto readx;
o15Bx: 		RFL(106) a8=A;d=d1reg; goto readx;
o15Ex: 		RFL(107) a8=C;d=d0reg; goto readx;
o15Fx: 		RFL(108) a8=C;d=d1reg; 
			readx:loopCount=0;
			asm { move.b (a1)+,d5 }  // assumptions loopCount is in d5, a1 -> I[3], we need I[3]
			loopCount++;
			pc+=4;
			//PCPLUSONE - took care of this with the postincrement asm above
			if ((d<mcIOCeil)&&(d>=mcIOBase))  //IO
			{
				d-=mcIOBase;
				goto read_io;
		 	}
			else if ((d>=mcRAMBase)&&(d<mcRAMCeil))  // RAM
			{
				n8=d+(char*)mcRAMDelta;
			}
			else if ((d>=mcCE2Base)&&(d<mcCE2Ceil))
			{
				n8=d+(char*)mcCE2Delta;
			}
			else if ((d>=mcCE1Base)&&(d<mcCE1Ceil))
			{
				if (status49GActive)
				{
					RomSwitch(d);
					OPEND
				}
				else
					n8=d+(char*)mcCE1Delta;
			}
			else if ((d>=mcNCE3Base)&&(d<mcNCE3Ceil))
			{
				//FlashRead(d,loopCount);
				//OPEND;
				n8=d+(char*)mcNCE3Delta;
			}
			else   //ROM
			{
  				if (status49GActive)
  				{
  					e=d&0x7ffff;
  					if (e<0x40000)  // lo bank
  						n8=flashRomLo+e;
  					else			// hi bank
  						n8=flashRomHi+e;
  				}
  				else
					n8=rom+d;
			}
			do { *a8++=*n8++; } while (--loopCount);
			OPEND


read_io:	read_io_m(d,a8,loopCount);
			OPEND

o16x: 		RFL(109)  
			//d0reg+=I[2]+1; 
			asm {
				move.b 		-1(a1),d0  // assumptions: a1 -> I[3], we need I[2]
				ext.w		d0
				addq.w		#1,d0
				ext.l		d0
				add.l		d0,d0reg
			}
			if (d0reg>0xfffff) { d0reg&=0xfffff; CARRY=1; } else CARRY=0; pc+=3; OPEND
			
o17x: 		RFL(110) 
			//d1reg+=I[2]+1; 
			asm {
				move.b 		-1(a1),d0  // assumptions: a1 -> I[3], we need I[2]
				ext.w		d0
				addq.w		#1,d0
				ext.l		d0
				add.l		d0,d1reg
			}
			if (d1reg>0xfffff) { d1reg&=0xfffff; CARRY=1; } else CARRY=0; pc+=3; OPEND
			
o18x: 		RFL(111) 
			//d0reg-=I[2]+1; 
			asm {
				move.b 		-1(a1),d0  // assumptions: a1 -> I[3], we need I[2]
				ext.w		d0
				addq.w		#1,d0
				ext.l		d0
				sub.l		d0,d0reg
			}
			if (d0reg<0) { d0reg&=0xfffff; CARRY=1; } else CARRY=0; pc+=3; OPEND
			
o19d2: 		RFL(112) *((UInt8*)(&d0reg)+3)=Npack2I2ASM(); pc+=4; PCPLUSONE OPEND   //OPTIMIZE Npack2I2ASM
o1Ad4: 		RFL(113) *((UInt16*)(&d0reg)+1)=Npack4I2ASM(); pc+=6; PCPLUSTHREE OPEND //OPTIMIZE Npack4I2ASM
o1Bd5: 		RFL(114) d0reg=Npack5I2ASM(); pc+=7; PCPLUSFOUR OPEND                   //OPTIMIZE Npack5I2ASM

o1Cx: 		RFL(115) 
			//d1reg-=I[2]+1; 
			asm {
				move.b 		-1(a1),d0  // assumptions: a1 -> I[3], we need I[2]
				ext.w		d0
				addq.w		#1,d0
				ext.l		d0
				sub.l		d0,d1reg
			}
			if (d1reg<0) { d1reg&=0xfffff; CARRY=1; } else CARRY=0; pc+=3; OPEND
			
o1Dd2: 		RFL(116) *((UInt8*)(&d1reg)+3)=Npack2I2ASM(); pc+=4; PCPLUSONE OPEND   //OPTIMIZE Npack2I2ASM
o1Ed4: 		RFL(117) *((UInt16*)(&d1reg)+1)=Npack4I2ASM(); pc+=6; PCPLUSTHREE OPEND //OPTIMIZE Npack4I2ASM
o1Fd5: 		RFL(118) d1reg=Npack5I2ASM(); pc+=7; PCPLUSFOUR OPEND                   //OPTIMIZE Npack5I2ASM

o2n: 		RFL(119)  
			//P=I[1];
			asm {
				move.b		-2(a1),P  // assumptions: a1 -> I[3], we need I[1]
			} 
			PCHANGED pc+=2; PCMINUSONE OPEND
			
o3X: 		RFL(120)
			//n=I[1]+1; 
			n=0;
			asm {
				move.b		-2(a1),d5  // assumptions: n is in d5,a1 -> I[3], we need I[1]
				addq.b		#1,d5
				ext.w		d5
			}
			Ncopy_circular_C_pc_minus_one( ); 
			pc+=2+n; OPEND


o4d2: 		RFL(121) if ( CARRY) { jmp=Npack2I1ASM_m3(); if (jmp) {pc+=jmp+1; PCPLUSd0 OPEND} else {rstkpop_to_a1(); OPEND} } pc+=3; OPEND
o5d2: 		RFL(122) if (!CARRY) { jmp=Npack2I1ASM_m3(); if (jmp) {pc+=jmp+1; PCPLUSd0 OPEND} else {rstkpop_to_a1(); OPEND} } pc+=3; OPEND
o6d3: 		RFL(123) d=Npack3I1ASM_m3(); if (d&0x800) {pc-=(d^0xfff); PCMINUSd0} else {pc+=d+1; PCPLUSd0} OPEND
o7d3: 		RFL(124) rstkpush_pc4(); d=Npack3I1ASM_m3(); if (d&0x800) {pc-=(d^0xfff)-3; PCMINUSd0} else {pc+=d+4; PCPLUSd0} OPEND



o_invalid3: pc+=3; OPEND
o_invalid4: pc+=4; OPEND
o_invalid5: pc+=5; OPEND
o_invalid6: pc+=6; OPEND

o_goyes:
    if (!CARRY) { pc+=2; PCPLUSTWO OPEND }
    jmp = Npack2I0ASM();
    if (jmp) 
    {
    	pc+=jmp;
    	PCPLUSd0
    	OPEND
    }
    else 
    {
    	rstkpop_to_a1(); 
    }
    
o_done:



#ifdef INCLUDE_PROFILER
	timeEnd=*timeReg;
	if (timeEnd<timeStart) timeEnd+=0xD7E4;
	if ((timeIndex>=0)&&(timeIndex<500))
	{
		count[timeIndex]++;
		time[timeIndex]+=timeEnd-timeStart;
	}
#endif




	if (icounter>0) continue;



o_done2:
	icounter = saturn_speed;
#ifdef PRINT_IPS
	asm { add.l  d7,instCount }   // assumptions: icounter is in d7...
#endif
	
		
		
	if (lcdRedrawBuffer[lcdCurrentLine])
	{
#ifdef INCLUDE_TRACER
		if (collecting)
		{
			SAVEPC
			StrPrintF(tmp,"Draw line\n");
			FileWrite(tracerStream, tmp, StrLen(tmp), 1, NULL);
			RESTOREPC
		}
#endif
	
		if (statusDisplayMode2x2)
			DrawDisplayLine16x2();
		else
			DrawDisplayLine16x1();
	}
	lcdCurrentLine++;
	
	if (lcdCurrentLine&0x0001) continue;

	if (lcdCurrentLine>=0x40)
	{
		SAVEPC
		
		lcdCurrentLine=0;
		if (delayRefresh)
		{
			RecalculateDisplayParameters();
			lcdPreviousAnnunciatorState=0;
			change_ann(NULL,NULL);
			delayRefresh = false;
		}
		
#ifdef PRINT_IPS
		if (!--ipsLoop)
		{
			update_ips();
			ipsLoop=40;
		}
#endif

		if (ioram[0x10]&8)   // clear serial buffer byte.
		{
			if (ioram[0x12]&1) 
			{
				// Dispose of byte in buffer at 0x16, 0x17
				ioram[0x12] = 0;
				if (ioram[0x10]&4) INTERRUPT("IO");
			}
		}
	    
		if (delayEvent)
		{
			delayEvent--;
			if (delayEvent==0)
			{
				//perform event;
				if (delayKey==virtKeyVideoSwap)
				{
					if (statusDisplayMode2x2) toggle_lcd();
				}
				else
				{
					kbd_handler(delayKey,0,false);
					if (delayEvent2)
					{
						kbd_handler(delayKey2,1,false);
						if (INTE) { if (!INTP) INTERRUPT("KP") else { INTD=1; } }
						delayEvent=delayEvent2;
						delayKey=delayKey2;
						delayKey2=noKey;
						delayEvent2=0;
					}
				}
			}
		}
		
		MemSemaphoreRelease(true);

		if ((optionPowerSave)&&(SHUTDN))
			EvtGetEvent(&event,evtWaitForever);  // Allow Palm CPU to doze
		else
			EvtGetEvent(&event,0);

		if (event.eType!=nilEvent)
	    	if ((ProcessPalmEvent(&event)) && (!IR15X))
	    	{
	    		WAKEUP=1;
	    		if (INTE)
	    		{
	    			if (!INTP)
	    				INTERRUPT("KP")
	    			else
	    			{
	    				INTD=1;
	    			}
	    		}
	    	}
		if (IR15X) 
		{
			WAKEUP=1;
			if (!INTP) INTERRUPT("ON");
		}
		if (statusPerformReset)
		{
			PCRESET();
			statusPerformReset=false;
		}
		MemSemaphoreReserve(true);				
		RESTOREPC
		if (statusQuit) break;
    }


	if (!IO_TIMER_RUN) 
	{
		continue; // T2, T1 disabled
	}
	


/******************************************************************************
* 
* Timer updating section
*
*******************************************************************************/

	// Handle T2 timer updating
	asm
	{
			//currTimeStamp = *timeReg;
			movea.l   timeReg,a0
			moveq     #0,d2
			move.w    (a0),d2            // d2 = currTimeStamp before branch
			move.w    #674,d4            // d4 = constant #674
			
			//if (currTimeStamp>prevTimeStamp)
			cmp.w     prevTimeStamp,d2
			bls.s     t2label1
			
				//timeDelta = (currTimeStamp-prevTimeStamp)/674;
			sub.w     prevTimeStamp,d2
			divu.w    d4,d2              // d2 = timeDelta
			
				//prevTimeStamp+=timeDelta*674;
			move.w    d2,d0
			mulu.w    d4,d0
			add.w     d0,prevTimeStamp
			
			//else
			bra.s     t2label2
			
				//timeDelta = ((currTimeStamp+55268)-prevTimeStamp)/674;
t2label1:	move.w    #55268,d5          // d5 = constant #55268
			add.w     d5,d2
			sub.w     prevTimeStamp,d2
			divu.w    d4,d2              // d2 = timeDelta
			
				//prevTimeStamp+=timeDelta*674;
			move.w    d2,d0
			mulu.w    d4,d0
			add.w     d0,prevTimeStamp

				//if (prevTimeStamp>55268)
			cmp.w     prevTimeStamp,d5
			bcc.s     t2label2
					//prevTimeStamp-=55268;

			sub.w     d5,prevTimeStamp
			
t2label2:	ext.l     d2
			cmp.l     t2,d2
			bls.s     t2label3
	}
		// assumpions: timeDelta in d2 - do not add anything that uses d2
		if (ioram[0x2f]&4) { ioram[0x2f]|=8; WAKEUP=1; }
		if (ioram[0x2f]&2) { ioram[0x2f]|=8; if (!INTP) INTERRUPT("T2"); }
		
	asm
	{
t2label3:	sub.l     d2,t2
	}
		

	// Handle T1 timer updating
	if (!--update_t1)
	{
		update_t1=4;
		SAVEPC
		currTickStamp = TimGetTicks();
		RESTOREPC
		if ((currTickStamp-prevTickStamp)>=8)
		{
			prevTickStamp=currTickStamp;
			if (t1)
				t1--;
			else
			{
				t1 = 0xf;
				if (ioram[0x2e]&4) { ioram[0x2e]|=8; WAKEUP=1; }
				if (ioram[0x2e]&2) { ioram[0x2e]|=8; if (!INTP) INTERRUPT("T1"); }
			}
		}
	}
}

pc_reg=pc;
return;







decode808:
	asm 
	{		                // a1 -> I[3]
		move.b	(a1)+,d0    // a1 -> I[4]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt13:jmp		o8080
		jmp		fin801
		jmp		o8082X
		jmp		o8083
		jmp		o8084n
		jmp		o8085n
		jmp		o8086n
		jmp		o8087n
		jmp		o8088n
		jmp		o8089n
		jmp		o808An
		jmp		o808Bn
		jmp		o808C
		jmp		o808D
		jmp		o808E
		jmp		o808F
fin801: cmpi.b	#0,(a1)+   // a1 ->I[5]
		bne.s	fininv
		jmp		o80810
fininv:	jmp		o_invalid5
	}

decode818:
	asm 
	{		                // a1 -> I[3]
		addq.l  #1,a1       // a1 -> I[4]
		move.b	(a1)+,d0    // a1 -> I[5]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt15:jmp		o818f0x
		jmp		o818f1x
		jmp		o818f2x
		jmp		o818f3x
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o818f8x
		jmp		o818f9x
		jmp		o818fAx
		jmp		o818fBx
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
	}
		
decode819:
	asm 
	{		                // a1 -> I[3]
		addq.l  #1,a1       // a1 -> I[4]
		move.b	(a1)+,d0    // a1 -> I[5]
		beq     o819f0
		subq.b  #1,d0
		beq     o819f1
		subq.b  #1,d0
		beq     o819f2
		subq.b  #1,d0
		beq     o819f3
		jmp     o_invalid5
	}
	
decode81A:
	asm 
	{		                // a1 -> I[3]
		addq.l  #1,a1		// a1 -> I[4]
		move.b	(a1)+,d0	// a1 -> I[5]
		bne.s   skip1
		
decode81Af0:		
		move.b	(a1)+,d0    // a1 -> I[6]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt19:jmp		o81Af00
		jmp		o81Af01
		jmp		o81Af02
		jmp		o81Af03
		jmp		o81Af04
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o81Af08
		jmp		o81Af09
		jmp		o81Af0A
		jmp		o81Af0B
		jmp		o81Af0C
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6

skip1:	subq.b  #1,d0
		bne.s   skip2
		
decode81Af1:
		move.b	(a1)+,d0    // a1 -> I[6]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt20:jmp		o81Af10
		jmp		o81Af11
		jmp		o81Af12
		jmp		o81Af13
		jmp		o81Af14
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o81Af18
		jmp		o81Af19
		jmp		o81Af1A
		jmp		o81Af1B
		jmp		o81Af1C
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6

skip2:	subq.b  #1,d0
		bne.s   skip3

decode81Af2:
		move.b	(a1)+,d0    // a1 -> I[6]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt21:jmp		o81Af20
		jmp		o81Af21
		jmp		o81Af22
		jmp		o81Af23
		jmp		o81Af24
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o81Af28
		jmp		o81Af29
		jmp		o81Af2A
		jmp		o81Af2B
		jmp		o81Af2C
		jmp		o_invalid6
		jmp		o_invalid6
		jmp		o_invalid6

skip3:	jmp     o_invalid5
	}


decode81B:
	asm 
	{		                // a1 -> I[3]
		move.b	(a1)+,d0	// a1 -> I[4]
		ext.w   d0
		add.w	d0,d0
		add.w   d0,d0
		jmp		2(pc,d0)
jstrt18:jmp		o_invalid4
		jmp		o81B1
		jmp		o81B2
		jmp		o81B3
		jmp		o81B4
		jmp		o81B5
		jmp		o81B6
		jmp		o81B7
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
		jmp		o_invalid4
	}



 	
 	
o800: 		RFL(125) pc+=3; OUT&=0xff0; OUT|=C[0]; OPEND
o801: 		RFL(126) pc+=3; OUT = Npack3CASM(); update_out(); OPEND
o802: 		RFL(127) pc+=3; update_in(); Nunpack4(A, IN); OPEND
o803: 		RFL(128) pc+=3; update_in(); Nunpack4(C, IN); OPEND
o804: 		RFL(129) pc+=3; unconfig(); OPEND
o805: 		RFL(130) pc+=3; config(); OPEND
o806: 		RFL(131) pc+=3; c_eq_id(); OPEND
o807: 		RFL(132) if (SHUTDN) { if (WAKEUP) { SHUTDN=0; pc+=3; WAKEUP=0; } else PCMINUSTHREE } else { SHUTDN=1; WAKEUP=0; PCMINUSTHREE } OPEND


o809: 		RFL(149) pc+=3; d=Npack5CASM(); d+=P+1; if (d>0xfffff) { d&=0xfffff; CARRY=1; } else CARRY=0; Nunpack5(C, d); OPEND
o80A: 		RFL(150) pc+=3; reset(); OPEND
o80B: 		RFL(151) pc+=3; OPEND

o80Cn: 		RFL(152) 
//			C[(int)I[3]] = P; 
			asm {
				move.b    (a1)+,d0  // assumptions: a1 -> I[3], we need I[3], will increment to I[4] to avoid PCPLUSONE
				ext.w     d0
				lea       C,a0
				move.b    P,(a0,d0.w)
			}
			pc+=4; OPEND
			
o80Dn: 		RFL(153)
//			P = C[(int)I[3]]; 
			asm {
				move.b    (a1)+,d0  // assumptions: a1 -> I[3], we need I[3], will increment to I[4] to avoid PCPLUSONE
				ext.w     d0
				lea       C,a0
				move.b    (a0,d0.w),P
			}
			PCHANGED pc+=4; OPEND
			
o80E: 		RFL(154) pc+=3; C[0]=0; OPEND
o80Fn: 		RFL(155) 
			n = P;
			P = 0;
//			P = C[(int)I[3]]; C[(int)I[3]] = n; 
			asm {
				move.b    (a1),d0  // assumptions: a1 -> I[3], we need I[3]
				ext.w     d0
				lea       C,a0
				move.b    (a0,d0.w),P
				move.b    (a1)+,d0  // assumptions: a1 -> I[3], we need I[3], will increment to I[5] to avoid PCPLUSONE
				ext.w     d0
				lea       C,a0
				move.b    d5,(a0,d0.w) // assumptions: n is in d5
			}
			PCHANGED pc+=4; OPEND
			
o810: 		RFL(156) a8=A; goto nslc16finish;
o811: 		RFL(157) a8=B; goto nslc16finish;
o812: 		RFL(158) a8=C; goto nslc16finish;
o813: 		RFL(159) a8=D; 
			nslc16finish:
			a8+=16;
			b8=a8;
			m=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=*--b8;
			*--a8=m;
			pc+=3; OPEND
			
o814: 		RFL(160) a8=A; goto nsrc16finish;
o815: 		RFL(161) a8=B; goto nsrc16finish;
o816: 		RFL(162) a8=C; goto nsrc16finish;
o817: 		RFL(163) a8=D; 
			nsrc16finish:
			b8=a8;
			m=*b8++;
			
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8=m;
			pc+=3; OPEND

// o818 through o81B located further down...

o81C: 		RFL(176) pc+=3; Nsrb(A, 16); OPEND
o81D: 		RFL(177) pc+=3; Nsrb(B, 16); OPEND
o81E: 		RFL(178) pc+=3; Nsrb(C, 16); OPEND
o81F: 		RFL(179) pc+=3; Nsrb(D, 16); OPEND

o82n: 		RFL(180) 
			//HST&=~I[2]; 
			asm {
				move.b    -1(a1),d0    // assumptions: a1 -> I[3], we need I[2]
				not.b     d0
				and.b     d0,HST
			}
			pc+=3; OPEND
			
o83n: 		RFL(181)
			//n=I[2];
			n=0;
			asm { move.b    -1(a1),d5 }   // assumptions: n is in d5, a1 -> I[3], we need I[2]
			CARRY=(HST&n)?0:1; 
			pc+=3; goto o_goyes;
			
o84n: 		RFL(182) Nbit0_2(ST); pc+=3; OPEND
o85n: 		RFL(183) Nbit1_2(ST); pc+=3; OPEND
o86n: 		RFL(184) Tbit0_2(ST); pc+=3; goto o_goyes;
o87n: 		RFL(185) Tbit1_2(ST); pc+=3; goto o_goyes;

o88n: 		RFL(186) 
			//n=I[2];
			n=0;
			asm { move.b    -1(a1),d5 }   // assumptions: n is in d5, a1 -> I[3], we need I[2]
			if (P!=n) CARRY=1; else CARRY=0; 
			pc+=3; goto o_goyes;

o89n: 		RFL(187)
			//n=I[2];
			n=0;
			asm { move.b    -1(a1),d5 }   // assumptions: n is in d5, a1 -> I[3], we need I[2]
			if (P==n) CARRY=1; else CARRY=0; 
			pc+=3; goto o_goyes;
			




/**********************************************************************************
*                                                                                 *
*         NFadd, and NFdbl                                                        *
*                                                                                 *
**********************************************************************************/

oAfp0: 		RFL(256) a8=A;b8=B;goto npaddfinish;
oAfp1: 		RFL(257) a8=B;b8=C;goto npaddfinish;
oAfp2: 		RFL(258) a8=C;b8=A;goto npaddfinish;
oAfp3: 		RFL(259) a8=D;b8=C;goto npaddfinish;
oAfp4: 		RFL(260) a8=A;b8=A;goto npaddfinish;
oAfp5: 		RFL(261) a8=B;b8=B;goto npaddfinish;
oAfp6: 		RFL(262) a8=C;b8=C;goto npaddfinish;
oAfp7: 		RFL(263) a8=D;b8=D;goto npaddfinish;
oAfp8: 		RFL(264) a8=B;b8=A;goto npaddfinish;
oAfp9: 		RFL(265) a8=C;b8=B;goto npaddfinish;
oAfpA: 		RFL(266) a8=A;b8=C;goto npaddfinish;
oAfpB: 		RFL(267) a8=C;b8=D;
			npaddfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=P;b8+=P;
			*a8+=*b8;
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND
			
oAfwp0: 	RFL(256) a8=A;b8=B;goto nwpaddfinish;
oAfwp1: 	RFL(257) a8=B;b8=C;goto nwpaddfinish;
oAfwp2: 	RFL(258) a8=C;b8=A;goto nwpaddfinish;
oAfwp3: 	RFL(259) a8=D;b8=C;goto nwpaddfinish;
oAfwp4: 	RFL(260) a8=A;b8=A;goto nwpaddfinish;
oAfwp5: 	RFL(261) a8=B;b8=B;goto nwpaddfinish;
oAfwp6: 	RFL(262) a8=C;b8=C;goto nwpaddfinish;
oAfwp7: 	RFL(263) a8=D;b8=D;goto nwpaddfinish;
oAfwp8: 	RFL(264) a8=B;b8=A;goto nwpaddfinish;
oAfwp9: 	RFL(265) a8=C;b8=B;goto nwpaddfinish;
oAfwpA: 	RFL(266) a8=A;b8=C;goto nwpaddfinish;
oAfwpB: 	RFL(267) a8=C;b8=D;
			nwpaddfinish:
			o=0;
			n=(char)P+1;
			m=(char)MODE;
			do 
  			{
    			*a8 += (char)(*b8++ + o);
    			if (*a8 < m)
      			{ a8++;o = 0; }
   				else
      			{ *a8++ -= m; o = 1; }
			} while (--n);
  			CARRY=o;
			pc+=3;OPEND
			
oAfxs0: 	RFL(256) a8=A;b8=B;goto nxsaddfinish;
oAfxs1: 	RFL(257) a8=B;b8=C;goto nxsaddfinish;
oAfxs2: 	RFL(258) a8=C;b8=A;goto nxsaddfinish;
oAfxs3: 	RFL(259) a8=D;b8=C;goto nxsaddfinish;
oAfxs4: 	RFL(260) a8=A;b8=A;goto nxsaddfinish;
oAfxs5: 	RFL(261) a8=B;b8=B;goto nxsaddfinish;
oAfxs6: 	RFL(262) a8=C;b8=C;goto nxsaddfinish;
oAfxs7: 	RFL(263) a8=D;b8=D;goto nxsaddfinish;
oAfxs8: 	RFL(264) a8=B;b8=A;goto nxsaddfinish;
oAfxs9: 	RFL(265) a8=C;b8=B;goto nxsaddfinish;
oAfxsA: 	RFL(266) a8=A;b8=C;goto nxsaddfinish;
oAfxsB: 	RFL(267) a8=C;b8=D;
			nxsaddfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=2;b8+=2;
			*a8+=*b8;
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND

oAfx0: 		RFL(256) a8=A;b8=B;goto nxaddfinish;
oAfx1: 		RFL(257) a8=B;b8=C;goto nxaddfinish;
oAfx2: 		RFL(258) a8=C;b8=A;goto nxaddfinish;
oAfx3: 		RFL(259) a8=D;b8=C;goto nxaddfinish;
oAfx4: 		RFL(260) a8=A;b8=A;goto nxaddfinish;
oAfx5: 		RFL(261) a8=B;b8=B;goto nxaddfinish;
oAfx6: 		RFL(262) a8=C;b8=C;goto nxaddfinish;
oAfx7: 		RFL(263) a8=D;b8=D;goto nxaddfinish;
oAfx8: 		RFL(264) a8=B;b8=A;goto nxaddfinish;
oAfx9: 		RFL(265) a8=C;b8=B;goto nxaddfinish;
oAfxA: 		RFL(266) a8=A;b8=C;goto nxaddfinish;
oAfxB: 		RFL(267) a8=C;b8=D;
			nxaddfinish:
			CARRY=0;
			m=(char)MODE;
			*(UInt16*)a8 += *(UInt16*)b8;
			*(UInt8*)(a8+2) += *(UInt8*)(b8+2);
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND
			
oAfs0: 		RFL(256) a8=A;b8=B;goto nsaddfinish;
oAfs1: 		RFL(257) a8=B;b8=C;goto nsaddfinish;
oAfs2: 		RFL(258) a8=C;b8=A;goto nsaddfinish;
oAfs3: 		RFL(259) a8=D;b8=C;goto nsaddfinish;
oAfs4: 		RFL(260) a8=A;b8=A;goto nsaddfinish;
oAfs5: 		RFL(261) a8=B;b8=B;goto nsaddfinish;
oAfs6: 		RFL(262) a8=C;b8=C;goto nsaddfinish;
oAfs7: 		RFL(263) a8=D;b8=D;goto nsaddfinish;
oAfs8: 		RFL(264) a8=B;b8=A;goto nsaddfinish;
oAfs9: 		RFL(265) a8=C;b8=B;goto nsaddfinish;
oAfsA: 		RFL(266) a8=A;b8=C;goto nsaddfinish;
oAfsB: 		RFL(267) a8=C;b8=D;
			nsaddfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=15;b8+=15;
			*a8+=*b8;
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND
			
oAfm0: 		RFL(256) a8=A;b8=B;goto nmaddfinish;
oAfm1: 		RFL(257) a8=B;b8=C;goto nmaddfinish;
oAfm2: 		RFL(258) a8=C;b8=A;goto nmaddfinish;
oAfm3: 		RFL(259) a8=D;b8=C;goto nmaddfinish;
oAfm4: 		RFL(260) a8=A;b8=A;goto nmaddfinish;
oAfm5: 		RFL(261) a8=B;b8=B;goto nmaddfinish;
oAfm6: 		RFL(262) a8=C;b8=C;goto nmaddfinish;
oAfm7: 		RFL(263) a8=D;b8=D;goto nmaddfinish;
oAfm8: 		RFL(264) a8=B;b8=A;goto nmaddfinish;
oAfm9: 		RFL(265) a8=C;b8=B;goto nmaddfinish;
oAfmA: 		RFL(266) a8=A;b8=C;goto nmaddfinish;
oAfmB: 		RFL(267) a8=C;b8=D;
			nmaddfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=3;b8+=3;
			asm
			{
				move.l   a2,a0
				move.b   (a3)+,d0
				add.b    d0,(a0)+
				move.l   (a3)+,d0
				add.l    d0,(a0)+
				move.l   (a3)+,d0
				add.l    d0,(a0)+
				move.w   (a3)+,d0
				add.w    d0,(a0)+
				move.b   (a3)+,d0
				add.b    d0,(a0)+
			}
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND
			
oAfb0: 		RFL(256) a8=A;b8=B;goto nbaddfinish;
oAfb1: 		RFL(257) a8=B;b8=C;goto nbaddfinish;
oAfb2: 		RFL(258) a8=C;b8=A;goto nbaddfinish;
oAfb3: 		RFL(259) a8=D;b8=C;goto nbaddfinish;
oAfb4: 		RFL(260) a8=A;b8=A;goto nbaddfinish;
oAfb5: 		RFL(261) a8=B;b8=B;goto nbaddfinish;
oAfb6: 		RFL(262) a8=C;b8=C;goto nbaddfinish;
oAfb7: 		RFL(263) a8=D;b8=D;goto nbaddfinish;
oAfb8: 		RFL(264) a8=B;b8=A;goto nbaddfinish;
oAfb9: 		RFL(265) a8=C;b8=B;goto nbaddfinish;
oAfbA: 		RFL(266) a8=A;b8=C;goto nbaddfinish;
oAfbB: 		RFL(267) a8=C;b8=D;
			nbaddfinish:
			CARRY=0;
			m=(char)MODE;
			*(UInt16*)a8 += *(UInt16*)b8;
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND
			
oAfw0: 		RFL(256) a8=A;b8=B;goto nwaddfinish;
oAfw1: 		RFL(257) a8=B;b8=C;goto nwaddfinish;
oAfw2: 		RFL(258) a8=C;b8=A;goto nwaddfinish;
oAfw3: 		RFL(259) a8=D;b8=C;goto nwaddfinish;
oAfw4: 		RFL(260) a8=A;b8=A;goto nwaddfinish;
oAfw5: 		RFL(261) a8=B;b8=B;goto nwaddfinish;
oAfw6: 		RFL(262) a8=C;b8=C;goto nwaddfinish;
oAfw7: 		RFL(263) a8=D;b8=D;goto nwaddfinish;
oAfw8: 		RFL(264) a8=B;b8=A;goto nwaddfinish;
oAfw9: 		RFL(265) a8=C;b8=B;goto nwaddfinish;
oAfwA: 		RFL(266) a8=A;b8=C;goto nwaddfinish;
oAfwB: 		RFL(267) a8=C;b8=D;
			nwaddfinish:
			CARRY=0;
			m=(char)MODE;
			asm
			{
				move.l   a2,a0
				move.l   (a3)+,d0
				add.l    d0,(a0)+
				move.l   (a3)+,d0
				add.l    d0,(a0)+
				move.l   (a3)+,d0
				add.l    d0,(a0)+
				move.l   (a3)+,d0
				add.l    d0,(a0)+
			}
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8++ >= m) { *--a8 -= m; (*++a8)++; }
			if (*a8 >= m) { *a8 -= m; CARRY=1; }
			pc+=3;OPEND





/**********************************************************************************
*                                                                                 *
*         NFsub                                                                   *
*                                                                                 *
**********************************************************************************/

oBfp0: 		RFL(289) a8=A,b8=B;goto npsubfinish;
oBfp1: 		RFL(290) a8=B,b8=C;goto npsubfinish;
oBfp2: 		RFL(291) a8=C,b8=A;goto npsubfinish;
oBfp3: 		RFL(292) a8=D,b8=C;goto npsubfinish;
oBfp8: 		RFL(297) a8=B,b8=A;goto npsubfinish;
oBfp9: 		RFL(298) a8=C,b8=B;goto npsubfinish;
oBfpA:		RFL(299) a8=A,b8=C;goto npsubfinish;
oBfpB: 		RFL(300) a8=C,b8=D;
			npsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=P;b8+=P;
			*a8-=*b8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfwp0: 	RFL(289) a8=A,b8=B;goto nwpsubfinish;
oBfwp1: 	RFL(290) a8=B,b8=C;goto nwpsubfinish;
oBfwp2: 	RFL(291) a8=C,b8=A;goto nwpsubfinish;
oBfwp3: 	RFL(292) a8=D,b8=C;goto nwpsubfinish;
oBfwp8: 	RFL(297) a8=B,b8=A;goto nwpsubfinish;
oBfwp9: 	RFL(298) a8=C,b8=B;goto nwpsubfinish;
oBfwpA:		RFL(299) a8=A,b8=C;goto nwpsubfinish;
oBfwpB: 	RFL(300) a8=C,b8=D;
			nwpsubfinish:
			o=0;
			n=(char)P+1;
			m=(char)MODE;
			do 
  			{
    			*a8 -= (char)(*b8++ + o);
    			if (*a8 < 0)
      			{ *a8++ += m; o = 1; }
   				else
      			{ a8++;o = 0; }
			} while (--n);
  			CARRY=o;
			pc+=3;OPEND

oBfxs0: 	RFL(289) a8=A,b8=B;goto nxssubfinish;
oBfxs1: 	RFL(290) a8=B,b8=C;goto nxssubfinish;
oBfxs2: 	RFL(291) a8=C,b8=A;goto nxssubfinish;
oBfxs3: 	RFL(292) a8=D,b8=C;goto nxssubfinish;
oBfxs8: 	RFL(297) a8=B,b8=A;goto nxssubfinish;
oBfxs9: 	RFL(298) a8=C,b8=B;goto nxssubfinish;
oBfxsA:		RFL(299) a8=A,b8=C;goto nxssubfinish;
oBfxsB: 	RFL(300) a8=C,b8=D;
			nxssubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=2;b8+=2;
			*a8-=*b8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfx0: 		RFL(289) a8=A,b8=B;goto nxsubfinish;
oBfx1: 		RFL(290) a8=B,b8=C;goto nxsubfinish;
oBfx2: 		RFL(291) a8=C,b8=A;goto nxsubfinish;
oBfx3: 		RFL(292) a8=D,b8=C;goto nxsubfinish;
oBfx8: 		RFL(297) a8=B,b8=A;goto nxsubfinish;
oBfx9: 		RFL(298) a8=C,b8=B;goto nxsubfinish;
oBfxA:		RFL(299) a8=A,b8=C;goto nxsubfinish;
oBfxB: 		RFL(300) a8=C,b8=D;
			nxsubfinish:
			CARRY=0;
			m=(char)MODE;
			*a8++ -= *b8++;
			*a8++ -= *b8++;
			*a8 -= *b8;
			a8-=2;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfs0: 		RFL(289) a8=A,b8=B;goto nssubfinish;
oBfs1: 		RFL(290) a8=B,b8=C;goto nssubfinish;
oBfs2: 		RFL(291) a8=C,b8=A;goto nssubfinish;
oBfs3: 		RFL(292) a8=D,b8=C;goto nssubfinish;
oBfs8: 		RFL(297) a8=B,b8=A;goto nssubfinish;
oBfs9: 		RFL(298) a8=C,b8=B;goto nssubfinish;
oBfsA:		RFL(299) a8=A,b8=C;goto nssubfinish;
oBfsB: 		RFL(300) a8=C,b8=D;
			nssubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=15;b8+=15;
			*a8-=*b8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfm0: 		RFL(289) a8=A,b8=B;goto nmsubfinish;
oBfm1: 		RFL(290) a8=B,b8=C;goto nmsubfinish;
oBfm2: 		RFL(291) a8=C,b8=A;goto nmsubfinish;
oBfm3: 		RFL(292) a8=D,b8=C;goto nmsubfinish;
oBfm8: 		RFL(297) a8=B,b8=A;goto nmsubfinish;
oBfm9: 		RFL(298) a8=C,b8=B;goto nmsubfinish;
oBfmA:		RFL(299) a8=A,b8=C;goto nmsubfinish;
oBfmB: 		RFL(300) a8=C,b8=D;
			nmsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=3;b8+=3;c8=a8;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8 -= *b8;
			//a8-=11;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfb0: 		RFL(289) a8=A,b8=B;goto nbsubfinish;
oBfb1: 		RFL(290) a8=B,b8=C;goto nbsubfinish;
oBfb2: 		RFL(291) a8=C,b8=A;goto nbsubfinish;
oBfb3: 		RFL(292) a8=D,b8=C;goto nbsubfinish;
oBfb8: 		RFL(297) a8=B,b8=A;goto nbsubfinish;
oBfb9: 		RFL(298) a8=C,b8=B;goto nbsubfinish;
oBfbA:		RFL(299) a8=A,b8=C;goto nbsubfinish;
oBfbB: 		RFL(300) a8=C,b8=D;
			nbsubfinish:
			CARRY=0;
			m=(char)MODE;
			*a8++ -= *b8++;
			*a8 -= *b8;
			a8--;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfw0: 		RFL(289) a8=A,b8=B;goto nwsubfinish;
oBfw1: 		RFL(290) a8=B,b8=C;goto nwsubfinish;
oBfw2: 		RFL(291) a8=C,b8=A;goto nwsubfinish;
oBfw3: 		RFL(292) a8=D,b8=C;goto nwsubfinish;
oBfw8: 		RFL(297) a8=B,b8=A;goto nwsubfinish;
oBfw9: 		RFL(298) a8=C,b8=B;goto nwsubfinish;
oBfwA:		RFL(299) a8=A,b8=C;goto nwsubfinish;
oBfwB: 		RFL(300) a8=C,b8=D;
			nwsubfinish:
			CARRY=0;
			m=(char)MODE;
			c8=a8;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8++ -= *b8++;
			*c8 -= *b8;
			//a8-=15;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND






/**********************************************************************************
*                                                                                 *
*         NFrsub                                                                  *
*                                                                                 *
**********************************************************************************/

oBfpC: 		RFL(301) a8=A,b8=B;goto nprsubfinish;
oBfpD: 		RFL(302) a8=B,b8=C;goto nprsubfinish;
oBfpE: 		RFL(303) a8=C,b8=A;goto nprsubfinish;
oBfpF: 		RFL(304) a8=D,b8=C;
			nprsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=P;b8+=P;
			*a8=*b8-*a8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfwpC: 	RFL(301) a8=A,b8=B;goto nwprsubfinish;
oBfwpD: 	RFL(302) a8=B,b8=C;goto nwprsubfinish;
oBfwpE: 	RFL(303) a8=C,b8=A;goto nwprsubfinish;
oBfwpF: 	RFL(304) a8=D,b8=C;
			nwprsubfinish:
			o=0;
			n=(char)P+1;
			m=(char)MODE;
			do 
  			{
    			*a8 = *b8++ - (*a8 + o);
    			if (*a8 < 0)
      			{ *a8++ += m; o = 1; }
   				else
      			{ a8++;o = 0; }
			} while (--n);
  			CARRY=o;
			pc+=3;OPEND

oBfxsC: 	RFL(301) a8=A,b8=B;goto nxsrsubfinish;
oBfxsD: 	RFL(302) a8=B,b8=C;goto nxsrsubfinish;
oBfxsE: 	RFL(303) a8=C,b8=A;goto nxsrsubfinish;
oBfxsF: 	RFL(304) a8=D,b8=C;
			nxsrsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=2;b8+=2;
			*a8=*b8-*a8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfxC: 		RFL(301) a8=A,b8=B;goto nxrsubfinish;
oBfxD: 		RFL(302) a8=B,b8=C;goto nxrsubfinish;
oBfxE: 		RFL(303) a8=C,b8=A;goto nxrsubfinish;
oBfxF: 		RFL(304) a8=D,b8=C;
			nxrsubfinish:
			CARRY=0;
			m=(char)MODE;
			*a8++ = *b8++ - *a8;
			*a8++ = *b8++ - *a8;
			*a8=*b8-*a8;
			a8-=2;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND

oBfsC: 		RFL(301) a8=A,b8=B;goto nsrsubfinish;
oBfsD: 		RFL(302) a8=B,b8=C;goto nsrsubfinish;
oBfsE: 		RFL(303) a8=C,b8=A;goto nsrsubfinish;
oBfsF: 		RFL(304) a8=D,b8=C;
			nsrsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=15;b8+=15;
			*a8=*b8-*a8;
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfmC: 		RFL(301) a8=A,b8=B;goto nmrsubfinish;
oBfmD: 		RFL(302) a8=B,b8=C;goto nmrsubfinish;
oBfmE: 		RFL(303) a8=C,b8=A;goto nmrsubfinish;
oBfmF: 		RFL(304) a8=D,b8=C;
			nmrsubfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=3;b8+=3;c8=a8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8=*b8-*c8;
			//a8-=11;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfbC: 		RFL(301) a8=A,b8=B;goto nbrsubfinish;
oBfbD: 		RFL(302) a8=B,b8=C;goto nbrsubfinish;
oBfbE: 		RFL(303) a8=C,b8=A;goto nbrsubfinish;
oBfbF: 		RFL(304) a8=D,b8=C;
			nbrsubfinish:
			CARRY=0;
			m=(char)MODE;
			*a8++ = *b8++ - *a8;
			*a8=*b8-*a8;
			a8--;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3;OPEND
			
oBfwC: 		RFL(301) a8=A,b8=B;goto nwrsubfinish;
oBfwD: 		RFL(302) a8=B,b8=C;goto nwrsubfinish;
oBfwE: 		RFL(303) a8=C,b8=A;goto nwrsubfinish;
oBfwF: 		RFL(304) a8=D,b8=C;
			nwrsubfinish:
			CARRY=0;
			m=(char)MODE;
			c8=a8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8++ = *b8++ - *c8;
			*c8=*b8-*c8;
			//a8-=15;
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8++ < 0) { *--a8 += m; (*++a8)--; }
			if (*a8 < 0) { *a8 += m; CARRY=1; }
			pc+=3; goto o_done;





/**********************************************************************************
*                                                                                 *
*         NFdec                                                                   *
*                                                                                 *
**********************************************************************************/

oAfpC: 		RFL(268) a8=A; goto npdecfinish;    // P field, 1 nibble, [P]
oAfpD: 		RFL(269) a8=B; goto npdecfinish;
oAfpE: 		RFL(270) a8=C; goto npdecfinish;
oAfpF: 		RFL(271) a8=D; 
			npdecfinish:
			CARRY=0;
			a8+=P;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto npdecend; } *a8++ = m;
			CARRY=1;
			npdecend:pc+=3; goto o_done;

oAfwpC: 	RFL(268) a8=A; goto nwpdecfinish;   // WP field, P+1 nibble, [0..P]
oAfwpD: 	RFL(269) a8=B; goto nwpdecfinish;
oAfwpE: 	RFL(270) a8=C; goto nwpdecfinish;
oAfwpF: 	RFL(271) a8=D; 
			nwpdecfinish:
			CARRY=0;
			n=(char)P+1;
			m=(char)MODE_MINUS_ONE;
			do
			{
				if ((*a8)--) { goto nwpdecend; }
				*a8++ = m;
			} while (--n);
			CARRY = 1;
			nwpdecend:pc+=3; goto o_done;
			
oAfxsC: 	RFL(268) a8=A; goto nxsdecfinish;   // XS field, 1 nibble, [2]
oAfxsD: 	RFL(269) a8=B; goto nxsdecfinish;
oAfxsE: 	RFL(270) a8=C; goto nxsdecfinish;
oAfxsF: 	RFL(271) a8=D; 
			nxsdecfinish:
			CARRY=0;
			a8+=2;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nxsdecend; } *a8++ = m;
			CARRY=1;
			nxsdecend:pc+=3; goto o_done;

oAfxC: 		RFL(268) a8=A; goto nxdecfinish;    // X field, 3 nibbles, [0..2]
oAfxD: 		RFL(269) a8=B; goto nxdecfinish;
oAfxE: 		RFL(270) a8=C; goto nxdecfinish;
oAfxF: 		RFL(271) a8=D; 
			nxdecfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nxdecend; } *a8++ = m;
			if ((*a8)--) { goto nxdecend; } *a8++ = m;
			if ((*a8)--) { goto nxdecend; } *a8++ = m;
			CARRY=1;
			nxdecend:pc+=3; goto o_done;

oAfsC: 		RFL(268) a8=A; goto nsdecfinish;    // S field, 1 nibble, [15]
oAfsD: 		RFL(269) a8=B; goto nsdecfinish;
oAfsE: 		RFL(270) a8=C; goto nsdecfinish;
oAfsF: 		RFL(271) a8=D; 
			nsdecfinish:
			CARRY=0;
			a8+=15;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nsdecend; } *a8++ = m;
			CARRY=1;
			nsdecend:pc+=3; goto o_done;

oAfmC: 		RFL(268) a8=A; goto nmdecfinish;    // M field, 12 nibbles, [3..14]
oAfmD: 		RFL(269) a8=B; goto nmdecfinish;
oAfmE: 		RFL(270) a8=C; goto nmdecfinish;
oAfmF: 		RFL(271) a8=D; 
			nmdecfinish:
			CARRY=0;
			a8+=3;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			if ((*a8)--) { goto nmdecend; } *a8++ = m;
			CARRY=1;
			nmdecend:pc+=3; goto o_done;

oAfbC: 		RFL(268) a8=A; goto nbdecfinish;
oAfbD: 		RFL(269) a8=B; goto nbdecfinish;    // B field, 2 nibbles, [0..1]
oAfbE: 		RFL(270) a8=C; goto nbdecfinish;
oAfbF: 		RFL(271) a8=D; 
			nbdecfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nbdecend; } *a8++ = m;
			if ((*a8)--) { goto nbdecend; } *a8++ = m;
			CARRY=1;
			nbdecend:pc+=3; goto o_done;

oAfwC: 		RFL(268) a8=A; goto nwdecfinish;    // W field, 16 nibbles, [0..15]
oAfwD: 		RFL(269) a8=B; goto nwdecfinish;
oAfwE: 		RFL(270) a8=C; goto nwdecfinish;
oAfwF: 		RFL(271) a8=D; 
			nwdecfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			if ((*a8)--) { goto nwdecend; } *a8++ = m;
			CARRY=1;
			nwdecend:pc+=3; goto o_done;





/**********************************************************************************
*                                                                                 *
*         NFzero                                                                  *
*                                                                                 *
**********************************************************************************/

oAbfp0: 	RFL(272) a8=A; goto npzerofinish;    // P field, 1 nibble, [P]
oAbfp1: 	RFL(273) a8=B; goto npzerofinish;
oAbfp2: 	RFL(274) a8=C; goto npzerofinish;
oAbfp3: 	RFL(275) a8=D; 
			npzerofinish:
			a8+=P;
			*a8=0;
			pc+=3; goto o_done;

oAbfwp0: 	RFL(272) a8=A; goto nwpzerofinish;   // WP field, P+1 nibble, [0..P]
oAbfwp1: 	RFL(273) a8=B; goto nwpzerofinish;
oAbfwp2: 	RFL(274) a8=C; goto nwpzerofinish;
oAbfwp3: 	RFL(275) a8=D; 
			nwpzerofinish:
			n=(char)P+1;
			do { *a8++ = 0; } while (--n);
			pc+=3; goto o_done;
			
oAbfxs0: 	RFL(272) a8=A; goto nxszerofinish;   // XS field, 1 nibble, [2]
oAbfxs1: 	RFL(273) a8=B; goto nxszerofinish;
oAbfxs2: 	RFL(274) a8=C; goto nxszerofinish;
oAbfxs3: 	RFL(275) a8=D; 
			nxszerofinish:
			a8+=2;
			*a8=0;
			pc+=3; goto o_done;

oAbfx0: 	RFL(272) a8=A; goto nxzerofinish;    // X field, 3 nibbles, [0..2]
oAbfx1: 	RFL(273) a8=B; goto nxzerofinish;
oAbfx2: 	RFL(274) a8=C; goto nxzerofinish;
oAbfx3: 	RFL(275) a8=D; 
			nxzerofinish:
			asm
			{
				clr.w  (a2)+
				clr.b  (a2)
			}
			pc+=3; goto o_done;

oAbfs0: 	RFL(272) a8=A; goto nszerofinish;    // S field, 1 nibble, [15]
oAbfs1: 	RFL(273) a8=B; goto nszerofinish;
oAbfs2: 	RFL(274) a8=C; goto nszerofinish;
oAbfs3: 	RFL(275) a8=D; 
			nszerofinish:
			a8+=15;
			*a8=0;
			pc+=3; goto o_done;

oAbfm0: 	RFL(272) a8=A; goto nmzerofinish;    // M field, 12 nibbles, [3..14]
oAbfm1: 	RFL(273) a8=B; goto nmzerofinish;
oAbfm2: 	RFL(274) a8=C; goto nmzerofinish;
oAbfm3: 	RFL(275) a8=D; 
			nmzerofinish:
			a8+=3;
			asm
			{
				clr.b  (a2)+
				clr.l  (a2)+
				clr.l  (a2)+
				clr.w  (a2)+
				clr.b  (a2)
			}
			pc+=3; goto o_done;

oAbfb0: 	RFL(272) a8=A; goto nbzerofinish;
oAbfb1: 	RFL(273) a8=B; goto nbzerofinish;    // B field, 2 nibbles, [0..1]
oAbfb2: 	RFL(274) a8=C; goto nbzerofinish;
oAbfb3: 	RFL(275) a8=D; 
			nbzerofinish:
			asm
			{
				clr.w  (a2)
			}
			pc+=3; goto o_done;

oAbfw0: 	RFL(272) a8=A; goto nwzerofinish;    // W field, 16 nibbles, [0..15]
oAbfw1: 	RFL(273) a8=B; goto nwzerofinish;
oAbfw2: 	RFL(274) a8=C; goto nwzerofinish;
oAbfw3: 	RFL(275) a8=D; 
			nwzerofinish:
			asm
			{
				clr.l  (a2)+
				clr.l  (a2)+
				clr.l  (a2)+
				clr.l  (a2)
			}
			pc+=3; goto o_done;





/**********************************************************************************
*                                                                                 *
*         NFcopy (small)                                                          *
*                                                                                 *
**********************************************************************************/

oAbfp4: 	RFL(276) a8=A;b8=B; goto npcopysmallfinish;
oAbfp5: 	RFL(277) a8=B;b8=C; goto npcopysmallfinish;
oAbfp6: 	RFL(278) a8=C;b8=A; goto npcopysmallfinish;
oAbfp7: 	RFL(279) a8=D;b8=C; goto npcopysmallfinish;
oAbfp8: 	RFL(280) a8=B;b8=A; goto npcopysmallfinish;
oAbfp9: 	RFL(281) a8=C;b8=B; goto npcopysmallfinish;
oAbfpA: 	RFL(282) a8=A;b8=C; goto npcopysmallfinish;
oAbfpB: 	RFL(283) a8=C;b8=D;
			npcopysmallfinish:
			*(a8+P)=*(b8+P);
			pc+=3;goto o_done;
			
oAbfwp4: 	RFL(276) a8=A;b8=B; goto nwpcopysmallfinish;
oAbfwp5: 	RFL(277) a8=B;b8=C; goto nwpcopysmallfinish;
oAbfwp6: 	RFL(278) a8=C;b8=A; goto nwpcopysmallfinish;
oAbfwp7: 	RFL(279) a8=D;b8=C; goto nwpcopysmallfinish;
oAbfwp8: 	RFL(280) a8=B;b8=A; goto nwpcopysmallfinish;
oAbfwp9: 	RFL(281) a8=C;b8=B; goto nwpcopysmallfinish;
oAbfwpA: 	RFL(282) a8=A;b8=C; goto nwpcopysmallfinish;
oAbfwpB: 	RFL(283) a8=C;b8=D;
			nwpcopysmallfinish:
			n=(char)P+1;
			do {*a8++=*b8++;} while (--n);
			pc+=3;goto o_done;

oAbfxs4: 	RFL(276) a8=A;b8=B; goto nxscopysmallfinish;
oAbfxs5: 	RFL(277) a8=B;b8=C; goto nxscopysmallfinish;
oAbfxs6: 	RFL(278) a8=C;b8=A; goto nxscopysmallfinish;
oAbfxs7: 	RFL(279) a8=D;b8=C; goto nxscopysmallfinish;
oAbfxs8: 	RFL(280) a8=B;b8=A; goto nxscopysmallfinish;
oAbfxs9: 	RFL(281) a8=C;b8=B; goto nxscopysmallfinish;
oAbfxsA: 	RFL(282) a8=A;b8=C; goto nxscopysmallfinish;
oAbfxsB: 	RFL(283) a8=C;b8=D;
			nxscopysmallfinish:
			*(a8+2)=*(b8+2);
			pc+=3;goto o_done;
			
oAbfx4: 	RFL(276) a8=A;b8=B; goto nxcopysmallfinish;
oAbfx5: 	RFL(277) a8=B;b8=C; goto nxcopysmallfinish;
oAbfx6: 	RFL(278) a8=C;b8=A; goto nxcopysmallfinish;
oAbfx7: 	RFL(279) a8=D;b8=C; goto nxcopysmallfinish;
oAbfx8: 	RFL(280) a8=B;b8=A; goto nxcopysmallfinish;
oAbfx9: 	RFL(281) a8=C;b8=B; goto nxcopysmallfinish;
oAbfxA: 	RFL(282) a8=A;b8=C; goto nxcopysmallfinish;
oAbfxB: 	RFL(283) a8=C;b8=D;
			nxcopysmallfinish:
			asm	
			{
				move.w (a3)+,(a2)+
				move.b (a3),(a2)
			}
			pc+=3;goto o_done;

oAbfs4: 	RFL(276) a8=A;b8=B; goto nscopysmallfinish;
oAbfs5: 	RFL(277) a8=B;b8=C; goto nscopysmallfinish;
oAbfs6: 	RFL(278) a8=C;b8=A; goto nscopysmallfinish;
oAbfs7: 	RFL(279) a8=D;b8=C; goto nscopysmallfinish;
oAbfs8: 	RFL(280) a8=B;b8=A; goto nscopysmallfinish;
oAbfs9: 	RFL(281) a8=C;b8=B; goto nscopysmallfinish;
oAbfsA: 	RFL(282) a8=A;b8=C; goto nscopysmallfinish;
oAbfsB: 	RFL(283) a8=C;b8=D;
			nscopysmallfinish:
			*(a8+15)=*(b8+15);
			pc+=3;goto o_done;

oAbfm4: 	RFL(276) a8=A;b8=B; goto nmcopysmallfinish;
oAbfm5: 	RFL(277) a8=B;b8=C; goto nmcopysmallfinish;
oAbfm6: 	RFL(278) a8=C;b8=A; goto nmcopysmallfinish;
oAbfm7: 	RFL(279) a8=D;b8=C; goto nmcopysmallfinish;
oAbfm8: 	RFL(280) a8=B;b8=A; goto nmcopysmallfinish;
oAbfm9: 	RFL(281) a8=C;b8=B; goto nmcopysmallfinish;
oAbfmA: 	RFL(282) a8=A;b8=C; goto nmcopysmallfinish;
oAbfmB: 	RFL(283) a8=C;b8=D;
			nmcopysmallfinish:
			a8+=3;b8+=3;
			asm 
			{
				move.b (a3)+,(a2)+
				move.l (a3)+,(a2)+
				move.l (a3)+,(a2)+
				move.w (a3)+,(a2)+
				move.b (a3),(a2)
			}
			pc+=3;goto o_done;
			
oAbfb4: 	RFL(276) a8=A;b8=B; goto nbcopysmallfinish;
oAbfb5: 	RFL(277) a8=B;b8=C; goto nbcopysmallfinish;
oAbfb6: 	RFL(278) a8=C;b8=A; goto nbcopysmallfinish;
oAbfb7: 	RFL(279) a8=D;b8=C; goto nbcopysmallfinish;
oAbfb8: 	RFL(280) a8=B;b8=A; goto nbcopysmallfinish;
oAbfb9: 	RFL(281) a8=C;b8=B; goto nbcopysmallfinish;
oAbfbA: 	RFL(282) a8=A;b8=C; goto nbcopysmallfinish;
oAbfbB: 	RFL(283) a8=C;b8=D;
			nbcopysmallfinish:
			asm
			{
				move.w (a3),(a2)
			}
			pc+=3;goto o_done;

oAbfw4: 	RFL(276) a8=A;b8=B; goto nwcopysmallfinish;
oAbfw5: 	RFL(277) a8=B;b8=C; goto nwcopysmallfinish;
oAbfw6: 	RFL(278) a8=C;b8=A; goto nwcopysmallfinish;
oAbfw7: 	RFL(279) a8=D;b8=C; goto nwcopysmallfinish;
oAbfw8: 	RFL(280) a8=B;b8=A; goto nwcopysmallfinish;
oAbfw9: 	RFL(281) a8=C;b8=B; goto nwcopysmallfinish;
oAbfwA: 	RFL(282) a8=A;b8=C; goto nwcopysmallfinish;
oAbfwB: 	RFL(283) a8=C;b8=D;
			nwcopysmallfinish:
			asm
			{
				move.l (a3)+,(a2)+
			 	move.l (a3)+,(a2)+
				move.l (a3)+,(a2)+
				move.l (a3),(a2)
			}
			pc+=3;goto o_done;
			
			
			
			
			

/**********************************************************************************
*                                                                                 *
*         NFxchg (small)                                                          *
*                                                                                 *
**********************************************************************************/

oAbfpC: 	RFL(284) a8=A;b8=B; goto npxchgsmallfinish;
oAbfpD: 	RFL(285) a8=B;b8=C; goto npxchgsmallfinish;
oAbfpE: 	RFL(286) a8=C;b8=A; goto npxchgsmallfinish;
oAbfpF: 	RFL(287) a8=D;b8=C; 
			npxchgsmallfinish:c8=X;
			*c8=*(a8+P);
			*(a8+P)=*(b8+P);
			*(b8+P)=*c8;
			pc+=3;goto o_done;

oAbfwpC: 	RFL(284) a8=A;b8=B; goto nwpxchgsmallfinish;
oAbfwpD: 	RFL(285) a8=B;b8=C; goto nwpxchgsmallfinish;
oAbfwpE: 	RFL(286) a8=C;b8=A; goto nwpxchgsmallfinish;
oAbfwpF: 	RFL(287) a8=D;b8=C; 
			nwpxchgsmallfinish:c8=X;
			n=(char)P+1;
			do {*c8=*a8;*a8++=*b8;*b8++=*c8++;} while (--n);
			pc+=3;goto o_done;

oAbfxsC: 	RFL(284) a8=A;b8=B; goto nxsxchgsmallfinish;
oAbfxsD: 	RFL(285) a8=B;b8=C; goto nxsxchgsmallfinish;
oAbfxsE: 	RFL(286) a8=C;b8=A; goto nxsxchgsmallfinish;
oAbfxsF: 	RFL(287) a8=D;b8=C; 
			nxsxchgsmallfinish:c8=X;
			*c8=*(a8+2);
			*(a8+2)=*(b8+2);
			*(b8+2)=*c8;
			pc+=3;goto o_done;

oAbfxC: 	RFL(284) a8=A;b8=B; goto nxxchgsmallfinish;
oAbfxD: 	RFL(285) a8=B;b8=C; goto nxxchgsmallfinish;
oAbfxE: 	RFL(286) a8=C;b8=A; goto nxxchgsmallfinish;
oAbfxF: 	RFL(287) a8=D;b8=C; 
			nxxchgsmallfinish:c8=X;
			asm	
			{
				move.w (a2),(a4)
				move.w (a3),(a2)+
				move.w (a4)+,(a3)+
				move.b (a2),(a4)
				move.b (a3),(a2)
				move.b (a4),(a3)
			}
			pc+=3;goto o_done;

oAbfsC: 	RFL(284) a8=A;b8=B; goto nsxchgsmallfinish;
oAbfsD: 	RFL(285) a8=B;b8=C; goto nsxchgsmallfinish;
oAbfsE: 	RFL(286) a8=C;b8=A; goto nsxchgsmallfinish;
oAbfsF: 	RFL(287) a8=D;b8=C; 
			nsxchgsmallfinish:c8=X;
			*c8=*(a8+15);
			*(a8+15)=*(b8+15);
			*(b8+15)=*c8;
			pc+=3;goto o_done;

oAbfmC: 	RFL(284) a8=A;b8=B; goto nmxchgsmallfinish;
oAbfmD: 	RFL(285) a8=B;b8=C; goto nmxchgsmallfinish;
oAbfmE: 	RFL(286) a8=C;b8=A; goto nmxchgsmallfinish;
oAbfmF: 	RFL(287) a8=D;b8=C; 
			nmxchgsmallfinish:c8=X;
			a8+=3;b8+=3;c8+=3;
			asm 
			{
				move.b (a2),(a4)
				move.b (a3),(a2)+
				move.b (a4)+,(a3)+
				move.l (a2),(a4)
				move.l (a3),(a2)+
				move.l (a4)+,(a3)+
				move.l (a2),(a4)
				move.l (a3),(a2)+
				move.l (a4)+,(a3)+
				move.w (a2),(a4)
				move.w (a3),(a2)+
				move.w (a4)+,(a3)+
				move.b (a2),(a4)
				move.b (a3),(a2)
				move.b (a4),(a3)
			}
			pc+=3;goto o_done;
			
oAbfbC: 	RFL(284) a8=A;b8=B; goto nbxchgsmallfinish;
oAbfbD: 	RFL(285) a8=B;b8=C; goto nbxchgsmallfinish;
oAbfbE: 	RFL(286) a8=C;b8=A; goto nbxchgsmallfinish;
oAbfbF: 	RFL(287) a8=D;b8=C; 
			nbxchgsmallfinish:c8=X;
			asm
			{
				move.w (a2),(a4)
				move.w (a3),(a2)
				move.w (a4),(a3)
			}
			pc+=3;goto o_done;

oAbfwC: 	RFL(284) a8=A;b8=B; goto nwxchgsmallfinish;
oAbfwD: 	RFL(285) a8=B;b8=C; goto nwxchgsmallfinish;
oAbfwE: 	RFL(286) a8=C;b8=A; goto nwxchgsmallfinish;
oAbfwF: 	RFL(287) a8=D;b8=C; 
			nwxchgsmallfinish:c8=X;
			asm
			{
				move.l (a2),(a4)
				move.l (a3),(a2)+
				move.l (a4)+,(a3)+
				move.l (a2),(a4)
				move.l (a3),(a2)+
				move.l (a4)+,(a3)+
				move.l (a2),(a4)
				move.l (a3),(a2)+
				move.l (a4)+,(a3)+
				move.l (a2),(a4)
				move.l (a3),(a2)
				move.l (a4),(a3)
			}
			pc+=3;goto o_done;




/**********************************************************************************
*                                                                                 *
*         NFinc                                                                   *
*                                                                                 *
**********************************************************************************/

oBfp4: 		RFL(293) a8=A; goto npincfinish;
oBfp5: 		RFL(294) a8=B; goto npincfinish;
oBfp6: 		RFL(295) a8=C; goto npincfinish;
oBfp7: 		RFL(296) a8=D; 
			npincfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=P;
			if (++(*a8)<m) { goto npincend; } *a8++=0;
			CARRY=1;
			npincend:pc+=3; goto o_done;

oBfwp4: 	RFL(293) a8=A; goto nwpincfinish;
oBfwp5: 	RFL(294) a8=B; goto nwpincfinish;
oBfwp6: 	RFL(295) a8=C; goto nwpincfinish;
oBfwp7: 	RFL(296) a8=D; 
			nwpincfinish:
			CARRY=0;
			n=(char)P+1;
			m=(char)MODE;
			do
			{
				if (++(*a8)<m) { goto nwpincend; }
				*a8++=0;
			} while (--n);
			CARRY = 1;
			nwpincend:pc+=3; goto o_done;
			
oBfxs4: 	RFL(293) a8=A; goto nxsincfinish;
oBfxs5: 	RFL(294) a8=B; goto nxsincfinish;
oBfxs6: 	RFL(295) a8=C; goto nxsincfinish;
oBfxs7: 	RFL(296) a8=D; 
			nxsincfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=2;
			if (++(*a8)<m) { goto nxsincend; } *a8++=0;
			CARRY=1;
			nxsincend:pc+=3; goto o_done;

oBfx4: 		RFL(293) a8=A; goto nxincfinish;
oBfx5: 		RFL(294) a8=B; goto nxincfinish;
oBfx6: 		RFL(295) a8=C; goto nxincfinish;
oBfx7: 		RFL(296) a8=D; 
			nxincfinish:
			CARRY=0;
			m=(char)MODE;
			if (++(*a8)<m) { goto nxincend; } *a8++=0;
			if (++(*a8)<m) { goto nxincend; } *a8++=0;
			if (++(*a8)<m) { goto nxincend; } *a8++=0;
			CARRY=1;
			nxincend:pc+=3; goto o_done;

oBfs4: 		RFL(293) a8=A; goto nsincfinish;
oBfs5: 		RFL(294) a8=B; goto nsincfinish;
oBfs6: 		RFL(295) a8=C; goto nsincfinish;
oBfs7: 		RFL(296) a8=D; 
			nsincfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=15;
			if (++(*a8)<m) { goto nsincend; } *a8++=0;
			CARRY=1;
			nsincend:pc+=3; goto o_done;

oBfm4: 		RFL(293) a8=A; goto nmincfinish;
oBfm5: 		RFL(294) a8=B; goto nmincfinish;
oBfm6: 		RFL(295) a8=C; goto nmincfinish;
oBfm7: 		RFL(296) a8=D; 
			nmincfinish:
			CARRY=0;
			m=(char)MODE;
			a8+=3;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			if (++(*a8)<m) { goto nmincend; } *a8++=0;
			CARRY=1;
			nmincend:pc+=3; goto o_done;

oBfb4: 		RFL(293) a8=A; goto nbincfinish;
oBfb5: 		RFL(294) a8=B; goto nbincfinish;
oBfb6: 		RFL(295) a8=C; goto nbincfinish;
oBfb7: 		RFL(296) a8=D; 
			nbincfinish:
			CARRY=0;
			m=(char)MODE;
			if (++(*a8)<m) { goto nbincend; } *a8++=0;
			if (++(*a8)<m) { goto nbincend; } *a8++=0;
			CARRY=1;
			nbincend:pc+=3; goto o_done;

oBfw4: 		RFL(293) a8=A; goto nwincfinish;
oBfw5: 		RFL(294) a8=B; goto nwincfinish;
oBfw6: 		RFL(295) a8=C; goto nwincfinish;
oBfw7: 		RFL(296) a8=D; 
			nwincfinish:
			CARRY=0;
			m=(char)MODE;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			if (++(*a8)<m) { goto nwincend; } *a8++=0;
			CARRY=1;
			nwincend:pc+=3; goto o_done;




/**********************************************************************************
*                                                                                 *
*         NFsl                                                                    *
*                                                                                 *
**********************************************************************************/

oBbfp0: 	RFL(305) a8=A; goto npslfinish;
oBbfp1: 	RFL(306) a8=B; goto npslfinish;
oBbfp2: 	RFL(307) a8=C; goto npslfinish;
oBbfp3: 	RFL(308) a8=D;
			npslfinish:
			a8+=P;
			*a8=0;
			pc+=3; goto o_done;

oBbfwp0: 	RFL(305) a8=A; goto nwpslfinish;
oBbfwp1: 	RFL(306) a8=B; goto nwpslfinish;
oBbfwp2: 	RFL(307) a8=C; goto nwpslfinish;
oBbfwp3: 	RFL(308) a8=D;
			nwpslfinish:
			n=(char)P+1;
			a8+=n;
			b8=a8-1;
			while (--n) {*--a8 = *--b8;}
			*--a8=0;
			pc+=3; goto o_done;

oBbfxs0: 	RFL(305) a8=A; goto nxsslfinish;
oBbfxs1: 	RFL(306) a8=B; goto nxsslfinish;
oBbfxs2: 	RFL(307) a8=C; goto nxsslfinish;
oBbfxs3: 	RFL(308) a8=D;
			nxsslfinish:
			a8+=2;
			*a8=0;
			pc+=3; goto o_done;

oBbfx0: 	RFL(305) a8=A; goto nxslfinish;
oBbfx1: 	RFL(306) a8=B; goto nxslfinish;
oBbfx2: 	RFL(307) a8=C; goto nxslfinish;
oBbfx3: 	RFL(308) a8=D;
			nxslfinish:
			a8+=3;
			b8=a8-1;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8=0;
			pc+=3; goto o_done;

oBbfs0: 	RFL(305) a8=A; goto nsslfinish;
oBbfs1: 	RFL(306) a8=B; goto nsslfinish;
oBbfs2: 	RFL(307) a8=C; goto nsslfinish;
oBbfs3: 	RFL(308) a8=D;
			nsslfinish:
			a8+=15;
			*a8=0;
			pc+=3; goto o_done;

oBbfm0: 	RFL(305) a8=A; goto nmslfinish;
oBbfm1: 	RFL(306) a8=B; goto nmslfinish;
oBbfm2: 	RFL(307) a8=C; goto nmslfinish;
oBbfm3: 	RFL(308) a8=D;
			nmslfinish:
			a8+=15;
			b8=a8-1;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8=0;
			pc+=3; goto o_done;

oBbfb0: 	RFL(305) a8=A; goto nbslfinish;
oBbfb1: 	RFL(306) a8=B; goto nbslfinish;
oBbfb2: 	RFL(307) a8=C; goto nbslfinish;
oBbfb3: 	RFL(308) a8=D;
			nbslfinish:
			a8+=2;
			b8=a8-1;
			*--a8 = *--b8;
			*--a8=0;
			pc+=3; goto o_done;

oBbfw0: 	RFL(305) a8=A; goto nwslfinish;
oBbfw1: 	RFL(306) a8=B; goto nwslfinish;
oBbfw2: 	RFL(307) a8=C; goto nwslfinish;
oBbfw3: 	RFL(308) a8=D;
			nwslfinish:
			a8+=16;
			b8=a8-1;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8 = *--b8;
			*--a8=0;
			pc+=3; goto o_done;






/**********************************************************************************
*                                                                                 *
*         NFsr                                                                    *
*                                                                                 *
**********************************************************************************/

oBbfp4: 	RFL(309) a8=A; goto npsrfinish;
oBbfp5: 	RFL(310) a8=B; goto npsrfinish;
oBbfp6: 	RFL(311) a8=C; goto npsrfinish;
oBbfp7: 	RFL(312) a8=D;
			npsrfinish:
			a8+=P;
			if (*a8) HST|=SB;
			*a8=0;
			pc+=3; goto o_done;

oBbfwp4: 	RFL(309) a8=A; goto nwpsrfinish;
oBbfwp5: 	RFL(310) a8=B; goto nwpsrfinish;
oBbfwp6: 	RFL(311) a8=C; goto nwpsrfinish;
oBbfwp7: 	RFL(312) a8=D;
			nwpsrfinish:
			b8=a8+1;
			n=(char)P+1;
			if (*a8) HST|=SB;
			while (--n) {*a8++ = *b8++;}
			*a8=0;
			pc+=3; goto o_done;

oBbfxs4: 	RFL(309) a8=A; goto nxssrfinish;
oBbfxs5: 	RFL(310) a8=B; goto nxssrfinish;
oBbfxs6: 	RFL(311) a8=C; goto nxssrfinish;
oBbfxs7: 	RFL(312) a8=D;
			nxssrfinish:
			a8+=2;
			if (*a8) HST|=SB;
			*a8=0;
			pc+=3; goto o_done;

oBbfx4: 	RFL(309) a8=A; goto nxsrfinish;
oBbfx5: 	RFL(310) a8=B; goto nxsrfinish;
oBbfx6: 	RFL(311) a8=C; goto nxsrfinish;
oBbfx7: 	RFL(312) a8=D;
			nxsrfinish:
			b8=a8+1;
			if (*a8) HST|=SB;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8=0;
			pc+=3; goto o_done;

oBbfs4: 	RFL(309) a8=A; goto nssrfinish;
oBbfs5: 	RFL(310) a8=B; goto nssrfinish;
oBbfs6: 	RFL(311) a8=C; goto nssrfinish;
oBbfs7: 	RFL(312) a8=D;
			nssrfinish:
			a8+=15;
			if (*a8) HST|=SB;
			*a8=0;
			pc+=3; goto o_done;

oBbfm4: 	RFL(309) a8=A; goto nmsrfinish;
oBbfm5: 	RFL(310) a8=B; goto nmsrfinish;
oBbfm6: 	RFL(311) a8=C; goto nmsrfinish;
oBbfm7: 	RFL(312) a8=D;
			nmsrfinish:
			a8+=3;b8=a8+1;
			if (*a8) HST|=SB;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8=0;
			pc+=3; goto o_done;

oBbfb4: 	RFL(309) a8=A; goto nbsrfinish;
oBbfb5: 	RFL(310) a8=B; goto nbsrfinish;
oBbfb6: 	RFL(311) a8=C; goto nbsrfinish;
oBbfb7: 	RFL(312) a8=D;
			nbsrfinish:
			b8=a8+1;
			if (*a8) HST|=SB;
			*a8++ = *b8++;
			*a8=0;
			pc+=3; goto o_done;

oBbfw4: 	RFL(309) a8=A; goto nwsrfinish;
oBbfw5: 	RFL(310) a8=B; goto nwsrfinish;
oBbfw6: 	RFL(311) a8=C; goto nwsrfinish;
oBbfw7: 	RFL(312) a8=D;
			nwsrfinish:
			b8=a8+1;
			if (*a8) HST|=SB;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8++ = *b8++;
			*a8=0;
			pc+=3; goto o_done;






/**********************************************************************************
*                                                                                 *
*         NFneg                                                                   *
*                                                                                 *
**********************************************************************************/

oBbfp8: 	RFL(313) a8=A; goto npnegfinish;    // P field, 1 nibble, [P]
oBbfp9: 	RFL(314) a8=B; goto npnegfinish;
oBbfpA: 	RFL(315) a8=C; goto npnegfinish;
oBbfpB: 	RFL(316) a8=D; 
			npnegfinish:
			CARRY=0;
			a8+=P;
			m=(char)MODE_MINUS_ONE;
			if (*a8) { *a8 = m - *a8 + 1; CARRY=1;}
			pc+=3; goto o_done;

oBbfwp8: 	RFL(313) a8=A; goto nwpnegfinish;   // WP field, P+1 nibble, [0..P]
oBbfwp9: 	RFL(314) a8=B; goto nwpnegfinish;
oBbfwpA: 	RFL(315) a8=C; goto nwpnegfinish;
oBbfwpB: 	RFL(316) a8=D; 
			nwpnegfinish:
			CARRY=0;
			n=(char)P+1;
			m=(char)MODE_MINUS_ONE;
			do { if (*a8++) break; } while (--n);
			a8--;
			if (n==0) {CARRY=0; goto nwpnegend; }
			CARRY=1;
			*a8++ = ((char)MODE)-(*a8);
			n--;
			if (n==0) goto nwpnegend;
			do { *a8++=(char)(m-(*a8)); } while (--n);
			nwpnegend:pc+=3; goto o_done;
			
oBbfxs8: 	RFL(313) a8=A; goto nxsnegfinish;   // XS field, 1 nibble, [2]
oBbfxs9: 	RFL(314) a8=B; goto nxsnegfinish;
oBbfxsA: 	RFL(315) a8=C; goto nxsnegfinish;
oBbfxsB: 	RFL(316) a8=D; 
			nxsnegfinish:
			CARRY=0;
			a8+=2;
			m=(char)MODE_MINUS_ONE;
			if (*a8) { *a8 = m - *a8 + 1; CARRY=1;}
			pc+=3; goto o_done;

oBbfx8: 	RFL(313) a8=A; goto nxnegfinish;    // X field, 3 nibbles, [0..2]
oBbfx9: 	RFL(314) a8=B; goto nxnegfinish;
oBbfxA: 	RFL(315) a8=C; goto nxnegfinish;
oBbfxB: 	RFL(316) a8=D; 
			nxnegfinish:
			m=(char)MODE_MINUS_ONE;
			if (*a8++) {(*--a8)--;goto nxnegend1;}
			if (*a8++) {(*--a8)--;goto nxnegend2;}
			if (*a8) {(*a8)--;goto nxnegend3;}
			CARRY=0;
			goto nxnegendxx;
			nxnegend1: *a8++ = m-(*a8);
			nxnegend2: *a8++ = m-(*a8);
			nxnegend3: *a8 = m-(*a8);
			CARRY=1;
			nxnegendxx:pc+=3; goto o_done;

oBbfs8: 	RFL(313) a8=A; goto nsnegfinish;    // S field, 1 nibble, [15]
oBbfs9: 	RFL(314) a8=B; goto nsnegfinish;
oBbfsA: 	RFL(315) a8=C; goto nsnegfinish;
oBbfsB: 	RFL(316) a8=D; 
			nsnegfinish:
			CARRY=0;
			a8+=15;
			m=(char)MODE_MINUS_ONE;
			if (*a8) { *a8 = m - *a8 + 1; CARRY=1;}
			pc+=3; goto o_done;

oBbfm8: 	RFL(313) a8=A; goto nmnegfinish;    // M field, 12 nibbles, [3..14]
oBbfm9: 	RFL(314) a8=B; goto nmnegfinish;
oBbfmA: 	RFL(315) a8=C; goto nmnegfinish;
oBbfmB: 	RFL(316) a8=D; 
			nmnegfinish:
			a8+=3;
			m=(char)MODE_MINUS_ONE;
			if (*a8++) {(*--a8)--;goto nmnegend1;}
			if (*a8++) {(*--a8)--;goto nmnegend2;}
			if (*a8++) {(*--a8)--;goto nmnegend3;}
			if (*a8++) {(*--a8)--;goto nmnegend4;}
			if (*a8++) {(*--a8)--;goto nmnegend5;}
			if (*a8++) {(*--a8)--;goto nmnegend6;}
			if (*a8++) {(*--a8)--;goto nmnegend7;}
			if (*a8++) {(*--a8)--;goto nmnegend8;}
			if (*a8++) {(*--a8)--;goto nmnegend9;}
			if (*a8++) {(*--a8)--;goto nmnegend10;}
			if (*a8++) {(*--a8)--;goto nmnegend11;}
			if (*a8) {(*a8)--;goto nmnegend12;}
			CARRY=0;
			goto nmnegendxx;
			nmnegend1: *a8++ = m-(*a8);
			nmnegend2: *a8++ = m-(*a8);
			nmnegend3: *a8++ = m-(*a8);
			nmnegend4: *a8++ = m-(*a8);
			nmnegend5: *a8++ = m-(*a8);
			nmnegend6: *a8++ = m-(*a8);
			nmnegend7: *a8++ = m-(*a8);
			nmnegend8: *a8++ = m-(*a8);
			nmnegend9: *a8++ = m-(*a8);
			nmnegend10: *a8++ = m-(*a8);
			nmnegend11: *a8++ = m-(*a8);
			nmnegend12: *a8 = m-(*a8);
			CARRY=1;
			nmnegendxx:pc+=3; goto o_done;

oBbfb8: 	RFL(313) a8=A; goto nbnegfinish;
oBbfb9: 	RFL(314) a8=B; goto nbnegfinish;    // B field, 2 nibbles, [0..1]
oBbfbA: 	RFL(315) a8=C; goto nbnegfinish;
oBbfbB: 	RFL(316) a8=D; 
			nbnegfinish:
			m=(char)MODE_MINUS_ONE;
			if (*a8++) {(*--a8)--;goto nbnegend1;}
			if (*a8) {(*a8)--;goto nbnegend2;}
			CARRY=0;
			goto nbnegendxx;
			nbnegend1: *a8++ = m-(*a8);
			nbnegend2: *a8 = m-(*a8);
			CARRY=1;
			nbnegendxx:pc+=3; goto o_done;

oBbfw8: 	RFL(313) a8=A; goto nwnegfinish;    // W field, 16 nibbles, [0..15]
oBbfw9: 	RFL(314) a8=B; goto nwnegfinish;
oBbfwA: 	RFL(315) a8=C; goto nwnegfinish;
oBbfwB: 	RFL(316) a8=D; 
			nwnegfinish:
			m=(char)MODE_MINUS_ONE;
			if (*a8++) {(*--a8)--;goto nwnegend1;}
			if (*a8++) {(*--a8)--;goto nwnegend2;}
			if (*a8++) {(*--a8)--;goto nwnegend3;}
			if (*a8++) {(*--a8)--;goto nwnegend4;}
			if (*a8++) {(*--a8)--;goto nwnegend5;}
			if (*a8++) {(*--a8)--;goto nwnegend6;}
			if (*a8++) {(*--a8)--;goto nwnegend7;}
			if (*a8++) {(*--a8)--;goto nwnegend8;}
			if (*a8++) {(*--a8)--;goto nwnegend9;}
			if (*a8++) {(*--a8)--;goto nwnegend10;}
			if (*a8++) {(*--a8)--;goto nwnegend11;}
			if (*a8++) {(*--a8)--;goto nwnegend12;}
			if (*a8++) {(*--a8)--;goto nwnegend13;}
			if (*a8++) {(*--a8)--;goto nwnegend14;}
			if (*a8++) {(*--a8)--;goto nwnegend15;}
			if (*a8) {(*a8)--;goto nwnegend16;}
			CARRY=0;
			goto nwnegendxx;
			nwnegend1: *a8++ = m-(*a8);
			nwnegend2: *a8++ = m-(*a8);
			nwnegend3: *a8++ = m-(*a8);
			nwnegend4: *a8++ = m-(*a8);
			nwnegend5: *a8++ = m-(*a8);
			nwnegend6: *a8++ = m-(*a8);
			nwnegend7: *a8++ = m-(*a8);
			nwnegend8: *a8++ = m-(*a8);
			nwnegend9: *a8++ = m-(*a8);
			nwnegend10: *a8++ = m-(*a8);
			nwnegend11: *a8++ = m-(*a8);
			nwnegend12: *a8++ = m-(*a8);
			nwnegend13: *a8++ = m-(*a8);
			nwnegend14: *a8++ = m-(*a8);
			nwnegend15: *a8++ = m-(*a8);
			nwnegend16: *a8 = m-(*a8);
			CARRY=1;
			nwnegendxx:pc+=3; goto o_done;





/**********************************************************************************
*                                                                                 *
*         NFnot                                                                   *
*                                                                                 *
**********************************************************************************/

oBbfpC: 	RFL(317) a8=A; goto npnotfinish;    // P field, 1 nibble, [P]
oBbfpD: 	RFL(318) a8=B; goto npnotfinish;
oBbfpE: 	RFL(319) a8=C; goto npnotfinish;
oBbfpF: 	RFL(320) a8=D; 
			npnotfinish:
			CARRY=0;
			a8+=P;
			m=(char)MODE_MINUS_ONE;
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfwpC: 	RFL(317) a8=A; goto nwpnotfinish;   // WP field, P+1 nibble, [0..P]
oBbfwpD: 	RFL(318) a8=B; goto nwpnotfinish;
oBbfwpE: 	RFL(319) a8=C; goto nwpnotfinish;
oBbfwpF: 	RFL(320) a8=D; 
			nwpnotfinish:
			CARRY=0;
			n=(char)P+1;
			m=(char)MODE_MINUS_ONE;
			do { *a8++=(m-*a8); } while (--n);
			pc+=3; goto o_done;
			
oBbfxsC: 	RFL(317) a8=A; goto nxsnotfinish;   // XS field, 1 nibble, [2]
oBbfxsD: 	RFL(318) a8=B; goto nxsnotfinish;
oBbfxsE: 	RFL(319) a8=C; goto nxsnotfinish;
oBbfxsF: 	RFL(320) a8=D; 
			nxsnotfinish:
			CARRY=0;
			a8+=2;
			m=(char)MODE_MINUS_ONE;
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfxC: 	RFL(317) a8=A; goto nxnotfinish;    // X field, 3 nibbles, [0..2]
oBbfxD: 	RFL(318) a8=B; goto nxnotfinish;
oBbfxE: 	RFL(319) a8=C; goto nxnotfinish;
oBbfxF: 	RFL(320) a8=D; 
			nxnotfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfsC: 	RFL(317) a8=A; goto nsnotfinish;    // S field, 1 nibble, [15]
oBbfsD: 	RFL(318) a8=B; goto nsnotfinish;
oBbfsE: 	RFL(319) a8=C; goto nsnotfinish;
oBbfsF: 	RFL(320) a8=D; 
			nsnotfinish:
			CARRY=0;
			a8+=15;
			m=(char)MODE_MINUS_ONE;
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfmC: 	RFL(317) a8=A; goto nmnotfinish;    // M field, 12 nibbles, [3..14]
oBbfmD: 	RFL(318) a8=B; goto nmnotfinish;
oBbfmE: 	RFL(319) a8=C; goto nmnotfinish;
oBbfmF: 	RFL(320) a8=D; 
			nmnotfinish:
			CARRY=0;
			a8+=3;
			m=(char)MODE_MINUS_ONE;
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfbC: 	RFL(317) a8=A; goto nbnotfinish;
oBbfbD: 	RFL(318) a8=B; goto nbnotfinish;    // B field, 2 nibbles, [0..1]
oBbfbE: 	RFL(319) a8=C; goto nbnotfinish;
oBbfbF: 	RFL(320) a8=D; 
			nbnotfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			*a8++=(m-*a8);
			*a8=(m-*a8);
			pc+=3; goto o_done;

oBbfwC: 	RFL(317) a8=A; goto nwnotfinish;    // W field, 16 nibbles, [0..15]
oBbfwD: 	RFL(318) a8=B; goto nwnotfinish;
oBbfwE: 	RFL(319) a8=C; goto nwnotfinish;
oBbfwF: 	RFL(320) a8=D; 
			nwnotfinish:
			CARRY=0;
			m=(char)MODE_MINUS_ONE;
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8++=(m-*a8);
			*a8=(m-*a8);
			pc+=3; goto o_done;








o8A0: 		RFL(188) pc+=3; Te5(A, B); goto o_goyes;
o8A1: 		RFL(189) pc+=3; Te5(B, C); goto o_goyes;
o8A2:		RFL(190) pc+=3; Te5(C, A); goto o_goyes;
o8A3: 		RFL(191) pc+=3; Te5(D, C); goto o_goyes;
o8A4: 		RFL(192) pc+=3; Tne5(A, B); goto o_goyes;
o8A5: 		RFL(193) pc+=3; Tne5(B, C); goto o_goyes;
o8A6: 		RFL(194) pc+=3; Tne5(C, A); goto o_goyes;
o8A7: 		RFL(195) pc+=3; Tne5(D, C); goto o_goyes;
o8A8: 		RFL(196) pc+=3; Tz5(A); goto o_goyes;
o8A9: 		RFL(197) pc+=3; Tz5(B); goto o_goyes;
o8AA: 		RFL(198) pc+=3; Tz5(C); goto o_goyes;
o8AB: 		RFL(199) pc+=3; Tz5(D); goto o_goyes;
o8AC: 		RFL(200) pc+=3; Tnz5(A); goto o_goyes;
o8AD: 		RFL(201) pc+=3; Tnz5(B); goto o_goyes;
o8AE: 		RFL(202) pc+=3; Tnz5(C); goto o_goyes;
o8AF: 		RFL(203) pc+=3; Tnz5(D); goto o_goyes;
o8B0: 		RFL(204) pc+=3; Ta5(A, B); goto o_goyes;
o8B1: 		RFL(205) pc+=3; Ta5(B, C); goto o_goyes;
o8B2: 		RFL(206) pc+=3; Ta5(C, A); goto o_goyes;
o8B3: 		RFL(207) pc+=3; Ta5(D, C); goto o_goyes;
o8B4: 		RFL(208) pc+=3; Tb5(A, B); goto o_goyes;
o8B5: 		RFL(209) pc+=3; Tb5(B, C); goto o_goyes;
o8B6: 		RFL(210) pc+=3; Tb5(C, A); goto o_goyes;
o8B7: 		RFL(211) pc+=3; Tb5(D, C); goto o_goyes;
o8B8: 		RFL(212) pc+=3; Tae5(A, B); goto o_goyes;
o8B9: 		RFL(213) pc+=3; Tae5(B, C); goto o_goyes;
o8BA: 		RFL(214) pc+=3; Tae5(C, A); goto o_goyes;
o8BB: 		RFL(215) pc+=3; Tae5(D, C); goto o_goyes;
o8BC: 		RFL(216) pc+=3; Tbe5(A, B); goto o_goyes;
o8BD: 		RFL(217) pc+=3; Tbe5(B, C); goto o_goyes;
o8BE: 		RFL(218) pc+=3; Tbe5(C, A); goto o_goyes;
o8BF: 		RFL(219) pc+=3; Tbe5(D, C); goto o_goyes;
o8Cd4: 		RFL(220) d=Npack4I2ASM_m3(); if (d&0x8000) {pc-=(d^0xffff)-1;PCMINUSd0} else {pc+=d+2;PCPLUSd0} goto o_done;
o8Dd5: 		RFL(221) pc=Npack5I2ASM(); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o8Ed4: 		RFL(222) rstkpush_pc6(); d=Npack4I2ASM_m3(); if (d&0x8000) {pc-=(d^0xffff)-5;PCMINUSd0} else {pc+=d+6;PCPLUSd0} goto o_done;
o8Fd5: 		RFL(223) rstkpush_pc7(); pc=Npack5I2ASM(); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3





/**********************************************************************************
*                                                                                 *
*         TFe                                                                     *
*                                                                                 *
**********************************************************************************/

o9fp0: 		RFL(224) a8=A;b8=B; goto tpefinish;    // P field, 1 nibble, [P]
o9fp1: 		RFL(225) a8=B;b8=C; goto tpefinish;
o9fp2: 		RFL(226) a8=C;b8=A; goto tpefinish;
o9fp3: 		RFL(227) a8=D;b8=C; 
			tpefinish:
			a8+=P;
			b8+=P;
			CARRY=(*a8==*b8);
			pc+=3; goto o_goyes;

o9fwp0: 	RFL(224) a8=A;b8=B; goto twpefinish;   // WP field, P+1 nibble, [0..P]
o9fwp1: 	RFL(225) a8=B;b8=C; goto twpefinish;
o9fwp2: 	RFL(226) a8=C;b8=A; goto twpefinish;
o9fwp3: 	RFL(227) a8=D;b8=C; 
			twpefinish:
			n=(char)P+1;
			do { if (*a8++!=*b8++) { CARRY = 0; pc+=3; goto o_goyes; } } while (--n);
			CARRY=1;
			pc+=3; goto o_goyes;
			
o9fxs0: 	RFL(224) a8=A;b8=B; goto txsefinish;   // XS field, 1 nibble, [2]
o9fxs1: 	RFL(225) a8=B;b8=C; goto txsefinish;
o9fxs2: 	RFL(226) a8=C;b8=A; goto txsefinish;
o9fxs3: 	RFL(227) a8=D;b8=C; 
			txsefinish:
			a8+=2;
			b8+=2;
			CARRY=(*a8==*b8);
			pc+=3; goto o_goyes;

o9fx0: 		RFL(224) a8=A;b8=B; goto txefinish;    // X field, 3 nibbles, [0..2]
o9fx1: 		RFL(225) a8=B;b8=C; goto txefinish;
o9fx2: 		RFL(226) a8=C;b8=A; goto txefinish;
o9fx3: 		RFL(227) a8=D;b8=C; 
			txefinish:
			if (*(UInt16*)a8 != *(UInt16*)b8) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+2) == *(UInt8*)(b8+2));
			pc+=3; goto o_goyes;

o9fs0: 		RFL(224) a8=A;b8=B; goto tsefinish;    // S field, 1 nibble, [15]
o9fs1: 		RFL(225) a8=B;b8=C; goto tsefinish;
o9fs2: 		RFL(226) a8=C;b8=A; goto tsefinish;
o9fs3: 		RFL(227) a8=D;b8=C; 
			tsefinish:
			a8+=15;
			b8+=15;
			CARRY=(*a8==*b8);
			pc+=3; goto o_goyes;

o9fm0: 		RFL(224) a8=A;b8=B; goto tmefinish;    // M field, 12 nibbles, [3..14]
o9fm1: 		RFL(225) a8=B;b8=C; goto tmefinish;
o9fm2: 		RFL(226) a8=C;b8=A; goto tmefinish;
o9fm3: 		RFL(227) a8=D;b8=C; 
			tmefinish:
			a8+=3;
			b8+=3;
			if (*(UInt8*)a8 != *(UInt8*)b8) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+1) != *(UInt32*)(b8+1)) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+5) != *(UInt32*)(b8+5)) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt16*)(a8+9) != *(UInt16*)(b8+9)) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+11) == *(UInt8*)(b8+11));
			pc+=3; goto o_goyes;

o9fb0: 		RFL(224) a8=A;b8=B; goto tbefinish;
o9fb1: 		RFL(225) a8=B;b8=C; goto tbefinish;    // B field, 2 nibbles, [0..1]
o9fb2: 		RFL(226) a8=C;b8=A; goto tbefinish;
o9fb3: 		RFL(227) a8=D;b8=C; 
			tbefinish:
			CARRY = (*(UInt16*)a8 == *(UInt16*)b8);
			pc+=3; goto o_goyes;

o9fw0: 		RFL(224) a8=A;b8=B; goto twefinish;    // W field, 16 nibbles, [0..15]
o9fw1: 		RFL(225) a8=B;b8=C; goto twefinish;
o9fw2: 		RFL(226) a8=C;b8=A; goto twefinish;
o9fw3: 		RFL(227) a8=D;b8=C; 
			twefinish:
			if (*(UInt32*)a8 != *(UInt32*)b8) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+4) != *(UInt32*)(b8+4)) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+8) != *(UInt32*)(b8+8)) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt32*)(a8+12) == *(UInt32*)(b8+12));
			pc+=3; goto o_goyes;






/**********************************************************************************
*                                                                                 *
*         TFne                                                                    *
*                                                                                 *
**********************************************************************************/

o9fp4: 		RFL(228) a8=A;b8=B; goto tpnefinish;    // P field, 1 nibble, [P]
o9fp5: 		RFL(229) a8=B;b8=C; goto tpnefinish;
o9fp6: 		RFL(230) a8=C;b8=A; goto tpnefinish;
o9fp7: 		RFL(231) a8=D;b8=C; 
			tpnefinish:
			a8+=P;
			b8+=P;
			CARRY=(*a8!=*b8);
			pc+=3; goto o_goyes;

o9fwp4: 	RFL(228) a8=A;b8=B; goto twpnefinish;   // WP field, P+1 nibble, [0..P]
o9fwp5: 	RFL(229) a8=B;b8=C; goto twpnefinish;
o9fwp6: 	RFL(230) a8=C;b8=A; goto twpnefinish;
o9fwp7: 	RFL(231) a8=D;b8=C; 
			twpnefinish:
			n=(char)P+1;
			do { if (*a8++!=*b8++) { CARRY = 1; pc+=3; goto o_goyes; } } while (--n);
			CARRY=0;
			pc+=3; goto o_goyes;
			
o9fxs4: 	RFL(228) a8=A;b8=B; goto txsnefinish;   // XS field, 1 nibble, [2]
o9fxs5: 	RFL(229) a8=B;b8=C; goto txsnefinish;
o9fxs6: 	RFL(230) a8=C;b8=A; goto txsnefinish;
o9fxs7: 	RFL(231) a8=D;b8=C; 
			txsnefinish:
			a8+=2;
			b8+=2;
			CARRY=(*a8!=*b8);
			pc+=3; goto o_goyes;

o9fx4: 		RFL(228) a8=A;b8=B; goto txnefinish;    // X field, 3 nibbles, [0..2]
o9fx5: 		RFL(229) a8=B;b8=C; goto txnefinish;
o9fx6: 		RFL(230) a8=C;b8=A; goto txnefinish;
o9fx7: 		RFL(231) a8=D;b8=C; 
			txnefinish:
			if (*(UInt16*)a8 != *(UInt16*)b8) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+2) != *(UInt8*)(b8+2));
			pc+=3; goto o_goyes;

o9fs4: 		RFL(228) a8=A;b8=B; goto tsnefinish;    // S field, 1 nibble, [15]
o9fs5: 		RFL(229) a8=B;b8=C; goto tsnefinish;
o9fs6: 		RFL(230) a8=C;b8=A; goto tsnefinish;
o9fs7: 		RFL(231) a8=D;b8=C; 
			tsnefinish:
			a8+=15;
			b8+=15;
			CARRY=(*a8!=*b8);
			pc+=3; goto o_goyes;

o9fm4: 		RFL(228) a8=A;b8=B; goto tmnefinish;    // M field, 12 nibbles, [3..14]
o9fm5: 		RFL(229) a8=B;b8=C; goto tmnefinish;
o9fm6: 		RFL(230) a8=C;b8=A; goto tmnefinish;
o9fm7: 		RFL(231) a8=D;b8=C; 
			tmnefinish:
			a8+=3;
			b8+=3;
			if (*(UInt8*)a8 != *(UInt8*)b8) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+1) != *(UInt32*)(b8+1)) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+5) != *(UInt32*)(b8+5)) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt16*)(a8+9) != *(UInt16*)(b8+9)) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+11) != *(UInt8*)(b8+11));
			pc+=3; goto o_goyes;

o9fb4: 		RFL(228) a8=A;b8=B; goto tbnefinish;
o9fb5: 		RFL(229) a8=B;b8=C; goto tbnefinish;    // B field, 2 nibbles, [0..1]
o9fb6: 		RFL(230) a8=C;b8=A; goto tbnefinish;
o9fb7: 		RFL(231) a8=D;b8=C; 
			tbnefinish:
			CARRY = (*(UInt16*)a8 != *(UInt16*)b8);
			pc+=3; goto o_goyes;

o9fw4: 		RFL(228) a8=A;b8=B; goto twnefinish;    // W field, 16 nibbles, [0..15]
o9fw5: 		RFL(229) a8=B;b8=C; goto twnefinish;
o9fw6: 		RFL(230) a8=C;b8=A; goto twnefinish;
o9fw7: 		RFL(231) a8=D;b8=C; 
			twnefinish:
			if (*(UInt32*)a8 != *(UInt32*)b8) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+4) != *(UInt32*)(b8+4)) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+8) != *(UInt32*)(b8+8)) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt32*)(a8+12) != *(UInt32*)(b8+12));
			pc+=3; goto o_goyes;





/**********************************************************************************
*                                                                                 *
*         TFz                                                                     *
*                                                                                 *
**********************************************************************************/

o9fp8: 		RFL(232) a8=A; goto tpzfinish;    // P field, 1 nibble, [P]
o9fp9: 		RFL(233) a8=B; goto tpzfinish;
o9fpA: 		RFL(234) a8=C; goto tpzfinish;
o9fpB: 		RFL(235) a8=D; 
			tpzfinish:
			a8+=P;
			CARRY=(*a8==0);
			pc+=3; goto o_goyes;

o9fwp8: 	RFL(232) a8=A; goto twpzfinish;   // WP field, P+1 nibble, [0..P]
o9fwp9: 	RFL(233) a8=B; goto twpzfinish;
o9fwpA: 	RFL(234) a8=C; goto twpzfinish;
o9fwpB: 	RFL(235) a8=D; 
			twpzfinish:
			n=(char)P+1;
			do { if (*a8++!=0) { CARRY = 0; pc+=3; goto o_goyes; } } while (--n);
			CARRY=1;
			pc+=3; goto o_goyes;
			
o9fxs8: 	RFL(232) a8=A; goto txszfinish;   // XS field, 1 nibble, [2]
o9fxs9: 	RFL(233) a8=B; goto txszfinish;
o9fxsA: 	RFL(234) a8=C; goto txszfinish;
o9fxsB: 	RFL(235) a8=D; 
			txszfinish:
			a8+=2;
			CARRY=(*a8==0);
			pc+=3; goto o_goyes;

o9fx8: 		RFL(232) a8=A; goto txzfinish;    // X field, 3 nibbles, [0..2]
o9fx9: 		RFL(233) a8=B; goto txzfinish;
o9fxA: 		RFL(234) a8=C; goto txzfinish;
o9fxB: 		RFL(235) a8=D; 
			txzfinish:
			if (*(UInt16*)a8 != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+2) == 0);
			pc+=3; goto o_goyes;

o9fs8: 		RFL(232) a8=A; goto tszfinish;    // S field, 1 nibble, [15]
o9fs9: 		RFL(233) a8=B; goto tszfinish;
o9fsA: 		RFL(234) a8=C; goto tszfinish;
o9fsB: 		RFL(235) a8=D; 
			tszfinish:
			a8+=15;
			CARRY=(*a8==0);
			pc+=3; goto o_goyes;

o9fm8: 		RFL(232) a8=A; goto tmzfinish;    // M field, 12 nibbles, [3..14]
o9fm9: 		RFL(233) a8=B; goto tmzfinish;
o9fmA: 		RFL(234) a8=C; goto tmzfinish;
o9fmB: 		RFL(235) a8=D; 
			tmzfinish:
			a8+=3;
			if (*(UInt8*)a8 != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+1) != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+5) != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt16*)(a8+9) != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+11) == 0);
			pc+=3; goto o_goyes;

o9fb8: 		RFL(232) a8=A; goto tbzfinish;
o9fb9: 		RFL(233) a8=B; goto tbzfinish;    // B field, 2 nibbles, [0..1]
o9fbA: 		RFL(234) a8=C; goto tbzfinish;
o9fbB: 		RFL(235) a8=D; 
			tbzfinish:
			CARRY = (*(UInt16*)a8 == 0);
			pc+=3; goto o_goyes;

o9fw8: 		RFL(232) a8=A; goto twzfinish;    // W field, 16 nibbles, [0..15]
o9fw9: 		RFL(233) a8=B; goto twzfinish;
o9fwA: 		RFL(234) a8=C; goto twzfinish;
o9fwB: 		RFL(235) a8=D; 
			twzfinish:
			if (*(UInt32*)a8 != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+4) != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+8) != 0) { CARRY = 0; pc+=3; goto o_goyes; }
			CARRY = (*(UInt32*)(a8+12) == 0);
			pc+=3; goto o_goyes;





/**********************************************************************************
*                                                                                 *
*         TFnz                                                                    *
*                                                                                 *
**********************************************************************************/

o9fpC: 		RFL(236) a8=A; goto tpnzfinish;    // P field, 1 nibble, [P]
o9fpD: 		RFL(237) a8=B; goto tpnzfinish;
o9fpE: 		RFL(238) a8=C; goto tpnzfinish;
o9fpF: 		RFL(239) a8=D; 
			tpnzfinish:
			a8+=P;
			CARRY=(*a8!=0);
			pc+=3; goto o_goyes;

o9fwpC: 	RFL(236) a8=A; goto twpnzfinish;   // WP field, P+1 nibble, [0..P]
o9fwpD: 	RFL(237) a8=B; goto twpnzfinish;
o9fwpE: 	RFL(238) a8=C; goto twpnzfinish;
o9fwpF: 	RFL(239) a8=D; 
			twpnzfinish:
			n=(char)P+1;
			do { if (*a8++!=0) { CARRY = 1; pc+=3; goto o_goyes; } } while (--n);
			CARRY=0;
			pc+=3; goto o_goyes;
			
o9fxsC: 	RFL(236) a8=A; goto txsnzfinish;   // XS field, 1 nibble, [2]
o9fxsD: 	RFL(237) a8=B; goto txsnzfinish;
o9fxsE: 	RFL(238) a8=C; goto txsnzfinish;
o9fxsF: 	RFL(239) a8=D; 
			txsnzfinish:
			a8+=2;
			CARRY=(*a8!=0);
			pc+=3; goto o_goyes;

o9fxC: 		RFL(236) a8=A; goto txnzfinish;    // X field, 3 nibbles, [0..2]
o9fxD: 		RFL(237) a8=B; goto txnzfinish;
o9fxE: 		RFL(238) a8=C; goto txnzfinish;
o9fxF: 		RFL(239) a8=D; 
			txnzfinish:
			if (*(UInt16*)a8 != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+2) != 0);
			pc+=3; goto o_goyes;

o9fsC: 		RFL(236) a8=A; goto tsnzfinish;    // S field, 1 nibble, [15]
o9fsD: 		RFL(237) a8=B; goto tsnzfinish;
o9fsE: 		RFL(238) a8=C; goto tsnzfinish;
o9fsF: 		RFL(239) a8=D; 
			tsnzfinish:
			a8+=15;
			CARRY=(*a8!=0);
			pc+=3; goto o_goyes;

o9fmC: 		RFL(236) a8=A; goto tmnzfinish;    // M field, 12 nibbles, [3..14]
o9fmD: 		RFL(237) a8=B; goto tmnzfinish;
o9fmE: 		RFL(238) a8=C; goto tmnzfinish;
o9fmF: 		RFL(239) a8=D; 
			tmnzfinish:
			a8+=3;
			if (*(UInt8*)a8 != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+1) != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+5) != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt16*)(a8+9) != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt8*)(a8+11) != 0);
			pc+=3; goto o_goyes;

o9fbC: 		RFL(236) a8=A; goto tbnzfinish;
o9fbD: 		RFL(237) a8=B; goto tbnzfinish;    // B field, 2 nibbles, [0..1]
o9fbE: 		RFL(238) a8=C; goto tbnzfinish;
o9fbF: 		RFL(239) a8=D; 
			tbnzfinish:
			CARRY = (*(UInt16*)a8 != 0);
			pc+=3; goto o_goyes;

o9fwC: 		RFL(236) a8=A; goto twnzfinish;    // W field, 16 nibbles, [0..15]
o9fwD: 		RFL(237) a8=B; goto twnzfinish;
o9fwE: 		RFL(238) a8=C; goto twnzfinish;
o9fwF: 		RFL(239) a8=D; 
			twnzfinish:
			if (*(UInt32*)a8 != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+4) != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			if (*(UInt32*)(a8+8) != 0) { CARRY = 1; pc+=3; goto o_goyes; }
			CARRY = (*(UInt32*)(a8+12) != 0);
			pc+=3; goto o_goyes;





o9b0: 		RFL(240) TFa(A, B); pc+=3; goto o_goyes;
o9b1: 		RFL(241) TFa(B, C); pc+=3; goto o_goyes;
o9b2: 		RFL(242) TFa(C, A); pc+=3; goto o_goyes;
o9b3: 		RFL(243) TFa(D, C); pc+=3; goto o_goyes;
o9b4: 		RFL(244) TFb(A, B); pc+=3; goto o_goyes;
o9b5: 		RFL(245) TFb(B, C); pc+=3; goto o_goyes;
o9b6: 		RFL(246) TFb(C, A); pc+=3; goto o_goyes;
o9b7: 		RFL(247) TFb(D, C); pc+=3; goto o_goyes;
o9b8: 		RFL(248) TFae(A, B); pc+=3; goto o_goyes;
o9b9: 		RFL(249) TFae(B, C); pc+=3; goto o_goyes;
o9bA: 		RFL(250) TFae(C, A); pc+=3; goto o_goyes;
o9bB: 		RFL(251) TFae(D, C); pc+=3; goto o_goyes;
o9bC: 		RFL(252) TFbe(A, B); pc+=3; goto o_goyes;
o9bD: 		RFL(253) TFbe(B, C); pc+=3; goto o_goyes;
o9bE: 		RFL(254) TFbe(C, A); pc+=3; goto o_goyes;
o9bF: 		RFL(255) TFbe(D, C); pc+=3; goto o_goyes;



oC0: 		RFL(321) pc+=2; PCMINUSONE Nadd5(A, B); goto o_done;
oC1: 		RFL(322) pc+=2; PCMINUSONE Nadd5(B, C); goto o_done;
oC2: 		RFL(323) pc+=2; PCMINUSONE Nadd5(C, A); goto o_done;
oC3: 		RFL(324) pc+=2; PCMINUSONE Nadd5(D, C); goto o_done;
oC4: 		RFL(325) pc+=2; PCMINUSONE Ndbl5(A); goto o_done;
oC5: 		RFL(326) pc+=2; PCMINUSONE Ndbl5(B); goto o_done;
oC6: 		RFL(327) pc+=2; PCMINUSONE Ndbl5(C); goto o_done;
oC7: 		RFL(328) pc+=2; PCMINUSONE Ndbl5(D); goto o_done;
oC8: 		RFL(329) pc+=2; PCMINUSONE Nadd5(B, A); goto o_done;
oC9: 		RFL(330) pc+=2; PCMINUSONE Nadd5(C, B); goto o_done;
oCA: 		RFL(331) pc+=2; PCMINUSONE Nadd5(A, C); goto o_done;
oCB: 		RFL(332) pc+=2; PCMINUSONE Nadd5(C, D); goto o_done;
oCC: 		RFL(333) pc+=2; PCMINUSONE Ndec5(A); goto o_done;
oCD: 		RFL(334) pc+=2; PCMINUSONE Ndec5(B); goto o_done;  
oCE: 		RFL(335) pc+=2; PCMINUSONE Ndec5(C); goto o_done;
oCF: 		RFL(336) pc+=2; PCMINUSONE Ndec5(D); goto o_done;
oD0: 		RFL(337) pc+=2; PCMINUSONE Nzero5(A); goto o_done;
oD1: 		RFL(338) pc+=2; PCMINUSONE Nzero5(B); goto o_done;
oD2: 		RFL(339) pc+=2; PCMINUSONE Nzero5(C); goto o_done;
oD3: 		RFL(340) pc+=2; PCMINUSONE Nzero5(D); goto o_done;
oD4: 		RFL(341) pc+=2; PCMINUSONE Ncopy5(A, B); goto o_done;
oD5: 		RFL(342) pc+=2; PCMINUSONE Ncopy5(B, C); goto o_done;
oD6: 		RFL(343) pc+=2; PCMINUSONE Ncopy5(C, A); goto o_done;
oD7: 		RFL(344) pc+=2; PCMINUSONE Ncopy5(D, C); goto o_done;
oD8: 		RFL(345) pc+=2; PCMINUSONE Ncopy5(B, A); goto o_done;
oD9: 		RFL(346) pc+=2; PCMINUSONE Ncopy5(C, B); goto o_done;
oDA: 		RFL(347) pc+=2; PCMINUSONE Ncopy5(A, C); goto o_done;
oDB: 		RFL(348) pc+=2; PCMINUSONE Ncopy5(C, D); goto o_done;
oDC: 		RFL(349) pc+=2; PCMINUSONE Nxchg5(A, B); goto o_done;
oDD: 		RFL(350) pc+=2; PCMINUSONE Nxchg5(B, C); goto o_done;
oDE: 		RFL(351) pc+=2; PCMINUSONE Nxchg5(C, A); goto o_done;
oDF: 		RFL(352) pc+=2; PCMINUSONE Nxchg5(D, C); goto o_done;
oE0: 		RFL(353) pc+=2; PCMINUSONE Nsub5(A, B); goto o_done;
oE1: 		RFL(354) pc+=2; PCMINUSONE Nsub5(B, C); goto o_done;
oE2: 		RFL(355) pc+=2; PCMINUSONE Nsub5(C, A); goto o_done;
oE3: 		RFL(356) pc+=2; PCMINUSONE Nsub5(D, C); goto o_done;
oE4: 		RFL(357) pc+=2; PCMINUSONE Ninc5(A); goto o_done;
oE5: 		RFL(358) pc+=2; PCMINUSONE Ninc5(B); goto o_done;
oE6: 		RFL(359) pc+=2; PCMINUSONE Ninc5(C); goto o_done;
oE7: 		RFL(360) pc+=2; PCMINUSONE Ninc5(D); goto o_done;
oE8: 		RFL(361) pc+=2; PCMINUSONE Nsub5(B, A); goto o_done;
oE9: 		RFL(362) pc+=2; PCMINUSONE Nsub5(C, B); goto o_done;
oEA: 		RFL(363) pc+=2; PCMINUSONE Nsub5(A, C); goto o_done;
oEB: 		RFL(364) pc+=2; PCMINUSONE Nsub5(C, D); goto o_done;
oEC: 		RFL(365) pc+=2; PCMINUSONE Nrsub5(A, B); goto o_done;
oED: 		RFL(366) pc+=2; PCMINUSONE Nrsub5(B, C); goto o_done;
oEE: 		RFL(367) pc+=2; PCMINUSONE Nrsub5(C, A); goto o_done;
oEF: 		RFL(368) pc+=2; PCMINUSONE Nrsub5(D, C); goto o_done;
oF0: 		RFL(369) pc+=2; PCMINUSONE Nsl5(A); goto o_done;
oF1: 		RFL(370) pc+=2; PCMINUSONE Nsl5(B); goto o_done;
oF2: 		RFL(371) pc+=2; PCMINUSONE Nsl5(C); goto o_done;
oF3: 		RFL(372) pc+=2; PCMINUSONE Nsl5(D); goto o_done;
oF4: 		RFL(373) pc+=2; PCMINUSONE Nsr5(A); goto o_done;
oF5: 		RFL(374) pc+=2; PCMINUSONE Nsr5(B); goto o_done;
oF6: 		RFL(375) pc+=2; PCMINUSONE Nsr5(C); goto o_done;
oF7: 		RFL(376) pc+=2; PCMINUSONE Nsr5(D); goto o_done;
oF8: 		RFL(377) pc+=2; PCMINUSONE Nneg5(A); goto o_done;
oF9: 		RFL(378) pc+=2; PCMINUSONE Nneg5(B); goto o_done;
oFA: 		RFL(379) pc+=2; PCMINUSONE Nneg5(C); goto o_done;
oFB: 		RFL(380) pc+=2; PCMINUSONE Nneg5(D); goto o_done;
oFC: 		RFL(381) pc+=2; PCMINUSONE Nnot5(A); goto o_done;
oFD: 		RFL(382) pc+=2; PCMINUSONE Nnot5(B); goto o_done;
oFE: 		RFL(383) pc+=2; PCMINUSONE Nnot5(C); goto o_done;
oFF: 		RFL(384) pc+=2; PCMINUSONE Nnot5(D); goto o_done;


o8080: 		RFL(133) pc+=4; INTE=1; if (INTD&&(!INTP)) {INTD=0;INTERRUPT("DL");} goto o_done;
o80810: 	RFL(134) pc+=5; if (IN) {if (!INTP) INTERRUPT("RS") else INTD=1;} goto o_done;

o8082X: 	RFL(135) 
			//n=I[4]+1; 
			n=0;
			asm {
				move.b		(a1),d5  // assumptions: n is in d5,a1 -> I[4], we need I[4]
				addq.b		#1,d5
				ext.w		d5
			}
			Ncopy_circular_A_pc_plus_one( ); 
			pc+=5+n; goto o_done;

o8083: 		RFL(136) pc+=4; goto o_done;
o8084n: 	RFL(137) Nbit0_4(A); pc+=5; goto o_done;
o8085n: 	RFL(138) Nbit1_4(A); pc+=5; goto o_done;
o8086n: 	RFL(139) Tbit0_4(A); pc+=5; goto o_goyes;
o8087n:		RFL(140) Tbit1_4(A); pc+=5; goto o_goyes;
o8088n: 	RFL(141) Nbit0_4(C); pc+=5; goto o_done;
o8089n: 	RFL(142) Nbit1_4(C); pc+=5; goto o_done;
o808An: 	RFL(143) Tbit0_4(C); pc+=5; goto o_goyes;
o808Bn: 	RFL(144) Tbit1_4(C); pc+=5; goto o_goyes;
o808C: 		RFL(145) pc = Npack5I0ASM(GetRealAddressNoIO(Npack5AASM())); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o808D: 		RFL(146) pc+=4; goto o_done;
o808E: 		RFL(147) pc = Npack5I0ASM(GetRealAddressNoIO(Npack5CASM())); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o808F: 		RFL(148) pc+=4; INTE = 0; goto o_done;

o818f0x: 	RFL(164) NFaddconstant(A); pc+=6; goto o_done;
o818f1x: 	RFL(165) NFaddconstant(B); pc+=6; goto o_done;
o818f2x: 	RFL(166) NFaddconstant(C); pc+=6; goto o_done;
o818f3x: 	RFL(167) NFaddconstant(D); pc+=6; goto o_done;
o818f8x: 	RFL(168) NFsubconstant(A); pc+=6; goto o_done;
o818f9x: 	RFL(169) NFsubconstant(B); pc+=6; goto o_done;
o818fAx: 	RFL(170) NFsubconstant(C); pc+=6; goto o_done;
o818fBx: 	RFL(171) NFsubconstant(D); pc+=6; goto o_done; 	
o819f0: 	RFL(172) NFsrb(A); pc+=5; goto o_done;
o819f1: 	RFL(173) NFsrb(B); pc+=5; goto o_done;
o819f2: 	RFL(174) NFsrb(C); pc+=5; goto o_done;
o819f3: 	RFL(175) NFsrb(D); pc+=5; goto o_done;

o81Af00: 	RFL(385) a8=R0;b8=A; goto nfcopyfinish;
o81Af01: 	RFL(386) a8=R1;b8=A; goto nfcopyfinish;
o81Af02: 	RFL(387) a8=R2;b8=A; goto nfcopyfinish;
o81Af03: 	RFL(388) a8=R3;b8=A; goto nfcopyfinish;
o81Af04: 	RFL(389) a8=R4;b8=A; goto nfcopyfinish;
o81Af08: 	RFL(390) a8=R0;b8=C; goto nfcopyfinish;
o81Af09: 	RFL(391) a8=R1;b8=C; goto nfcopyfinish;
o81Af0A: 	RFL(392) a8=R2;b8=C; goto nfcopyfinish;
o81Af0B: 	RFL(393) a8=R3;b8=C; goto nfcopyfinish;
o81Af0C: 	RFL(394) a8=R4;b8=C; goto nfcopyfinish;
o81Af10: 	RFL(395) a8=A;b8=R0; goto nfcopyfinish;
o81Af11: 	RFL(396) a8=A;b8=R1; goto nfcopyfinish;
o81Af12: 	RFL(397) a8=A;b8=R2; goto nfcopyfinish;
o81Af13: 	RFL(398) a8=A;b8=R3; goto nfcopyfinish;
o81Af14: 	RFL(399) a8=A;b8=R4; goto nfcopyfinish;
o81Af18: 	RFL(400) a8=C;b8=R0; goto nfcopyfinish;
o81Af19: 	RFL(401) a8=C;b8=R1; goto nfcopyfinish;
o81Af1A: 	RFL(402) a8=C;b8=R2; goto nfcopyfinish;
o81Af1B: 	RFL(403) a8=C;b8=R3; goto nfcopyfinish;
o81Af1C: 	RFL(404) a8=C;b8=R4;
			nfcopyfinish:pc+=6;
			//assumptions: a2 = a8, a3 = b8, a1 -> I[6]
			asm 
				{
					move.b  -3(a1),d0	// we need I[3]
					ext.w	d0
					add.w	d0,d0
					jmp		2(pc,d0)
			nfcj1:	bra.s   nfcopy0
					bra.s   nfcopy1
					bra.s   nfcopy2
					bra.s   nfcopy3
					bra.s   nfcopy4
					bra.s   nfcopy5
					bra.s   nfcopy6
					bra.s   nfcopy7
					bra.s   nfcopyexit
					bra.s   nfcopyexit
					bra.s   nfcopyexit
					bra.s   nfcopyexit
					bra.s   nfcopyexit	
					bra.s   nfcopyexit		
					bra.s   nfcopyexit		
					bra.s   nfcopyf		
				}

			nfcopy0:	*(a8+P)=*(b8+P);				// P field (nibble P) 
						goto o_done;
			nfcopy1:	n=(char)P+1;
						do {*a8++=*b8++;} while (--n);
						goto o_done;
			nfcopy2:	*(a8+2)=*(b8+2);    			// XS field (nibble 2)
						goto o_done;
			nfcopy3:	asm								// X field (nibbles 0 through 2)
						{
							move.w (a3)+,(a2)+
							move.b (a3),(a2)
						}
						goto o_done;
			nfcopy4:	*(a8+15)=*(b8+15);				// S field (nibble 15)
						goto o_done;
			nfcopy5:	a8+=3;b8+=3;					// M field (nibble 3 through 14)
						asm 
						{
							move.b (a3)+,(a2)+
							move.l (a3)+,(a2)+
							move.l (a3)+,(a2)+
							move.w (a3)+,(a2)+
							move.b (a3),(a2)
						}
						goto o_done;
			nfcopy6:	asm  							// B field (nibbles 0 through 1)
						{
							move.w (a3),(a2)
						}
						goto o_done;
			nfcopy7:	asm								// W field (nibbles 0 through 15)
						{
							move.l (a3)+,(a2)+
			 				move.l (a3)+,(a2)+
							move.l (a3)+,(a2)+
							move.l (a3),(a2)
						}
						goto o_done;
			nfcopyf: 	asm								// A field (nibbles 0 through 4)
						{
							move.l (a3)+,(a2)+
							move.b (a3),(a2)
						}
			nfcopyexit: goto o_done;


o81Af20: 	RFL(405) a8=A;b8=R0; goto nfxchgfinish;
o81Af21: 	RFL(406) a8=A;b8=R1; goto nfxchgfinish;
o81Af22: 	RFL(407) a8=A;b8=R2; goto nfxchgfinish;
o81Af23: 	RFL(408) a8=A;b8=R3; goto nfxchgfinish;
o81Af24: 	RFL(409) a8=A;b8=R4; goto nfxchgfinish;
o81Af28: 	RFL(410) a8=C;b8=R0; goto nfxchgfinish;
o81Af29: 	RFL(411) a8=C;b8=R1; goto nfxchgfinish;
o81Af2A: 	RFL(412) a8=C;b8=R2; goto nfxchgfinish;
o81Af2B: 	RFL(413) a8=C;b8=R3; goto nfxchgfinish;
o81Af2C: 	RFL(414) a8=C;b8=R4;
			nfxchgfinish:pc+=6;c8=X;
			//assumptions: a2 = a8, a3 = b8,a4 = c8, a1 -> I[6]	
			asm 
				{
					move.b  -3(a1),d0	// we need I[3]
					ext.w	d0
					add.w	d0,d0
					add.w	d0,d0
					jmp		2(pc,d0)
					jmp   	nfxchg0
					jmp		nfxchg1
					jmp		nfxchg2
					jmp		nfxchg3
					jmp		nfxchg4
					jmp		nfxchg5
					jmp		nfxchg6
					jmp		nfxchg7
					jmp		nfxchgexit
					jmp		nfxchgexit
					jmp		nfxchgexit
					jmp		nfxchgexit
					jmp		nfxchgexit	
					jmp		nfxchgexit		
					jmp		nfxchgexit		
					jmp		nfxchgf		
				}

			nfxchg0:	*c8=*(a8+P);				// P field (nibble P) 
						*(a8+P)=*(b8+P);
						*(b8+P)=*c8;
						goto o_done;
			nfxchg1:	n=(char)P+1;						// WP field (nibble0 through P)
						do {*c8=*a8;*a8++=*b8;*b8++=*c8++;} while (--n);
						goto o_done;
			nfxchg2:	*c8=*(a8+2);    			// XS field (nibble 2)
						*(a8+2)=*(b8+2);
						*(b8+2)=*c8;
						goto o_done;
			nfxchg3:	asm								// X field (nibbles 0 through 2)
						{
							move.w (a2),(a4)
							move.w (a3),(a2)+
							move.w (a4)+,(a3)+
							move.b (a2),(a4)
							move.b (a3),(a2)
							move.b (a4),(a3)
						}
						goto o_done;
			nfxchg4:	*c8=*(a8+15);				// S field (nibble 15)
						*(a8+15)=*(b8+15);
						*(b8+15)=*c8;
						goto o_done;
			nfxchg5:	a8+=3;b8+=3;c8+=3;				// M field (nibble 3 through 14)
						asm 
						{
							move.b (a2),(a4)
							move.b (a3),(a2)+
							move.b (a4)+,(a3)+
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.w (a2),(a4)
							move.w (a3),(a2)+
							move.w (a4)+,(a3)+
							move.b (a2),(a4)
							move.b (a3),(a2)
							move.b (a4),(a3)
						}
						goto o_done;
			nfxchg6:	asm  							// B field (nibbles 0 through 1)
						{
							move.w (a2),(a4)
							move.w (a3),(a2)
							move.w (a4),(a3)
						}
						goto o_done;
			nfxchg7:	asm								// W field (nibbles 0 through 15)
						{
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.l (a2),(a4)
							move.l (a3),(a2)
							move.l (a4),(a3)
						}
						goto o_done;
			nfxchgf: 	asm								// A field (nibbles 0 through 4)
						{
							move.l (a2),(a4)
							move.l (a3),(a2)+
							move.l (a4)+,(a3)+
							move.b (a2),(a4)
							move.b (a3),(a2)
							move.b (a4),(a3)
						}
			nfxchgexit: goto o_done;
			



o81B1: 		RFL(415) EmulateBeep(); PCHANGED rstkpop_to_a1(); goto o_done;
o81B2: 		RFL(416) pc=Npack5AASM(); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o81B3: 		RFL(417) pc=Npack5CASM(); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o81B4: 		RFL(418) pc+=4; Nunpack5(A, pc); goto o_done;
o81B5: 		RFL(419) pc+=4; Nunpack5(C, pc); goto o_done;
o81B6: 		RFL(420) d = pc+4; pc=Npack5AASM(); Nunpack5(A, d); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3
o81B7: 		RFL(421) d = pc+4; pc=Npack5CASM(); Nunpack5(C, d); ResetPCinRegA1(pc); goto o_done;  // assumptions: pc is in d3

goto o_done;
}

